//// File / Fichier :   HGPLAbout.cpp
//// Copyright (C) 2004 Hugues Vanden Bossche
//// Last modification / Dernire modification : 08/01/2004
//// Implementation of a function GPLabout that display a typical "About"
//// windows including a view of the GPL, with fltk

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// =============================================================================  

#include "HGPLAbout.h"
using namespace std;

//==============================================================================
//  Const declaration
//==============================================================================

const char* GPLFILE = "resources/gpl.htm";
const char* LOGOFILE = "resources/logo.bmp";
const int AX = 450;
const int AY = 400;

//==============================================================================
//  Classes declaration
//==============================================================================

/// Class : AboutWin
/// This class is used to pop-up a "About" window that includes a BMP file
class AboutWindow : public Fl_Window
{   ostringstream os;
    Fl_BMP_Image *logo; 
  public :
  // constructor & destructor
  AboutWindow(const char *, const char *);
  ~AboutWindow()
  { delete logo; }
  // other functions
  void draw();
};

//==============================================================================
//  Member functions definition
//==============================================================================

/// Class : AboutWin
/// This class is used to pop-up a "About" window that includes a BMP file

// constructor
AboutWindow::AboutWindow(const char *thePrgName, const char *theFileName):
Fl_Window(AX,AY)
{
  os.str("");
  os << "About " << thePrgName;
  label(os.str().c_str());
  
  logo = new Fl_BMP_Image(theFileName);
}

//dessin
void AboutWindow::draw()
{
  Fl_Window::draw();
  logo->draw(10,20); 
}

//==============================================================================
//  Eventually, the function definition
//==============================================================================

/// Function : GPLAbout()
/// draw a About fonction including the GPL Lisence
void GPLAbout(const char* prgm, const char* version, int yyyy, const char* author)
{
  AboutWindow *win = new AboutWindow(prgm, LOGOFILE);
#ifdef rsr  
  // Display informations
  ostringstream os;
  Fl_Box *info;
  
  // program name
  info = new Fl_Box(AX/2-30, 20, AX/2+30, 50, prgm);
  info->labelsize(20);
  info->labelfont(FL_HELVETICA_BOLD_ITALIC);
  info->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
  info->labelcolor(HV_TURQUOISE);
  
  // Version and Copyright
  os << "Version " << version << endl;
  os << "Copyright (C) " << yyyy << " " << author;
  info = new Fl_Box(AX/2-30, 70, AX/2+30, 50, os.str().c_str());
  info->labelsize(11);
  info->labelfont(FL_HELVETICA_BOLD);
  info->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  info->labelcolor(HV_TURQUOISE);

  // Get the GPL text from a file and display it
  Fl_Help_View *helpView = new Fl_Help_View(0, 170, AX, AY-170);
  helpView->color(HV_GRAY_LL);
  helpView->load(GPLFILE);
#endif
  win->end ();
  win->show ();
}

