//// PoloCalc for windows
//// File / Fichier :   HInterface.cpp
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 18/05/2003
//// This file defines specifics graphics classes, derived from fltk
//// to build the calculator interface

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// =============================================================================  

#include "HInterface.h"

// =============================================================================
//  Global variables
// =============================================================================

extern HV_SmartInput *inpt;
extern HV_TextScroll *outpt;

// ============================================================================
// Definition of functions
// ============================================================================

/// Class : HDigitKey
/// virtual key that generates the echo of a char on the target field

// constructors
HV_DigitKey::HV_DigitKey(int x, int y, int w, int h, const char *label, char theDigit)
: Fl_Button(x, y, w, h, label)
{
  color( HV_KEYC );
  labelfont( HV_KEYF );
  labelsize( HV_KEYLS );
  labelcolor( HV_KEYTC );
  align( FL_ALIGN_CENTER );
  box( FL_PLASTIC_UP_BOX );
  digit = theDigit;
  callback((Fl_Callback *)forwardDigit);  
}

HV_DigitKey::HV_DigitKey(int x, int y, const char *label, char theDigit)
: Fl_Button(x, y, HV_KEYW, HV_KEYH, label)
{
  color( HV_KEYC );
  labelfont( HV_KEYF );
  labelsize( HV_KEYLS );
  labelcolor( HV_KEYTC );
  align( FL_ALIGN_CENTER );
  box( FL_PLASTIC_UP_BOX );
  digit = theDigit;
  callback((Fl_Callback *)forwardDigit, NULL);  
}

/// Class : HOperationKey
/// virtual key that launches an action

// constructors
HV_OperationKey::HV_OperationKey(int x, int y, int w, int h, const char *label, Fl_Callback* action)
: Fl_Button(x, y, w, h, label)
{
  color( HV_KEYC );
  labelfont( HV_KEYF );
  labelsize( HV_KEYLS );
  labelcolor( HV_KEYTC );
  align( FL_ALIGN_CENTER );
  box( FL_PLASTIC_UP_BOX );
  callback(action);  
}

HV_OperationKey::HV_OperationKey(int x, int y, const char *label, Fl_Callback *action)
: Fl_Button(x, y, HV_KEYW, HV_KEYH, label)
{
  color( HV_KEYC );
  labelfont( HV_KEYF );
  labelsize( HV_KEYLS );
  labelcolor( HV_KEYTC );
  align( FL_ALIGN_CENTER );
  box( FL_PLASTIC_UP_BOX );
  callback(action);  
}

void HV_OperationKey::overLabel(const char *label)
{
  int X = x(), Y = y(), W=w(), H=h()/2;
  Fl_Box *labelBox = new Fl_Box(X,Y-H,W,H,label);
  labelBox->labelsize(8);
  labelBox->labelcolor(HV_ORANGE_L);
  labelBox->labelfont(FL_HELVETICA);  
  parent()->add(labelBox);
}  

/// Class : HDoubleOperationKey
/// Like operation key, but is able to lauche two actions according to its state

// constructors
HV_DoubleOperationKey::HV_DoubleOperationKey( int x, int y , int w, int h, const char *label1 , const char *label2, Fl_Callback* action)
: HV_OperationKey(x,y,w,h,label1,action) 
{
  normalLabel = label1;  
  otherLabel = label2;
  inversed = false;  
  labelfont( FL_HELVETICA );
  labelsize( HV_KEYLS -2);  
}

HV_DoubleOperationKey::HV_DoubleOperationKey( int x, int y , const char *label1 , const char *label2, Fl_Callback* action):
HV_OperationKey(x, y, HV_KEYW, HV_KEYSH, label1, action)
{
  normalLabel = label1;  
  otherLabel = label2;
  inversed = false;
  labelfont( FL_HELVETICA );
  labelsize( HV_KEYLS -2);   
}

// inversion
void HV_DoubleOperationKey::inverse()
{
  if(inversed)
  {
    inversed = false;
    labelcolor( HV_KEYTC );
    label(normalLabel);
    redraw();
  }
  else
  {
    inversed = true;
    labelcolor( HV_KEYINV );
    label(otherLabel);
    redraw();
  }
}

// ============================================================================
// Other function 
// ============================================================================

/// function forwardDigit() : Transmission of a character to the input field
void forwardDigit(HV_DigitKey *theKey, void* nothing)
{
  char digit[2];
  digit[0] = theKey->getDigit();
  digit[1]='\0';
  inpt->insert(digit);
  inpt->take_focus();
}
