//// PoloCalc for windows
//// File / Fichier :   HInterface.h
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 18/05/2003
//// This file defines specifics graphics classes, derived from fltk
//// to build the calculator interface

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// =============================================================================

#ifndef HINTERFACE
#define HINTERFACE 1

#include <FL/Fl.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>
#include <string>
#include <vector>
#include <algorithm> 
#include <iostream>
#include <fstream>

#include "HVSmartInput.h"
#include "HVTextScroll.h"
#include "HVColor.h"

// ============================================================================
// Constantes definitions / Definition des constantes
// ============================================================================

// Button size
const int HV_KEYW = 35;
const int HV_KEYH = 22;
const int HV_KEYSH = 16;
const int HV_KEYLS = 12;

// Size data for the screen
const int HV_MENUH = 25;
const int HV_MENUTS = 9;
const int HV_SPACE = 15;
const int HV_SSPACE = 5;
const int HV_OUTH = 100;
const int HV_OUTW = 4*HV_KEYW+3*HV_SPACE;
const int HV_INH = 25;
const int HV_BOARDW = 4*HV_KEYW+5*HV_SPACE;
const int HV_BOARDH = 9*HV_KEYH+3*HV_SPACE+12*HV_SSPACE+HV_MENUH+HV_OUTH+HV_INH;

// Color Data
const Fl_Color  HV_OUTPTC = HV_GRAY;
const Fl_Color  HV_OUTPTTC = HV_ORANGE_L;
const Fl_Color  HV_INPTC = HV_GRAY_D;
const Fl_Color  HV_INPTTC = HV_ORANGE;
const Fl_Color  HV_WINDC = HV_TURQUOISE_D;
const Fl_Color  HV_KEYC = HV_GREEN_D;
const Fl_Color  HV_KEYTC = HV_PURPLE_D;
const Fl_Color  HV_KEYINV = HV_GREEN;

// Text style
const Fl_Font HV_KEYF = FL_HELVETICA_BOLD;
const Fl_Font HV_OUTPTF = FL_COURIER;
const Fl_Font HV_INPTF = FL_HELVETICA_BOLD;


// ============================================================================
// Classes definition / Dfinition des classes
// ============================================================================

/// Class : HDigitKey
/// Classe derived from Fl_Button, to insert enter a new digit on the
/// Input field of the calculator
class HV_DigitKey : public Fl_Button
{
  private :
    char digit;    // comporte le digit  retourner

  public :
    // constructor & destructor
    HV_DigitKey( int, int, int , int , const char*, char);
    HV_DigitKey( int, int, const char*, char);    
    ~HV_DigitKey() {}
    
    // access to member
    char getDigit()
    { return digit; }
    
};

/// Class : HOperationKey
/// Classe derived from Fl_Button, to peform an operation 
class HV_OperationKey : public Fl_Button
{
  public :
    HV_OperationKey( int, int , int, int, const char*, Fl_Callback*);
    HV_OperationKey( int, int, const char*, Fl_Callback*);  
    ~HV_OperationKey() {}
    void overLabel(const char *);  
};

/// Class : HV_DoubleOperationKey
/// Action key that can have two states
class HV_DoubleOperationKey : public HV_OperationKey
{
  private :
    bool inversed;
    const char * normalLabel;
    const char * otherLabel;
  public :
    HV_DoubleOperationKey( int, int , int, int, const char*, const char*, Fl_Callback*);
    HV_DoubleOperationKey( int, int , const char*, const char*, Fl_Callback*);
    ~HV_DoubleOperationKey() {}
    bool isInversed() 
    { return inversed; }
    void inverse();    
    
};

// ============================================================================
// Functions definition / Dfinition des fonctions
// ============================================================================

/// function forwardDigit() : permet de transmettre le caractre de la 
/// touche frappe  une cible de type input
void forwardDigit( HV_DigitKey *, void *);

#endif
