//// PoloCalc for windows
//// File / Fichier :   HV_List.cpp
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 18/05/2003
//// This file defines the list of data and the various kind of data that
//// are used in calculator PoloCalc 

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// =============================================================================

#include "HVList.h"
#include "HVTextScroll.h"

#define M_PI 3.14159265358979f

//==============================================================================
//  Global variables
//==============================================================================

extern HV_TextScroll *outpt;

// =============================================================================
// Dfinition des fonctions membres des classes
// =============================================================================

/// Class : HV_List
/// Liste stockant les donnes

// push() : instance d'une classe  determiner et insertion dans la liste
void HV_List::push(const char * theValue)
{
  char* error;
  double theDouble = strtod(theValue, &error);
  if(*error)
  {
    HV_Text * Ptr = new HV_Text(theValue);
    push_front(Ptr);   
  }
  else
    push(theDouble);  
 
}

// push() : instance d'une classe HV_Data et insertion dans la liste
void HV_List::push(double theValue)
{
  HV_Double* Ptr = new HV_Double(theValue);
  push_front(Ptr);
}

// pushCopy() : instance a appropriate HData to insert a copy of an element
void HV_List::pushCopy(const HV_DataPtr theData)
{
  if(theData)
  {
    char dataType = theData->who();
    switch(dataType)
    {
      case HV_DOUBLE :
        push(((HV_Double*)theData)->getValue());
        break;
      case HV_TEXT :
        push(((HV_Text*)theData)->getStr().c_str());
        break;
      default :
        break;            
    }
  }
}

// whoIsFirst() : retourne la nature du premier lment de la liste
char HV_List::whoIsFirst()
{
  if(!size())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);
  }
  HV_Data* Ptr = front();
  return Ptr->who();
}

// whoAreFirsts() : retourne la nature combine des deux premiers lment 
char HV_List::whoAreFirsts()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);
  }
    
  iterator it = begin();
  HV_Data* Ptr = (*it);
  char firstWho, secondWho;
  
  firstWho = Ptr->who();
  it++;
  Ptr = (*it);
  secondWho = Ptr->who();
  
  return (firstWho|secondWho);
}

// getFirstValue() : retourne la valeur du premier lment de la liste
double HV_List::getFirstValue()
{
  if(!size())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);
  } 
    
  HV_DataPtr Ptr = front();
  if (Ptr->who()!=HV_DOUBLE)
  {
    HExcptWrongType Excpt(HV_DOUBLE);
    throw(Excpt);
  }
  else
    return ((HV_Double*)Ptr)->getValue();    
}

// getFirstStr() : first Data of the list as string
string HV_List::getFirstStr()
{
  if(!size())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);
  } 
    
  HV_DataPtr Ptr = front();
  return Ptr->getStr();    
}

// getSecondValue() : retourne la valeur du second lment de la liste
double HV_List::getSecondValue()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);
  }
  
  HV_List::iterator it = begin();
  it++;
  HV_DataPtr Ptr = *it;
  if (Ptr->who()!=HV_DOUBLE)
  {
    HExcptWrongType Excpt(HV_DOUBLE);
    throw(Excpt);
  }
  else
    return ((HV_Double*)Ptr)->getValue();
}

// getSecondStr() : : first Data of the list as string
string HV_List::getSecondStr()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);
  }
  
  HV_List::iterator it = begin();
  it++;
  HV_DataPtr Ptr = *it;
  return Ptr->getStr();
}

// getValue() : retourne la valeur du nbime lment de la liste
double HV_List::getValue(unsigned nb)
{
  if(size()<nb)
  {
    HExcptWrongNb Excpt(nb);
    throw(Excpt);
  }
    
  HV_List::iterator it = begin();
  for(unsigned i=1; i<nb; i++)
    it++;
  HV_DataPtr Ptr = *it;  
  if (Ptr->who()!=HV_DOUBLE)
  {
    HExcptWrongType Excpt(HV_DOUBLE);
    throw(Excpt);
  }
  else
    return ((HV_Double*)Ptr)->getValue();
}

// toString() : return a string with all the elements of the list
string  HV_List::toString()
{
  if(empty())
    return "";
  else  
  {
// rsr: Ouch! was memory stack error when returned const char*
	ostringstream os;
    HV_List::reverse_iterator it;
    for( it=rbegin(); it!=rend(); it++)
    {
      HV_DataPtr Ptr = *it;
      string data=Ptr->getStr();
      os << data << endl;
    }
    string data=os.str();
    return data;
  }
}

// toFile() : Write the content of the list into a file
void  HV_List::toFile(const char * theFile)
{
  ofstream target(theFile);
  if(target.is_open()) 
  {  
    HV_List::reverse_iterator it;
    HV_DataPtr Ptr;
    for( it=rbegin(); it!=rend(); it++)
    {
      Ptr = *it;
      target << Ptr->getStr() << endl;
    }
    target.close();
  }
  else
  {
    HExcptFile Excpt(theFile);
    throw(Excpt);
  }
}

//copy : copy the content of the list in another one
void HV_List::copyTo(HV_List* target)
{
  if(!target->empty())
    target->clear();
  if(!empty())
  {  
    HV_List::reverse_iterator it;
    HV_DataPtr Ptr;
    for( it=rbegin(); it!=rend(); it++)
    {
      Ptr = *it;      
      target->push(Ptr->getStr().c_str());
    }  
  }     
}

// fromFile() : Write the content of the file into the list
void  HV_List::fromFile(const char * theFile)
{
  ifstream source(theFile);
  if(source.is_open()) 
  {   
    string theData;
    do
    {
      source >> theData;
      if(!source.fail())
        push(theData.c_str());
    }
    while (!source.eof()) ;
    source.close();
  }
  else
  {
    HExcptFile Excpt(theFile);
    throw(Excpt);
  }
}



// drop() : destroy first element of the list
int HV_List::drop()
{
  if(size())
  {
    HV_DataPtr Ptr=front();
    delete Ptr;
    pop_front();
    return 1;
  }
  else
  return 0;
}

// clear() : destroy of the whole list
void HV_List::clear()
{
  while(drop());
}

// dup() : copy the data of the top of the list
int HV_List::dup(int nb)
{
  if(nb && !empty()) 
  {
    HV_List::iterator it = begin(); 
    int i = min((int)size(),nb), j=i;
    while(--i)
      it++; 
    while(j--)
    {
      pushCopy(*it);
      it--;
    }  
    return 1;
  }
  else
    return 0;
}

// add() : addition des deux premiers lments de la liste
void HV_List::add()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);  
  }

  char who = whoAreFirsts();
  switch( who )
  {
    // classical situation
    case (HV_DOUBLE|HV_DOUBLE) :
    {
      double result;
      result = getFirstValue() + getSecondValue();
      drop(); drop();
      push(result);
    }  
    break;
    
    // append text data        
    case (HV_DOUBLE|HV_TEXT) : 
    case (HV_TEXT|HV_TEXT) : 
    {
      ostringstream os;
      os << getSecondStr() << getFirstStr();
      drop(); drop();
      push(os.str().c_str());
    }  
    break;
    
    // wrong types  
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// sub() : substraction of the two top first elements in the list
void HV_List::sub()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);  
  }

  char who = whoAreFirsts();
  switch( who )
  {
    // classical situation
    case (HV_DOUBLE|HV_DOUBLE) :
      double result;
      result = getSecondValue() - getFirstValue() ;
      drop(); drop();
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// mul() : Multiplication of the two top first elements in the list
void HV_List::mul()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);  
  }

  char who = whoAreFirsts();
  switch( who )
  {
    // classical situation
    case (HV_DOUBLE|HV_DOUBLE) :
      double result;
      result = getFirstValue() * getSecondValue();
      drop(); drop();
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// div() : division of the two top first elements in the list
void HV_List::div()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);  
  }
  char who = whoAreFirsts();
  switch( who )
  {
    // classical situation
    case (HV_DOUBLE|HV_DOUBLE) :
      if(getFirstValue())
      {
        double result;
        result = getSecondValue() / getFirstValue();
        drop(); drop();
        push(result);
      }
      else
      {
        HExcptDivBy0 Excpt;
        throw(Excpt);
      } 
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// pow() : second value power first value
void HV_List::pw()
{
  if(size()<2)
  {
    HExcptWrongNb Excpt(2);
    throw(Excpt);  
  }
  char who = whoAreFirsts();
  switch( who )
  {
    // classical situation
    case (HV_DOUBLE|HV_DOUBLE) :  
      if(!(getSecondValue())&&(getFirstValue()<0))
      {
        HExcptDivBy0 Excpt;
        throw(Excpt);        
      }
      else
      {   
        double result;
        result = pow(getSecondValue(),getFirstValue());
        drop(); drop();
        push(result);     
      }  
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// opp() : transforms a number into its opposite
void HV_List::opp()
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }

  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case (HV_DOUBLE) :
      double result;
      result = - getFirstValue() ;
      drop();
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// inv() : put inverse of the first value at top of list
void HV_List::inv()
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      if(getFirstValue())
      {
        double result;
        result = 1 / getFirstValue();
        drop();
        push(result);
      }
      else
      {
        HExcptDivBy0 Excpt;
        throw(Excpt);
      } 
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// ln() : log neperien of last element
void HV_List::ln()
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      if(getFirstValue())
      {
        double result;
        result = log(getFirstValue());
        drop();
        push(result);
      }
      else
      {
        HExcptlog0 Excpt;
        throw(Excpt);
      } 
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// llog10() : decimal log of last element
void HV_List::llog10()
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      if(getFirstValue())
      {
        double result;
        result = log10(getFirstValue());
        drop();
        push(result);
      }
      else
      {
        HExcptlog0 Excpt;
        throw(Excpt);
      } 
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// lexp() : exp of last element
void HV_List::lexp()
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = exp(getFirstValue());
      drop();
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// pw10() : last element rized at power 10
void HV_List::pw10()
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = pow(double(10),getFirstValue());
      drop();
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// lcos() : cos of last element
void HV_List::lcos(HV_TRIGO unit)
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = getFirstValue();
      drop();
      if(unit==DEG)
        result = result / 180 * M_PI;
      if(unit==GRAD)  
        result = result / 200 * M_PI;      
      push(cos(result));
      break;        
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// lacos() : acos of last element
void HV_List::lacos(HV_TRIGO unit)
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = acos(getFirstValue());
      drop();
      if(unit==DEG)
        result = result / 180 * M_PI;
      if(unit==GRAD)  
        result = result / 200 * M_PI;      
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// lsin() : sin of last element
void HV_List::lsin(HV_TRIGO unit)
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = getFirstValue();
      drop();
      if(unit==DEG)
        result = result / 180 * M_PI;
      if(unit==GRAD)  
        result = result / 200 * M_PI;      
      push(sin(result));
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// ltan() : tangente of last element
void HV_List::ltan(HV_TRIGO unit)
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = getFirstValue();
      drop();
      if(unit==DEG)
        result = result / 180 * M_PI;
      if(unit==GRAD)  
        result = result / 200 * M_PI;      
      push(tan(result));
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// lasin() : asin of last element
void HV_List::lasin(HV_TRIGO unit)
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = asin(getFirstValue());
      drop();
      if(unit==DEG)
        result = result / 180 * M_PI;
      if(unit==GRAD)  
        result = result / 200 * M_PI;      
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// latan() : tangente of last element
void HV_List::latan(HV_TRIGO unit)
{
  if(empty())
  {
    HExcptWrongNb Excpt(1);
    throw(Excpt);  
  }
  char who = whoIsFirst();
  switch( who )
  {
    // classical situation
    case HV_DOUBLE :
      double result;
      result = atan(getFirstValue());
      drop();
      if(unit==DEG)
        result = result / 180 * M_PI;
      if(unit==GRAD)  
        result = result / 200 * M_PI;      
      push(result);
      break;  
      
    default :
      HExcptWrongType Excpt(HV_DOUBLE);
      throw(Excpt);
      break;    
  } 
}

// swap() : exchange of the two top first elements in the list
int HV_List::swap()
{
  if(size()<2)
    return 0;  
  else
  {
    HV_DataPtr ptrFirst = (HV_DataPtr) front();
    pop_front();    
    HV_DataPtr ptrSecond = (HV_DataPtr) front();
    pop_front();
    push_front(ptrFirst);    
    push_front(ptrSecond);   
    return 1;
  } 
}

// pick() : get the x ime lement in the list
int HV_List::pick(bool era)
{
  if(empty())
    return 0;
  else
  {
    unsigned nb = abs((int) getFirstValue());
    if((nb)&&!(size()<nb))
    {
      drop();
      HV_List::iterator it = begin(); 
      for(unsigned i=1; i<nb; i++) 
        it++;   
      HV_Data* Ptr = (*it);  
      pushCopy(Ptr);
      
      if(era)
        erase(it);   
      return 1;              
    }
    else
      return 0;
  }   
}


