//// PoloCalc for windows
//// File / Fichier :   HV_List.h
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 18/05/2003
//// This file declares the list of data and the various kind of data that
//// are used in calculator PoloCalc 

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// =============================================================================

#ifndef HVList
#define HVList 1

#include <list>
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <math.h>

using namespace std;

// ============================================================================
// Constantes definitions / Definition des constantes
// ============================================================================

const char HV_DATA = 1;   // retourn par la classe mre de toutes les donnes
                          // lors d'une appel de la fonction who()
const char HV_DOUBLE = 2; // retourn par la classe HFloat
const char HUNIT = 4;     // retourn par la classe HUnit
const char HV_TEXT = 9;   // retourn par la classe HText 
enum HV_TRIGO {DEG, RAD, GRAD}; // mesure d'angles

// ============================================================================
// Classes definition / Dfinition des classes
// ============================================================================

/// Class : HV_Data
/// Classe mre des autres classes encapsulants les donnes introduites dans
/// la liste - C'est une classe abstraite
class HV_Data
{
  public :
    virtual char who()
    { return HV_DATA;}
    virtual string getStr()=0;
};

/// Definition d'un type pour les facilits de l'criture
typedef HV_Data* HV_DataPtr;

/// Class : HV_Double
/// Classe permettant de stocker des donnes numriques
class HV_Double : public HV_Data
{
    double value;  // valeur stocke 
  	ostringstream os;
  public :
    HV_Double(double theValue)
    { value = theValue; }
    ~HV_Double()
    {}
    char who()
    { return HV_DOUBLE; }
    double getValue()
    { return value; }
/// Storage of float value (double precision)
	string HV_Double::getStr()
	{	ostringstream os;
		os << value;
		if(0!=os.bad())
		{	return "Internal Error";
		}
		return os.str();
	}
};

/// Class : HV_Text
/// Classe permettant de stocker des donnes alphanumriques
class HV_Text : public HV_Data
{
  private :
    string value;  // valeur stocke 
  public :
    HV_Text(const char * theValue)
    { value = theValue; } 
    ~HV_Text()
    {}
    char who()
    { return HV_TEXT; }
    string getStr()
   { return value; }
};

/// Class : HV_List
/// Liste stockant les donnes 
class HV_List : public list<HV_DataPtr> 
{	ostringstream os;
  public :
    void push(const char *);  // ajoute un lment, instance  dterminer
    void push(double);        // ajoute une instance du type HV_Double
    char whoIsFirst();        // nature du premier lment de la liste
    char whoAreFirsts();      // nature combine des deux premiers lments
    double getFirstValue();   // valeur du premier lment de la liste
    double getSecondValue();  // valeur du second lment de la liste  
    string getFirstStr();    // first Data of the list as string
    string getSecondStr();   // second Data of the list as string
    double getValue(unsigned nb);     // retourne la valeur du nbime lment
    int drop();               // destruction of the top element
    void clear();             // destruction of the list content
    int dup(int nb);          // copy the data of the top of the list
    void add();               // addition of the two first members of the list
    void sub();               // substraction of the two first members of the list
    void mul();               // multiplication of the two first members of the list 
    void div();               // division of the two first members of the list 
    void pw();                // second element power  first   
    void opp();               // transform last data in its opposite
    void inv();               // transform last data in its inverse number    
    void ln();                // ln of last element
    void lexp();              // exp of last element
    void llog10();            // decimal log of last element
    void pw10();              // rise last element to power 10
    void lcos(HV_TRIGO = RAD);  // cosinus of last element
    void lacos(HV_TRIGO = RAD); // arc cosinus of last element  
    void lsin(HV_TRIGO = RAD);  // cosinus of last element
    void lasin(HV_TRIGO = RAD); // arc cosinus of last element
    void ltan(HV_TRIGO = RAD);  // tangente of last element
    void latan(HV_TRIGO = RAD); // arc tangente of last element    
    void pushCopy(const HV_DataPtr);    
                             // Make a copy of the data and push it into the list 
    void toFile(const char *);    // Write the string in a file  
    void fromFile( const char *); // get data from a file                           
    int swap();               // swaps the first two elements
    int pick(bool);           // swaps the first two elements    
    string toString();  // retourne le contenu de tous les lments
    void copyTo(HV_List *);   // copy dans une autre liste   
    
  // ============================================================================
  // Exception classes definition / Dfinition des classes d'exceptions
  // ============================================================================
  
  // Class : HExcptWrongNb
  // Classe utilis lorsque le nombre d'argument de la liste n'autorise pas 
  // une opration
  class HExcptWrongNb
  {
    private :
      int rightNb;
    public :
      HExcptWrongNb(int theRightNb)
      { rightNb = theRightNb; }
      ~HExcptWrongNb()
      {}      
      int getRightNb()
      { return rightNb; }  
  };
  
  // Class : HExcptWrongType
  // Classe utilis lorsque le type d'arguments de la liste n'autorise pas 
  // une opration
  class HExcptWrongType
  {
    private :
      char rightType;
    public :
      HExcptWrongType(char theRightType)
      { rightType = theRightType; }
      ~HExcptWrongType()
      {}
      char getRightType()
      { return rightType; }  
  };
  
  // Class : HExcptDivBy0
  // Class used in case of a division by 0 trial
  class HExcptDivBy0
  {
    public :
      HExcptDivBy0()
      {}
      ~HExcptDivBy0()
      {}  
  };
  
  // Class : HExcptln0
  // Class used if ln(0) or log10(0) is asked
  class HExcptlog0 
  {
    public :
      HExcptlog0()
      {}
      ~HExcptlog0()
      {}  
  };  
  // Class : HExcptln0
  // Class used if ln(0) or log10(0) is asked
  class HExcptFile 
  {
    private :
      const char * name;
    public :
      HExcptFile(const char* theName)
      {name = theName;}
      ~HExcptFile()
      {}  
      const char * getName()
      { return name; }        
  };      
};

/// Definition d'un type pour les facilits de l'criture
typedef HV_List::iterator HIt;


#endif



