//// PoloCalc for windows
//// File / Fichier :   HVSmartInput.h
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 24/04/2003
//// Declaration of an input that filters keys before accepting them
//// and check for specific operations

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// ============================================================================= 

#ifndef HVINTERFACE
#define HVINTERFACE 1

#include <FL/Fl.H>
#include <FL/Fl_Input.H>
#include <vector>
#include <algorithm> 

using namespace std;

//==============================================================================
//  Classes declaration
//==============================================================================

/// Class : HV_SmartInput
/// Class derived from Fl_Input, that allows the lauching of a fonction if
/// specifics characters are keyed.
class HV_SmartInput : public Fl_Input
{
  private :
    vector<int> hotKeys ;                   // List of specific key
    void (*hotAction)( int );  // fuction to launch  
    
  public :
    HV_SmartInput( int x, int y, int w, int h, const char* label = 0 )
    :Fl_Input(x, y, w, h, label) {}
    ~HV_SmartInput () {}
    int handle( int );
    void behaviour( void (*theHotAction)(int) )
    { hotAction = theHotAction; }
    void addHotKey( int theHotKey )
    { hotKeys.push_back(theHotKey); }
    void clear()
    { this->value(""); } 
};

#endif

