//// PoloCalc for windows
//// File / Fichier : HVTextScroll.cpp
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 12/04/2003
//// Definition of a Scroll type window designed to adapt its size to a text
//// content

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// ============================================================================= 

#include "HVTextScroll.h"

//==============================================================================
//  Member functions definition
//==============================================================================

// constructor
HV_TextScroll::HV_TextScroll(int x, int y, int w, int h, const char * label):
Fl_Scroll(x, y, w, h, label)
{     
    content = "";
    textBox = new Fl_Box(x, y, w, h);
    textBox->box(FL_FLAT_BOX);    
    textBox->align(HV_TS_ALIGN);
    textBox->labelfont(HV_TS_FONT);
    textBox->labelsize(HV_TS_SIZE);
    textBox->labelcolor(HV_TS_COLOR);   
    color(HV_TS_BKCOLOR);
    type(0);
    bottomFlag = false;
    rightFlag = false;
        
    this->add(textBox); 
}

// destructor
HV_TextScroll::~HV_TextScroll()
{
    remove(textBox);
    delete(textBox);
}

// drawing : setting the box before
void HV_TextScroll::updateAndDraw()
{
  if(content.length())
  {
    int boxW, boxH, deltaW, deltaH;
    deltaW=0; deltaH=0;
    
    // Size
    fl_font(textBox->labelfont(), textBox->labelsize());
    fl_measure(content.c_str(), boxW, boxH);
    
    // No explanation for that : it works better thats all
    boxW += 2; boxH += 2;
    
    if( ( boxW <= w() ) && ( boxH <= h() ) )
    // Ici tout est de longueur suffisante
    {
      type(0);
      boxW = w();
      boxH = h();
    }
    else
    {
      if(boxW <= w())
      // Largeur OK, mais il faudra une scrollbar verticale
      {       
        deltaW = HV_BARSIZE;        
        if(boxW <= (w()-HV_BARSIZE))
        {
          type(Fl_Scroll::VERTICAL); 
          boxW = w() - HV_BARSIZE;    
        }   
        else
        // la largeur, rduite par la scroll ncessite maintenant une scroll horizontale
        {      
          type(Fl_Scroll::BOTH);        
          deltaH = HV_BARSIZE;                               
        }  
      }
      else if(boxH <= h())
      // Longueur OK, mais il faudra une scrollbar horizontale
      {
        deltaH = HV_BARSIZE;      
        if(boxH <= (h()-HV_BARSIZE))
        {
          type(Fl_Scroll::HORIZONTAL);  
          boxH = h() - HV_BARSIZE;  
        }             
        else
        // la hauteur, rduite par la scroll ncessite maintenant une scroll verticale
        {
          type(Fl_Scroll::BOTH);    
          deltaW = HV_BARSIZE;                                   
        }             
      }      
      else
      // tout trop court
      {
        type(Fl_Scroll::BOTH);
        deltaW = HV_BARSIZE;         
        deltaH = HV_BARSIZE;                                                    
      }               
    }  
        
    // ResizeBox    
    if(bottomFlag)
      deltaH += boxH - h();
    else
      deltaH = 0;
      
    if(rightFlag)
      deltaW += boxW - w();
    else
      deltaW = 0;   
           
    textBox->resize(x()-deltaW, y()-deltaH, boxW, boxH);    
    textBox->label(content.c_str());             
  }
  else
  {
    textBox->resize(x(),y(),w(),h());
    textBox->label("");
  }  
  redraw();  
}
