//// PoloCalc for windows
//// File / Fichier : HVTextScroll.h
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 12/04/2003
//// Declaration of a Scroll type window designed to adapt its size to a text
//// content

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// ============================================================================= 

#ifndef HVTEXTSCROLL
#define HVTEXTSCROLL 1

#include <FL/Fl.H>
#include <FL/Fl_Scroll.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_draw.H> 
#include <string>
#include <iostream>

using namespace std;

//==============================================================================
//  Constants declarations
//==============================================================================

const Fl_Align HV_TS_ALIGN = (Fl_Align)(FL_ALIGN_INSIDE|FL_ALIGN_CENTER);
const Fl_Font HV_TS_FONT = FL_HELVETICA;
const int HV_TS_SIZE = 20;//10; rsr: make Fl_Box wider, numbers not round so quickly
const Fl_Color HV_TS_COLOR = FL_BLACK ;
const Fl_Color HV_TS_BKCOLOR = FL_BACKGROUND2_COLOR;
const int HV_BARSIZE = 16;

//==============================================================================
//  Classe declaration
//==============================================================================

/// Class : HV_TextScroll
class HV_TextScroll : public Fl_Scroll
{
  private :
    string content;
    Fl_Box *textBox;
    bool bottomFlag, rightFlag;
    
  public :
    // constructors destructor
    HV_TextScroll(int, int, int, int, const char * = 0);
    ~HV_TextScroll();
    
    // access to memeber
    template <class T> void setc(T);
    template <class T> void addtc(T);
    void newLine()
    { content += '\n'; }
    void clearText()
    { content = ""; }
    void textAlign(Fl_Align theAlign)
    { textBox->align(theAlign); }
    void textFont(Fl_Font theFont)
    { textBox->labelfont(theFont); }
    void textColor(Fl_Color theColor)
    { textBox->labelcolor(theColor); }        
    void textSize(int theSize)
    { textBox->labelsize(theSize); } 
    void color(Fl_Color theColor)
    { textBox->color(theColor); }   
    void bottom()
    { bottomFlag = true; } 
    void right()
    { rightFlag = true; }  
    void bottomRight()
    { bottomFlag = true; rightFlag = true; }      
    
    // other functions
    void updateAndDraw();  
  
};

//==============================================================================
//  Template functions definition
//==============================================================================

template <class T> void HV_TextScroll::setc(T theData)
{
  ostringstream os;
  os << theData;
  content = os.str();
}

template <class T> void HV_TextScroll::addtc(T theData)
{
  ostringstream os;
  os << content << theData;
  content = os.str();
}

#endif

