//// PoloCalc for windows
//// File/Fichier : main.cpp
//// Copyright (C) 2003 Hugues Vanden Bossche
//// Last modification / Dernire modification : 18/05/2003
//// This is the main file, that contains the program to emule an HP type
//// calculator

// =============================================================================
//    This file is part of PoloCalc, a Windows desk caculator using
//    the Reverse Polish Notation
//
//    PoloCalc is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    PoloCalc is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with PoloCalc; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// =============================================================================
#include <list>
#include <string>

#include <FL/Fl_Window.H>
#include <FL/Fl_Multiline_Output.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/x.H> 
#include <FL/fl_ask.H>
#include <FL/Fl_Help_View.h>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Menu_Button.H>

#include "HVSmartInput.h"
#include "HVTextScroll.h"
#include "HVList.h"
#include "HVColor.h"
#include "HInterface.h"
#include "HGPLAbout.h"
// resource.h doesn't seem to be needed - R.H (amspire)
// #include "resource.h"  
#include "main.h"

//==============================================================================
//  Functions declaration
//==============================================================================

// Performed after each action to update output
void update()
{ string data=HPPile->toString();
  outpt->setc(data);
  outpt->updateAndDraw();   
  inpt->value("");  
  inpt->take_focus();
}

// Message wrongNb, used each time the wong nb of argument is used
void wrongNb(int theNb)
{
// rsr: Bug? memory
  ostringstream os;
  if(theNb==1)
    os << HV_messages[lang][NBARG1] ;
  else  
    os << HV_messages[lang][NBARGB] << theNb << HV_messages[lang][NBARGE] ; 
  outpt->setc(HPPile->toString());  
  outpt->updateAndDraw();
  fl_beep(FL_BEEP_ERROR);
  fl_message_font(FL_HELVETICA_BOLD, 12);  
  fl_alert(os.str().c_str());  
  inpt->value("");  
  inpt->take_focus();      
} 

// Message wrongType, used each time the wong type of argument appears in an operation
void wrongType()
{
  outpt->setc(HPPile->toString());  
  outpt->updateAndDraw();
  fl_beep(FL_BEEP_ERROR);
  fl_message_font(FL_HELVETICA_BOLD, 12);  
  fl_alert(HV_messages[lang][TYPEARG]);  
  inpt->value("");  
  inpt->take_focus();      
} 

// Message division by 0
void div0()
{
  outpt->setc(HPPile->toString());  
  outpt->updateAndDraw();
  fl_beep(FL_BEEP_ERROR);
  fl_message_font(FL_HELVETICA_BOLD, 12);  
  fl_alert(HV_messages[lang][DIV0]);    
  inpt->value("");  
  inpt->take_focus();  
} 

// Message : function log(0) has been asked
void ln0()
{
  outpt->setc(HPPile->toString());  
  outpt->updateAndDraw();
  fl_beep(FL_BEEP_ERROR);
  fl_message_font(FL_HELVETICA_BOLD, 12);  
  fl_alert(HV_messages[lang][LN0]);       
  inpt->value("");  
  inpt->take_focus();  
} 

// Message warning a problem in reading or writing into a file
void wrongFile(const char * theName)
{
  string buffer(HV_messages[lang][FILEMESS]);
  buffer += theName;
  fl_beep(FL_BEEP_ERROR);
  fl_message_font(FL_HELVETICA_BOLD, 12);  
  fl_alert(buffer.c_str());     
  inpt->take_focus();  
} 

// Function called when a constant value is clicked on the corresponding menu
void dataToInpt(Fl_Menu_Button *w, void *theData) 
{
  string data = (const char *)theData;
  inpt->value(data.c_str());
  inpt->redraw();
  inpt->take_focus();
}

// Function called when INV is clicked on the screen
// set invKey to the opposite value
void invK(void)
{
  for(unsigned i=0; i<listOfDoubleKeys.size(); i++)
    listOfDoubleKeys[i]->inverse();
  doubleKeysInversed = !doubleKeysInversed;  
  inpt->take_focus();    
}

// Function called when + is clicked on the screen or equivalent action 
void addition(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  try
  {
    HPPile->copyTo(copyPile);
    HPPile->add();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    wrongNb(Excpt.getRightNb());
  }
}

// Function called when - is clicked on the screen or equivalent action 
void substraction(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  try
  {
    HPPile->copyTo(copyPile);
    HPPile->sub();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    wrongNb(Excpt.getRightNb());
  }
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }
  
}

// Function called when * is clicked on the screen or equivalent action 
void multiplication(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  try
  {
    HPPile->copyTo(copyPile);
    HPPile->mul();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    wrongNb(Excpt.getRightNb());
  }
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }
}

// Function called when / is clicked on the screen or equivalent action 
void division(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  try
  {
    HPPile->copyTo(copyPile);
    HPPile->div();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    wrongNb(Excpt.getRightNb());
  }
  catch(HV_List::HExcptDivBy0 Excpt)
  {
    div0();    
  }
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }  
}

// Function called when +/- is clicked on the screen or equivalent action 
void opp(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  try
  {
    HPPile->copyTo(copyPile);
    HPPile->opp();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    wrongNb(Excpt.getRightNb());
  }  
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }
}

// Function called when 1/x is clicked on the screen or equivalent action 
void inv(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  try
  {
    HPPile->copyTo(copyPile);
    HPPile->inv();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    wrongNb(Excpt.getRightNb());
  }
  catch(HV_List::HExcptDivBy0 Excpt)
  {
    div0();     
  }
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }  
}

// Function called when SWAP is clicked on the screen or equivalent action 
// inverse the order of the two last member of the pile
void swp(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);  
  HPPile->swap();
  update();    
}

// Function called when Enter is clicked on the screen
// push the value of Inpt or double last value
void enterData(void)
{
  HPPile->copyTo(copyPile); 
  if(inpt->size())
    HPPile->push(inpt->value());
  else
    HPPile->dup(1);  
  update();
}

// Function called when DUP is actionned in the menu
void dup()
{
  if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);  
  HPPile->dup(1);  
  update();
}

// Function called when Dupn is actionned in the menu
void dupn()
{
  if(inpt->size())
    HPPile->push(inpt->value());
  if(HPPile->whoIsFirst()==HV_DOUBLE)
  {
    HPPile->copyTo(copyPile);
    int nb = abs((int)HPPile->getFirstValue());
    HPPile->drop(); 
    if(!HPPile->dup(nb))
      HPPile->push(inpt->value());
  } 
  update();
}

// Function called when PICK is actionned in the menu
void pick(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);  
  HPPile->pick(false);  
  update();
}

// Function called when ROLL is actionned in the menu
void roll(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);  
  HPPile->pick(true);  
  update();
}

// Function called when DROP is clicked on the screen
// erase last member
void drop(void)
{
  HPPile->drop();
  update();
}

// Function called when down arrow is clicked
// droplast member nack in the input field
void dropToInpt(void)
{
  if(HPPile->size())
  {
    inpt->value(HPPile->getFirstStr().c_str());
    HPPile->drop();
    outpt->setc(HPPile->toString());
    outpt->updateAndDraw();
  }
}

// Function called when DEL is clicked on the screen
// erase everything
void clear(void)
{
  HPPile->copyTo(copyPile);
  HPPile->clear();
  update();
}

// Function to rize x at power y where y is the last element entered
void power(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  try
  {
    HPPile->copyTo(copyPile);
    if(doubleKeysInversed)
    {
      HPPile->inv();
      invK();
    }
    HPPile->pw();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    wrongNb(Excpt.getRightNb());
  }
  catch(HV_List::HExcptDivBy0 Excpt)
  {
    div0();       
  } 
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }   
}

// According to the value of invKey can do square or square root
void sqr(void)
{  
  if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);  
  if(doubleKeysInversed)
  {
    HPPile->push(0.5);
    invK();
  }
  else
    HPPile->push(2);  
  try
  {
    HPPile->pw();
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {  
    HPPile->drop();
    wrongNb(Excpt.getRightNb()-1);
  }
  catch(HV_List::HExcptWrongType Excpt)
  {  
    HPPile->drop();  
    wrongType();
  }  
}

// According to the value of invKey can do ln or exp
void logNeperien(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);  
  try  
  {  
    if(doubleKeysInversed)
    {
      HPPile->lexp();
      invK();    
    }  
    else
      HPPile->ln();  
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {
    wrongNb(Excpt.getRightNb());
  }
  catch(HV_List::HExcptlog0 Excpt)
  {
    ln0();
  }
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }  
}

// According to the value of invKey can do Log or 10^
void logDecimal(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
    
  try  
  {  
    HPPile->copyTo(copyPile);
    if(doubleKeysInversed)
    {
      HPPile->pw10();
      invK();    
    }  
    else
      HPPile->llog10();  
    update();    
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {
    wrongNb(Excpt.getRightNb());
  }
  catch(HV_List::HExcptlog0 Excpt)
  {
    ln0();
  }
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }
}

// changeAngle() : modify the angle unit taken into account by triginometric fonctions
void changeAngle()
{
  string actual(angle->label());
  if(!actual.compare("deg"))
  {
    angle->label("rad"); 
    trigo = RAD;
  }  
  else
  {
    if (!actual.compare("rad"))
    {
      angle->label("grad");
      trigo = GRAD;
    }   
    else
    {
      angle->label("deg");
      trigo = DEG;   
    }
  } 
  inpt->take_focus();
}

// According to the value of invKey can do cos or acos
void cosinus(void)
{
 if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);
  try
  {
    if(doubleKeysInversed)
    {
      HPPile->lacos(trigo);
      invK();    
    }  
    else
      HPPile->lcos(trigo);  
    update(); 
  }   
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }  
  catch(HV_List::HExcptWrongNb Excpt)
  {
    wrongNb(Excpt.getRightNb());
  }  
}

// According to the value of invKey can do sin or as
void sinus(void)
{
 if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);
  try
  {
    if(doubleKeysInversed)
    {
      HPPile->lasin(trigo);
      invK();    
    }  
    else
      HPPile->lsin(trigo);  
    update();  
  } 
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  } 
  catch(HV_List::HExcptWrongNb Excpt)
  {
    wrongNb(Excpt.getRightNb());
  }  
}

// According to the value of invKey can do tan or atan
void tangente(void)
{
 if(inpt->size())
    HPPile->push(inpt->value());
  HPPile->copyTo(copyPile);
  try
  {
    if(doubleKeysInversed)
    {
      HPPile->latan(trigo);
      invK();    
    }  
    else
      HPPile->ltan(trigo);  
    update(); 
  }   
  catch(HV_List::HExcptWrongType Excpt)
  {  
    wrongType();
  }  
  catch(HV_List::HExcptWrongNb Excpt)
  {
    wrongNb(Excpt.getRightNb());
  }  
}

// Copy last value to clipBoard
void cpLast(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
    
  if(HPPile->size()) 
  {
    Fl_Input *tmpInput = new Fl_Input(0, 0, 0, 0);
    tmpInput->value(HPPile->getFirstStr().c_str()); 
    tmpInput->position(0,tmpInput->size());
    tmpInput->copy(1);      
    delete tmpInput;   
  }                 
}

// Copy all the values to clipBoard
void cpAll(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
    
  if(HPPile->size()) 
  {
    Fl_Input *tmpInput = new Fl_Input(0, 0, 0, 0);
    tmpInput->value(HPPile->toString().c_str()); 
    tmpInput->position(0,tmpInput->size());
    tmpInput->copy(1);      
    delete tmpInput;   
  }  
}           

// According to the value of invKey, copy last element to the clipboard or all list
void cpToClip(void)
{
  
  if(doubleKeysInversed) 
  {
    cpAll(); 
    invK();
  } 
  else
    cpLast();       
  update();  
}

// copy the content of the List to the default file
void toDefault(void)
{
  try
  {
    HPPile->toFile("DefaultFile.poc");
  }  
  catch(HV_List::HExcptFile Excpt)
  {
    wrongFile(Excpt.getName());
  }
}

// copy the content of the List to the default file, and close the main window
// this is the callback fonction for the main window
void toDefaultAndClose(Fl_Window *theWindow)
{
  if(!HPPile->empty())
  {
    try
    {
      HPPile->toFile(default_poc_file);
    }  
    catch(HV_List::HExcptFile Excpt)
    {
      wrongFile(Excpt.getName());
    }
  }
  theWindow->hide();
}

// copy the content of the List to a file choosen by user
void toFile(void)
{
  try
  {
    const char *fileName = fl_file_chooser(HV_messages[lang][TARGET], "*.poc", ".poc");
    if( fileName )
      HPPile->toFile(fileName);
  }  
  catch(HV_List::HExcptFile Excpt)
  {
    wrongFile(Excpt.getName());
  }
}

// get the content of the List from the default file
void fromDefault(void)
{
  HPPile->copyTo(copyPile);
  try
  {
    HPPile->fromFile(default_poc_file);
  }  
  catch(HV_List::HExcptFile Excpt)
  {
    wrongFile(Excpt.getName());
  }
  update();
}

// get the content of the List from the default file
void fromFile(void)
{
  HPPile->copyTo(copyPile);
  try
  {
    const char *fileName = fl_file_chooser(HV_messages[lang][SOURCE], "*.poc", "DefaultFile.poc");
    if( fileName )
      HPPile->fromFile(fileName);
  }  
  catch(HV_List::HExcptFile Excpt)
  {
    wrongFile(Excpt.getName());
  }
  update();
}

// get the content of the List from the copy Pile
void recoverFromCopy(void)
{
  HV_List *tempPile = new HV_List();
  HPPile->copyTo(tempPile);  
  copyPile->copyTo(HPPile);
  tempPile->copyTo(copyPile);
  delete tempPile;
  update();
}

// Function called to recover a value from the constants or the variables list
void dropTheVal(Fl_Menu_Button *w, void* theData)
{
  string* value = (string *) theData; 
  inpt->value(value->c_str());
  inpt->redraw();
  inpt->take_focus();
}

// Deletion of a variable
void rm(Fl_Menu_Button *w)
{
  int itemNb = w->value();
  varMenu->remove(itemNb); 
  rmMenu->remove(itemNb); 
  if(varMenu->size()==1)
  {
    varMenu->labelcolor(HV_OUTPTC);
    rmMenu->labelcolor(HV_OUTPTC);
  }
  varMenu->redraw();  
  rmMenu->redraw();
}

// Creation of a new item in the Var list
void sto(void)
{
  if(inpt->size())
    HPPile->push(inpt->value());
  if(HPPile->size()>1)
  {
    if(HPPile->whoIsFirst()!=HV_TEXT)
      wrongType();
    else
    {
      string constName = HPPile->getFirstStr();
      string *value = new string(HPPile->getSecondStr());
      varMenu->add(constName.c_str(), 0,(Fl_Callback *) dropTheVal, (void *) value, 0);      
      varMenu->labelcolor(HV_INPTTC);
      varMenu->redraw();
      rmMenu->add(constName.c_str(), 0,(Fl_Callback *) rm);      
      rmMenu->labelcolor(HV_INPTTC);
      rmMenu->redraw();
      HPPile->drop();
      update();
    }          
  }
  else
    wrongNb(2);          
}

// Interception of a HotKey, bi inpt
void hotKeyPressed(int theKey)
{
  int i=0;
  while(correspond[i][0]!=theKey)
    i++;
  switch(correspond[i][1])
  {
    case ENTER :
      enterData();
      break;
    case ADD :
      addition();
      break;  
    case SUB :
      substraction();
      break;  
    case MUL :
      multiplication();
      break;  
    case DIV :
      division();
      break;  
    case LN :
      logNeperien();
      break;  
    case LOG10 :
      logDecimal();
      break;  
    case COS :
      cosinus();
      break;  
    case SIN:
      sinus();
      break;  
    case TAN:
      tangente();
      break;  
    case SETUNIT:
      changeAngle();
      break;                                                    
    case DROP :
      drop();
      break;  
    case PICK :
      pick();
      break;        
    case DROPTOINPT :
      dropToInpt();
      break;  
    case CLEAR :
      clear();
      break;
    case UNDO :
      recoverFromCopy();      
      break;            
    case INV :
      inv();
      break;
    case OPP :
      opp();
      break;
    case SWAP :
      swp();
      break;   
    case INVK :
      invK();
      break;
    case SQR :
      sqr();      
      break; 
    case POW :
      power();      
      break;   
    case COPY :
      cpToClip();      
      break;                                             
    default :
      outpt->addtc("operation  implmenter");
      outpt->updateAndDraw();   
      inpt->value("");  
      inpt->take_focus();
      break;
  }  
}

// showAbout : show the about datas
void showAbout(void)
{
  GPLAbout(PRGNAME, VERSION, YYYY, AUTHOR);
}

// showAbout : show the help datas
void showHelp(void)
{  
  // window
  Fl_Window *helpWindow = new Fl_Window (HV_BOARDH, HV_BOARDH);
  Fl_Help_View *helpView = new Fl_Help_View(5,5,HV_BOARDH-10, HV_BOARDH-10);
  helpView->load(help_file);
  helpWindow->end ();
  helpWindow->show ();
}

//==============================================================================
//  Main
//==============================================================================

int main (int argc, char *argv[])
{   
  int KY, KX; 
  Fl_Menu_Button *menuButton;
  HV_OperationKey *opKey;
  HV_DoubleOperationKey *dopKey;
  HV_DigitKey *digKey;
  
  // List FIFO were operations are performed :  the list of the calculator
  // and a list to save previous
  HPPile = new HV_List; copyPile = new HV_List;
  
  // Get the langage (default english)
  ifstream config;
  config.open("config.poc");  
  lang = ENGLISH; 
  if (config.is_open())
    config >> lang;
  if(lang > FRENCH)
    lang = ENGLISH;
  config.close();  
    
  // Main Window (board)        
  Fl_Window *window = new Fl_Window (HV_BOARDW, HV_BOARDH);
  window->resizable(window);
#ifdef rsr
  window->icon((char *) LoadIcon(fl_display, MAKEINTRESOURCE(IDI_MYICON))); 
#endif
  string title = PRGNAME;
#ifdef rsr
  title += "  ";
  title += VERSION;
#endif
  window->label(title.c_str());
  window->color(HV_WINDC);
  window->callback((Fl_Callback *) toDefaultAndClose);

  // menu bar
  Fl_Menu_Item menuItems[] =
  {
    {HV_messages[lang][MENUFILE], FL_ALT + 'f', 0, 0, FL_SUBMENU },
      {HV_messages[lang][MENUSD], FL_ALT + 's', (Fl_Callback *) toDefault },
      {HV_messages[lang][MENULD], FL_ALT + 'l', (Fl_Callback *) fromDefault },
      {HV_messages[lang][MENUSF], 0, (Fl_Callback *) toFile },
      {HV_messages[lang][MENULF], 0, (Fl_Callback *) fromFile },     
      {0},
    {HV_messages[lang][MENUEDIT], 0, 0, 0, FL_SUBMENU},
      {HV_messages[lang][MENUC], FL_ALT + 'c', (Fl_Callback *) cpLast},
      {HV_messages[lang][MENUCA], FL_ALT + FL_SHIFT + 'c', (Fl_Callback *) cpAll},
      {HV_messages[lang][MENUU], FL_ALT + 'u', (Fl_Callback *) recoverFromCopy },       
      {0},
    {HV_messages[lang][MENUSTACK], FL_ALT + 'p', 0, 0, FL_SUBMENU },
      {"Swap", 0, (Fl_Callback *) swp },
      {"Dup", 0, (Fl_Callback *) dup },
      {"Dupn", 0, (Fl_Callback *) dupn },
      {"Pick", 0, (Fl_Callback *) pick },
      {"Roll", 0, (Fl_Callback *) roll },
      {0},    
    {HV_messages[lang][MENUHELP], 0, 0, 0, FL_SUBMENU },
      {HV_messages[lang][MENUHELP], FL_ALT + 'h', (Fl_Callback *) showHelp},
      {HV_messages[lang][MENUABOUT], FL_ALT + 'a', (Fl_Callback *) showAbout},
      {0},
    {0}  
  };

  Fl_Menu_Bar *menubar = new Fl_Menu_Bar(0,0,HV_BOARDW,HV_MENUH);
  menubar->menu(menuItems);
  menubar->textsize(HV_MENUTS);
  
  // Output
  KY = HV_MENUH + HV_SPACE;
  outpt = new HV_TextScroll(HV_SPACE, KY, HV_OUTW, HV_OUTH);
  outpt->color(HV_OUTPTC);
  outpt->textColor(HV_OUTPTTC);
  outpt->box (FL_EMBOSSED_BOX);
  outpt->textSize(12);
  outpt->textFont(HV_OUTPTF);
  outpt->textAlign((Fl_Align)(FL_ALIGN_INSIDE|FL_ALIGN_BOTTOM_RIGHT));
  outpt->bottomRight();
  outpt->end();

  // Line of special button
  KY += HV_OUTH + HV_SSPACE;
  KX = HV_SPACE;
  
  // Constant button  
  menuButton = new Fl_Menu_Button (KX, KY, HV_KEYW, HV_KEYSH, "Cst"); 
  menuButton->color(HV_INPTC);
  menuButton->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
  menuButton->labelfont(FL_HELVETICA_BOLD);
  menuButton->labelcolor(HV_INPTTC);  
  menuButton->textcolor(HV_INPTTC);  
  menuButton->labelsize(HV_KEYLS-2);
  ifstream constData;
  constData.open("resources/Constantes.poc");  
  
  // reading of the constantes  
  if (constData.is_open())
  {
    char *error;
    string path = "", item, itemPath, buffer;
    constData >> item >> buffer;
    
    do
    {
      strtod(buffer.c_str(), &error);
      if(*error)
      {
        if(item!="")
          path = item + "/";
        item = buffer;   
      }
      else
      {        
        string* value = new string(buffer.c_str());
        itemPath = path + item;     
        menuButton->add(itemPath.c_str(), 0,(Fl_Callback *) dropTheVal, (void *) value, 0);
        item = "";
      }
      constData >> buffer;
    }
    while(!constData.eof());
  }

  // Sto button (to create a var)
  KX += HV_KEYW + HV_SPACE; 
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "Sto", (Fl_Callback *) sto); 
  opKey->color(HV_INPTC);
  opKey->labelfont(FL_HELVETICA_BOLD);
  opKey->labelcolor(HV_INPTTC);  
  opKey->box(FL_UP_BOX);
  opKey->labelsize( HV_KEYLS - 2);  
  
  // Variable Button
  KX += HV_KEYW + HV_SPACE;
  varMenu = new Fl_Menu_Button (KX, KY, HV_KEYW, HV_KEYSH, "Var"); 
  varMenu->color(HV_INPTC);
  varMenu->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
  varMenu->labelfont(FL_HELVETICA_BOLD);
  varMenu->labelcolor(HV_OUTPTC);  
  varMenu->textcolor(HV_INPTTC);  
  varMenu->labelsize(HV_KEYLS-2);
  
  // Rm menu : used to remove a variable
  KX += HV_KEYW + HV_SPACE; 
  rmMenu = new Fl_Menu_Button (KX, KY, HV_KEYW, HV_KEYSH, "Rm"); 
  rmMenu->color(HV_INPTC);
  rmMenu->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
  rmMenu->labelfont(FL_HELVETICA_BOLD);
  rmMenu->labelcolor(HV_OUTPTC);  
  rmMenu->textcolor(HV_INPTTC);  
  rmMenu->labelsize(HV_KEYLS-2); 

  // Input   
  KY += HV_KEYSH + HV_SSPACE;  
  inpt = new HV_SmartInput(HV_SPACE,  KY, HV_OUTW, HV_INH);
  inpt->behaviour( hotKeyPressed );
  inpt->textfont(HV_INPTF);
  inpt->color(HV_INPTC);
  inpt->textcolor(HV_INPTTC);
  for(int i=0; i<sizeof(correspond)/(2*sizeof(int)); i++)
    inpt->addHotKey( correspond[i][0]);    
  
  // First line of keys    
  KY += HV_INH + HV_SPACE;
  KX = HV_SPACE;   
  dopKey = new HV_DoubleOperationKey( KX, KY, "@8->", "@2->", (Fl_Callback *) invK ); 
  dopKey->labeltype(FL_SYMBOL_LABEL);
  listOfDoubleKeys.push_back(dopKey);
  dopKey->overLabel("F1");   
  KX += HV_SPACE + HV_KEYW;   
  dopKey = new HV_DoubleOperationKey( KX, KY, "Copy", "CpAll", (Fl_Callback *) cpToClip);    
  listOfDoubleKeys.push_back(dopKey); 
  dopKey->overLabel("F2");
  KX += HV_SPACE + HV_KEYW;   
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "+/-", (Fl_Callback *) opp); 
  opKey->labelfont( FL_HELVETICA );
  opKey->overLabel("F3");
  KX += HV_SPACE + HV_KEYW;
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "1/x", (Fl_Callback *) inv); 
  opKey->labelfont( FL_HELVETICA );
  opKey->overLabel("F4");  

  // Second line of keys  
  KY += HV_KEYH + HV_SSPACE;
  KX = HV_SPACE;   
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "Drop", (Fl_Callback *) drop); 
  opKey->labelfont( FL_HELVETICA );
  opKey->labelsize( HV_KEYLS - 2);
  opKey->overLabel("F5");
  KX += HV_SPACE + HV_KEYW;
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "Swap", (Fl_Callback *) swp); 
  opKey->labelfont( FL_HELVETICA );
  opKey->labelsize( HV_KEYLS - 2);
  opKey->overLabel("F6");  
  KX += HV_SPACE + HV_KEYW;
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "Pick", (Fl_Callback *) pick); 
  opKey->labelfont( FL_HELVETICA );
  opKey->labelsize( HV_KEYLS - 2);  
  opKey->overLabel("F7");  
  KX += HV_SPACE + HV_KEYW;
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "Undo", (Fl_Callback *)  recoverFromCopy ); 
  opKey->labelfont( FL_HELVETICA );  
  opKey->labelsize( HV_KEYLS - 2);   
  opKey->overLabel("F8");  

  // Third line of keys    
  KY += HV_KEYH + HV_SSPACE;
  KX = HV_SPACE;   
  dopKey = new HV_DoubleOperationKey( KX, KY, "x", "Sqr", (Fl_Callback *) sqr);
  listOfDoubleKeys.push_back(dopKey);  
  KX += HV_SPACE + HV_KEYW;
  dopKey->overLabel("F9");  
  dopKey = new HV_DoubleOperationKey( KX, KY, "Pow", "Root", (Fl_Callback *) power);   
  listOfDoubleKeys.push_back(dopKey);  
  dopKey->overLabel("F10");    
  KX += HV_SPACE + HV_KEYW;   
  dopKey = new HV_DoubleOperationKey( KX, KY, "ln", "exp", (Fl_Callback *) logNeperien );  
  listOfDoubleKeys.push_back(dopKey);
  dopKey->overLabel("F11");      
  KX += HV_SPACE + HV_KEYW;
  dopKey = new HV_DoubleOperationKey( KX, KY, "Log10", "10^x", (Fl_Callback *) logDecimal);      
  listOfDoubleKeys.push_back(dopKey);  
  dopKey->overLabel("F12");    
  
  // Fourth line of keys         
  KY += HV_KEYH + HV_SSPACE;
  KX = HV_SPACE;     
  dopKey = new HV_DoubleOperationKey( KX, KY, "cos", " acos", (Fl_Callback *) cosinus ); 
  KX += HV_SPACE + HV_KEYW;
  listOfDoubleKeys.push_back(dopKey);  
  dopKey = new HV_DoubleOperationKey( KX, KY, "sin", "asin", (Fl_Callback *) sinus );    
  KX += HV_SPACE + HV_KEYW;
  listOfDoubleKeys.push_back(dopKey);    
  dopKey = new HV_DoubleOperationKey( KX, KY, "tan", "atan", (Fl_Callback *) tangente ); 
  KX += HV_SPACE + HV_KEYW;
  listOfDoubleKeys.push_back(dopKey);    
  angle = new HV_OperationKey( KX, KY, HV_KEYW, HV_KEYSH, "deg", (Fl_Callback *) changeAngle );     
  angle->labelfont( FL_TIMES_BOLD );
  angle->labelsize( HV_KEYLS - 2);    
  angle->labelcolor( HV_PURPLE );
  
  // New Line of keys
  KY += HV_KEYH + HV_SSPACE;
  KX = HV_SPACE;   
  opKey = new HV_OperationKey( KX, KY, "CLR", (Fl_Callback *) clear);
  opKey->labelcolor(HV_GREEN);
  KX += HV_SPACE + HV_KEYW;
  opKey = new HV_OperationKey( KX, KY, "/", (Fl_Callback *) division);
  KX += HV_SPACE + HV_KEYW;
  opKey = new HV_OperationKey( KX, KY, "X", (Fl_Callback *) multiplication); 
  opKey->labelsize( HV_KEYLS - 2);   
  KX += HV_SPACE + HV_KEYW;     
  opKey = new HV_OperationKey( KX, KY, "-", (Fl_Callback *) substraction); 
  
  // New line of keys  
  KY += HV_KEYH + HV_SSPACE;
  KX = HV_SPACE; 
  digKey = new HV_DigitKey(KX, KY, "7", '7');  
  KX += HV_SPACE + HV_KEYW;    
  digKey = new HV_DigitKey(KX, KY, "8", '8');
  KX += HV_SPACE + HV_KEYW;    
  digKey = new HV_DigitKey(KX, KY,"9", '9');
  KX += HV_SPACE + HV_KEYW;     
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, 2*HV_KEYH + HV_SSPACE, "+", (Fl_Callback *) addition);  
       
  // New line of keys  
  KY += HV_KEYH + HV_SSPACE;
  KX = HV_SPACE;     
  digKey = new HV_DigitKey(KX, KY, "4", '4');    
  KX += HV_SPACE + HV_KEYW;     
  digKey = new HV_DigitKey(KX, KY, "5", '5');
  KX += HV_SPACE + HV_KEYW;     
  digKey = new HV_DigitKey(KX, KY, "6", '6');
   
  // New line of keys  
  KY += HV_KEYH + HV_SSPACE;
  KX = HV_SPACE;     
  digKey = new HV_DigitKey(KX, KY, "1", '1');    
  KX += HV_SPACE + HV_KEYW;   
  digKey = new HV_DigitKey(KX, KY, "2", '2');
  KX += HV_SPACE + HV_KEYW;  
  digKey = new HV_DigitKey(KX, KY, "3", '3');
  KX += HV_SPACE + HV_KEYW;  
  opKey = new HV_OperationKey( KX, KY, HV_KEYW, 2*HV_KEYH + HV_SSPACE, "Enter", (Fl_Callback *) enterData);
     
  // New line of keys  
  KY += HV_KEYH + HV_SSPACE;     
  digKey = new HV_DigitKey(HV_SPACE, KY, 2*HV_KEYW + HV_SPACE, HV_KEYH, "0", '0');     
  digKey = new HV_DigitKey(3*HV_SPACE + 2*HV_KEYW, KY, ".", '.');
  
  window->end ();
 try {
  window->show (argc, argv);  
}catch(...)
{	printf("Unhandled exception caught!\n");
}
  inpt->take_focus();

  int state = Fl::run();


}

