/* A GL window with a trackball interface.  This is a subclass of Fl_Gl_Window
that only implements the handle() method to keep track of mouse motions.
You subclass Tb_Window and implement draw(), and call the transform() method
to orient the scene. */

#ifndef Tb_Window_h
#define Tb_Window_h

#include <FL/Fl.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/gl.h>
#include "math.h"
#include "../ewave/vecmath.h"

class Tb_Window : public Fl_Gl_Window {
public:
  Tb_Window(int x, int y, int w, int h, const char *l = 0)
    : Fl_Gl_Window(x, y, w, h, l) { init(); }
  Tb_Window(int w, int h, const char *l = 0)
    : Fl_Gl_Window(w, h, l) { init(); }
  ~Tb_Window();
  
  /* Set the trackball size and mouse scale. */
  
  void tbsize(float f) { Tbsize = f; }
  void mscale(float f) { Mscale = 30.0f * f; }
  
  /* Set the initial trackball origin, translation and rotation. */
  
  void origin(float x, float y, float z);
  void translate(sgVec3 v);
  void rotate(sgVec3 axis, float theta);
  void initialize();
  
  /* Call this from draw() to transform the view. */
  
  void transform();
  
  /* Save and restore the trackball state. */
  
  void save(char *name);
  void restore(char *name);

  void set_zoom (double z);
  double get_zoom ();
  
  /* Call this when the geometry changes to force a redraw(),
   or just call redraw(). */
  
  void set_changed() { changed = 1; }
  
  /* Call idle_redraw(TRUE) to force a redraw() every time idle() is
   called.  It is initially FALSE. */
  
  void idle_redraw(int b) { idle_redraw_ = b; }

 protected:  
  int handle(int);

 private:
  void init();
  static void idle_cb(Tb_Window *);
  void idle();
  
  sgQuat Spin;            // quaternion incremental rotation
  sgQuat Rot;             // quaternion total rotation
  sgVec3 Trans;           // total translation
  sgVec3 OTrans;          // object total translation
  
  float Tbsize;           // fraction of window to fill with trackball
  float Mscale;           // mouse movement multiplier
  
  ulong Event;            // mouse button and modifier keys
  int Mx;                 // place where mouse is now
  int My;
  int Oldx;               // place where mouse used to be
  int Oldy;
  
  int spinning;           // if the user did a SPIN
  int changed;            // if the view changed
  int idle_redraw_;       // call redraw() in any case
  
  void trackball();
  void window_to_tb(float mx, float my, float *x, float *y);
  float tb_project_to_sphere(float, float, float);
};

#endif
