// Extreme Wave -- A 3d modelling and animation environment
// Copyright (C) 1999 James Dean Palmer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// Please report all bugs and problems to "james@tiger-marmalade.com".

#include <unistd.h>
#include <stdio.h>

#include <ewave/objects/ew_object.h>
#include <ewave/materials/material.h>
#include <ewave/exporters/nativeio.h>
#include <ewave/kernel/global.h>

EW_bool 
save_native (EW_Object * a, char *filename ) 
{
  FILE * file;
  int i, j;

  file = fopen(filename, "w+");

  if (file == NULL)
    {
      printf ("Error: Could not save native data file \"%s\"", filename);
      return 0;
    }

  fprintf (file, "(!format \"Extreme Wave\" )\n");
  fprintf (file, "(!version \"%d.%d.%d\" )\n", 
	   EWAVE_MAJOR_VERSION,
	   EWAVE_MINOR_VERSION,
	   EWAVE_PATCH_VERSION);

  fprintf (file, "(!pointlist ");
  
  /*
  for (i=0; i<point_db.size(); i++) 
    fprintf (file, "  %g %g %g\n", 
	     point_db[i].xyz[0],
	     point_db[i].xyz[1],
	     point_db[i].xyz[2]);
  */
  
  fprintf (file, ")\n\n");

  if (material_list.size() > 0)
    {
      fprintf (file, "(!materiallist ");
      for (i=0; i<material_list.size(); i++)
	material_list[i]->write(file, 2);
      fprintf(file, ")\n");
    }
/*
  if (bone_list.size() > 0)
    {
      fprintf (file, "(!bonelist ");
      
      for (i=0; i<bone_list.size(); i++) 
	{
	  fprintf (file, "  (bone \"%s\" ", bone_list[i]->name);
	  
	  for (j=0; j<bone_list[i]->a.size(); j++)
	    fprintf (file, " %d %d", 
		     bone_list[i]->a[j], 
		     bone_list[i]->b[j]);
	  
	  fprintf (file, ")\n");
	}
      
      fprintf (file, ")\n");
    }
*/
  //a->write (file, 0);

  fclose (file);
  return 1;
}

// This is the old spatch import code.  A previous incarnation of
// Extreme Wave was much better at doing Bezier curves but the code
// was pretty awful.  This needs to be converted into a guile
// script anyway.  I've kept it in the source for file format 
// reference purposes.  I don't think there is any documentation on
// this format.

/*
EW_Object * 
Tio::import_spatch (char * filename) 
{
   EW_Object * a=NULL, * b=NULL;
   in_file.open(filename);
   if (in_file.bad()) 
     {
       error = FileReadError;
       cerr << "Error: Could not import sPatch data file: " << filename << "\n";
       return(NULL);
     }
   in_file.unsetf(ios::skipws);

   while (!in_file.eof()) 
     {
       a = read_spatch(NULL);
       if (a!=NULL) {  // This is a very dangerous way of assigning
	 b = a;        // read data to the Master Scene.  We should
       }               // run some kind of check to make sure there
     }                 // is only one root node.
   in_file.close();
   return(b);
}

EW_Object * Tio::read_spatch (EW_Object * p) 
{  
  EW_Object *a, *a2;
  char value[64];
  char op[16];
  a = NULL;
  a2 = NULL;
  skip_char();
  skip_ws();
  read_word((char *)op);
  if (!strcmp(op, "sPatches")) {
    //do some error checking here.
  }
  if (!strcmp(op, "version")) {
    //do some error checking here.
  }
  if (!strcmp(op, "num_points")) {
    int i, j;
    import_offset = point_db.size();
    skip_ws();
    read_word((char *)value);
    i = atoi(value);
    for (j=0; j<i; j++) {
      Vector a;
      skip_ws();
      read_word((char *)value);
      a.x = atof(value);
      skip_ws();
      read_word((char *)value);
      a.y = atof(value);
      skip_ws();
      read_word((char *)value);
      a.z = atof(value);
      skip_ws();
      read_word((char *)value); // int
      skip_ws();
      read_word((char *)value); // int
      int g = atoi(value);
      for (int h=0; h<g; h++) {
	skip_ws();
	read_word((char *)value); // int
	skip_ws();
	read_word((char *)value); // int
      }
      point_db.append();
      point_db.last() = a;
    }
    //do some error checking here.
  }
  // The bezier operator
  if (!strcmp(op, "num_curves")) {
    int num_curves;
    char value[64];
    skip_ws();
    read_word((char *)value);  
    num_curves = atoi(value);   // Number of Curves

    a = new EW_ObjectGroup();
    a->type = DTgroup;
    ((EW_ObjectGroup *)a)->name = "sPatch Import";
    ((EW_ObjectGroup *)a)->comment = "sPatch Imported material";

    for (int iterator=0; num_curves>iterator; iterator++) {
      int num_points;
      int closed;
      a2 = new EW_Object();
      a2->type = DTbezier;

      skip_ws();
      read_word((char *)value);
      num_points = atoi(value);

      skip_ws();
      read_word((char *)value); 
      closed = atoi(value);
      // Do I really need to know this information??
      //      if (closed) a2->closed = 1;
      //      else a2->closed = 0;

      for (int iteratorb = 0; num_points > iteratorb; iteratorb++) {
	Vector d;
	skip_ws();
	read_word((char *)value);
	a2->point.append();
	a2->point.last() = atoi(value) + import_offset;

	skip_ws();
	read_word((char *)value);
	// What is this??? (int)

	skip_ws();
	read_word((char *)value);
	d.x = atof(value); // hold this value for later..

	// Add two control points.  We'll have to compute them after this loop.
	point_db.append();
	point_db.last() = d;
	a2->point.append();
	a2->point.last() = point_db.size() - 1;
	point_db.append();
	point_db.last() = d;
	a2->point.append();
	a2->point.last() = point_db.size() - 1;
      }

      for (int i=0; i<a2->point.size(); i++) 
	 {
	Vector t, s, u;
	// rp = 0, 3, 6,  9
	switch (i% 3) {
	case 0: // 0/3, 3/3
	  // This is control point.
	  break;
	case 1: // 1/3
	  if ((i > 3) && (a2->point.size() > i+2)) {
	    t = point_db[a2->point[i-4]] - point_db[a2->point[i-1]];
	    s = point_db[a2->point[i+2]] - point_db[a2->point[i-1]];
	    u = s + (s.magnitude()*t.unit(-t));
	      //u = VectorAdd(s, VectorScalarMultiply(VectorUnit(VectorNegative(t)), VectorMagnitude(s)));
	      //u = VectorScalarMultiply(VectorUnit(u), point_db[a2->point[i]].x*0.2);  // use another magnitude
	    u = u.unit(u)*(point_db[a2->point[i]].x*0.2);  // use another magnitude
	    //u = VectorAdd(u, point_db[a2->point[i-1]]);  // Position correctly.
	    u = u + point_db[a2->point[i-1]];  // Position correctly.
	  } else {
	    u = point_db[a2->point[i-1]];
	  }
	  point_db[a2->point[i]] = u;
	  break;
	case 2: // 2/3
	  if ((i > 1) && (a2->point.size() > i+4)) {
	    //t = VectorFromPoints(point_db[a2->point[i+1]], point_db[a2->point[i+4]]);
	    //s = VectorFromPoints(point_db[a2->point[i+1]], point_db[a2->point[i-2]]);
	    //u = VectorAdd(s, VectorScalarMultiply(VectorUnit(VectorNegative(t)), VectorMagnitude(s)));
	    //u = VectorScalarMultiply(VectorUnit(u), point_db[a2->point[i]].x*0.2);  // use another magnitude
	    //u = VectorAdd(u, point_db[a2->point[i+1]]);  // Position correctly.
	    t = point_db[a2->point[i+4]] - point_db[a2->point[i+1]];
	    s = point_db[a2->point[i-2]] - point_db[a2->point[i+1]];
	    u = s + (t.unit(-t) * s.magnitude());
	    u = u.unit(u) * (point_db[a2->point[i]].x*0.2);  // use another magnitude
	    u = u + point_db[a2->point[i+1]];  // Position correctly.

	  } else {
	    if (a2->point[i+1] < point_db.size()) u = point_db[a2->point[i+1]];
	  }
	  point_db[a2->point[i]] = u;
	  break;
	}
      }
      ((EW_ObjectGroup *)a)->add(a2);
    }
  }
  // The polygon operator
  if (!strcmp(op, "end_patches")) {
    return(NULL);
  }
  if (p != NULL) ((EW_ObjectGroup *)p)->add(a);
  return(a);

  return NULL;
}

*/
