%{ 
  // Extreme Wave -- A 3d modelling and animation environment
  // Copyright (C) 1999 James Dean Palmer
  //
  // This program is free software; you can redistribute it and/or modify
  // it under the terms of the GNU General Public License as published by
  // the Free Software Foundation; either version 2 of the License, or
  // (at your option) any later version.
  //
  // This program is distributed in the hope that it will be useful,
  // but WITHOUT ANY WARRANTY; without even the implied warranty of
  // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  // GNU General Public License for more details.
  //
  // You should have received a copy of the GNU General Public License
  // along with this program; if not, write to the Free Software
  // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  //
  // Please report all bugs and problems to "james@tiger-marmalade.com".
  
  // This file is a bison grammar which defines the syntax for
  // Extreme Wave's native file format.  The grammar has been designed
  // to be flexible enough for any new objects to plug into the
  // file format without making any changes to the grammar.

#include <stdio.h> 
#include <string.h> 
#include <gFL/gfl.h>
#include <ewave/objects/ew_object.h>
#include <ewave/materials/material.h>
#include <ewave/kernel/global.h>

  // The "object_stack" is the stack of objects that 
  // the grammar operates on.  At the end of yyparse() there should
  // only be one object on the stack -- the root node.
  static EW_Object_array object_stack;

  // Arguments are also kept on a "stack" of sorts.  All arguments
  // are kept in a scheme list called "argument_stack".
  SCM argument_stack;

  void yyerror (const char *error); 
  int  yylex ();
 
%} 
 
%union 
{ 
  char *estring; 
  double edouble; 
  int eint; 
  struct 
  { 
    double x; 
    double y; 
    double z; 
  } evector; 
}

%token FORMAT VERSION
%token OSTART VSTART END 
%token POINTLIST MATERIALLIST BONELIST MATERIAL
%token <estring> TEXT 
%token <estring> WORD 
%token <edouble> DOUBLE 
%token <eint> INTEGER 
%type  <evector> point_triplet
%%

valid_stuff:    valid_stuff valid | valid;

valid:          object
                {
		  if (global.verbose) 
		    printf("grammar: valid object\n");
		}
        |       point_list
                {
		  if (global.verbose) 
		    printf("grammar: valid point list.\n");
		}
        |       bone_list
                {
		  if (global.verbose) 
		    printf("grammar: valid bone list.\n");
		}
        |       material_list
                {
		  if (global.verbose) 
		    printf("grammar: valid material list.\n");
		}
        |       format
                {
		  if (global.verbose) 
		    printf("grammar: valid format.\n");
		}
        |       version
                {
		  if (global.verbose) 
		    printf("grammar: valid version.\n");
		}
                ;

format: OSTART FORMAT TEXT END;

version: OSTART VERSION TEXT END;

material_list:  start_material_list materials END
                {
		  if (global.verbose) 
		    printf("grammar: valid material.\n");
		}
                ;

start_material_list: OSTART MATERIALLIST
                {
		  if (global.verbose) 
		    printf("grammar: start material_list (resets material_list)\n");
		  material_list.reset();
                } 
                ;

materials:      materials material | material ;

material:       material_start material_arguments END;

material_start: OSTART MATERIAL
                {
		  if (global.verbose) 
		    printf("material_list.append() = new Material();\n");
		  material_list.append() = new Material();
		}
                ;

material_arguments: material_arguments material_argument | material_argument;

material_argument: VSTART 

material_argument: VSTART WORD TEXT END 
                {
		  if (material_list.last() != NULL)
		    material_list.last()->set_attribute($2, $3);
		  argument_stack = SCM_EOL;
                }
        |       VSTART WORD numbers END
                {
		  if (material_list.last() != NULL)
		    {
		      material_list.last()->set_attribute($2, 
					    gh_reverse(argument_stack));
		    }
		  argument_stack = SCM_EOL;
		}
                ;

point_list:     start_point_list point_triplets END 
                { 
		  if (global.verbose) 
		    printf("grammar: valid point list.\n");
	       } 
	       ;

start_point_list: OSTART POINTLIST
               {
		 if (global.verbose) 
		   printf("grammar: start pointlist (resets point_db).\n");

//point_db.reset();
	       }
               ;

point_triplets: point_triplets point_triplet 
               {
		 if (global.verbose) 
		   printf("append point <%g, %g, %g> (1)\n", $2.x, $2.y, $2.z);

		 //point_db.append();
		 //sgSetVec3(point_db.last().xyz, $2.x, $2.y, $2.z);
               }
               | point_triplet 
	       {
		 if (global.verbose) 
		   printf("append point <%g, %g, %g> (2)\n", $1.x, $1.y, $1.z);
		 
		 //point_db.append();
		 //sgSetVec3(point_db.last().xyz, $1.x, $1.y, $1.z);
	       }
	       ;

point_triplet: number number number
               {
		 $$.z = gh_scm2double(gh_car(argument_stack));
		 $$.y = gh_scm2double(gh_cadr(argument_stack));
		 $$.x = gh_scm2double(gh_caddr(argument_stack));
		 argument_stack = SCM_EOL;
	       }
               ;

bone_list: start_bone_list bones END;

bones: bones bone | bone;

bone: VSTART WORD TEXT numbers END
               {
		 printf("grammar: valid bone\n");
		 /*
		 bone_list.append() = new EW_Bone(0);
		 bone_list.last()->name = strdup($3);
		 SCM a = gh_reverse(argument_stack);
		 int i=0;
		 while (a != SCM_EOL)
		   {
		     bone_list.last()->a.append();
		     bone_list.last()->b.append();

		     bone_list.last()->a[i] = gh_scm2int(gh_car(a));
		     a = gh_cdr(a);
		     bone_list.last()->b[i] = gh_scm2int(gh_car(a));
		     a = gh_cdr(a);
		     i++;
		   }
		 frame_count = i;
		 */
		 argument_stack = SCM_EOL;
	       }
               ;

start_bone_list: OSTART BONELIST
               {
		 if (global.verbose) 
		   printf("grammar: start bonelist.\n");

		 //bone_list.reset();
	       }
               ;

object: 	start_object arguments END
               {
		 printf("grammar: valid object\n");
	       }
               ;

start_object: 	OSTART WORD
		{
		  if (global.verbose) 
		    printf("grammar: start object \"%s\"\n", $2);

		  EW_Object * o = NULL;
		  for (int i=0; i<kernel->object_registry.size(); i++)
		    if (!strcmp(kernel->object_registry[i]->name(), $2))
		      o = (kernel->object_registry[i]->create())();
		  if (o == NULL)
		      printf("WARNING: \"%s\" was not found.\n"
			     "         Skipping object's properties.\n", $2);

		  object_stack.append() = o;
		}
		;

arguments: 	arguments argument | argument; 

argument: 	VSTART WORD TEXT END 
                {
		  if (object_stack.last() != NULL)
		    {
		      char a[128];
		      SCM rvalue = SCM_EOL;
		      SCM obj = gh_cons(gh_int2scm(object_stack.last()->vklass().id()), 
					gh_long2scm((long)object_stack.last()));
		      //char * b = 
		      sprintf(a, "%s::set-%s", object_stack.last()->vklass().name(), $2);
		      rvalue = gh_cons(gh_str02scm($3), rvalue);
		      rvalue = gh_cons(obj, rvalue);
		      gfl_exe(a, rvalue);
		    }
		  argument_stack = SCM_EOL;
                }
        |       VSTART WORD numbers END
                {
		  if (object_stack.last())
		    {
		      SCM obj = gh_cons(gh_int2scm(object_stack.last()->vklass().id()), 
					gh_long2scm((SCM)object_stack.last()));
		      char a[128];
		      //char * b = ;
		      sprintf(a, "%s::set-%s", object_stack.last()->vklass().name(), $2);
		      gfl_exe(a, gh_cons(obj, gh_reverse(argument_stack)));
		    }
		  argument_stack = SCM_EOL;
		}
        |       object 
                {
		  EW_Object * o;
		  if (object_stack.last() != NULL)
		    {
		      o = object_stack.last();
		      object_stack.shrink();
		      if (object_stack.last() != NULL)
			{
			  EW_Group * p = (EW_Group *)object_stack.last();
			  p->add(o);
			}
		      else
			delete o;
		    }
		  else
		    object_stack.shrink();
		}
                ;

numbers: 	numbers number | number ;

number:         DOUBLE 
                {
		  argument_stack = gh_cons(gh_double2scm($1), argument_stack);
		}
	| 	INTEGER 
                { 
		  argument_stack = gh_cons(gh_int2scm($1), argument_stack);
		} 
		; 
 
%% 
 
extern FILE *yyin; 
extern char *yytext;

EW_Object * 
load_native(char *s) 
{
  FILE *file;
  file = fopen( s, "r" );
  yyin = file;
  //point_db.reset();
  object_stack.reset();

  global.loading = 1;
  if (global.verbose) 
    printf("load_native(): reading file %s\n", s);

  yyparse();
  global.loading = 0;
  if (global.verbose) 
    printf("load_native(): file successfully read\n");
  return (object_stack.last());
}
 
void 
yyerror (const char *error) 
{ 
  fprintf(stderr, "%s at ? text=\"%s\"\n", error, yytext); 
} 
