/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/

double x[4];
double y[4];
double z[4];

void 
Bezier (double ax, double ay, double az, 
	double bx, double by, double bz, 
	double cx, double cy, double cz, 
	double dx, double dy, double dz) 
{

  // Bezier(t) = t^3*(C3-3*C2+3*C1-C0) + 
  //             t^2*(3*C2-6*C1+3*C0) + 
  //             t*(3*C1-3*C0) + C0
   
  x[0] = ax;
  x[1] = 3.0*(bx-ax);
  x[2] = 3.0*(cx-2.0*bx+ax);
  x[3] = dx+3.0*(bx-cx)-ax;
  
  y[0] = ay;
  y[1] = 3.0*(by-ay);
  y[2] = 3.0*(cy-2.0*by+ay);
  y[3] = dy+3.0*(by-cy)-ay;

  z[0] = az;
  z[1] = 3.0*(bz-az);
  z[2] = 3.0*(cz-2.0*bz+az);
  z[3] = dz+3.0*(bz-cz)-az;
}

double 
BezierX (double t) 
{
  double result;
  result = t*t*t*x[3] + t*t*x[2] + t*x[1] + x[0];
  return(result);
}

double 
BezierY (double t) 
{
  double result;
  result = t*t*t*y[3] + t*t*y[2] + t*y[1] + y[0];
  return(result);
}

double 
BezierZ (double t) 
{
  double result;
  result = t*t*t*z[3] + t*t*z[2] + t*z[1] + z[0];
  return(result);
}

/*
void 
draw_bezier_points (Object * a) 
{
  int i = 0;
  while (i < a->control_points.size()) 
    {
      glColor3f(0.3, 0.3, 0.8);
      draw_point( a->point[i] );

      // if we want control points
      glColor3f(0.2, 0.5, 0.2);
      if ((i+1) < a->control_points.size())
	draw_point( a->point[i+1] );
      if ((i+2) < a->control_points.size())
	draw_point( a->point[i+2] );

      // if we want control point lines
      glBegin(GL_LINES);
      if ((i+1) < a->control_points.size()) 
	{
	  glVertex3f(point_db[a->point[i]].x,
		     point_db[a->point[i]].y,
		     point_db[a->point[i]].z);
	  glVertex3f(point_db[a->point[i+1]].x,
		     point_db[a->point[i+1]].y,
		     point_db[a->point[i+1]].z);
	}
      if ((i+3) < a->control_points.size()) 
	{
	  glVertex3f(point_db[a->point[i+2]].x,
		     point_db[a->point[i+2]].y,
		     point_db[a->point[i+2]].z);
	  glVertex3f(point_db[a->point[i+3]].x,
		     point_db[a->point[i+3]].y,
		     point_db[a->point[i+3]].z);
	}
      glEnd();
      i+=3;
    }
}
*/

/*:
  @subsection Vector deCasteljau1(Vector *P, int n, double u)

  Compute a point on a Bezier curve with the deCasteljau algorithm.
  [Piegl 24]
*/

/*
Vector
deCasteljau1(Vector *P, int n, double u)
{
  int i, k;
  Vector *Q = new Vector[n];

  for (i=0; i<=n; i++)
    Q[i] = P[i];

  for (k=1; k<=n; k++)
    for (i=1; i<=n; i++)
      Q[i] = (1.0-u)*Q[i] + u*Q[i+1];

  return Q[0];
}
*/

/*
Vector
deCasteljau2(Vector *P, int n, int m, double u0, double v0)
{
  int j;

  if (n<=m)
    {
      Vector *Q = new Vector[m];

      for (j=0; j<=m; j++)
	Q[j] = deCasteljau1(P[j][], n, u0);
      return deCasteljau1(Q, m, v0);
    }
  else
    {
      Vector *Q = new Vector[n];

      for (j=0; j<=n; j++)
	Q[j] = deCasteljau1(P[][j], m, v0);
      return deCasteljau1(Q, n, u0);
    }
}
*/

/*
double 
bezier_blend(int k, double mu, int n)
{
  int nn, kn, nkn;
  double blend=1;

  nn = n;
  kn = k;
  nkn = n - k;

  while (nn >= 1) 
    {
      blend *= nn;
      nn--;
      if (kn > 1) 
	{
	  blend /= (double)kn;
	  kn--;
	}
      if (nkn > 1) 
	{
	  blend /= (double)nkn;
	  nkn--;
	}
    }
  if (k > 0)
    blend *= pow(mu,(double)k);
  if (n-k > 0)
    blend *= pow(1-mu,(double)(n-k));

  return(blend);
}
*/

/*
  // Create a random surface
  srandom(1111);
   for (i=0;i<=NI;i++) {
   for (j=0;j<=NJ;j++) {
         inp[i][j].x = i;
         inp[i][j].y = j;
         inp[i][j].z = (random() % 10000) / 5000.0 - 1;
      }
   }
*/

/*
draw_gl()
{
  int NI=5;
  int NJ=4;

  int RESOLUTIONI 10*NI;
  int RESOLUTIONJ 10*NJ;

  Vector * inp = new Vector[NI+1][NJ+1];
  Vector * outp = new Vector[RESOLUTIONI][RESOLUTIONJ];

  int i,j,ki,kj;
  double mui,muj,bi,bj;

  for (i=0;i<RESOLUTIONI;i++) 
    {
      mui = i / (double)(RESOLUTIONI-1);
      for (j=0;j<RESOLUTIONJ;j++) 
	{
	  muj = j / (double)(RESOLUTIONJ-1);
	  outp[i][j].x = 0;
	  outp[i][j].y = 0;
	  outp[i][j].z = 0;
	  for (ki=0;ki<=NI;ki++) 
	    {
	      bi = BezierBlend(ki,mui,NI);
	      for (kj=0;kj<=NJ;kj++) 
		{
		  bj = BezierBlend(kj,muj,NJ);
		  outp[i][j].x += (inp[ki][kj].x * bi * bj);
		  outp[i][j].y += (inp[ki][kj].y * bi * bj);
		  outp[i][j].z += (inp[ki][kj].z * bi * bj);
		}
	    }
	}
    }

   printf("LIST\n");

// Display the surface, in this case in OOGL format for GeomView 
   printf("{ = CQUAD\n");
   for (i=0;i<RESOLUTIONI-1;i++) 
     {
       for (j=0;j<RESOLUTIONJ-1;j++) 
	 {
	   printf("%g %g %g 1 1 1 1\n",
		  outp[i][j].x, outp[i][j].y, outp[i][j].z);
	   printf("%g %g %g 1 1 1 1\n",
		  outp[i][j+1].x, outp[i][j+1].y, outp[i][j+1].z);
	   printf("%g %g %g 1 1 1 1\n",
		  outp[i+1][j+1].x, outp[i+1][j+1].y, outp[i+1][j+1].z);
	   printf("%g %g %g 1 1 1 1\n",
		  outp[i+1][j].x, outp[i+1][j].y, outp[i+1][j].z);
	 }
     }
   printf("}\n");

// Control point polygon
   for (i=0;i<NI;i++) 
     {
       for (j=0;j<NJ;j++) 
	 {
	   printf("{ = SKEL 4 1  \n");
	   printf("%g %g %g \n",inp[i][j].x,inp[i][j].y,inp[i][j].z);
	   printf("%g %g %g \n",inp[i][j+1].x,inp[i][j+1].y,inp[i][j+1].z);
	   printf("%g %g %g \n",inp[i+1][j+1].x,inp[i+1][j+1].y,inp[i+1][j+1].z);
	   printf("%g %g %g \n",inp[i+1][j].x,inp[i+1][j].y,inp[i+1][j].z);
	   printf("5 0 1 2 3 0\n");
	   printf("}\n");
	 }
     }
} 
*/
