#ifndef EW_MATH_
#define EW_MATH_

#include <limits.h>
#include <stdio.h>
#include <math.h>

#define EW_ZERO      0.0f
#define EW_ONE       1.0f
#define EW_TWO       2.0f
#define EW_180       180.0f
#define EW_MAX       FLT_MAX
#define EW_INFINITY  999999000.0

#ifndef M_PI
#define EW_PI 3.1415926535
#else
#define EW_PI M_PI
#endif

#define DEGREES_TO_RADIANS 0.0174532925194
#define RADIANS_TO_DEGREES 57.295779516179

inline double ewSqrt   ( double x ) { return sqrt ( x ) ; }
inline double ewSquare ( double x ) { return x * x ; }
inline double ewAbs    ( double a ) { return ( a < EW_ZERO ) ? -a : a ; }
inline double ewMin    ( double a, double b) { return ( a < b ) ? a : b ; }
inline double ewMax    ( double a, double b) { return ( a > b ) ? a : b ; }
inline double ewSign   ( double a ) { return ( a < EW_ZERO ) ? -a : ((a > EW_ZERO ) ? EW_ONE : EW_ZERO ); }

inline int ewCompareDouble ( double a, double b, double tol )
{
  if ( ( a + tol ) < b ) return -1 ;
  if ( ( b + tol ) < a ) return  1 ;
  return 0 ;
}

#include <ewave/kernel/base.h>
#include <ewave/geometry/vec3.h>
#include <ewave/geometry/point.h>

#endif
