#ifndef EW_QUAT__
#define EW_QUAT__

#include <stdio.h>

class HMatrix : public Base {
 public:
  HMatrix () {}
  void set (int x, int y, double value) { data[x][y] = value; }
  double get (int x, int y) { return data[x][y]; }
  void zero () { 
    for (int i=0; i<4; i++)
      for (int j=0; j<4; j++)
	data[i][j] = 0.;
  }
  void identity () { 
    zero ();
    for (int i=0; i<4; i++)
      data[i][i] = 1.;
  }

  double * rowd (int i) { return &data[i][0]; }
  const double * value () { return &data[0][0]; }  

  void display ()
    {
      for (int i = 0; i < 4; i++)
	printf("[ %g,  %g,  %g,  %g ]\n", data[i][0], data[i][1], data[i][2], data[i][3]);
    }
  
 protected:
  double data[4][4];
};

class Quat : public Base
{
 public:
  Quat () {}
  Quat (const double x, const double y, const double z) { set(x, y, z, 0); }
  Quat (const double x, const double y, const double z, const double w) { set(x, y, z, w); }
  Quat (double f) { set(f,f,f,f); }

  const double * value () { return data; }

  virtual int read (EIFF *eiff)
    {
      data[0] = eiff_read_double (eiff);
      data[1] = eiff_read_double (eiff);
      data[2] = eiff_read_double (eiff);
      data[3] = eiff_read_double (eiff);
      return 0;//bug
    }

  virtual int write (EIFF *eiff)
    {
      eiff_write_double (eiff, data[0]);
      eiff_write_double (eiff, data[1]);
      eiff_write_double (eiff, data[2]);
      eiff_write_double (eiff, data[3]);
      return 0;//bug
    }

  void set (double x, double y, double z, double w) { data[0] = x; data[1] = y; data[2] = z; data[3] = w; }
  double & operator [](const int i) { return data[i]; }
  double x() { return data[0]; }
  void x(double t) { data[0] = t; }
  double y() { return data[1]; }
  void y(double t) { data[1] = t; }
  double z() { return data[2]; }
  void z(double t) { data[2] = t; }
  double w() { return data[3]; }
  void w(double t) { data[3] = t; }

  Vec3 toVec3 ()
    {
      return Vec3(data[0], data[1], data[2]); 
    }
  
  HMatrix toMatrix ()
    { 
      HMatrix out;
      double Nq = data[0] * data[0] + data[1] * data[1] + data[2] * data[2] + data[3] * data[3];
      double s = (Nq > 0.0) ? (2.0 / Nq) : 0.0;
      double xs = data[0]*s,	      ys = data[1]*s,	  zs = data[2]*s;
      double wx = data[3]*xs,	      wy = data[3]*ys,	  wz = data[3]*zs;
      double xx = data[0]*xs,	      xy = data[0]*ys,	  xz = data[0]*zs;
      double yy = data[1]*ys,	      yz = data[1]*zs,	  zz = data[2]*zs;
      out.zero ();
      out.set (0, 0, 1.0 - (yy + zz));
      out.set (1, 0, xy + wz);
      out.set (2, 0, xz - wy);
      out.set (0, 1, xy - wz);
      out.set (1, 1, 1.0 - (xx + zz));
      out.set (2, 1, yz + wx);
      out.set (0, 2, xz + wy);
      out.set (1, 2, yz - wx);
      out.set (2, 2, 1.0 - (xx + yy));
      out.set (3, 3, 1.0);
      return out;
    }

  Quat conjugate ()
    {
      return Quat(-data[0], -data[1], -data[2], data[3] );
    }
  
  friend Quat operator * (const Quat& v1, const Quat& v2);
  
 protected:
  double data[4];
};

  /* Return quaternion product qL * qR.  Note: order is important!
 * To combine rotations, use the product Mul(qSecond, qFirst),
 * which gives the effect of rotating by qFirst then qSecond. */

inline Quat 
operator * (const Quat& qL, const Quat& qR)
{
  Quat qq;
  qq.data[3] = qL.data[3]*qR.data[3] - qL.data[0]*qR.data[0] - qL.data[1]*qR.data[1] - qL.data[2]*qR.data[2];
  qq.data[0] = qL.data[3]*qR.data[0] + qL.data[0]*qR.data[3] + qL.data[1]*qR.data[2] - qL.data[2]*qR.data[1];
  qq.data[1] = qL.data[3]*qR.data[1] + qL.data[1]*qR.data[3] + qL.data[2]*qR.data[0] - qL.data[0]*qR.data[2];
  qq.data[2] = qL.data[3]*qR.data[2] + qL.data[2]*qR.data[3] + qL.data[0]*qR.data[1] - qL.data[1]*qR.data[0];
  return qq;
}

#endif
