#ifndef EW_VEC3__
#define EW_VEC3__

#include <ewave/geometry/math.h>

class Vec3 : public Base
{
 public:
  Vec3 () {}
  Vec3 (const double x, const double y, const double z) { set(x, y, z); }
  Vec3 (double f) { set(f,f,f); }

  const double * value () { return data; }
  
  virtual int read (EIFF *eiff)
    {
      data[0] = eiff_read_double (eiff);
      data[1] = eiff_read_double (eiff);
      data[2] = eiff_read_double (eiff);
      return 0;//bug
    }

  virtual int write (EIFF *eiff)
    {
      eiff_write_double (eiff, data[0]);
      eiff_write_double (eiff, data[1]);
      eiff_write_double (eiff, data[2]);
      return 0;//bug
    }
  
  void set (double x, double y, double z) { data[0] = x; data[1] = y; data[2] = z; }
  double & operator [](const int i) { return data[i]; }
  double x() { return data[0]; }
  void x(double t) { data[0] = t; }
  double y() { return data[1]; }
  void y(double t) { data[1] = t; }
  double z() { return data[2]; }
  void z(double t) { data[2] = t; }
  
  void normalize ()
    {
      double z  = (1 / length ());
      data[0] *= z;
      data[1] *= z;
      data[2] *= z;
    }

  double norm ()
    {
      return data[0]*data[0] + data[1]*data[1] + data[2]*data[2];
    }
  
  double length ()
    {
      return sqrt (norm());
    }

  static Vec3 bisect (Vec3 v1, Vec3 v2)
    {
      Vec3 v = v1 + v2;
      double Nv = v.norm();
      if (Nv < 1.0e-5) {
	v.set (0, 0, 1);
      } else {
	v *= 1 / sqrt (Nv);
      }
      return v;
    }
  
  void operator+= (const Vec3& v) { data[0] += v.data[0]; data[1] += v.data[1]; data[2] += v.data[2]; }
  void operator-= (const Vec3& v) { data[0] -= v.data[0]; data[1] -= v.data[1]; data[2] -= v.data[2]; }
  void operator/= (const double& v) { data[0] /= v; data[1] /= v; data[2] /= v; }
  void operator*= (const double& v) { data[0] *= v; data[1] *= v; data[2] *= v; }

  Vec3 operator + () const;
  Vec3 operator - () const;
  friend Vec3 operator + (const Vec3& v1, const Vec3& v2);
  friend Vec3 operator - (const Vec3& v1, const Vec3& v2);
  friend double operator * (const Vec3& v1, const Vec3& v2);
  friend Vec3 operator % (const Vec3& v1, const Vec3& v2);
  friend Vec3 operator * (const Vec3& v, double f);
  friend double operator * (double f, const Vec3& v);
  friend Vec3 operator / (const Vec3& v, double f);
  double data[3];
};

inline Vec3 Vec3::operator + () const { return *this; }
inline Vec3 Vec3::operator - () const { return Vec3(-data[0], -data[1], -data[2]); }

inline short 
operator == (const Vec3& v1, const Vec3& v2)
{ 
  if ((v1.data[0] == v2.data[0]) &&
      (v1.data[1] == v2.data[1]) &&
      (v1.data[2] == v2.data[2]))
    return 1;
  return 0;
}

inline Vec3 
operator + (const Vec3& v1, const Vec3& v2)
{ 
  return Vec3 (v1.data[0]+v2.data[0], v1.data[1]+v2.data[1], v1.data[2]+v2.data[2]);
}

inline Vec3 
operator - (const Vec3& v1, const Vec3& v2)
{ 
  return Vec3 (v1.data[0]-v2.data[0], v1.data[1]-v2.data[1], v1.data[2]-v2.data[2]); 
}

inline Vec3 
operator % (const Vec3& v1, const Vec3& v2)
{ 
  return Vec3 (v1.data[1] * v2.data[2] - v1.data[2] * v2.data[1], 
	       v1.data[2] * v2.data[0] - v1.data[0] * v2.data[2], 
	       v1.data[0] * v2.data[1] - v1.data[1] * v2.data[0] ); 
}

inline double 
operator * (const Vec3& v1, const Vec3& v2)
{ 
  return v1.data[0] * v2.data[0] +
    v1.data[1] * v2.data[1] +
    v1.data[2] * v2.data[2] ;
}

inline Vec3 
operator * (const Vec3& v, double f)
{ 
  return Vec3 (v.data[0]*f, v.data[1]*f, v.data[2]*f); 
}

inline Vec3 
operator / (const Vec3& v, double d)
{
  if ( d != 0.0f ) 
    {
      double e = 1.0f/d;
      return Vec3 (v.data[0]*e, v.data[1]*e, v.data[2]*e); 
    } 
  else 
    {
      return Vec3 (EW_INFINITY, EW_INFINITY, EW_INFINITY);
    }
}

#endif
