/***** Ball.c *****/
/* Ken Shoemake, 1993 */
#include <unistd.h>
#include <GL/gl.h>
//#include <GL/glu.h>
#include <ewave/geometry/math.h>
#include <ewave/vecmath.h>
#include "ArcBall.h"

#define LG_NSEGS 4
#define NSEGS (1<<LG_NSEGS)
#define RIMCOLOR()    glColor3f(1.0f, 1.0f, 1.0f)
#define FARCOLOR()    glColor3f(float(195./255.0), float(127./255.), float(31./255.))
#define NEARCOLOR()   glColor3f(1.0f, 1.0f, float(63./255.))
#define DRAGCOLOR()   glColor3f(float(127./255.), 1.0f, 1.0f)
#define RESCOLOR()    glColor3f(float(195./255.), float(31./255.), float(31./255.))


void gluOrtho2D(float x,float y,float dx,float dy)
{	glOrtho(x,y,dx,dy,0.0f,1.0f);
}

             
HMatrix mId; // = {{1,0,0,0},{0,1,0,0},{0,0,1,0},{0,0,0,1}};
double otherAxis[][4] = {{-0.48, 0.80, 0.36, 1}};

void 
simple_circle (Vec3 *cen, Vec3 *n, float r, int nsides) 
{
  /* doesn't assume n is a unit vector */
  
  int i;
  float ang, cosang, sinang;
  Vec3 u, v, pt;
  
  if ((*n)[0] * (*n)[0] < .33) 
    u = Vec3 (0, (*n)[2], -(*n)[1]);   /* n X (1,0,0) */
  else 
    u = Vec3 (-(*n)[2], 0, -(*n)[0]);   /* n X (1,0,0) */
  u = u * (r / u.length ());
  v = (*n) % u;
  v = v * (r / v.length ());
  //vec_cross(n, &u, &v);                       /* v is another tangent vector*/
  //vec_smul(r/VEC_LEN(&v), &v, &v);
  /* u, v, and n are mutually orthogonal, u and v have length r */
  
  /* draw a disk */
  glBegin(GL_POLYGON);
  for (i=0; i<nsides; i++) {
    ang = float(2.0*M_PI*i/nsides);
    cosang = cosf(ang);
    sinang = sinf(ang);
    pt[0] = (*cen)[0] + cosang*u[0] + sinang*v[0];
    pt[1] = (*cen)[1] + cosang*u[1] + sinang*v[1];
    pt[2] = (*cen)[2] + cosang*u[2] + sinang*v[2];
    glVertex3dv(pt.value ());
  }
  glEnd();
}

void circ (double x, double y, double z)
{
  Vec3 center (x, y, z);
  simple_circle (&center, &center, 1, 20);
}

/* Establish reasonable initial values for controller. */
ArcBall::ArcBall ()
{
  center.set (0, 0, 0);
  radius = 1.0;
  vDown.set (0, 0, 0);
  vNow.set (0, 0, 0); 
  qDown.set (0, 0, 0, 1);
  qNow.set (0, 0, 0, 1);
  mNow.identity ();
  mDown.identity ();
  mId.identity ();
  ShowResult = dragging = 0;
  axisSet = NoAxes;

  sets[CameraAxes] = mId.rowd (0); //&(mId.value ())[0*4]; 
  setSizes[CameraAxes] = 3;

  sets[BodyAxes] = mDown.rowd (0); //&(mDown.value ())[0*4]; 
  setSizes[BodyAxes] = 3;
  
  sets[OtherAxes] = otherAxis[0]; 
  setSizes[OtherAxes] = 1;
}

/* Set the center and size of the controller. */
void 
ArcBall::place (Vec3 center_, double radius_)
{
  center = center_;
  radius = radius_;
}

/* Incorporate new mouse position. */
void 
ArcBall::mouse (Vec3 vNow_)
{
  vNow = vNow_;
}

/* Choose a constraint set, or none. */
void 
ArcBall::useSet (AxisSet axisSet_)
{
  if (!dragging) axisSet = axisSet_;
}

/* Begin drawing arc for all drags combined. */
void 
ArcBall::showResult ()
{
  ShowResult = 1;
}

/* Stop drawing arc for all drags combined. */
void 
ArcBall::hideResult ()
{
  ShowResult = 0;
}

/* Using vDown, vNow, dragging, and axisSet, compute rotation etc. */
void 
ArcBall::update ()
{
  int setSize = setSizes[axisSet];
  Vec3 *set = (Vec3 *)(sets[axisSet]);
  vFrom = mouseOnSphere(vDown, center, radius);
  vTo = mouseOnSphere(vNow, center, radius);
  if (dragging) {
    if (axisSet!=NoAxes) {
      vFrom = constrainToAxis(vFrom, set[axisIndex]);
      vTo = constrainToAxis(vTo, set[axisIndex]);
    }
    qDrag = quatFromBallPoints (vFrom, vTo);
    qNow = qDrag * qDown;
  } else {
    if (axisSet!=NoAxes) {
      axisIndex = nearestConstraintAxis(vTo, set, setSize);
    }
  }
  quatToBallPoints (qDown, &vrFrom, &vrTo);
  //quatToMatrix (qNow.conjugate (), mNow); /* Gives transpose for GL. */
  mNow = (qNow.conjugate ()).toMatrix ();
}

/* Return rotation matrix defined by controller use. */
HMatrix ArcBall::value ()
{
  return mNow; 
}

/* Begin drag sequence. */
void ArcBall::beginDrag ()
{
  dragging = 1;
  vDown = vNow;
}

/* Stop drag sequence. */
void 
ArcBall::endDrag ()
{
  dragging = 0;
  qDown = qNow;
  mDown = mNow;
  
  printf("mDown:\n");
  mDown.display ();
  printf("mNow:\n");
  mNow.display ();
  //for (i=15; i>=0; i--)
  //  ((double *)mDown)[i] = ((double *)mNow)[i];
}

/* Draw the controller with all its arcs. */
void 
ArcBall::draw ()
{
  double r = radius;
  glMatrixMode (GL_MODELVIEW);
  glPushMatrix ();
    {
      glLoadIdentity ();
      glMatrixMode (GL_PROJECTION);
      gluOrtho2D (-1.0, 1.0, -1.0, 1.0);
      glMatrixMode (GL_MODELVIEW);
      RIMCOLOR ();
      //glScalef (r, r, r);
      // FIXME!! 
      circ (0.0, 0.0, 0.0);
      drawResultArc ();
      drawConstraints ();
      drawDragArc ();
    }
  glMatrixMode (GL_MODELVIEW);
  glPopMatrix();
}

/* Draw an arc defined by its ends. (static) */
void 
ArcBall::drawAnyArc (Vec3 vFrom, Vec3 vTo)
{
  int i;
  Vec3 pts[NSEGS+1];
  double dot;
  pts[0] = vFrom;
  pts[1] = pts[NSEGS] = vTo;
  for (i=0; i<LG_NSEGS; i++) 
    pts[1] = Vec3::bisect (pts[0], pts[1]);
  
  dot = 2.0 * (pts[0] * pts[1]); // dot product
  for (i=2; i<NSEGS; i++)
    pts[i] = (pts[i-1] * dot) - pts[i-2];

  glBegin (GL_LINE_STRIP);
    {
      for (i=0; i<=NSEGS; i++)
	glVertex3dv(pts[i].value ());
    }
  glEnd ();
}

/* Draw the arc of a semi-circle defined by its axis. (static) */
void 
ArcBall::drawHalfArc (Vec3 n)
{
  Vec3 p, m;
  p[2] = 0;
  if (n[2] != 1.0) {
    p[0] = n[1]; p[1] = -n[0];
    p.normalize();
  } else {
    p[0] = 0; p[1] = 1;
  }
  m = p % n;
  drawAnyArc (p, m);
  drawAnyArc (m, -p);
}

/* Draw all constraint arcs. */
void 
ArcBall::drawConstraints ()
{
  ConstraintSet set;
  Vec3 axis;
  int axisI, setSize = setSizes[axisSet];
  if (axisSet==NoAxes) return;
  set = sets[axisSet];
  for (axisI=0; axisI<setSize; axisI++) {
    if (axisIndex!=axisI) {
      if (dragging) continue;
      FARCOLOR();
    } else NEARCOLOR();
    axis = *(Vec3 *)&set[4*axisI];
    if (axis[2] == 1.0) {
      //FIXME! 
      circ (0.0, 0.0, 1.0);
    } else {
      drawHalfArc (axis);
    }
  }
}

/* Draw "rubber band" arc during dragging. */
void 
ArcBall::drawDragArc ()
{
  DRAGCOLOR();
  if (dragging) 
    drawAnyArc (vFrom, vTo);
}

/* Draw arc for result of all drags. */
void 
ArcBall::drawResultArc ()
{
  RESCOLOR();
  if (ShowResult) 
    drawAnyArc (vrFrom, vrTo);
}


/**** BallMath.c - Essential routines for ArcBall.  ****/

/* Convert window coordinates to sphere coordinates. */
Vec3 
ArcBall::mouseOnSphere(Vec3 mouse, Vec3 ballCenter, double ballRadius)
{
  Vec3 ballMouse;
  register double mag;
  ballMouse[0] = (mouse[0] - ballCenter[0]) / ballRadius;
  ballMouse[1] = (mouse[1] - ballCenter[1]) / ballRadius;
  mag = ballMouse[0]*ballMouse[0] + ballMouse[1]*ballMouse[1];
  if (mag > 1.0) {
    register double scale = 1.0/sqrt(mag);
    ballMouse[0] *= scale; ballMouse[1] *= scale;
    ballMouse[2] = 0.0;
  } else {
    ballMouse[2] = sqrt(1 - mag);
  }
  //ballMouse[3] = 0.0;
  return (ballMouse);
}

/* Construct a unit quaternion from two points on unit sphere */
Quat 
ArcBall::quatFromBallPoints(Vec3 from, Vec3 to)
{
  Quat qu;
  qu[0] = from[1]*to[2] - from[2]*to[1];
  qu[1] = from[2]*to[0] - from[0]*to[2];
  qu[2] = from[0]*to[1] - from[1]*to[0];
  qu[3] = from[0]*to[0] + from[1]*to[1] + from[2]*to[2];
  return (qu);
}

/* Convert a unit quaternion to two points on unit sphere */
void 
ArcBall::quatToBallPoints(Quat q, Vec3 *arcFrom, Vec3 *arcTo)
{
  double s;
  s = sqrt(q[0]*q[0] + q[1]*q[1]);
  if (s == 0.0) {
    *arcFrom = Vec3 (0.0, 1.0, 0.0);
  } else {
    *arcFrom = Vec3 (-q[1]/s, q[0]/s, 0.0);
  }
  (*arcTo)[0] = q[3]*(*arcFrom)[0] - q[2]*(*arcFrom)[1];
  (*arcTo)[1] = q[3]*(*arcFrom)[1] + q[2]*(*arcFrom)[0];
  (*arcTo)[2] = q[0]*(*arcFrom)[1] - q[1]*(*arcFrom)[0];
  if (q[3] < 0.0) *arcFrom = Vec3 (-(*arcFrom)[0], -(*arcFrom)[1], 0.0);
}

/* Force sphere point to be perpendicular to axis. */
Vec3 
ArcBall::constrainToAxis (Vec3 loose, Vec3 axis)
{
  Vec3 onPlane;
  register float norm;
  onPlane = loose - (axis * (axis * loose));
  norm = (float) onPlane.norm();
  if (norm > 0.0) {
    if (onPlane[2] < 0.0) onPlane = -onPlane;
    return onPlane * (1/sqrt(norm));
  }
  if (axis[2] == 1) {
    onPlane = Vec3 (1.0, 0.0, 0.0);
  } else {
    Vec3 tmp (-axis[1], axis[0], 0.0);
    tmp.normalize ();
    onPlane = tmp;
  }
  return (onPlane);
}

/* Find the index of nearest arc of axis set. */
int 
ArcBall::nearestConstraintAxis(Vec3 loose, Vec3 *axes, int nAxes)
{
  Vec3 onPlane;
  register float max, dot;
  register int i, nearest;
  max = -1; 
  nearest = 0;
  for (i=0; i<nAxes; i++) {
    onPlane = constrainToAxis (loose, axes[i]);
    dot = float(onPlane * loose);
    if (dot > max) {
      max = dot; 
      nearest = i;
    }
  }
  return (nearest);
}
