// Extreme Wave -- A 3d modelling and animation environment
// Copyright (C) 1999 James Dean Palmer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// Please report all bugs and problems to "james@tiger-marmalade.com".

#include <FL/Fl.H>
#include "keys.h"

Key_Binding_List key_bindings;
static Array<long> sequence;
static short meta;

void
print_sequence (Array<long> & sequence)
{
  if (sequence.size() > 0)
    {
      printf("Sequence: ");
      for (unsigned long i=0; i<sequence.size(); i++)
	{
	  char * s = (char *)malloc(sequence.size()*2+1);
	  s[0] = '\0';
	  long k = sequence[i];
	  
	  if (k > FL_META)
	    {
	      k -= FL_META;
	      s = strcat(s, "M-");
	    }
	  
	  if (k > FL_ALT)
	    {
	      k -= FL_ALT;
	      s = strcat(s, "A-");
	    }
	  
	  if (k > FL_CTRL)
	    {
	      k -= FL_CTRL;
	      s = strcat(s, "C-");
	    }
	  
	  if (k > FL_SHIFT)
	    {
	      k -= FL_SHIFT;
	      s = strcat(s, "S-");
	    }
	  
	    {
	      char c = (char)k;
	      s = strncat(s, (char *)&c, 1);
	      s = strcat(s, " ");
	    }
	  printf(s);
	  free(s);
	}
      printf("\n");
    }
}

void
keyboard_handler ()
{
  unsigned long i;
  long key = Fl::event_key();

  if ((key == 'g') && (Fl::event_state(FL_CTRL)))
    {
      keyboard_handler_clear();
      return;
    }

  if ((key == FL_Meta_L) || (key == FL_Meta_R) 
      || (key == FL_Shift_L) || (key == FL_Shift_R) 
      || (key == FL_Alt_L) || (key == FL_Alt_R) 
      || (key == FL_Control_L) || (key == FL_Control_R))
    return;

  if (key == FL_Escape) 
    {
      meta = 1;
      return;
    }

  if (Fl::event_state(FL_META) || meta) 
    key += FL_META;
  if (Fl::event_state(FL_SHIFT))
    key += FL_SHIFT;
  if (Fl::event_state(FL_ALT))
    key += FL_ALT;
  if (Fl::event_state(FL_CTRL))
    key += FL_CTRL;

  sequence.push_back (key);

  print_sequence (sequence);
  
  for (i=0; i<key_bindings.size(); i++)
    {
      if (*(key_bindings[i]->sequence) == sequence)
	{
	  if (SCM_CLOSUREP(key_bindings[i]->call_back))
	    scm_apply(key_bindings[i]->call_back, SCM_EOL, SCM_EOL);
	  keyboard_handler_clear();
	  break;
	}
    }

  meta = 0;
}

void keyboard_handler_clear()
{
  sequence.clear ();
}
