/*
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".
*/

#include <ewave/geometry/math.h>
#include "viewport.h"
#include <FL/Fl_Tile.H>
//#include <gFL/guile_fltk.h>
#include "../FakeGuile.h"

//long viewport_tag;

GFL_EXTERN_TYPE (fltk_window);
GFL_EXTERN_TYPE (fltk_widget);

static 
SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static 
scm_sizet
Sfree (SCM widget_smob)
{
  delete (EW_Fl_Viewport *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (GFL_Widget) + sizeof (EW_Fl_Viewport));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  EW_Fl_Viewport *widget = (EW_Fl_Viewport *) gh_cdr (widget_smob);
  scm_puts ("#<widget EW_Fl_Viewport ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("ew-viewport",
		   "ew-viewport description",
		   ew_viewport, fltk_group,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/
#if 1
GFL_PROCEDURE_5_1
( ew_viewport_new,
  "ew-viewport-new",
  "description",
  x, y, w, h, t,
  int, int, int, int, int,
  ew_viewport,
  fltk_window)
#else
fltk_window ew_viewport_new(
  int x, int y, int w, int h, int t)
#endif
{
  GFL_ASSERT_5 (ew_viewport_new, x, y, w, h, t);  
  SCM widget_smob;
  EW_Fl_Viewport * widget = new EW_Fl_Viewport(gh_scm2int(x), 
						   gh_scm2int(y), 
						   gh_scm2int(w), 
						   gh_scm2int(h), 0);
  widget->gl->view_type (gh_scm2int(t));

/*  switch (widget->gl->view_type)
    {
    case ortho_right:
      widget->output->value("Orthographic Right");
      break;
    case ortho_left:
      widget->output->value("Orthographic Left");
      break;
    case ortho_top:
      widget->output->value("Orthographic Top");
      break;
    case ortho_bottom:
      widget->output->value("Orthographic Bottom");
      break;
    case ortho_front:
      widget->output->value("Orthographic Front");
      break;
    case ortho_back:
      widget->output->value("Orthographic Back");
      break;
    }
 */
  GFL_NEW_SMOB (ew_viewport, widget_smob, widget);
  // Double protection
  //scm_protect_object(widget_smob);

  // Register this as a viewport that will be updated by Extreme Wave.
  viewport_list.push_back (widget);

//  return widget_smob;
	return widget;
}
#error
//deprecated:

/*
GFL_PROCEDURE_1_0
  (viewport_gl_show,
   "viewport-gl-show",
   "description",
   widget_smob,
   fltk_window)
{
  GFL_ASSERT_1 (viewport_gl_show, widget_smob);
  EW_Fl_Viewport *widget;
  widget = (EW_Fl_Viewport *) gh_cdr (widget_smob);
  widget->gl->show();

  return SCM_UNSPECIFIED;
}
*/

GFL_PROCEDURE_1_0
  (ew_viewport_show,
   "ew-viewport-show",
   "description",
   widget_smob,
   ew_viewport)
{
  GFL_ASSERT_1 (ew_viewport_show, widget_smob);
  EW_Fl_Viewport *widget;
  widget = (EW_Fl_Viewport *) gh_cdr (widget_smob);
  widget->show();
  //widget->gl->show();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
  (ew_viewport_hide,
   "ew-viewport-hide",
   "description",
   widget_smob,
   ew_viewport)
{
  GFL_ASSERT_1 (ew_viewport_hide, widget_smob);
  EW_Fl_Viewport *widget;
  widget = (EW_Fl_Viewport *) gh_cdr (widget_smob);
  widget->hide();
  return SCM_UNSPECIFIED;
}

/*
 * TODO: This next bit is uneccessarily long. Remove redundancies.
 */

GFL_PROCEDURE_2_0
  (ew_viewport_set_type,
   "ew-viewport-set-type",
   "Set the view type for this viewport.",
   widget, type,
   ew_viewport, symbol)
{
  GFL_ASSERT_2 (ew_viewport_set_type, widget, type);
  EW_Fl_Viewport *w = (EW_Fl_Viewport *) gh_cdr (widget);
  char * type_c = gh_symbol2newstr(type, 0);
  if (!strcmp(type_c, "ortho-top"))
    { 
      w->type (ortho_top); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "ortho-bottom"))
    { 
      w->type (ortho_bottom); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "ortho-front"))
    { 
      w->type (ortho_front); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "ortho-back"))
    { 
      w->type (ortho_back); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "ortho-left"))
    { 
      w->type (ortho_left); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "ortho-right"))
    { 
      w->type (ortho_right); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "ortho-arbitrary"))
    { 
      w->type (ortho_arbitrary); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "perspective-top"))
    { 
      w->type (perspective_top); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "perspective-bottom"))
    { 
      w->type (perspective_bottom); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "perspective-front"))
    { 
      w->type (perspective_front); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "perspective-back"))
    { 
      free (type_c);
      return SCM_UNSPECIFIED;
      w->type (perspective_back); 
    }
  if (!strcmp(type_c, "perspective-left"))
    { 
      w->type (perspective_left); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "perspective-right"))
    { 
      w->type (perspective_right); 
      free (type_c);
      return SCM_UNSPECIFIED;
    }
  if (!strcmp(type_c, "perspective-arbitrary"))
    { 
      w->type (perspective_arbitrary); 
      free (type_c);
      return SCM_UNSPECIFIED;
   } 
  free (type_c);
  return SCM_UNSPECIFIED;
}

/*
GFL_PROCEDURE_1_1
  (viewport_gl,
   "viewport-gl",
   "description",
   widget_smob,
   fltk_window,
   widg,
   fltk_widget)
{
  GFL_ASSERT_1 (viewport_gl, widget_smob);
  EW_Fl_Viewport *widget;
  widget = (EW_Fl_Viewport *) gh_cdr (widget_smob);

  return gfl_lookup((Fl_Widget *) widget->gl);
}
*/

GFL_PROCEDURE_1_1
  (ew_viewport_menu,
   "ew-viewport-menu",
   "description",
   widget_smob,
   ew_viewport,
   widg,
   fltk_widget)
{
  GFL_ASSERT_1 (ew_viewport_menu, widget_smob);
  EW_Fl_Viewport *widget;
  widget = (EW_Fl_Viewport *) gh_cdr (widget_smob);

  //return gfl_lookup((Fl_Widget *) widget->menu);
  extern Gfl_Class fltk_choice_class_tag;
  return gfl_lookup2((Fl_Widget *) widget->menu, fltk_choice_class_tag.tag);
}

/*
GFL_PROCEDURE_1_1
  (ew_viewport_menu_bar,
   "ew-viewport-menu-bar",
   "description",
   widget_smob,
   ew_viewport,
   widg,
   fltk_widget)
{
  GFL_ASSERT_1 (ew_viewport_menu_bar, widget_smob);
  EW_Fl_Viewport *widget;
  widget = (EW_Fl_Viewport *) gh_cdr (widget_smob);

  extern Gfl_Class fltk_menu_bar_class_tag;
  return gfl_lookup2((Fl_Widget *) widget->menu_bar, fltk_menu_bar_class_tag.tag);
}
*/

GFL_PROCEDURE_0_1
  (ew_get_active_viewport,
   "ewave::get-active-viewport",
   "Returns the active viewport.",
   widg,
   ew_viewport)
{
  if (current_gl)
    return gfl_lookup2((Fl_Widget *) current_gl, ew_viewport_class_tag.tag);
  return SCM_BOOL_F;
}

GFL_PROCEDURE_0_1
  (ew_get_active_viewport_window,
   "ewave::get-active-viewport-window",
   "Returns the active viewport window.",
   widg,
   ew_viewport)
{
  if ((current_gl) && (current_gl->parent ()))
    return gfl_lookup2((Fl_Widget *) current_gl->parent(), ew_viewport_class_tag.tag);
  return SCM_BOOL_F;
}

/*GFL_PROCEDURE_0_0
  (fill_right,
   "fill-right",
   "description" )
{
  EW_Fl_Viewport *c = 0;
  extern Fl_Tile* main_window_tile;
  for (int i=0; i < viewport_list.size(); i++)
    if (current_gl == viewport_list[i]->gl)
      c = viewport_list[i];

  if (c) tile_fill_right (main_window_tile, c);
} */

//tile_split_horizontal();
//tile_split_vertical();


void
viewport_init ()
{
  GFL_NEW_TYPE (ew_viewport);
  //  viewport_tag = scm_newsmob (&widget_smobfuns);
  // inherit parent's methods
  //fltk_group_subclass("viewport", viewport_tag);

  GFL_NEW_PROCEDURE (ew_viewport_new);
  //GFL_NEW_PROCEDURE (ew_viewport_gl_show);
  //GFL_NEW_PROCEDURE (ew_viewport_gl);
  GFL_NEW_PROCEDURE (ew_viewport_menu);
  //GFL_NEW_PROCEDURE (ew_viewport_menu_bar);
  GFL_NEW_PROCEDURE (ew_get_active_viewport);
  GFL_NEW_PROCEDURE (ew_get_active_viewport_window);
  GFL_NEW_PROCEDURE (ew_viewport_show);
  GFL_NEW_PROCEDURE (ew_viewport_hide);

  scm_sysintern ("ortho_left", gh_int2scm(ortho_left));
  scm_sysintern ("ortho_right", gh_int2scm(ortho_right));
  scm_sysintern ("ortho_top", gh_int2scm(ortho_top));
  scm_sysintern ("ortho_bottom", gh_int2scm(ortho_bottom));
  scm_sysintern ("ortho_front", gh_int2scm(ortho_front));
  scm_sysintern ("ortho_back", gh_int2scm(ortho_back));
  scm_sysintern ("ortho_arbitrary", gh_int2scm(ortho_arbitrary));
  scm_sysintern ("perspective_arbitrary", gh_int2scm(perspective_arbitrary));
}
