/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <stdio.h>
#include <GL/Glu.h>
#include <ewave/objects/ew_object.h>
#include <ewave/objects/ew_camera.h>
#include <ewave/gui/viewport.h>
#include <ewave/kernel/global.h>
#include <ewave/gui/keys.h>

#define BUFFER_LENGTH 64   // Select Buffer Length
#define RADIUS (10)

extern void           draw_grid();
//rsr: extern 
int            tmp_shade;
static Vec3           last_pnt;
Viewport_List         viewport_list;
EW_Fl_Gl_Window*      current_gl;
static Vec3        translation_origin;

EW_Fl_Gl_Window::EW_Fl_Gl_Window(int x, int y, int w, int h, const char *l) :
  Tb_Window(x,y,w,h,l)
{
  look_at.set(0, 0, 0);
  last_pnt.set(0, 0, 0);
  pan.set(0, 0, 0);
  view_type_ = ortho_left;
  current_camera = 0;
  arcball.showResult ();
}

void 
EW_Fl_Gl_Window::resize (int x2, int y2, int w2, int h2)
{
  Fl_Gl_Window::resize(x2, y2, w2, h2);

  if (global.aspect_ratio > 0.0)
    {
      double xcenter = (ortho_clip_left / 2.0) + (ortho_clip_right / 2.0);
      double halfw = global.aspect_ratio * w() * ratio_y() / 2.0;
      ortho_clip_left = xcenter - halfw;
      ortho_clip_right = xcenter + halfw;
    }
  ratio_x();
  ratio_y();
}

int
EW_Fl_Gl_Window::handle(int event)
{
  if (Fl::event_button() == 2)
    {
      // Switch to arbitrary mode:
      if (event == FL_PUSH)
	{
	  EW_Fl_Viewport *c = (EW_Fl_Viewport *)parent();
	  if (c->menu->value() < 8)
	    {
	      c->menu->value(7);
	      view_type_ = ortho_arbitrary;
	    }
	  else
	    {
	      if (c->menu->value() < 16)
		{
		  c->menu->value(16);
		  view_type_ = perspective_arbitrary;
		}
	      else
		{
		  // Camera Type:  We need to use this event to extract 
		  // camera inforamtion..
		}
	    }
	}
      //return Tb_Window::handle(event);
      switch (event)
	{
	 case FL_PUSH:
	  arcball.place (Vec3(0,0,0), RADIUS);
	  arcball.beginDrag ();
	  break;
	 case FL_RELEASE:
	  arcball.endDrag ();
	  break;
	 case FL_DRAG:
	  Vec3 tmp = Vec3( (2.0*Fl::event_x ()) / w() - 1.0, (2.0 * Fl::event_y ()) / h() - 1.0, 0);
	  printf("T=%g, %g, %g\n", tmp[0], tmp[1], tmp[2]);
	  arcball.mouse (tmp);
	  arcball.update ();
	  redraw ();
	  break;
	}
      return 1;
    }
  
  Vec3 a;

  switch(event)
    {
     case FL_MOVE:
      current_gl = this;
      if (kernel->current_object)
	{
	  x_now = Fl::event_x();
	  y_now = Fl::event_y();
	  redraw();
	}
      previous_event = FL_MOVE;
      break;

    case FL_PUSH:
      current_gl = this;
      x_begin = Fl::event_x();
      y_begin = Fl::event_y();
      
      switch (global.tool) {
      case TS_weld:
	{
	  // FIXME
	  global.tool = TS_pick;
	}
	break;
      case TS_pick:
	{
	  // The user just clicked down.  Find what point that click 
	  // corresponds to and set the current_object and current_point
	  // appropriately.

	  int selected_point = selection (x_begin, y_begin);

	  if (selected_point == -1)
	    {
	      kernel->current_object = 0;
	      kernel->current_point[0] = -1;
	      global.set_properties("");
	    }
	  else 
	    {
	      if ((selected_point != kernel->current_point[0]) 
		  || (kernel->current_object == 0))
		{
		  kernel->current_object = 
		    kernel->scene->object_from_point (selected_point);
		  
		  kernel->current_point = 
		    kernel->current_object->lookup_point (selected_point);
		}

	      if (kernel->previous_object != kernel->current_object)
		{
		  const char *d = kernel->current_object->vklass().name();
		  global.set_properties (d);
		  kernel->current_object->refresh ();
		}
	    }
	}

	kernel->previous_object = kernel->current_object;
	break;
      case TS_zoom:
      case TS_move:
	opan = pan;
	old_ortho_clip_left = ortho_clip_left;
	old_ortho_clip_right = ortho_clip_right;
	old_ortho_clip_bottom = ortho_clip_bottom;
	old_ortho_clip_top = ortho_clip_top;
	break;
      case TS_add_object:
	project (Vec3 (0, 0, 0));
	a = unproject (winx (Fl::event_x()), winy (Fl::event_y()));
	last_pnt = a;

	if (kernel->current_object == 0)
	  {
	    kernel->current_object = global.new_object ();
	    if (!kernel->current_group)
	      kernel->current_group = kernel->scene;
	    kernel->current_group->add (kernel->current_object);
	    
	    const char *d = kernel->current_object->vklass().name();
	    global.set_properties (d);
	  }
	kernel->current_object->add_point_down (a);	
	break;
      case TS_translate:
	project (Vec3 (0, 0, 0));
	translation_origin = unproject (winx (Fl::event_x()), winy (Fl::event_y()));
	break;
      }
      redraw();
      previous_event = FL_PUSH;
      return 1;
    case FL_DRAG:
      x_now = Fl::event_x();
      y_now = Fl::event_y();
      
      switch (global.tool) 
	{
	case TS_zoom:
	  {
	    pan[0] = opan[0] + (x_now - x_begin);
	    pan[1] = opan[1] + (y_now - y_begin);
	    double side;
	    
	    if (x_now < x_begin) 
	      side = 1.0f;
	    else
	      side = -1.0f;
	    
	    distance = sqrt ((double)((x_now-x_begin) * (x_now-x_begin) +
				      (y_now-y_begin) * (y_now-y_begin)));
	    
	    if (side < .0f) 
	      distance /= -1000.0;
	    else
	      distance /= 500.0;

	    double xcenter = ((old_ortho_clip_left + old_ortho_clip_right)
			      / 2.0);
	    double ycenter = ((old_ortho_clip_bottom + old_ortho_clip_top)
			      / 2.0);
	    double halfw = ((old_ortho_clip_right - old_ortho_clip_left) 
			    / 2.0);
	    double halfh = ((old_ortho_clip_top - old_ortho_clip_bottom) 
			    / 2.0);
	    
	    ortho_clip_left = xcenter - halfw - (halfw * distance);
	    ortho_clip_right = xcenter + halfw + (halfw * distance);
	    ortho_clip_top = ycenter + halfh + (halfh * distance);
	    ortho_clip_bottom = ycenter - halfh - (halfh * distance);

	    if ((ortho_clip_left == ortho_clip_right) ||
		(ortho_clip_top == ortho_clip_bottom))
	      {
		ortho_clip_left = old_ortho_clip_left;
		ortho_clip_right = old_ortho_clip_right;
		ortho_clip_bottom = old_ortho_clip_bottom;
		ortho_clip_top = old_ortho_clip_top;
	      }
	    redraw();
	  }
	  break;
	case TS_weld:
	  {
	    /*
	      FIXME:

	      Point Welding is broken.  To fix it, a "point-weld" plug-in needs 
	      to be designed for the coefficient database.

	      // If we are dragging something, and we suddenly want to weld it..
	      int weld_1;
	      int weld_2 = current_point;
	      weld_1 = selection ((double)x_now, (double)y_now, 0);
	      if (weld_1 == current_point)
	      weld_1 = selection ((double)x_now, (double)y_now, 1);
	      if ((weld_2 >= 0) || (weld_1 >= 0))
	      {
		//scene->swap_point(weld_1, weld_2);
		//POINT_DB point_db[weld_1].hidden = 1;
		//POINT_DB point_db[weld_1].locked = 1;
		current_point = weld_2;
	      }
	    */
	    global.tool = TS_pick;
	  }
	  break;
	case TS_move:
	  pan[0] = opan[0] + (double)(x_now - x_begin);
	  pan[1] = opan[1] + (double)(y_now - y_begin);
	  pan[2] = 0;
	  redraw();
	  break;
	case TS_pick:
	  if ((kernel->current_point[0] != -1) && (kernel->current_object))
	    {
	      //if (previous_event != FL_DRAG)
	      // printf ("\nWORK abs:%g %g %g\n", work[0], work[1], work[2]);
	      work = kernel->current_object->get_point_absolute (kernel->current_point);
	      project (work);
	      work = unproject (winx (Fl::event_x ()), winy (Fl::event_y ()));
	      kernel->current_object->select_point_drag (kernel->current_point, work);
	      redraw();
	    }
	  break;
	case TS_add_object:
	  project (Vec3 (0, 0, 0));
	  a = unproject (winx (Fl::event_x ()), winy (Fl::event_y ()));
	  
	  if (kernel->current_object == NULL)
	    {
	      // Sometimes we don't get a push event??  Get rid of me?
	      kernel->current_object = global.new_object ();
	      kernel->scene->add (kernel->current_object);
	      const char *d = kernel->current_object->vklass().name();
	      global.set_properties(d);
	    }
	  kernel->current_object->add_point_drag (Vec3(a[0], a[1], a[2]));
	  redraw ();
	  break;
	 case TS_translate:
	  if (kernel->current_object) 
	    {
	      kernel->current_object->set_damage (EW_Object::Dmg_translate);
	      project (Vec3 (0, 0, 0));
	      a = unproject (winx (Fl::event_x ()), winy (Fl::event_y ()));
	      a -= translation_origin;
	      translation_origin += a;
	      kernel->current_object->translate (a);
	      redraw_viewports ();
	    }
	  break;

	}
      previous_event = FL_DRAG;
      return 1;

    case FL_RELEASE:
      switch (global.tool)
	{
	case TS_pick:
	  if ((kernel->current_point[0] !=-1) && (kernel->current_object))
	    {
	      // project
	      // raw_project (kernel->current_object->get_point_absolute (kernel->current_point));
	      Vec3 work = kernel->current_object->get_point_absolute (kernel->current_point);
	      project (work);
	      work = unproject (winx (Fl::event_x ()), winy (Fl::event_y ()));
	      kernel->current_object->select_point_drag (kernel->current_point, work);
	      kernel->current_object->refresh ();
	      redraw_viewports();
	    }
	  break;
	case TS_translate:
	  if (kernel->current_object) 
	    {
	      kernel->current_object->set_damage (EW_Object::Dmg_translate);
	      project (Vec3 (0, 0, 0));
	      a = unproject (winx (Fl::event_x ()), winy (Fl::event_y ()));
	      a -= translation_origin;
	      translation_origin += a; //a.data;
	      kernel->current_object->translate (a);
	      redraw_viewports ();
	    }
	  break;

	case TS_zoom:
	  ratio_x();
	  ratio_y();
	  break;

	case TS_move:
	  break;

	case TS_add_object:
	  project (Vec3 (0, 0, 0));
	  a = unproject (winx (Fl::event_x ()), winy (Fl::event_y ()));

	  if (kernel->current_object)
	    {
	      kernel->current_object->add_point_release(a);
	      kernel->current_object->refresh ();
	    }
	  redraw_viewports();
	  break;
	}
      previous_event = FL_RELEASE;
      break;

    case FL_ENTER:
      previous_event = FL_ENTER;
      return 1;

    case FL_LEAVE:
      previous_event = FL_LEAVE;
      return 0;

    case FL_FOCUS: 
      previous_event = FL_FOCUS;
      return 1;

    case FL_UNFOCUS: 
      previous_event = FL_UNFOCUS;
      return 0;

    case FL_KEYBOARD:
      previous_event = FL_KEYBOARD;
      keyboard_handler();
      return 1;
    }
   return 0;
}

int
EW_Fl_Gl_Window::selection (int xPos, int yPos) // ProcessSelection
{
  //printf("(%d, %d)\n", xPos, yPos);
  // Space for selection buffer 
  GLuint selectBuff[BUFFER_LENGTH]; 
  // Hit counter and viewport storeage 
  GLint hits, viewport[4]; 
  // Setup selection buffer 
  glSelectBuffer (BUFFER_LENGTH, selectBuff); 
  glInitNames ();

  // Change render mode 
  glRenderMode (GL_SELECT); 

  // Get the viewport 
  glGetIntegerv (GL_VIEWPORT, viewport); 

  // Switch to projection and save the matrix 
  glMatrixMode (GL_PROJECTION); 
  glPushMatrix(); 
  glLoadIdentity(); 

  // Establish new clipping volume to be unit cube around 
  // mouse cursor point (xPos, yPos) and extending two pixels 
  // in the vertical and horzontal direction 
  gluPickMatrix(xPos, viewport[3]-yPos, 6, 6, viewport); 
  // Apply perspective matrix 
  // gluPerspective(60.0f, fAspect, 1.0, 425.0); 

  camera ();  
  //transform ();
  glMultMatrixd ((arcball.value ()).value ());
  
  // Draw the scene 
  // RenderScene(); 
  glMatrixMode (GL_MODELVIEW); 
  glPointSize(4.0);
  kernel->scene->draw (GL_SELECT, EW_Object::Drw_points);
  glPointSize(1.0);

  // Collect the hits 
  hits = glRenderMode(GL_RENDER); 
  // Restore the projection matrix 

  glMatrixMode (GL_PROJECTION); 
  glPopMatrix(); 

  // Go back to modelview for normal rendering 
  //glMatrixMode(GL_MODELVIEW); 

  GLuint lastz = 0, *ptr = selectBuff, sel = 0; //-1;
  for (int i = 0; i < hits; i++) {
    /* If the first element is greater than one, we made a mistake. */
    if (*ptr != 1)
      printf ("ERROR: Too many names on stack in uiptsSelect.");
    ptr++;
    if (i==0) {
      lastz = *ptr;
      ptr += 2;
      sel = *ptr;
    } else {
      if (*ptr < lastz) {
        lastz = *ptr;
        sel = *(ptr+2);
      }
      ptr += 2;
    }
    ptr++;
  }

  if (hits==0) return -1;
  return sel;
}

/*
  If we have a point Z that we want to move in 2D, we first need to 
  project it into screen coordinates.  Once it is projected, we
  can then unproject the point with a new x,y (screen) value to find
  the new points x,y,z coordinates.
*/

void
EW_Fl_Gl_Window::raw_project (Vec3 v)
{
  glGetDoublev (GL_MODELVIEW_MATRIX, modelMatrix);
  glGetDoublev (GL_PROJECTION_MATRIX, projMatrix);
  glGetIntegerv (GL_VIEWPORT, viewport);
  
  double winx, winy;
  gluProject(v[0],
	     v[1],
	     v[2],
	     modelMatrix,
	     projMatrix,
	     viewport,
	     &winx,
	     &winy,
	     &winz);
  printf ("EW_Fl_Gl_Window::raw_project %g %g %g -> %g %g %g\n", 
	  v[0], v[1], v[2], winx, winy, winz);
}

void
EW_Fl_Gl_Window::project (Vec3 v)
{
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();

  glLoadIdentity();
  camera ();
  //transform ();
  glMultMatrixd ((arcball.value ()).value ());
  
  raw_project (v);

  glMatrixMode( GL_PROJECTION );
  glPopMatrix();
}

Vec3
EW_Fl_Gl_Window::unproject (double mouse_x, double mouse_y)
{
  double objx, objy, objz;
  gluUnProject(mouse_x,
	       mouse_y,
	       winz,
	       modelMatrix,
	       projMatrix,
	       viewport,
	       &objx,
	       &objy,
	       &objz);
  printf("EW_Fl_Gl_Window::update_vector %g %g %g -> %g %g %g\n", 
	 mouse_x, mouse_y, winz, objx, objy, objz);

  return Vec3 (objx, objy, objz);
}

void
EW_Fl_Gl_Window::draw()
{
  glViewport (0,0,w(),h());
  glClearColor (0.3f, 0.3f, 0.3f, 0.3f);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glFlush ();

  // These should be definable per viewport
  glCullFace(GL_FRONT);
  glEnable(GL_DEPTH_TEST);

  glEnable(GL_DITHER);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
  glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);

  glLoadIdentity ();

//    {
//      int i;
//      glGetIntegerv(GL_MODELVIEW_STACK_DEPTH, &i);
//      printf("top stack depth = %d\n", i);
//    }

  /*

    FIXME:

    Lighting information should be extracted from the scene.
    If there is no lighting information it should fall back on
    some defaults like this.

  */

  glMatrixMode (GL_MODELVIEW); 
  glPushMatrix ();
  glLoadIdentity ();

//    {
//      int i;
//      glGetIntegerv(GL_MODELVIEW_STACK_DEPTH, &i);
//      printf("stack depth = %d\n", i);
//    }
  
  /*
  GLfloat light0_pos[4]   = { -5000.0, 5000.0, 0.0, 0.0 };
  GLfloat light0_color[4] = { .6, .6, .8, 1.0 };
  glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);
  glLightfv(GL_LIGHT0, GL_DIFFUSE,  light0_color);  
  glEnable(GL_LIGHT0);
*/
  
  //  GLfloat light1_pos[4]   = {  5000.0, 5000.0, 0.0, 0.0 };
//  GLfloat light1_color[4] = { .4, .4, 1, 1.0 };
//  glLightfv(GL_LIGHT1, GL_POSITION, light1_pos);
//  glLightfv(GL_LIGHT1, GL_DIFFUSE,  light1_color);
//  glEnable(GL_LIGHT1);
  
  glCullFace(GL_FRONT);

  if (tmp_shade == 0)
    {
      glDisable(GL_LIGHTING);
      glDisable(GL_COLOR_MATERIAL);  
      glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    }
  else
    {
      glEnable(GL_LIGHTING);
      glEnable(GL_COLOR_MATERIAL);  
      glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
      glPolygonMode(GL_FRONT, GL_FILL); // GL_FRONT_AND_BACK
      if (tmp_shade == 2)
	glShadeModel(GL_FLAT);
      else
	glShadeModel(GL_SMOOTH);
    }

  glMatrixMode (GL_MODELVIEW); 
  glPopMatrix();

  glMatrixMode (GL_PROJECTION);
  glPushMatrix();

  arcball.draw ();  
  
  glMatrixMode (GL_PROJECTION);
  glPopMatrix();
  glPushMatrix();
  
  glLoadIdentity();
  camera ();
  //transform ();
  glMultMatrixd ((arcball.value ()).value ());

  //#define RADIUS 30
  //glScalef (RADIUS, RADIUS, RADIUS);
  //drawbody(mNow);
  //popmatrix();

  glColor3f(0.7f, 0.3f, 0.3f);
  kernel->scene->draw (GL_RENDER, EW_Object::Drw_polygons);

  glDisable(GL_LIGHTING);
  glDisable(GL_COLOR_MATERIAL);  
  glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

  // Render all the points in the scene
  kernel->point_color.set(0.6, 0.2, 0.2);
  kernel->selected_point_color.set(1.0, 0.4, 0.4);

  glColor3dv (kernel->point_color.value());
  glPointSize (4.0);
  kernel->scene->draw (GL_RENDER, EW_Object::Drw_points);
  glPointSize (1.0);

  draw_grid ();
  
        GLfloat light0_pos[4]   = { -5000.0, 5000.0, 0.0, 0.0 };
      GLfloat light0_color[4] = { .6f, .6f, .8f, 1.0f };
      
      glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);
      glLightfv(GL_LIGHT0, GL_DIFFUSE,  light0_color);  
      glEnable(GL_LIGHT0);

  /*
  if ((global.tool == TS_add_object) && (kernel->current_object))
    {
      glBegin(GL_LINES);
      {
	raw_project (Vec3 (0, 0, 0));
	Vec3 curr_pnt = unproject (winx (x_now), winy (y_now));
	printf("\ncurr %g %g %g\n", curr_pnt[0], curr_pnt[1], curr_pnt[2]); 
	printf("last %g %g %g\n", last_pnt[0], last_pnt[1], last_pnt[2]); 
	printf("win = %g, %g\n", winx (x_now), winy (y_now));
	glVertex3dv( last_pnt.value() );
	glVertex3dv( curr_pnt.value() );
      }
      glEnd();
    }
   */

  glMatrixMode (GL_PROJECTION);  
  glPopMatrix();
}

double
EW_Fl_Gl_Window::ratio_x()
{
  if (view_type_ == perspective_arbitrary)
    {
      // non-trivial
      return 1.0;
    }
  _ratio_x = (ortho_clip_right - ortho_clip_left) / (double)w();
  return _ratio_x;
}


double
EW_Fl_Gl_Window::ratio_y()
{
  if (view_type_ == perspective_arbitrary)
    {
      // non-trivial
      return 1.0;
    }
  _ratio_y = (ortho_clip_top - ortho_clip_bottom) / (double)h();
  return _ratio_y;
}

void
EW_Fl_Gl_Window::camera ()
{
  Vec3 t_eye;
  Vec3 t_look_at;
  Vec3 t_up;
  
  double z = get_zoom();
  if (z <= 0) z = 1;

  if (view_type_ > 8)
    {
      if (view_type_ > 16) // camera
	{

#ifdef REM
	  if ((view_type_ == camera_primary) && (kernel->primary_camera))
	    {
	      t_eye = kernel->primary_camera->get_center ();
	      t_look_at = kernel->primary_camera.get_look_at ();
	      t_up = kernel->primary_camera.get_up ();
	    }
	  else
	    {
	      if ((view_type_ == camera_secondary) && (kernel->secondary_camera))
		{
		  t_eye = kernel->secondary_camera->get_center ();
		  t_look_at = kernel->secondary_camera.get_look_at ();
		  t_up = kernel->secondary_camera.get_up ();		  
		}
	      else
		{
		  // Reset to something sane - perspective arbitrary:
		  view_type_ = perspective_arbitrary;
		}
	    }
#endif

	}
      if (view_type_ < 16)
	{
	  t_eye = ((eye - look_at) * z) + look_at;
	  t_look_at = look_at;
	  t_up = up;
	  
	  //t_eye = t_eye + pan;
	  //t_look_at = t_look_at + pan;
	  //glTranslatef(pan[0]*z, pan[1]*z, pan[2]*z);
	  glTranslatef(pan[0]*.01f, -pan[1]*.01f, 0.0f);
	}
      gluPerspective(60.0, 1.0, 10, 10000);
    }
  else // orthographic
    {
      glOrtho ((w()*z/-2.)-(pan[0]*z), (w()*z/2.)-(pan[0]*z), 
	       (h()*z/-2)+(pan[1]*z), (h()*z/2)+(pan[1]*z), 
	       -10000, 10000);
      t_eye = eye;
      t_look_at = look_at;
      t_up = up;
    }
  gluLookAt (t_eye[0], t_eye[1], t_eye[2],
	     t_look_at[0], t_look_at[1], t_look_at[2],
	     t_up[0], t_up[1], t_up[2]);
#ifdef REM
  printf ("gluLookAt (%g, %g, %g,   %g, %g, %g,   %g, %g, %g)\n", 
	   t_eye[0], t_eye[1], t_eye[2],
	   t_look_at[0], t_look_at[1], t_look_at[2],
	   t_up[0], t_up[1], t_up[2]);
#endif

}

void
EW_Fl_Gl_Window::view_type (int vt)
{
  view_type_ = vt;

  EW_Fl_Viewport *c = (EW_Fl_Viewport *)parent();
  switch (vt)
    {
     case ortho_left:
      c->menu->value(1);
      eye.set (400, 0, 0);
      up.set (0, 1, 0);
      break;
     case ortho_right:
      c->menu->value(2);
      eye.set (-400, 0, 0);
      up.set (0, 1, 0);
      break;
     case ortho_top:
      c->menu->value(3);
      eye.set (0, 400, 0);
      up.set (0, 0, 1);
      break;
     case ortho_bottom:
      c->menu->value(4);
      eye.set (0, -400, 0);
      up.set (0, 0, 1);
      break;
     case ortho_front:
      c->menu->value(5);
      eye.set (0, 0, -400);
      up.set (0, 1, 0);
      break;
     case ortho_back:
      c->menu->value(6);
      eye.set (0, 0, 400);
      up.set (0, 1, 0);
      break;
     case ortho_arbitrary:
      c->menu->value(7);
      break;
      
     case perspective_left:
      c->menu->value(10);
      eye.set (400, 0, 0);
      up.set (0, 1, 0);
      break;
     case perspective_right:
      c->menu->value(11);
      eye.set (-400, 0, 0);
      up.set (0, 1, 0);
      break;
     case perspective_top:
      c->menu->value(12);
      eye.set (0, 400, 0);
      up.set (0, 0, 1);
      break;
     case perspective_bottom:
      c->menu->value(13);
      eye.set (0, -400, 0);
      up.set (0, 0, 1);
      break;
     case perspective_front:
      c->menu->value(14);
      eye.set (0, 0, -400);
      up.set (0, 1, 0);
      break;
     case perspective_back:
      c->menu->value(15);
      eye.set (0, 0, 400);
      up.set (0, 1, 0);
      break;
     case perspective_arbitrary:
      c->menu->value(16);
      break;
      
     case camera_primary:  // Extract this information from the camera(s).
      c->menu->value(15);
      eye.set (0, 0, 400);
      up.set (0, 1, 0);
      break;
     case camera_secondary:
      c->menu->value(15);
      eye.set (0, 0, 400);
      up.set (0, 1, 0);
      break;

    }
  redraw();
}

static void set_type_cb (Fl_Widget *a, void *b)
{
  EW_Fl_Viewport *c = (EW_Fl_Viewport *)(a->parent());
  c->gl->view_type((int)b);
  c->gl->initialize();
}

static Fl_Menu_Item menutable[] = {
    {"Orthographic",0,0,0,FL_SUBMENU},
    {"Orthographic Left", 0, set_type_cb, (void *)ortho_left, 0},
    {"Orthographic Right", 0, set_type_cb, (void *)ortho_right, 0},
    {"Orthographic Top", 0, set_type_cb, (void *)ortho_top, 0},
    {"Orthographic Bottom", 0, set_type_cb, (void *)ortho_bottom, 0},
    {"Orthographic Front", 0, set_type_cb, (void *)ortho_front, 0},
    {"Orthographic Back", 0, set_type_cb, (void *)ortho_back, 0},
    {"Orthographic Arbitrary", 0, set_type_cb, (void *)ortho_arbitrary, 0},
    {0},
    {"Perspective",0,0,0,FL_SUBMENU},
    {"Perspective Left", 0, set_type_cb, (void *)perspective_left, 0},
    {"Perspective Right", 0, set_type_cb, (void *)perspective_right, 0},
    {"Perspective Top", 0, set_type_cb, (void *)perspective_top, 0},
    {"Perspective Bottom", 0, set_type_cb, (void *)perspective_bottom, 0},
    {"Perspective Front", 0, set_type_cb, (void *)perspective_front, 0},
    {"Perspective Back", 0, set_type_cb, (void *)perspective_back, 0},
    {"Perspective Arbitrary", 0, set_type_cb, (void *)perspective_arbitrary, 0},
    {0},
    {"Camera",0,0,0,(FL_SUBMENU|FL_MENU_DIVIDER)},
    {"Camera - Primary", 0, set_type_cb, (void *)camera_primary, 0},
    {"Camera - Secondary", 0, set_type_cb, (void *)camera_secondary, 0},
    {0},
    {"Object Inspector", 0, set_type_cb, (void *)perspective_left, 0},
    {"Material Inspector", 0, set_type_cb, (void *)perspective_left, 0},
    {"Scene Graph", 0, set_type_cb, (void *)perspective_left, 0},
    {0}};

EW_Fl_Viewport::EW_Fl_Viewport(int x, int y, int w, int h, const char *t) :
    Fl_Group(x,y,w,h,t)
{
  menu = new Fl_Choice (x, y+(h-22), w, 22);
  menu->box (FL_THIN_UP_BOX);
  menu->selection_color (46);
  menu->labelsize(12);
  menu->textsize(12);
  menu->textcolor(52);
  menu->menu(menutable);
     
  box = new Fl_Box (x, y, w, h-menu->h(), t);
  box->color (FL_BLACK);
  box->box (FL_FLAT_BOX);
  box->show ();

  gl = new EW_Fl_Gl_Window(x+1, y+1, w-2, h-2-menu->h(),t);
  add_resizable(*(Fl_Widget *)gl);
}

void redraw_viewports()
{
  for (unsigned long i=0; i < viewport_list.size(); i++)
    viewport_list[i]->gl->redraw();
}

void resize_viewports()
{
  if (viewport_list.size() > 0)
    current_gl = viewport_list[0]->gl;

  for (unsigned long i=0; i<viewport_list.size(); i++)
    viewport_list[i]->resize(viewport_list[i]->x(),
			     viewport_list[i]->y(),
			     viewport_list[i]->w(),
			     viewport_list[i]->h());
}
