#ifndef EXTREME_VIEWPORT_H
#define EXTREME_VIEWPORT_H 1

#include <FL/Fl.H>
#include "Fl_Tb_Window/Fl_Tb_Window.h"
#include <FL/Fl_Window.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Menu_Bar.H>
//#include <FL/glut.H>
#include <FL/gl.h>

#include <ewave/kernel/array.h>
#include <ewave/gui/ArcBall.h>

// This is a subclass of Fl_Group that has an Fl_Gl_Window widget
// attached.

enum { 
  ortho_left=0, 
  ortho_right=1, 
  ortho_top=2, 
  ortho_bottom=3, 
  ortho_front=4, 
  ortho_back=5, 
  ortho_arbitrary=6, 
  perspective_left=9, 
  perspective_right=10, 
  perspective_top=11, 
  perspective_bottom=12, 
  perspective_front=13, 
  perspective_back=14, 
  perspective_arbitrary=15,
  camera_primary=18,
  camera_secondary=19
};

class EW_Camera;

class EW_Fl_Gl_Window : public Tb_Window //Fl_Gl_Window
{
public:
  EW_Fl_Gl_Window(int x, int y, int w, int h, const char *l=0);
  double ratio_x();
  double ratio_y();
  inline double winx (double wx) { return wx; }
  inline double winy(double wy) { return viewport[3] - wy; }
  Vec3 unproject (double mouse_x, double mouse_y); //update_vectori

  sgVec3 arbitrary_eye;
  sgVec3 arbitrary_look_at;
  sgVec3 arbitrary_up;

  // glOrtho stuff
  double ortho_clip_left;    // = -ViewPort[cview].nbx+ViewPort[cview].nobx
  double ortho_clip_right;   // =  ViewPort[cview].nbx+ViewPort[cview].nobx
  double ortho_clip_bottom;  // = -ViewPort[cview].nby+ViewPort[cview].noby
  double ortho_clip_top;     // =  ViewPort[cview].nby+ViewPort[cview].noby

  EW_Camera * current_camera;
  void resize(int x,int y,int w,int h);
  void view_type (int);

  void raw_project (Vec3 v);
  void project (Vec3 v);
  
protected:
  void draw();
  int selection(int, int);
  void draw_overlay() {}
  int handle(int event);
  Vec3 work;
private:

  // gluPerspective stuff
  double perspective_fovy;   // field of view (in degrees) in y direction
  double perspective_aspect; // aspect ratio of x to y

  // gluPerspective and glOrtho stuff
  double clip_near;          // (-10000.0 / 10)
  double clip_far;           // 10000.0

  // misc
  void camera();             // Sets the camera correctly for this window.
  int view_type_;             // perspective, ortho, oddball
  int x_begin;               // The point.x where a drag originated.
  int y_begin;               // The point.y where a drag originated.
  int x_now;
  int y_now;
  double distance;           // Distance between xy_begin and xy_now
  
  double old_ortho_clip_left;    // = -ViewPort[cview].nbx+ViewPort[cview].nobx
  double old_ortho_clip_right;   // =  ViewPort[cview].nbx+ViewPort[cview].nobx
  double old_ortho_clip_bottom;  // = -ViewPort[cview].nby+ViewPort[cview].noby
  double old_ortho_clip_top;     // =  ViewPort[cview].nby+ViewPort[cview].noby

  double _ratio_x;               // the last ratio_x()
  double _ratio_y;               // the last ratio_x()

  int click_count;
  int previous_event;

  GLfloat win[3];
  GLdouble modelMatrix[16], projMatrix[16];
  GLint viewport[4];
  double winz;
  
  Vec3 eye;
  Vec3 up;
  Vec3 look_at;

  Vec3 pan;
  Vec3 opan;
  ArcBall arcball;
};

extern EW_Fl_Gl_Window * current_gl;

class EW_Fl_Viewport : public Fl_Group
{
public:
  EW_Fl_Gl_Window *gl;
  Fl_Choice *menu;
  //Fl_Menu_Bar *menu_bar;
  Fl_Box *box;
  FL_EXPORT void show() {
    Fl_Group::show();
    if (gl)
      gl->show();
  }

  FL_EXPORT void set_type (int a)
    {
      //printf("new value=%d\n", a);
      //printf("current value=%d\n", gl->view_type);
      if (gl)
	gl->view_type (a);
      //gl->view_type_ = a;
    }
  
  FL_EXPORT void hide() {
    if (gl)
      gl->hide();
    Fl_Group::hide();
  }

  EW_Fl_Viewport(int x,int y,int w,int h, const char *t = 0);
  ~EW_Fl_Viewport ()
    {
      //delete menu_bar;
      delete menu;
      delete gl;
    }
};

typedef PArray<EW_Fl_Viewport> Viewport_List;
extern  Viewport_List viewport_list;

/*
typedef EW_Fl_Viewport *                 EW_Fl_Viewport_ptr;

class EW_Fl_Viewport_array : public EW_ptr_array 
{
public:
  EW_Fl_Viewport_ptr & first() 
    { return (EW_Fl_Viewport_ptr &)EW_array::first(); }
  EW_Fl_Viewport_ptr & last() 
    { return (EW_Fl_Viewport_ptr &)EW_array::last(); }
  EW_Fl_Viewport_ptr & operator[](EW_int i) 
    { return (EW_Fl_Viewport_ptr &)_data[i*_item_size]; }
  EW_Fl_Viewport_ptr & append(EW_int s) 
    { return (EW_Fl_Viewport_ptr &)EW_array::append(s); }
  EW_Fl_Viewport_ptr & append() 
    { return append(1); }
  EW_Fl_Viewport_ptr & insert(EW_int n, EW_int s) 
    { return (EW_Fl_Viewport_ptr &)EW_array::insert(n, s); }
  EW_Fl_Viewport_ptr * data() 
    { return (EW_Fl_Viewport_ptr *)_data; }
};

extern EW_Fl_Viewport_array viewport_list;
*/


void redraw_viewports();
void resize_viewports();

#endif
