
#include "3ds.h"

H3DSFile::H3DSFile()
{
	fh = NULL;
	error = 0;
	meshobjs = 0;
	meshobjlist = NULL;
}

H3DSFile::~H3DSFile()
{
	if (fh) fclose(fh);
	if (error) printf("Error code: %d\n",error);

	if(meshobjlist) {
		for(int n=meshobjs-1; n>=0; n--) {
			H3dsMeshObj * mobj = &meshobjlist[n];
			if(mobj->maplist)  delete mobj->maplist;
			if(mobj->facelist) delete mobj->facelist;
			if(mobj->vertlist) delete mobj->vertlist;
		}
		delete meshobjlist;
	}
}

short H3DSFile::openFile(char * filename)
{
	if ((fh = fopen(filename,"r")))
		return(1);
	else
		return(0);
}

void H3DSFile::fileRead(void * dest, int len)
{
	if(fread(dest, len, 1, fh) != 1) {
		printf("Error reading file\n");
		error = 1;
	}
}

void H3DSFile::fileSetPos(unsigned long pos)
{
	if(fseek(fh, (long)pos, SEEK_SET) != 0) {
		printf("Error seeking in file\n");
		error = 1;
	}
}

unsigned long H3DSFile::fileGetPos(void)
{
	long pos;
	if((pos=ftell(fh)) == -1L) {
		printf("Error getting fileposition\n");
		error = 1;
		return 0;
	}
	return (unsigned long) pos;
}



H3dsMeshObj *H3DSFile::getMeshObj(void)
{
	void *mem;
	if((mem=realloc(meshobjlist,
			sizeof(H3dsMeshObj)*(meshobjs+1))) == 0) {
		printf("Error reallocating mem\n");
		error = 1;
		return NULL;
	}
	meshobjlist=(H3dsMeshObj *) mem;
	H3dsMeshObj *obj=&meshobjlist[meshobjs++];
	memset(obj, 0, sizeof(H3dsMeshObj));
	return obj;
}

void *H3DSFile::getmem(int size)
{
	void *mem;
	if((mem=malloc(size))==0) {
		printf("Error allocating mem\n");
		error = 1;
		return NULL;
	}
	return mem;
}

void H3DSFile::readVertList(H3dsMeshObj * meshobj)
{
	H3dsword nv;
	fileRead(&nv, sizeof(nv));
	meshobj->vertices=nv;
	int k=nv;
	meshobj->vertlist=(H3dsVert *) getmem(sizeof(H3dsVert)*k);
	for(int n=0; n<k; n++) {
		fileRead(&meshobj->vertlist[n], sizeof(H3dsfloat)*3);
	}
}

void H3DSFile::readFaceList(H3dsMeshObj * meshobj)
{
	H3dsword nv;
	fileRead(&nv, sizeof(nv));
	meshobj->faces=nv;
	int k=nv;
	meshobj->facelist=(H3dsFace *) getmem(sizeof(H3dsFace)*k);
	for(int n=0; n<k; n++) {
		fileRead(&meshobj->facelist[n], sizeof(H3dsword)*4);
	}
}

void H3DSFile::readMapList(H3dsMeshObj * meshobj)
{
	H3dsword nv;
	fileRead(&nv, sizeof(nv));
	meshobj->maps=nv;
	int k=nv;
	meshobj->maplist=(H3dsMap *) getmem(sizeof(H3dsMap)*k);
	for(int n=0; n<k; n++) {
		fileRead(&meshobj->maplist[n], sizeof(H3dsfloat)*2);
	}
}

void H3DSFile::readTransMatrix(H3dsMeshObj * meshobj)
{
	fileRead(&meshobj->transmatrix, sizeof(H3dsfloat)*12);
	meshobj->matrix=1;
}

void H3DSFile::readTriMeshBlocks(H3dslong p, char * name)
{
	H3dsword id;
	H3dslong len, pc;
	H3dsMeshObj * meshobj=getMeshObj();
	strcpy(meshobj->name, name);
	while((pc=fileGetPos()) < p)
	{
		fileRead(&id, sizeof(id));
		fileRead(&len, sizeof(len));
		switch((int)id)
		{
			case CHUNK_VERTLIST: readVertList (meshobj); break;
			case CHUNK_FACELIST: readFaceList (meshobj); break;
			case CHUNK_MAPLIST:  readMapList  (meshobj); break;
			case CHUNK_TRMATRIX: readTransMatrix(meshobj); break;

			//case CHUNK_FACEMAT:
			//case CHUNK_SMOOLIST:
			default: fileSetPos(pc+len);
		}
	}
}

void H3DSFile::readObjBlocks(H3dslong p)
{
	H3dsword id;
	H3dslong len, pc;
	char name[16];

	// The object name is the first item
        unsigned int n=0;
	do {
		fileRead(&name[n++], 1);
	} while((name[n-1] != '\0') && (n < sizeof(name)));
	name[n-1]='\0';

	while((pc=fileGetPos()) < p)
	{
		fileRead(&id, sizeof(id));
		fileRead(&len, sizeof(len));
		switch((int)id)
		{
			case CHUNK_TRIMESH:	readTriMeshBlocks(pc+len, name); break;

			//case CHUNK_LIGHT:
			//case CHUNK_CAMERA:
			default: fileSetPos(pc+len);
		}
	}
}

void H3DSFile::readObjMeshBlocks(H3dslong p)
{
	H3dsword id;
	H3dslong len, pc;

	while((pc=fileGetPos()) < p)
	{
		fileRead(&id, sizeof(id));
		fileRead(&len, sizeof(len));
		switch((int)id)
		{
			case CHUNK_OBJBLOCK: readObjBlocks(pc+len); break;

			//case CHUNK_AMBCOLOR:
			//case CHUNK_BKGCOLOR:
			default: fileSetPos(pc+len);
		}
	}
}

void H3DSFile::readMainBlocks(H3dslong p)
{
	H3dsword id;
	H3dslong len, pc;

	while((pc=fileGetPos()) < p)
	{
		fileRead(&id, sizeof(id));
		fileRead(&len, sizeof(len));
		switch((int)id)
		{
			case CHUNK_OBJMESH: readObjMeshBlocks(pc+len); break;

			//case CHUNK_MATERIAL:
			//case CHUNK_KEYFRAMER:
			default: fileSetPos(pc+len);
		}
	}
}

short H3DSFile::read3DS(char *filename)
{
	H3dsword id;
	H3dslong len, pc;

	if (!openFile(filename)) return(0);

	pc = fileGetPos();

	fileRead(&id, sizeof(id));
	fileRead(&len, sizeof(len));

	if((int)id!=CHUNK_MAIN) {
			printf("Not 3ds data\n");
			return(0);
	}

	readMainBlocks(pc+len);

	if (error) return(0);

	return(1);
}

