/* Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <stdio.h>
//#include <gFL/guile_fltk.h>
#include <ewave/FakeGuile.h>
#include <FL/filename.H>
#include <ewave/gui/gui.h>
#include <ewave/kernel/global.h>

#include <ewave/objects/ew_object.h>
#include <ewave/objects/ew_group.h>
#include <ewave/objects/ew_camera.h>
#include <ewave/objects/ew_generic.h>
#include <ewave/objects/ew_sphere.h>
#include <ewave/objects/ew_nurbs.h>
#include <ewave/objects/ew_capsul.h>
#include <ewave/objects/ew_cube.h>
#include <ewave/objects/ew_cone.h>
#include <ewave/objects/ew_cylinder.h>
#include <ewave/objects/grid.h>

#include <ewave/materials/material.h>
#include <ewave/gui/keys.h>
#include <ewave/xpm_icons/world.xpm>

extern void init_scheme_interface();
extern void viewport_init();
extern void resize_viewports();
extern SCM exception_handler(void *, SCM, SCM);
extern void GridBrowserClear();
extern void pdb_functions_init();
extern void update_pdb_browser();
extern void ergun_mesh();

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

int
file_is_directory (char *file_name)
{
  struct stat x;
  if (stat(file_name, &x) == 0)
    {
      if (S_ISDIR(x.st_mode))
	return 1;
    }
  return 0;
}
    
int
check_user_dir ()
{
  char name[128];

  // Create user directory and defaults if not found.
  
  filename_expand (name, "~/.ewave/");
  if (!(file_is_directory (name)))
    {
      printf("Creating directory \"~/.ewave/\"...\n");
    }

  filename_expand (name, "~/.ewave/plug-ins/");
  if (file_is_directory (name))
    {
      printf("loading user plugins..\n");
    }
  
  // Load plug-ins from current directory
  filename_expand (name, "./plug-ins/");
  if (file_is_directory (name))
    {
      printf("loading user plugins..\n");
    }
  
  // Load plug-ins in PREFIX/share/ewave/plugins/.
  filename_expand (name, "$PREFIX/share/ewave/plug-ins/");
  if (file_is_directory (name))
    {
      printf("loading user plugins..\n");
    }

  return 0;
}

static void
inner_main (void *closure, int argc, char **argv)
{
  check_user_dir();
  
  if (global.verbose) printf("Initializing guile-fltk\n");
  scm_init_fltk();

  printf("Registering core functions..\n");
  pdb_functions_init();
#ifdef SCHEME
  if (global.verbose) printf("Initializing ewave scheme interface..\n");
  init_scheme_interface();
#endif
  if (global.verbose) printf("Loading objects..\n");
  Fl_Group::current (0);
#ifdef SCHEME
  extern void ew_base_scheme_init ();
  extern void ew_ubase_scheme_init ();
  printf("base: \n");
  ew_base_scheme_init ();
  printf("ubase: \n");
  ew_ubase_scheme_init ();
  printf("Registering objects: \n");
#endif
  EW_Object::initialize ();
  EW_Group::initialize ();
  EW_Camera::initialize ();
  EW_Generic::initialize ();
  EW_Sphere::initialize ();
  EW_Cube::initialize ();
  EW_Capsul::initialize ();
  EW_Cone::initialize ();
  EW_Cylinder::initialize ();
  EW_NURBS::initialize ();
   
  kernel->scene = new EW_Group();
  kernel->scene->set_name (strdup("World"));
  kernel->current_group = kernel->scene;
  kernel->current_object = NULL;
  kernel->current_point.set(-1, -1, -1);
  
  
  
  if (global.verbose) printf("Initializing scheme_viewport..\n");
  viewport_init();

  extern Gfl_Class fltk_tile_class_tag;
  extern Gfl_Class fltk_window_class_tag;
  if (global.verbose) printf("Exporting Fltk objects to scheme space..\n");
  gfl_export_object("prime-tile", main_window_tile, fltk_tile_class_tag.tag);
  gfl_export_object("prime-window", main_window, fltk_window_class_tag.tag);

  if (global.verbose) printf("Evaluating scheme files..\n");
  gh_eval_file_with_catch("scheme/goosh.scm", &exception_handler);
  gh_eval_file_with_catch("scheme/keys.scm", &exception_handler);
  gh_eval_file_with_catch("scheme/init.scm", &exception_handler);

  if (global.verbose) printf("Resizing viewports..\n");
  resize_viewports ();

  if (global.verbose) printf("Adding global group to scene graph..\n");
  scene_graph->add_sub("Global", 1, new Fl_Pixmap(world_xpm), kernel->scene);

  if (global.verbose) printf("Evaluating scheme export scripts..\n");

  gh_eval_file_with_catch("scheme/povray.scm", &exception_handler);

  if (global.verbose) printf("Setting version information..\n");

  int * cw = new int[2];
  cw[0] = 200;
  cw[1] = 0;
  about_browser->column_widths(cw);

  char buffer[80];
  sprintf(buffer, "Extreme Wave version\t= %d.%d.%d", 
	  EWAVE_MAJOR_VERSION,
	  EWAVE_MINOR_VERSION,
	  EWAVE_PATCH_VERSION);
  about_browser->add(buffer);
  
  sprintf(buffer, "Fltk version\t= %d.%d.%d",
	  FL_MAJOR_VERSION,
	  FL_MINOR_VERSION,
	  FL_PATCH_VERSION);
  about_browser->add(buffer);
  
#ifdef HAVE_STYLES
  sprintf(buffer, "Fltk styles\t= yes");
#else
  sprintf(buffer, "Fltk styles\t= no");
#endif
  about_browser->add(buffer);  
  
  char *minor = gh_scm2newstr(scm_minor_version(), NULL);
  char *major = gh_scm2newstr(scm_major_version(), NULL);
  sprintf(buffer, "Guile version\t= %s.(%s)", major, minor);
  //free(minor);
  //free(major);

  about_browser->add(buffer);
  
  sprintf(buffer, "guile-fltk version\t= %d.%d.%d", 
	  GUILE_FLTK_MAJOR, 
	  GUILE_FLTK_MINOR,
	  GUILE_FLTK_PATCH);
  about_browser->add(buffer);

  update_pdb_browser();
  
  if (global.verbose) printf("Running Fltk..\n");
  Fl::run();
}

void 
ewave_init (int argc, char** argv)
{
  kernel = new EW_Kernel();
  global.verbose = 1;

  printf("Extreme Wave (version %d.%d.%d)\n", 
	 EWAVE_MAJOR_VERSION,
	 EWAVE_MINOR_VERSION,
	 EWAVE_PATCH_VERSION);
  printf("(c) 1998-1999 James Dean Palmer\n");
  printf("A Tiger Marmalade Production\n\n");

  make_main_window();
  make_scene_graph_inspector();
  make_object_inspector();
  make_grid_inspector();
  make_material_inspector();
  make_bone_inspector();
  make_about_window();
  make_pdb_browser();

  // Initialize some interface elements.
  material_browser_clear();
     
  // Display the user interface.
  main_window->show(argc, argv);
  main_window->end();

  // Initialize some data structure elements.
  global.tool_delta = 2.0;
  global.tool = TS_pick;
  global.aspect_ratio = -1;
  GridBrowserClear();
}

void 
ewave_run (int argc, char** argv)
{
  if (global.verbose) printf("\nSanity Checking registered objects:\n");
  for (EW_Class_List::cursor h = kernel->object_registry.begin(); &h; h++)
    if (global.verbose) printf("  Found \"%s\".\n", (&h)->name());

  printf("\n");
  scm_boot_guile (argc, argv, inner_main, 0);
  int state = Fl::run();
}
