#ifndef _EIFF_H_
#define _EIFF_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>

typedef struct EIFF_node
{
  unsigned long size;
  unsigned long seek;
  EIFF_node *node;
} EIFF_node;

typedef struct EIFF
{
  FILE* file;
  unsigned long align;
  EIFF_node *node;
} EIFF;

typedef int (*reader_function_type)(EIFF*, unsigned long);

typedef struct rchunk
{
  unsigned long chunk_id;
  reader_function_type reader_function;
} rchunk;

// Call this before using EIFF
void eiff_init ();

// Assign a read routine to a particular chunk.
void   eiff_register_chunk (long chunk_id, 
			    reader_function_type reader_function);

// Open a file for reading or writing.
EIFF*  eiff_open (char *name, char* format);

// Read from the file.
int    eiff_read_chunk (EIFF *eiff, void *);
long   eiff_read_long (EIFF *eiff);
double eiff_read_double (EIFF *eiff);
char*  eiff_read_string (EIFF *eiff);

// Write to the file..
int    eiff_write_chunk (EIFF *eiff, long chunk_id);
int    eiff_finish_chunk (EIFF* eiff);
void   eiff_write_long (EIFF *eiff, long x);
void   eiff_write_double (EIFF *eiff, double x);
void   eiff_write_string (EIFF *eiff, char *string);


enum {
  // String CHUNK
  CHUNK_STRING            = 0x0001,
  CHUNK_NEXT              = 0x0002,
  CHUNK_FORM              = 0x0003,

  // EW_Object CHUNKS
  CHUNK_EW_OBJECT         = 0x0010,
  CHUNK_EW_COEFFICIENTS   = 0x1000,
  CHUNK_EW_POINTS         = 0x1010,
  CHUNK_EW_ATTRIBUTES     = 0x1020,
  CHUNK_EW_CENTER         = 0x1030,

  // EW_Generic CHUNKS
  CHUNK_EW_GENERIC_RADIUS = 0x1040,
  CHUNK_EW_GENERIC_SCALE  = 0x1041,
  CHUNK_EW_GENERIC_ROTATE = 0x1040

};

#ifdef LITTLE_ENDIAN
#pragma message( "LITTLE_ENDIAN" )
inline long long int bswap64(long long int x)
{	union { unsigned long long int __ll;                            
	unsigned long int __l[2]; } __w, __r;                   
	__w.__ll = (x);                                                 
	__r.__l[0] = htonl (__w.__l[1]);                                
	__r.__l[1] = htonl (__w.__l[0]);                                
	return __r.__ll;
}
#else
#pragma message( "BIG_ENDIAN" )
inline long long int bswap64(long long int x)
{	return x;
}
#endif

#endif
