#ifndef _EW_KERNEL_
#define _EW_KERNEL_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ewave/kernel/zavltree.h>
#include <ewave/kernel/iavltree.h>
#include <ewave/geometry/math.h>

class EW_Object;
class EW_Group;
class Fl_Pixmap;
class Fl_Group;

typedef EW_Object * (EW_Callback0 )();

class EW_Class
{
 public:
  EW_Class () 
    {
      name_ = 0;
      id_ = -1;
      create_ = 0;
      pixmap_ = 0;
      inspector_ = 0;
      uid++;
      id_ = uid;
    }
  const char*        name () { return name_; }
  void               name (char * c) { if (name_) free (name_); name_ = strdup (c); }
  long               id () { return id_; }
  void               id (long i) { id_ = i; }
  EW_Callback0*      create () { return create_; }
  void               create (EW_Callback0* x) { create_ = x; }
  Fl_Group*          inspector () { return inspector_; }
  void               inspector (Fl_Group *g);
  Fl_Pixmap*         pixmap () { return pixmap_; }
  void               pixmap (Fl_Pixmap *g) { pixmap_ = g; }
  EW_Object*         construct () { return create_(); }
  static const char* evaluator (EW_Class *);
 private:
  char*              name_;
  long               id_;
  EW_Callback0*      create_;
  Fl_Group*          inspector_;
  Fl_Pixmap*         pixmap_;
  static int         uid;
};

typedef ZAVLTree<EW_Class> EW_Class_List;

class EW_Kernel {
 public:
  EW_Kernel ()
    {
      object_registry.init (EW_Class::evaluator);
      current_record = 0;
      Click = 0;
    }
  EW_Group*      scene;
  EW_Group*      current_group;
  EW_Object*     current_object;
  EW_Class*      current_record;
  EW_Object*     previous_object;
  Point          current_point;
  EW_Class_List  object_registry;
  Vec3           point_color;
  Vec3           selected_point_color;
  double         time;
  
  void register_object (EW_Class *t)
    {
      object_registry.insert (t);
    }
  int get_clicks () { return Click; }
  void set_clicks (int x) { Click=x; }
  void click () { Click++; }
 protected:
  int Click;
};

extern EW_Kernel *kernel;

#endif
