/* Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <stdio.h>

//#include <FL/glut.H>
#include <FL/fl_file_chooser.H>

#include <ewave/gui/gui.h>
#include <ewave/objects/ew_object.h>
#include <ewave/materials/material.h>
#include <ewave/kernel/global.h>
#include <ewave/xpm_icons/world.xpm>
#include <ewave/FakeGuile.h>

extern void redraw_viewports();

void
update_frame_line()
{
  char t[20];
  sprintf (t, "BROKEN");
  //FIXME sprintf(t, "%d of %d", current_frame, frame_count-1);
  frame_line->value(t);
}

void 
update_pdb_browser()
{
  /*
  char t[256];
  for (int i=0; i<pdb_length; i++)
    {
      sprintf(t, "Name:\t%s", pdb[i].name);
      pdb_browser->add(t);
      sprintf(t, "Author:\t%s", pdb[i].author);
      pdb_browser->add(t);
      sprintf(t, "Desc:\t%s", pdb[i].docstring);
      pdb_browser->add(t);
      pdb_browser->add("\t");
    }
   */
}

#ifdef SCHEME

GFL_PROCEDURE_0_1
( ew_file_new,
  "ewave::file-new",
  "Clears the current scene and creates a new scene.",
  rval,
  bool)
{
  delete kernel->scene;
  material_list.clear();
  kernel->scene = new EW_Group();
  kernel->scene->set_name (strdup("World"));
  scene_graph->clear();
  scene_graph->add_sub("World", 1, new Fl_Pixmap(world_xpm), kernel->scene);
  redraw_viewports();
  global.tool = TS_pick;

  for (unsigned int i=0; i<viewport_list.size(); i++)
    (viewport_list[i])->gl->current_camera=0;

  return SCM_BOOL_T;
}

GFL_PROCEDURE_1_1
( ew_file_open,
  "ewave::file-open",
  "Opens a new file",
  file_name,
  str,
  rval,
  bool )
{
  char *b;
  b = fl_file_chooser("Open a file", "*.ew", NULL);
  if (b) 
    {
      //point_db.reset();
      material_list.clear();
      //bone_list.reset();
      delete kernel->scene;
      printf("loading scene\n");
      kernel->scene = 0; // (EW_Group *)load_native(b);
      printf("redrawing viewports\n");
      redraw_viewports();
      printf("update material browser\n");
      //material_browser_update();
      printf("update material browser 2\n");
      //material_browser_set_fields();
      kernel->current_group = kernel->scene;
      global.tool = TS_pick;
      printf("done w pdb..\n");
      EW_Object::read (b);
      return SCM_BOOL_T;
    }
  return SCM_BOOL_F;
}

GFL_PROCEDURE_0_1
( ew_file_save,
  "ewave::file-save",
  "Saves a file",
  rval,
  bool )
{
  char * a;
  a = fl_file_chooser("Save a file", "*.ew", NULL);
  if (a) 
    kernel->scene->write (a);
  else
    return SCM_BOOL_F;

  return SCM_BOOL_T;
}

GFL_PROCEDURE_1_1
( ew_file_save_as,
  "ewave::file-save-as",
  "Saves a file as..",
  file_name,
  str,
  rval,
  bool )
{
  char *b;
  b = fl_file_chooser("Save a file", "*.ew", NULL);
  if (b) 
    {
      kernel->scene->write (b);
      return SCM_BOOL_T;
    }
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
( ew_file_export_povray,
  "ewave::file-export-povray",
  "Export to povray",
  file_name,
  str,
  rval,
  bool )
{
  char *b;
  b = fl_file_chooser("Save a file", "*.pov", NULL);
  if (b) 
    {
      char temp[128];
      sprintf(temp, "(povray-export \"%s\")", b);
      gh_eval_str(temp);
      return SCM_BOOL_T;
    }
  return SCM_BOOL_F;
}

GFL_PROCEDURE_1_1
( ew_file_export_povray_anim,
  "ewave::file-export-povray-anim",
  "ewave::file-export-povray-anim",
  file_name,
  str,
  rval,
  bool )
{
  char *b;
  b = fl_file_chooser("Save a file", "*.pov", NULL);
  if (b) 
    {
      char temp[128];
      sprintf(temp, "(povray-export-anim \"%s\")", b);
      gh_eval_str(temp);
      return SCM_BOOL_T;
    }
  return SCM_BOOL_F;
}

GFL_PROCEDURE_0_0
( ew_quit,
  "ewave::quit",
  "Quit Extreme Wave? Are you crazy?" )
{
  // TODO: Add a file requester -- are you sure?
  // TODO: Check for unsaved work.
  exit (0);
}
#error

void
pdb_functions_init ()
{
  GFL_NEW_PROCEDURE (ew_file_new);
  GFL_NEW_PROCEDURE (ew_file_open);
  GFL_NEW_PROCEDURE (ew_file_save);
  GFL_NEW_PROCEDURE (ew_file_save_as);
  GFL_NEW_PROCEDURE (ew_file_export_povray);
  GFL_NEW_PROCEDURE (ew_file_export_povray_anim);
  GFL_NEW_PROCEDURE (ew_quit);
  update_frame_line();
}

#endif