/* Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/

#include <FL/Fl_ToggleTree.H>
#include <FL/Fl_Window.H>
#include <libguile.h>
#include <guile/gh.h>
#include <ewave/objects/ew_object.h>
#include <ewave/materials/material.h>
#include <ewave/kernel/global.h>
#include <ewave/gui/gui.h>
#include <ewave/gui/keys.h>
#include <gFL/gfl.h>

GFL_EXTERN_TYPE (ew_group);

extern int current_point;
int tmp_shade=0;

static SCM
ewave_shade_toggle ()
{
  if (tmp_shade) tmp_shade = 0;
  else tmp_shade = 1;
  redraw_viewports ();
  return SCM_UNSPECIFIED;
}

static SCM
ewave_shade_wire ()
{
  tmp_shade = 0;
  redraw_viewports ();
  return SCM_UNSPECIFIED;
}

static SCM
ewave_shade_smooth ()
{
  tmp_shade = 1;
  redraw_viewports ();
  return SCM_UNSPECIFIED;
}

static SCM
ewave_shade_flat ()
{
  tmp_shade = 2;
  redraw_viewports ();
  return SCM_UNSPECIFIED;
}

static SCM
ewave_set_tool (SCM tool)
{
  SCM_ASSERT (SCM_INUMP (tool), tool,  SCM_ARG1, "ewave-set-tool");
  global.tool = gh_scm2long (tool);
  if (global.tool == TS_add_object)
    kernel->current_object = 0;
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( kernel_set_object,
  "kernel::set-object",
  "kernel::set-object",
  object_type,
  str )
{
  GFL_ASSERT_1 (kernel_set_object, object_type);
  char *t = gh_scm2newstr(object_type, NULL);
  global.set_object(t);
  free (t);
  return SCM_UNSPECIFIED;
}

static SCM
ewave_get_tool (void)
{
  return gh_long2scm (global.tool);
}


static SCM
ewave_set_tool_delta (SCM delta)
{
  SCM_ASSERT ((SCM_NIMP (delta)), delta, SCM_ARG1, "ewave-set-tool-delta");
  global.tool_delta = gh_scm2double (delta);
  return SCM_UNSPECIFIED;
}


static SCM
ewave_get_tool_delta (void)
{
  return gh_double2scm (global.tool_delta);
}

static SCM
ewave_get_patch_version (void)
{
  return gh_long2scm (EWAVE_PATCH_VERSION);
}

static SCM
ewave_get_minor_version (void)
{
  return gh_long2scm (EWAVE_MINOR_VERSION);
}


static SCM
ewave_get_major_version (void)
{
  return gh_long2scm (EWAVE_MAJOR_VERSION);
}


static SCM
ewave_set_aspect_ratio (SCM aspect_ratio)
{
  SCM_ASSERT ((SCM_NIMP (aspect_ratio)), aspect_ratio, SCM_ARG1, "set-aspect-ratio");
  global.aspect_ratio = gh_scm2double (aspect_ratio);
  return SCM_UNSPECIFIED;
}


static SCM
ewave_get_aspect_ratio (void)
{
  return gh_double2scm (global.aspect_ratio);
}


static SCM
ewave_get_current_point (void)
{
  //point_db[current_point].display();
  return SCM_UNSPECIFIED;
}


static SCM
ewave_set_current_point (SCM a, SCM b, SCM c)
{
  /*
  SCM_ASSERT ((SCM_NIMP (a)), a, SCM_ARG1, "set-aspect-ratio");
  SCM_ASSERT ((SCM_NIMP (b)), b, SCM_ARG2, "set-aspect-ratio");
  SCM_ASSERT ((SCM_NIMP (c)), c, SCM_ARG3, "set-aspect-ratio");
  sgSetVec3(point_db[current_point].xyz,
	    gh_scm2double(a), 
	    gh_scm2double(b), 
	    gh_scm2double(c));
   */
  return SCM_UNSPECIFIED;
}


static SCM
ewave_export_povray (SCM a)
{
  SCM_ASSERT (SCM_NIMP (a) && SCM_STRINGP (a), a,
              SCM_ARG1, "export-povray");
  return SCM_UNSPECIFIED;
}

static SCM
get_attribute (SCM obj, SCM op)
{
  printf ("(get-attribute)\n");
  printf("  object_id = %ld\n", SCM_CAR (obj));
  return gfl_exe (SCM_CHARS(op), obj); //gh_cons(obj, SCM_EOL));
}

SCM
povray_bone_trick(SCM obj, SCM a)
{
  /*
  char r[128];
  EW_Generic * b = EW_Generic::scm2obj(obj);
  int bone = b->bone;
  double frame_offset = gh_scm2double(a);

  if ((current_frame > 0) && (bone >= 0))
    {
      sgVec3 tt;
      //double tt, rr, qq;
      bone_list[bone]->update( ((double)frame_offset)/10.0 );
      sgEULfromMat4(tt, bone_list[bone]->object_rotation);
      sgScaleVec3(tt, SG_RADIANS_TO_DEGREES);
      
      sgVec3 delta;
      sgScaleVec3(point_db[bone_list[bone]->a[0]].xyz, -1.0);
      sgVec3 delta2;
      if (current_frame < (frame_count - 1))
	sgLinInterpVec3(delta2, 
			point_db[bone_list[bone]->a[current_frame]].xyz, 
			point_db[bone_list[bone]->a[current_frame+1]].xyz,  
			(frame_offset)/10.0);
      else 
	sgCopyVec3(delta2, point_db[bone_list[bone]->a[current_frame]].xyz);

   //      sprintf(r, "  translate <%g, %g, %g>\n"  
   //      "  rotate <%g, %g, %g>\n"
   //      "  translate <%g, %g, %g>\n", delta[0], delta[1], delta[2],
   //      tt, rr, (qq*-1.0),
   //      delta2[0], delta2[1], delta2[2]);

   sprintf(r, "  translate <%g, %g, %g>\n"  
	      "  rotate <%g, %g, %g>\n"
	      "  translate <%g, %g, %g>\n", delta[0], delta[1], (-1*delta[2]),
	      tt[0], tt[1], (-1*tt[2]),
	      delta2[0], delta2[1], (-1*delta2[2]));

    }
  else
    sprintf(r, "\n");

  return gh_str02scm(r);
*/
}

static SCM
get_children (SCM obj)
{
  // FIXME - add conversions back!!
  EW_Group * a = 0; // EW_Group::scm2obj(obj);
  SCM list = SCM_EOL;
  for (EW_Object_List::cursor i = a->children.begin(); &i; i++)
    list = gh_cons(gh_cons(gh_int2scm((&i)->vklass().id()), 
			   gh_long2scm((long)(&i))), list);
  return list;
}

static SCM
flip_xx (SCM obj)
{
  /*
  EW_BlobbyGroup * a = (EW_BlobbyGroup *)gh_scm2long(obj);
  SCM list = SCM_EOL;
  for (int i=0; i<a->children.size(); i++)
    {
      int * b = 0;
      int l = 0;
      printf("flipping..\n");
      //a->children[i]->get_attribute("all-points", b, l);
      for (int j=0; j<l; j++)
	{
	  int q = *b;
	  printf("jajaflipping: %d..\n", q);
	  point_db[q].xyz[0] *= -1;
	}
    }
  return list;
  */
}


static SCM
is_group (SCM obj)
{
  EW_Object * a = (EW_Object *)gh_scm2long(obj);
  if (a->is_group())
    return SCM_BOOL_T;

  return SCM_BOOL_F;
}


  /*
static SCM
get_object_type (SCM obj)
{

   FIXME
  EW_Object * a = EW_Object::scm2obj(obj);
  return gh_str02scm((char *)a->klass().name());

  return SCM_BOOL_F;
}
   */

static SCM
get_material_attribute (SCM obj, SCM op)
{
  /*
  long a = gh_scm2long(obj);
  int l;
  double * b = 0;
  if (material_list[a]->get_attribute(SCM_CHARS(op), b, l))
    {
      SCM list = SCM_EOL;
      for (int i=0; i<l; i++)
	list = gh_cons(gh_double2scm(b[l-i-1]), list);
      return list;
    }
  int * c = 0;
  if (material_list[a]->get_attribute(SCM_CHARS(op), c, l))
    {
      SCM list = SCM_EOL;
      for (int i=0; i<l; i++)
	list = gh_cons(gh_int2scm(c[l-i-1]), list);
      return list;
    }
  char * d = 0;
  if (material_list[a]->get_attribute(SCM_CHARS(op), d, l))
    return gh_str02scm(d);
   */
  return SCM_UNSPECIFIED;  
}

static SCM
get_scene ()
{
  SCM smob;
  GFL_NEW_SMOB (ew_group, smob, kernel->scene);
  printf ("get_scene A = %ld\n", ew_group_class_tag.tag);
  printf ("get_scene B = %ld\n", SCM_CAR (smob) );
  return smob;
  // return gh_cons(gh_int2scm(kernel->scene->vklass().id()), gh_long2scm((long)kernel->scene));
}

static SCM
get_current_object ()
{
  return gh_cons(gh_int2scm(kernel->current_object->vklass().id()), 
		 gh_long2scm((long)kernel->current_object));
}

static SCM
get_point_db (SCM a)
{
  /*
  int b = gh_scm2int(a);
  SCM list = SCM_EOL;
  list = gh_cons(gh_double2scm(point_db[b].xyz[2]), list);
  list = gh_cons(gh_double2scm(point_db[b].xyz[1]), list);
  list = gh_cons(gh_double2scm(point_db[b].xyz[0]), list);
  return list;
   */
}

static SCM
set_point_db (SCM a, SCM b)
{
  /*
  int c = gh_scm2int(a);
  double x, y, z;
  SCM d = b;
  x = gh_scm2double(gh_car(d));
  d = gh_cdr(d);
  y = gh_scm2double(gh_car(d));
  d = gh_cdr(d);
  z = gh_scm2double(gh_car(d));
  sgSetVec3(point_db[c].xyz, x, y, z);
  return SCM_UNSPECIFIED;
   */
}

static SCM
get_frame_count ()
{
  //return gh_long2scm((long)frame_count);
}

static SCM
set_current_frame (SCM a)
{
  //current_frame = gh_scm2int(a);
  return SCM_UNSPECIFIED;
}

static SCM
activate_command_line ()
{
  command_line->activate();
  Fl::focus(command_line);
  return SCM_UNSPECIFIED;
}

static SCM
show_scene_graph ()
{
  //scene_graph_window->show();
  return SCM_UNSPECIFIED;
}

static SCM
group_new ()
{
  global.tool=TS_pick;
  global.set_object("group");
  kernel->current_object = global.new_object();
  kernel->current_group->add(kernel->current_object);
  kernel->current_group = (EW_Group *)kernel->current_object;

  Fl_ToggleNode * node;
  node = new Fl_ToggleNode(strdup(kernel->current_object->get_name()), 
			   (int)kernel->current_object->is_group(), 
			   kernel->current_object->vklass().pixmap(), 
			   (void *)kernel->current_object);
  scene_graph->traverse_start(scene_graph->find((void *)kernel->current_object->get_parent()));
  scene_graph->add_sub(node);
  
  return SCM_UNSPECIFIED;
}

static SCM
zoom (SCM a)
{
  SCM_ASSERT ((SCM_NIMP (a)), a, SCM_ARG1, "set-aspect-ratio");
  
  double d = gh_scm2double (a);
  double e = current_gl->get_zoom ();
  if (e==0) e=1;
  current_gl->set_zoom (d*e);
  current_gl->redraw();
  printf ("zoom %g %g\n", d, e);
  /*
  double d = gh_scm2double(a);
  double xcenter = ((current_gl->ortho_clip_left 
		     + current_gl->ortho_clip_right) / 2.0);
  double ycenter = ((current_gl->ortho_clip_bottom 
		     + current_gl->ortho_clip_top) / 2.0);
  double halfw = ((current_gl->ortho_clip_right 
		   - current_gl->ortho_clip_left) / (2.0 * d));
  double halfh = ((current_gl->ortho_clip_top 
		   - current_gl->ortho_clip_bottom) / (2.0 * d));
  
  current_gl->ortho_clip_left = xcenter - halfw;
  current_gl->ortho_clip_right = xcenter + halfw;
  current_gl->ortho_clip_top = ycenter + halfh;
  current_gl->ortho_clip_bottom = ycenter - halfh;
  
  current_gl->redraw();
  current_gl->ratio_x();
  current_gl->ratio_y();
*/
  
  return SCM_UNSPECIFIED;
}

static SCM
ewave_bind_key_raw (SCM a, SCM b)
{
  SCM_ASSERT (gh_list_p (a), a,  SCM_ARG1, "ewave::bind-key-raw");
  SCM_ASSERT (SCM_CLOSUREP (b), b,  SCM_ARG2, "ewave::bind-key-raw");

  SCM v = gh_list_to_vector (a);
  unsigned long length = gh_vector_length (v);
  unsigned long i=0;

  Array<long> *sequence = new Array<long>(length);

  for (i=0; i < length; i++)
    (*sequence)[i] = gh_scm2int(gh_vector_ref(v, gh_int2scm(i)));

  for (i=0; i<key_bindings.size(); i++)
    {
      if (key_bindings[i]->sequence->size () != length) 
	continue;
      
      if (*(key_bindings[i]->sequence) == *sequence)
	{
	  printf ("Warning: Duplicate keybinding found.\n");
	  // TODO: Don't unprotect if something else is using this.
	  scm_protect_object(b); 
	  scm_unprotect_object(key_bindings[i]->call_back);
	  key_bindings[i]->call_back = b;
	  delete sequence;
	  return SCM_UNSPECIFIED;
	}
    }

  scm_protect_object(b);
  EW_KeyBinding *t = new EW_KeyBinding();
  t->sequence = sequence;
  t->call_back = b;
  key_bindings.push_back (t);

  return SCM_UNSPECIFIED;
}

extern SCM scm_the_last_stack_fluid;

//#ifdef HAVE_SCM_THE_LAST_STACK_FLUID
/* from libguile/fluids.h --07/01/98 gjb */
SCM scm_fluid_ref SCM_P ((SCM fluid));
SCM scm_fluid_set_x SCM_P ((SCM fluid, SCM value));
#define DEREF_LAST_STACK scm_fluid_ref(gh_cdr(scm_the_last_stack_fluid))
#define SET_LAST_STACK(X) scm_fluid_set_x (gh_cdr (scm_the_last_stack_fluid), (X))

//#else
//#define DEREF_LAST_STACK gh_cdr(scm_the_last_stack_var)
//#define SET_LAST_STACK(X) gh_set_cdr_x(scm_the_last_stack_var, (X))
//#endif

SCM 
exception_handler(void *data, SCM tag, SCM throw_args)
{
  SCM port = scm_def_errp;
  if (scm_ilength (throw_args) >= 3)
    {
      SCM stack = DEREF_LAST_STACK;
      SCM subr = gh_car (throw_args);
      SCM message = SCM_CADR (throw_args);
      SCM args = SCM_CADDR (throw_args);
      
      scm_newline(port);
      scm_display_backtrace (stack, port, SCM_UNDEFINED, SCM_UNDEFINED);
      scm_newline(port);
      scm_display_error (stack, port, subr, message, args, SCM_EOL);
      return SCM_BOOL_F;
    }
  else
    {
      scm_puts ("uncaught throw to ", port);
      scm_prin1 (tag, port, 0);
      scm_puts (": ", port);
      scm_prin1 (throw_args, port, 1);
      scm_putc ('\n', port);
      exit (2);
    }
  /*  {
      char *skey;
      int l = 0;
      
      skey = gh_symbol2newstr(tag, &l);
      *(skey + l) = 0;

      printf("Scheme error in \"%s\"\nTag: %s.\n",
	     (char *) data, skey);
      if (l)
	free(skey);
    }*/
  return SCM_BOOL_F;
  
}

void
init_scheme_interface ()
{
  gh_new_procedure0_0 ("shade-toggle",
		       ewave_shade_toggle);
  gh_new_procedure0_0 ("shade-wire",
		       ewave_shade_wire);
  gh_new_procedure0_0 ("shade-smooth",
		       ewave_shade_smooth);
  gh_new_procedure0_0 ("shade-flat",
		       ewave_shade_flat);

  
  gh_new_procedure1_0 ("set-tool",
		       ewave_set_tool);
  gh_new_procedure0_0 ("get-tool",
		       ewave_get_tool);
  gh_new_procedure1_0 ("set-tool-delta",
		       ewave_set_tool_delta);
  gh_new_procedure0_0 ("get-tool-delta",
		       ewave_get_tool_delta);
  gh_new_procedure0_0 ("get-major-version",
		       ewave_get_major_version);
  gh_new_procedure0_0 ("get-minor-version",
		       ewave_get_minor_version);
  gh_new_procedure0_0 ("get-patch-version",
		       ewave_get_patch_version);
  gh_new_procedure1_0 ("set-aspect-ratio",
		       ewave_set_aspect_ratio);
  gh_new_procedure0_0 ("get-aspect-ratio",
		       ewave_get_aspect_ratio);
  gh_new_procedure3_0 ("set-current-point",
		       ewave_set_current_point);
  gh_new_procedure0_0 ("get-current-point",
		       ewave_get_current_point);
  gh_new_procedure1_0 ("export-povray",
		       ewave_export_povray);
  gh_new_procedure2_0 ("ewave::bind-key-raw",
		       ewave_bind_key_raw);
  gh_new_procedure1_0 ("zoom",
		       zoom);
  gh_new_procedure0_0 ("activate-command-line",
		       activate_command_line);
  gh_new_procedure0_0 ("scene-graph",
		       show_scene_graph);
  gh_new_procedure0_0 ("group-new",
		       group_new);
  gh_new_procedure2_0 ("get-attribute",
		       get_attribute);
  gh_new_procedure1_0 ("get-children",
		       get_children);
  gh_new_procedure1_0 ("is-group",
		       is_group);
  gh_new_procedure0_0 ("scene",
		       get_scene);
  gh_new_procedure0_0 ("current-object",
		       get_current_object);
  gh_new_procedure1_0 ("point-db",
		       get_point_db);
  gh_new_procedure2_0 ("set-point-db",
		       set_point_db);
  gh_new_procedure2_0 ("get-material-attribute",
		       get_material_attribute);
  gh_new_procedure1_0 ("flip-xx",
		       flip_xx);
  gh_new_procedure2_0 ("povray-bone-trick",
		       povray_bone_trick);
  gh_new_procedure0_0 ("frame-count",
		       get_frame_count);
  gh_new_procedure1_0 ("set-current-frame",
		       set_current_frame);

  GFL_NEW_PROCEDURE (kernel_set_object);

  scm_sysintern ("tool_add", SCM_MAKINUM(TS_add_object));
  scm_sysintern ("tool_null", SCM_MAKINUM(TS_null));
  scm_sysintern ("tool_pick", SCM_MAKINUM(TS_pick));
  scm_sysintern ("tool_zoom", SCM_MAKINUM(TS_zoom));
  scm_sysintern ("tool_move", SCM_MAKINUM(TS_move));
  scm_sysintern ("tool_translate", SCM_MAKINUM(TS_translate));
  scm_sysintern ("tool_weld", SCM_MAKINUM(TS_weld));
}
