// Extreme Wave -- A 3d modelling and animation environment
// Copyright (C) 1999 James Dean Palmer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// Please report all bugs and problems to "james@tiger-marmalade.com".
#include <unistd.h>
#include <stdio.h>

#include <FL/glut.H>
#include <FL/fl_draw.H>
#include <GL/gl.h>
#include <FL/glut.h>
#include <FL/fl_ask.H>
#include <FL/fl_show_colormap.H>
#include <FL/Fl_Color_Chooser.H>

#include <ewave/gui/gui.h>
#include <ewave/materials/material.h>

Material_List material_list;

void material_browser_update () {
   int j;
   j = MaterialBrowser->value();
   MaterialBrowser->clear();
  for (unsigned long i=0; i<material_list.size(); i++) {
     MaterialBrowser->add(material_list[i]->name);   
   }
   MaterialBrowser->value(j);
}

void material_browser_clear () {
  Material * a = new Material();
  material_list.clear ();
  material_list.push_back (a);
  material_browser_update();
  material_browser_set_fields();
}

void material_browser_set_fields () {
   int i, j;
   char s[30];
   j = MaterialBrowser->value();
   i=j-1;
   if (i<=0) i=0;
   MaterialBrowser->value(i+1);
   MaterialName->value(material_list[i]->name);
   NormalType->value(material_list[i]->normal.type);
   sprintf(s, "%g", material_list[i]->normal.perturb); NormalPerturb->value(s);
   sprintf(s, "%g", material_list[i]->normal.phase); NormalPhase->value(s);
   sprintf(s, "%g", material_list[i]->normal.frequency); NormalFrequency->value(s);
   sprintf(s, "%g, %g", material_list[i]->normal.scale[0],
	   material_list[i]->normal.scale[1]); 
   NormalScale->value(s);
   sprintf(s, "%g, %g", material_list[i]->normal.translate[0],
	   material_list[i]->normal.translate[1]); 
   NormalTranslate->value(s);
   sprintf(s, "%g, %g", material_list[i]->normal.rotate[0],
	   material_list[i]->normal.rotate[1]); 
   NormalRotate->value(s);

   sprintf(s, "%g", material_list[i]->finish.diffuse); FinishDiffuse->value(s);
   sprintf(s, "%g", material_list[i]->finish.brilliance); FinishBrilliance->value(s);
   sprintf(s, "%g", material_list[i]->finish.grain); FinishGrain->value(s);
   sprintf(s, "%g", material_list[i]->finish.ambient); FinishAmbient->value(s);
   sprintf(s, "%g", material_list[i]->finish.reflection); FinishReflection->value(s);
   sprintf(s, "%g", material_list[i]->finish.refraction); FinishRefraction->value(s);
   sprintf(s, "%g", material_list[i]->finish.ior); FinishIOR->value(s);
   sprintf(s, "%g", material_list[i]->finish.specular); FinishSpecular->value(s);
   sprintf(s, "%g", material_list[i]->finish.roughness); FinishRoughness->value(s);
   sprintf(s, "%g", material_list[i]->finish.phong); FinishPhong->value(s);
   sprintf(s, "%g", material_list[i]->finish.phong_size); FinishPhongSize->value(s);
}

void cb_material_browser_set_fields (Fl_Browser * a, void* b) {
  material_browser_set_fields();
}

void material_browser_get_fields () {
  int i;
  i = MaterialBrowser->value() - 1;
  if (i <= 0) i=0;
  material_list[i]->name = strdup(MaterialName->value());
  material_list[i]->normal.frequency = atof(NormalFrequency->value());
  material_list[i]->normal.phase = atof(NormalPhase->value());
  material_list[i]->normal.perturb = atof(NormalPerturb->value());
  material_list[i]->normal.type = NormalType->value();

  material_list[i]->finish.diffuse = atof(FinishDiffuse->value());
  material_list[i]->finish.brilliance = atof(FinishBrilliance->value());
  material_list[i]->finish.grain = atof(FinishGrain->value());
  material_list[i]->finish.ambient = atof(FinishAmbient->value());
  material_list[i]->finish.reflection = atof(FinishReflection->value());
  material_list[i]->finish.phong = atof(FinishPhong->value());
  material_list[i]->finish.phong_size = atof(FinishPhongSize->value());
  material_list[i]->finish.specular = atof(FinishSpecular->value());
  material_list[i]->finish.roughness = atof(FinishRoughness->value());
  material_list[i]->finish.refraction = atof(FinishRefraction->value());
  material_list[i]->finish.ior = atof(FinishIOR->value());
  
}

void cb_material_browser_remove (Fl_Button* a, void* b) {
  int i;
  if (material_list.size() <= 1) {
    fl_alert("Sorry, you must have at least one material value.");
    return;
  }
  i = MaterialBrowser->value() - 1;
  if (i<=0) i=0;
   
  delete (material_list[i]);
  material_list.erase ((unsigned int)i);
////  for (j=i+1; j<material_list.size(); j++) {
//    c = material_list[j-1];
//    material_list[j-1] = material_list[j];
//    material_list[j] = c;
//  }
//  material_list.shrink(1);
  material_browser_update();
  material_browser_set_fields();
}

void cb_material_browser_shift_down (Fl_Button* a, void* b) {
  unsigned long i=0, j;
  Material * c, * d;
  if (material_list.size() <= 1) return;
  if (MaterialBrowser->value() > 0)
    i = MaterialBrowser->value() - 1;
  j = i + 1;

   if (j >= material_list.size()) return;
   c = material_list[i];
   d = material_list[j];
   material_list[i] = d; 
   material_list[j] = c;
   
   material_browser_update();
   MaterialBrowser->value((j+1));
}

void cb_material_browser_shift_up(Fl_Button* a, void* b) {
   int i, j;
   Material * c;
   if (material_list.size() <= 1) return;
   i = MaterialBrowser->value() - 1;
   if (i<=0) i=0;
   j = i - 1;

   if (j<0) return;
   c = material_list[i];
   material_list[i] = material_list[j];
   material_list[j] = c;
   
   material_browser_update();
   MaterialBrowser->value((j+1));
}

void cb_material_browser_okay(Fl_Return_Button*, void*) {
  material_inspector->hide();
}

Material::Material() {
  name = strdup("default");
  pigment.color.set(0.8, 0.4, 0.4);
  normal.type = Nnone;
  normal.perturb = 0.2;
  normal.frequency = 0.2;
  normal.phase = 0.2;
  normal.translate.set(0,0,0);
  normal.scale.set(1,1,1);
  normal.rotate.set(0,0,0);
  finish.diffuse = 0.6;
  finish.brilliance = 1.0;
  finish.grain = 0.0;
  finish.ambient = 0.1;
  finish.reflection = 0.0;
  finish.phong = 0.0;
  finish.phong_size = 40;
  finish.metallic = 0;
  finish.specular = 0.0;
  finish.roughness = 0.02;
  finish.refraction = 0;
  finish.ior = 1.0;
}

static char * indent_it(int j)
{
  char * r = (char *)malloc(j+1);
  for (int i=0; i<j; i++)
    r[i] = ' ';
  r[j] = '\0';
  return r;
}

void
Material::write (FILE *file, int ind)
{
  char * indent = indent_it(ind);

  fprintf (file, "\n%s(!dmaterial ", indent);

  fprintf (file, "(name \"%s\")", name);

  fprintf (file, "\n%s  (diffuse_color %g %g %g)", 
	   indent, pigment.color[0], pigment.color[1], pigment.color[2]);

  if (normal.type == Nnone)
    fprintf (file, "\n%s  (normal_type \"none\")", indent);
  else if (normal.type == Nbumps)
    fprintf (file, "\n%s  (normal_type \"bumps\")", indent);
  else if (normal.type == Ndents)
    fprintf (file, "\n%s  (normal_type \"dents\")", indent);
  else if (normal.type == Nwaves)
    fprintf (file, "\n%s  (normal_type \"waves\")", indent);
  else if (normal.type == Nripples)
    fprintf (file, "\n%s  (normal_type \"ripples\")", indent);
  else if (normal.type == Nwrinkles)
    fprintf (file, "\n%s  (normal_type \"wrinkles\")", indent);
  else if (normal.type == Nbumpmap)
    fprintf (file, "\n%s  (normal_type \"bumpmap\")", indent);

  fprintf (file, "\n%s  (normal_perturbance %g)", indent, normal.perturb);
  fprintf (file, "\n%s  (normal_frequency %g)", indent, normal.frequency);
  fprintf (file, "\n%s  (normal_phase %g)", indent, normal.phase);
  fprintf (file, "\n%s  (normal_translate %g %g %g)", indent, 
	  normal.translate[0], normal.translate[1], normal.translate[2]);
  fprintf (file, "\n%s  (normal_scale %g %g %g)", indent, 
	  normal.scale[0], normal.scale[1], normal.scale[2]);
  fprintf (file, "\n%s  (normal_rotate %g %g %g)", indent, 
	   normal.rotate[0], normal.rotate[1], normal.rotate[2]);

  fprintf (file, "\n%s  (finish_diffuse %g)", indent, finish.diffuse);
  fprintf (file, "\n%s  (finish_brilliance %g)", indent, finish.brilliance);
  fprintf (file, "\n%s  (finish_grain %g)", indent, finish.grain);
  fprintf (file, "\n%s  (finish_ambient %g)", indent, finish.ambient);
  fprintf (file, "\n%s  (finish_reflection %g)", indent, finish.reflection);
  fprintf (file, "\n%s  (finish_phong %g)", indent, finish.phong);
  fprintf (file, "\n%s  (finish_phong_size %g)", indent, finish.phong_size);
  fprintf (file, "\n%s  (finish_phong_size %g)", indent, finish.phong_size);
  fprintf (file, "\n%s  (finish_metallic %d)", indent, (int)finish.metallic);
  fprintf (file, "\n%s  (finish_specular %g)", indent, finish.specular);
  fprintf (file, "\n%s  (finish_roughness %g)", indent, finish.roughness);
  fprintf (file, "\n%s  (finish_refraction %g)", indent, finish.refraction);
  fprintf (file, "\n%s  (finish_ior %g)", indent, finish.ior);

  fprintf (file, ")");

  free(indent);
}

short
Material::set_attribute(char *op, char *arg)
{
  if (!strcmp(op, "name"))
    {
      name = arg;
      return 1;
    }

  if (!strcmp(op, "normal_type"))
    {
      if (!strcmp(arg, "none"))
	normal.type = Nnone;
      else if (!strcmp(arg, "bumps"))
	normal.type = Nbumps;
      else if (!strcmp(arg, "dents"))
	normal.type = Ndents;
      else if (!strcmp(arg, "waves"))
	normal.type = Nwaves;
      else if (!strcmp(arg, "ripples"))
	normal.type = Nripples;
      else if (!strcmp(arg, "wrinkles"))
	normal.type = Nwrinkles;
      else if (!strcmp(arg, "bumpmap"))
	normal.type = Nbumpmap;
      return 1;
    }

  return 0;
}


short 
Material::get_attribute(char *op, char *&result, int &length)
{
  if (!strcmp(op, "name"))
    {
      result = name;
      return 1;
    }

  if (!strcmp(op, "normal_type"))
    {
      if (normal.type == Nnone)
	result = "none";
      if (normal.type == Nbumps)
	result = "bumps";
      if (normal.type == Ndents)
	result = "dents";
      if (normal.type == Nwaves)
	result = "waves";
      if (normal.type == Nwaves)
	result = "ripples";
      if (normal.type == Nwrinkles)
	result = "wrinkles";
      if (normal.type == Nbumpmap)
	result = "bumpmap";
      return 1;
    }

  return 0;
}


short 
Material::set_attribute(char *op, SCM arg)
{
  if (!strcmp(op, "diffuse_color"))
    {
      pigment.color.set (gh_scm2double(gh_car(arg)),
			 gh_scm2double(gh_cadr(arg)),
			 gh_scm2double(gh_caddr(arg)));
      return 1;
    }

  if (!strcmp(op, "normal_perturbance"))
    {
      normal.perturb = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "normal_frequency"))
    {
      normal.frequency = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "normal_phase"))
    {
      normal.phase = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "normal_translate"))
    {
      normal.translate.set (gh_scm2double(gh_car(arg)),
			    gh_scm2double(gh_cadr(arg)),
			    gh_scm2double(gh_caddr(arg)));
      return 1;
    }

  if (!strcmp(op, "normal_rotate"))
    {
      normal.rotate.set (gh_scm2double(gh_car(arg)),
			 gh_scm2double(gh_cadr(arg)),
			 gh_scm2double(gh_caddr(arg)));
      return 1;
    }

  if (!strcmp(op, "normal_scale"))
    {
      normal.scale.set (gh_scm2double(gh_car(arg)),
			gh_scm2double(gh_cadr(arg)),
			gh_scm2double(gh_caddr(arg)));
      return 1;
    }

  if (!strcmp(op, "finish_diffuse"))
    {
      finish.diffuse = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_brilliance"))
    {
      finish.brilliance = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_grain"))
    {
      finish.grain = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_ambient"))
    {
      finish.ambient = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_reflection"))
    {
      finish.reflection = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_phong"))
    {
      finish.phong = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_phong_size"))
    {
      finish.phong_size = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_specular"))
    {
      finish.specular = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_roughness"))
    {
      finish.roughness = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_refraction"))
    {
      finish.refraction = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_ior"))
    {
      finish.ior = gh_scm2double(gh_car(arg));
      return 1;
    }

  if (!strcmp(op, "finish_metallic"))
    {
      finish.metallic = gh_scm2int(gh_car(arg));
      return 1;
    }
  return 0;
}

/*
static void kludge(double *& result, Vec3 a)
{
  result = new double[3];
  result[0] = a[0];
  result[1] = a[1];
  result[2] = a[2];
}
*/

short 
Material::get_attribute(char *op, double *&result, int &length)
{
/*
  if (!strcmp(op, "diffuse_color"))
    {
      kludge(result, pigment.color);
      length = 3;
      return 1;
    }

  if (!strcmp(op, "normal_perturbance"))
    {
      result = &normal.perturb;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "normal_frequency"))
    {
      result = &normal.frequency;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "normal_phase"))
    {
      result = &normal.phase;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "normal_translate"))
    {
      kludge(result, normal.translate);
      length = 3;
      return 1;
    }

  if (!strcmp(op, "normal_rotate"))
    {
      kludge(result, normal.rotate);
      length = 3;
      return 1;
    }

  if (!strcmp(op, "normal_scale"))
    {
      kludge(result, normal.scale);
      length = 3;
      return 1;
    }

  if (!strcmp(op, "finish_diffuse"))
    {
      result = &finish.diffuse;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_brilliance"))
    {
      result = &finish.brilliance;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_grain"))
    {
      result = &finish.grain;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_ambient"))
    {
      result = &finish.ambient;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_reflection"))
    {
      result = &finish.reflection;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_phong"))
    {
      result = &finish.phong;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_phong_size"))
    {
      result = &finish.phong_size;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_specular"))
    {
      result = &finish.specular;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_roughness"))
    {
      result = &finish.roughness; 
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_refraction"))
    {
      result = &finish.refraction;
      length = 1;
      return 1;
    }

  if (!strcmp(op, "finish_ior"))
    {
      result = &finish.ior;
      length = 1;
      return 1;
    }
*/
  return 0;
}

short 
Material::get_attribute(char *op, int *&result, int &length)
{
  /*
  if (!strcmp(op, "finish_metallic"))
    {
      result = new int;
      *result = (int)finish.metallic;
      length = 1;
      return 1;
    }
*/
  return 0;
}


void cb_material_browser_duplicate(Fl_Button* a, void* b) {
   int i;
   i = MaterialBrowser->value() - 1;
   if (i<=0) i=0;
   Material *tmp = new Material();
   material_list.push_back (tmp);
   *tmp = *material_list[i];
   material_browser_update();
   material_browser_set_fields();
}


void cb_material_browser_update (Fl_Input* a, void* b) {
  material_browser_get_fields();
  material_browser_update();
}


void cb_material_browser_update2 (Fl_Choice* a, void* b) {
  material_browser_get_fields();
  material_browser_update();
}


void cb_material_browser_color (Fl_Button* a, void* b) {
  int i;
  i = MaterialBrowser->value() - 1;
  if (i <= 0) i=0;
  double t[3];
  t[0] = material_list[i]->pigment.color[0];
  t[1] = material_list[i]->pigment.color[1];
  t[2] = material_list[i]->pigment.color[2];
  fl_color_chooser("Pick a material color:", t[0], t[1], t[2]);
  material_list[i]->pigment.color.set (t[0], t[1], t[2]);
}
