#ifndef _MATERIAL_H_
#define _MATERIAL_H_

#include <ewave/geometry/math.h>
#include <ewave/kernel/array.h>
#include <ewave/FakeGuile.h>
//// Exactly uses povray's concept of texture.

// Tpigment will eventually be MUCH more complex

class Pigment {
 public:
  Vec3 color;
  Vec3 warm;
  Vec3 cool;
  Vec3 specular;
  double mood_level;
};

enum { Nnone=0, Nbumps=1, Ndents=2, Nwaves=3, Nripples=4, Nwrinkles=5, Nbumpmap=6 };

// TNormal

class Normal {
 public:
  int type;
  double perturb; // 0 - 1.0

  // only for ripples and waves:
  double frequency;
  double phase;

  // only uses x-y part of these
  Vec3 translate;
  Vec3 scale;
  Vec3 rotate;
};

class Finish {
 public:
  double diffuse; // 0.6      0-1
  double brilliance; // 1.0   0-INF
  double grain; // 0.0        0-1        crand
  double ambient; // 0.1      0-1
  double reflection; // 0.0   0-1

  //It is preferable to use specular OR phong, but you can use both.

  // phong only:
  double phong;      // 0.0   0-1
  double phong_size; // 40    0(dull) - 250(highly polished)
  short metallic;

  // specular only
  double specular; // 0.0 0-1
  double roughness; // 0.02   0.0001 (Smooth) - 1.0(Rough)

  double refraction; // 0 (off) and 1 (on) are the only "realistic" values.
  double ior; // index of refraction
  // The index of refraction for air is 1.0, water is 1.33, glass is 1.5, and diamond is 2.4
};

class Material {
 public:
  Material();
  virtual ~Material() {}
  char * name;
  Finish finish;
  Pigment pigment;
  Normal normal;
  void diffuse_lookup(Vec3 & dst) { pigment.color = dst; }
  virtual void write (FILE *, int i);
  virtual short set_attribute(char *, char *);
  virtual short set_attribute(char *, SCM);
  short get_attribute(char *op, char *&result, int &length);
  short get_attribute(char *op, int *&result, int &length);
  short get_attribute(char *op, double *&result, int &length);
};

typedef Material *                 EW_Material_ptr;

/*
 class EW_Material_array : public EW_ptr_array 
{
public:
  EW_Material_ptr & first() 
    { return (EW_Material_ptr &)EW_array::first(); }
  EW_Material_ptr & last() 
    { return (EW_Material_ptr &)EW_array::last(); }
  EW_Material_ptr & operator[](EW_int i) 
    { return (EW_Material_ptr &)_data[i*_item_size]; }
  EW_Material_ptr & append(EW_int s) 
    { return (EW_Material_ptr &)EW_array::append(s); }
  EW_Material_ptr & append() 
    { return append(1); }
  EW_Material_ptr & insert(EW_int n, EW_int s) 
    { return (EW_Material_ptr &)EW_array::insert(n, s); }
  EW_Material_ptr * data() 
    { return (EW_Material_ptr *)_data; }
};
 extern EW_Material_array material_list;

*/

typedef PArray<Material> Material_List;
extern Material_List material_list;
extern void material_browser_clear ();
extern void material_browser_set_fields ();
extern void material_browser_update ();

#endif
