#include <unistd.h>
#include <ewave/objects/blobby_group.h>
#include <ewave/objects/blobby_group_gui.h>
#include <ewave/xpm_icons/blobby_group.xpm>

EW_ObjectRecord EW_BlobbyGroup::record;

EW_BlobbyGroup::EW_BlobbyGroup ()
{ 
  default_name(type());
  threshold = 0.35;
}

static EW_Object *
EW_BlobbyGroup_alloc ()
{
  return new EW_BlobbyGroup();
}

void
EW_BlobbyGroup::initialize()
{
  // Create an ObjectRecord for EW_BlobbyGroup
  record.name = "EW_BlobbyGroup";
  record.tag = pdb_new_tag();
  record.create = EW_BlobbyGroup_alloc;
  record.pixmap = new Fl_Pixmap(blobby_group_xpm); 
  record.property_window = make_EW_BlobbyGroup_properties_window();
      
  // Register EW_BlobbyGroup methods in the procedural database
  pdb_register(record.name, record.tag);

  // Register EW_BlobbyGroup in the object registry.
  global.add_object(record);

  if (global.verbose)
    printf("EW_BlobbyGroup tag = %d\n", (int)record.tag);
}

void
EW_BlobbyGroup::write (FILE *file, int ind)
{
  char * indent = indenter(ind);

  if (tag() == record.tag)
    fprintf (file, "\n%s(!blobby-group ", indent);

  fprintf (file, "\n%s  (threshold %g)", 
	   indent, threshold);

  EW_Group::write(file, ind);

  if (tag() == record.tag)
    fprintf (file, ")");

  free(indent);
}

void
EW_BlobbyGroup::update_gui()
{
  BlobbyGroup_name->value(name);
}

EW_Object *
EW_BlobbyGroup::replicate()
{
  int i;
  EW_BlobbyGroup * rvalue = new EW_BlobbyGroup();
  *rvalue = *this;
  rvalue->children.reset();
  for (i=0; i<children.size(); i++)
    rvalue->add(children[i]->replicate());

  return rvalue;
}

EW_BlobbyGroup *
EW_BlobbyGroup::scm2obj(SCM a)
{
  return (EW_BlobbyGroup *)EW_Object::scm2obj(a, record.tag);
}

SCM
EW_BlobbyGroup::obj2scm(EW_Object *a)
{
  return gh_cons(gh_int2scm(record.tag), gh_long2scm((long)a));
}

void
EW_BlobbyGroup::pdb_register(char * n, long tag)
{
  EW_Group::pdb_register(n, tag);
  pdb_register_tag(tag, record.tag);

  char t[80];
  pdb_record w;

  w.author = "system";

  sprintf(t, "%s::set-threshold", n);
  w.name = strdup(t);
  w.docstring = "Set threshold.";
  w.procedure = EW_BlobbyGroup::set_threshold;
  add_pdb_record(w);

  sprintf(t, "%s::get-threshold", n);
  w.name = strdup(t);
  w.docstring = "Get threshold.";
  w.procedure = EW_BlobbyGroup::get_threshold;
  add_pdb_record(w);
}

SCM
EW_BlobbyGroup::set_threshold(SCM a)
{
  if (EW_BlobbyGroup * s = scm2obj(gh_car(a)))
      s->threshold = gh_scm2double(gh_cadr(a));
  else
    return SCM_BOOL_F;

  return SCM_BOOL_T;
}

SCM
EW_BlobbyGroup::get_threshold(SCM a)
{
  if (EW_BlobbyGroup * s = scm2obj(gh_car(a)))
    return gh_cons(gh_double2scm(s->threshold), SCM_EOL);

  return SCM_BOOL_F;
}

/*
Object * 
first_EW_BlobbyGroup (Object * a) 
{
  if (a->tag() == record.tag)
    return a;

  if (a->is_group()) 
    {
      Group * c = (Group *)a;
      for (int k=0; k<(c->children.size()); k++) 
	{
	  Object * b;
	  b = first_EW_BlobbyGroup(c->children[k]); 
	  if (b != NULL) return(b);
	}
    }
  return NULL;
}
*/
