/* Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include "blobby_sphere.h"
#include "../xpm_icons/blobby_sphere.xpm"

static Fl_Generic_Name * t_name;
static Fl_Generic_Radius * t_radius;
static Fl_Generic_Xform *t_xform;
static Fl_Generic_Material * t_material;
static Fl_Generic_Bone * t_bone;
static Fl_Generic_Radius * t_strength;
EW_bool show_blobby_field;
EW_ObjectRecord EW_BlobbySphere::record;

EW_BlobbySphere::EW_BlobbySphere () 
{
  default_name (type ());
  radius = 10.;
  sgSetVec3 (scale, 1., 1., 1.);
  sgSetVec3 (rotate, .0, .0, .0);
  material = 0;
  bone = -1;
  center = -1;
  t_bit = 1;
  d_bit = 1;
  strength = 1.0;
}

void
EW_BlobbySphere::write (FILE *file, int ind)
{
  char * indent = indenter(ind);

  if (tag () == record.tag)
    fprintf (file, "\n%s(!EW_BlobbySphere ", indent);

  EW_Sphere::write (file, ind);

  fprintf (file, "\n%s  (strength %g)", 
	  indent, strength);

  if (tag() == record.tag)
    fprintf (file, ")");

  free (indent);
}

void
EW_BlobbySphere::update_gui ()
{
  t_name->value->value (name);
  t_radius->value->value (radius);
  t_strength->value->value (radius);

  t_xform->scale_x->value (scale[0]);
  t_xform->scale_y->value (scale[1]);
  t_xform->scale_z->value (scale[2]);

  t_xform->rotate_x->value (rotate[0] * SG_RADIANS_TO_DEGREES);
  t_xform->rotate_y->value (rotate[1] * SG_RADIANS_TO_DEGREES);
  t_xform->rotate_z->value (rotate[2] * SG_RADIANS_TO_DEGREES);

  t_xform->translate_x->value (point_db[center].xyz[0]);
  t_xform->translate_y->value (point_db[center].xyz[1]);
  t_xform->translate_z->value (point_db[center].xyz[2]);

  t_material->update_material ();
  t_bone->update_bone ();
}

EW_Object *
EW_BlobbySphere::replicate ()
{
  EW_BlobbySphere * rvalue = new EW_BlobbySphere ();
  *rvalue = *this;
  rvalue->center = point_db.size ();
  point_db.append ();
  point_db.last () = point_db[center];
  return rvalue;
}

void 
EW_BlobbySphere::gl_draw () 
{
  int i, size;

  if (this != current_object) 
    glColor3f (0.7, 0.7, 0.9);
  else 
    glColor3f (0.8, 0.8, 0.4);

  if ((d_bit) || (t_bit))  // Use cached values if nothing has changed.
    {
      vertex_cache = sphere_mesh;
      size = vertex_cache.size ();

      // Scale the vertex points
      {
	sgVec3 new_scale;
	double threshold = 0.35;
	sgScaleVec3(new_scale, scale, ((sgAbs (strength) - threshold) 
				       / sgAbs (strength)) * radius);
	for (i = 0; i < size; i++) 
	  sgScaleVec3 (vertex_cache[i], new_scale);

	if (show_blobby_field)
	  {
	    sgScaleVec3(new_scale, scale, radius);
	    for (i = 0; i < size; i++) 
	      sgScaleVec3 (vertex_cache2[i], new_scale);
	  }
      }

      // Rotate the vertex points
      if ((rotate[0] != 0) || (rotate[1] != 0) || (rotate[2] != 0)) 
	{
	  sgMat4 new_rotate;
	  sgMakeRotMat4 (new_rotate, rotate);
	  for (i=0; i < size; i++) 
	    sgXformVec3 (vertex_cache[i], new_rotate);
	  if (show_blobby_field)
	    for (i=0; i < size; i++)
	      sgXformVec3 (vertex_cache2[i], new_rotate);
	}

      // Translate the vertex points
      {
	sgCopyVec3 (center_cache, point_db[center].xyz);
      
	for (i = 0; i < size; i++) 
	  {
	    sgAddVec3 (vertex_cache[i], center_cache);
	    if (bone > -1) 
	      bone_list[bone]->translate_point (vertex_cache[i], vertex_cache[i]);
	  }

	if (show_blobby_field)
	  for (i = 0; i < size; i++) 
	    {
	      sgAddVec3 (vertex_cache[i], center_cache);
	      if (bone > -1) 
		bone_list[bone]->translate_point (vertex_cache[i], vertex_cache[i]);
	    }

	if (bone > -1) 
	  bone_list[bone]->translate_point (center_cache, center_cache);
      }

      // Compute the normals
      {
	normal_cache = vertex_cache;
	for (i = 0; i < size; i++)
	  {
	    sgSubVec3 (normal_cache[i], vertex_cache[i], center_cache);
	    sgUnitVec3 (normal_cache[i]);
	  }
	if (show_blobby_field)
	  {
	    normal_cache2 = vertex_cache2;
	    for (i = 0; i < size; i++)
	      {
		sgSubVec3 (normal_cache2[i], vertex_cache2[i], center_cache);
		sgUnitVec3 (normal_cache2[i]);
	      }
	  }
      }
      d_bit = t_bit = 0;
    }
  else
    size = vertex_cache.size ();

  // Draw the vertex points
  {
    glBegin (GL_QUADS);
    sgVec3 * q = vertex_cache.data ();
    sgVec3 * r = normal_cache.data ();
    for (i = 0; i < size; i++)
      {
	glNormal3fv ((const GLfloat *)&r[i]);
	glVertex3fv ((const GLfloat *)&q[i]);
      }
    glEnd ();
  }

  if (show_blobby_field)
    {
      glBegin (GL_QUADS);
      sgVec3 * q = vertex_cache.data ();
      sgVec3 * r = normal_cache.data ();
      for (i = 0; i < size; i++)
	{
	  glNormal3fv ((const GLfloat *)&r[i]);
	  glVertex3fv ((const GLfloat *)&q[i]);
	}
      glEnd ();
    }
  
  draw_point (center);
}

EW_BlobbySphere *
EW_BlobbySphere::scm2obj (SCM a)
{
  return (EW_BlobbySphere *) EW_Object::scm2obj (a, record.tag);
}

SCM
EW_BlobbySphere::obj2scm (EW_Object *a)
{
  return gh_cons (gh_int2scm (record.tag), gh_long2scm ((long)a));
}

void
EW_BlobbySphere::pdb_register (char * n, long tag)
{
  EW_Sphere::pdb_register (n, tag);
  pdb_register_tag (tag, record.tag);

  char t[80];
  pdb_record w;

  w.author = "system";

  sprintf (t, "%s::set-strength", n);
  w.name = strdup (t);
  w.docstring = "Sets the blobby-sphere's strength.";
  w.procedure = EW_BlobbySphere::set_strength;
  add_pdb_record (w);

  sprintf (t, "%s::get-strength", n);
  w.name = strdup (t);
  w.docstring = "Gets the blobby-sphere's strength.";
  w.procedure = EW_BlobbySphere::get_strength;
  add_pdb_record (w);
}

SCM
EW_BlobbySphere::set_strength (SCM a)
{
  if (EW_BlobbySphere * s = scm2obj (gh_car (a)))
      s->strength = gh_scm2double (gh_cadr (a));
  else
    return SCM_BOOL_F;

  return SCM_BOOL_T;
}

SCM
EW_BlobbySphere::get_strength (SCM a)
{
  if (EW_BlobbySphere * s = scm2obj (gh_car (a)))
    return gh_cons (gh_double2scm (s->strength), SCM_EOL);

  return SCM_BOOL_F;
}

static EW_Object* 
EW_BlobbySphere_alloc ()
{
  return ((EW_Object*)(new EW_BlobbySphere ()));
}

void
EW_BlobbySphere::initialize ()
{
  // Create an ObjectRecord for EW_BlobbySphere
  record.name = "EW_BlobbySphere";
  record.tag = pdb_new_tag ();
  record.create = EW_BlobbySphere_alloc;
  {
    Fl_Pack *p = new Fl_Pack (0, 0, 285, 400);
    {
      t_name = new Fl_Generic_Name (0, 0, 285, 80, "Name");
      t_name->end ();
      t_radius = new Fl_Generic_Radius (0, 0, 285, 80, "Radius");
      t_radius->end ();
      t_strength = new Fl_Generic_Radius (0, 0, 285, 80, "Strength");
      t_strength->label("Strength");
      t_strength->end ();
      t_xform = new Fl_Generic_Xform (0, 0, 285, 165, "Transformations");
      t_xform->end ();
      t_material = new Fl_Generic_Material (0, 0, 285, 80, "Material");
      t_material->end ();
      t_bone = new Fl_Generic_Bone (0, 0, 285, 80, "Bone");
      t_bone->end ();
    }
    p->end ();
    record.property_window = p;
  }
  record.pixmap = new Fl_Pixmap (blobby_sphere_xpm);

  // Register EW_BlobbySphere methods in the procedural database.
  pdb_register (record.name, record.tag);
  
  // Register EW_BlobbySphere in the object registry.
  global.add_object (record);

  if (global.verbose)
    printf ("EW_BlobbySphere tag = %d\n", (int)record.tag);  
}

