#include <unistd.h>
#include <ewave/objects/difference_group.h>
#include <ewave/objects/difference_group_gui.h>
#include <ewave/xpm_icons/difference.xpm>

EW_ObjectRecord EW_DifferenceGroup::record;

EW_DifferenceGroup::EW_DifferenceGroup ()
{
  default_name(type());
}

static EW_Object *
EW_DifferenceGroup_alloc ()
{
  return new EW_DifferenceGroup();
}

void
EW_DifferenceGroup::initialize()
{
  // Set global properties for EW_DifferenceGroup
  record.name = "EW_DifferenceGroup";
  record.tag = pdb_new_tag();
  record.create = EW_DifferenceGroup_alloc;
  record.pixmap = new Fl_Pixmap(difference_xpm); 
  record.property_window = make_EW_DifferenceGroup_properties_window();

  // Register EW_DifferenceGroup methods in the procedural database
  pdb_register(record.name, record.tag);

  // Register EW_DifferenceGroup in the object registry.
  global.add_object(record);

  if (global.verbose)
    printf("EW_DifferenceGroup tag = %d\n", (int)record.tag);
}

void
EW_DifferenceGroup::write (FILE *file, int ind)
{
  char * indent = indenter(ind);

  if (tag() == record.tag)
    fprintf (file, "\n%s(!difference-group ", indent);

  EW_Group::write(file, ind);

  if (tag() == record.tag)
    fprintf (file, ")");
}

void
EW_DifferenceGroup::update_gui()
{
  DifferenceGroup_name->value(name);
}

EW_Object *
EW_DifferenceGroup::replicate()
{
  int i;
  EW_DifferenceGroup * rvalue = new EW_DifferenceGroup();
  *rvalue = *this;
  rvalue->children.reset();
  for (i=0; i<children.size(); i++)
    rvalue->add(children[i]->replicate());

  return rvalue;
}

EW_DifferenceGroup *
EW_DifferenceGroup::scm2obj(SCM a)
{
  return (EW_DifferenceGroup *)EW_Object::scm2obj(a, record.tag);
}

SCM
EW_DifferenceGroup::obj2scm(EW_Object *a)
{
  return gh_cons(gh_int2scm(record.tag), gh_long2scm((long)a));
}

void
EW_DifferenceGroup::pdb_register(char * n, long tag)
{
  EW_Group::pdb_register(n, tag);
  pdb_register_tag(tag, record.tag);
}

