/* Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_bones.h>

#include <FL/Fl_Button.H>
#include <FL/Fl_Browser.H>
extern Fl_Browser * BonesBrowser;

#include <FL/fl_ask.H>
#include <ewave/xpm_icons/bone.xpm>
int frame_count;
int current_frame;
int current_bone;
EW_Bone *theCurrentBone;
EW_Bone_array bone_list;

//EW_ObjectRecord EW_Bone::record;

EW_Bone::EW_Bone() 
{
  default_name(type());
  a.append(frame_count);
  b.append(frame_count);
  for (int i=0; i<frame_count; i++)
    {
      a[i] = point_db.size();
      point_db.append();
      sgZeroVec3(point_db.last().xyz);
      b[i] = point_db.size();
      point_db.append();
      sgZeroVec3(point_db.last().xyz);
    }
}

EW_Bone::EW_Bone(int a) 
{
  default_name(type());
}

static EW_Object * 
EW_Bone_alloc()
{
  return ((EW_Object *)(new EW_Bone()));
}

Fl_Window * make_EW_Bone_properties_window()
{
  return 0;
}

void
EW_Bone::initialize()
{
  // Set global properties for EW_Bone
  record.name = "EW_Bone";
  record.tag = pdb_new_tag();
  record.create = EW_Bone_alloc;
  record.pixmap = new Fl_Pixmap(bone_xpm); 
  record.property_window = 0; //make_EW_Bone_properties_window();

  // Register EW_Bone methods in the procedural database
  pdb_register(record.name, record.tag);

  // Register EW_Bone in the object registry.
  //global.add_object(record);

  if (global.verbose)
    printf("EW_Bone tag = %d\n", (int)record.tag);
}

/*:
  @subsection EW_Bone::update (double i)

  Update the rotation matrix to the current frame plus some
  interpolation towards the next frame i (0 <= i < 1.0).
*/

void 
EW_Bone::update(double i) 
{
  sgVec3 c;
  int next_a;
  int next_b;
  double angle;

  if (current_frame <= 0) 
    {
      printf("not updated..\n");
      return;
    }

  if (current_frame >= frame_count) 
    {
      printf("not updated..\n");
      return;
    }

  if ((i>1.0) || (i < 0.0)) 
    printf("Trouble in update.\n");
  
  int current_a = a[current_frame];
  int current_b = b[current_frame];

  sgVec3 na, nb;
  sgCopyVec3(na, point_db[current_a].xyz);
  sgCopyVec3(nb, point_db[current_b].xyz);
  
  if ((current_frame + 1) < frame_count) 
    {
      sgVec3 tmp;
      next_a = a[current_frame+1];
      next_b = b[current_frame+1];
      sgLinInterpVec3(tmp, na, point_db[next_a].xyz, i);
      sgCopyVec3(na, tmp);
      sgLinInterpVec3(tmp, nb, point_db[next_b].xyz, i);
      sgCopyVec3(nb, tmp);
    }
   
  sgVec3 w1, w2;
  sgSubVec3(w1, nb, na);
  sgUnitVec3(w1);
  sgSubVec3(w2, point_db[b[0]].xyz, point_db[a[0]].xyz);
  sgUnitVec3(w2);
  
  // Find the angle between these two vectors.
  angle = sgAngleBetweenUVec3(w1, w2);
  
  // Cross product of working vectors 1 and 2 define the axis by which we are
  // rotating.
  sgVectorProductVec3(c, w2, w1);

  // Equivalent angle axis representation.
  sgMakeRotMat4(object_rotation, angle, c);
}

void
EW_Bone::rot(sgMat4 &o_rotation) 
{
  sgVec3 c;
  double angle;
  
  int current_a = a[current_frame];
  int current_b = b[current_frame];

  sgVec3 na, nb;
  sgCopyVec3(na, point_db[current_a].xyz);
  sgCopyVec3(nb, point_db[current_b].xyz);

  sgVec3 w1, w2;
  sgSubVec3(w1, nb, na);
  sgUnitVec3(w1);
  w2[0] = w2[2] = 0;
  w2[1] = 1;
  
  // Find the angle between these two vectors.
  angle = sgAngleBetweenUVec3(w1, w2);

  // Cross product of working vectors 1 and 2 define the axis by which we are
  // rotating.
  sgVectorProductVec3(c, w2, w1);

  // Equivalent angle axis representation.
  sgMakeRotMat4(o_rotation, angle, c);
}

void
EW_Bone::write (FILE *file, int ind)
{
  char * indent = indenter(ind);

  if (tag() == record.tag)
    fprintf (file, "\n%s(!EW_Bone ", indent);

  fprintf (file, "(points ");

  for (int i=0; i<a.size(); i++)
    {
      fprintf(file, "%d ", a[i]);
    }

  fprintf (file, ")");

  fprintf (file, "\n%s  (rotation %g)", 
	   indent, rotation);

  if (tag() == record.tag)
    fprintf (file, ")");
  free(indent);
}

void
EW_Bone::update_gui()
{
}

EW_Object *
EW_Bone::replicate()
{/*
  EW_Bone * rvalue = new EW_Bone();
  *rvalue = *this;
  rvalue->a.reset();
  rvalue->b.reset();
  for (int i=0; i<a.size(); i++)
    {
      rvalue->a.append();
      rvalue->b.append();
      rvalue->a[i] = point_db.size();
      point_db.append();
      point_db.last() = point_db[a[i]];
      rvalue->b[i] = point_db.size();
      point_db.append();
      point_db.last() = point_db[b[i]];
    }
  return rvalue;
  */
  printf("NOT IMPLEMENTED: EW_Bone::replicate\n");
  return 0;
}

void EW_Bone::gl_draw() 
{
  int current_a = a[0];
  int current_b = b[0];

  if (current_frame>0)
    {
      current_a = a[current_frame];
      current_b = b[current_frame];
    }

  sgVec3 bb[4];
  sgSetVec3(bb[0], 0, 0, 0);
  sgSetVec3(bb[1], -.1, .8, 0);
  sgSetVec3(bb[2], 0, 1, 0);
  sgSetVec3(bb[3], .1, .8, 0);
  sgMat4 r;
  rot(r);
  sgVec3 ta, tb;
  sgCopyVec3(ta, point_db[a[current_frame]].xyz);
  sgCopyVec3(tb, point_db[b[current_frame]].xyz);
  double d = sgDistanceVec3(ta, tb);

  glColor3f(0.8, 1.0, 0.8);
  glBegin(GL_LINE_LOOP);
  {
  // translate point
  for (int i=0; i<4; i++)
    {
      sgScaleVec3( bb[i], d );
      sgXformVec3( bb[i], r );
      sgAddVec3( bb[i], point_db[a[current_frame]].xyz );
      glVertex3fv( bb[i] );
    }
  }
  glEnd();
   
  glPointSize(4.0);
  glBegin(GL_POINTS);
  {
    glVertex3fv(point_db[current_a].xyz);
    glVertex3fv(point_db[current_b].xyz);
  }
  glEnd();
  glPointSize(1.0);
}

/*:
  @subsection Vector EW_Bone::translate_point (Vector point)

  translate_point takes a Vector argument point and then "translates"
  that point based on the current EW_Bone to it's new location.
  Rotate the point given by Vector c about the rotation matrix.
  Then return the new translated position.
*/

void
EW_Bone::translate_point (sgVec3 & dst, sgVec3 & src) 
{
  sgCopyVec3(dst, src);

  if (current_frame <= 0) 
    return;

  int current_a = a[current_frame];

  update(0.0);

  // translate point
  sgSubVec3(dst, point_db[a[0]].xyz);
  sgXformVec3(dst, object_rotation);
  sgAddVec3(dst, point_db[current_a].xyz);
}

/*:
  @subsection void EW_Bone::mark_points()

  Mark points in the current frame as visible, and mark points that 
  aren't in the current frame as invisible.

void EW_Bone::mark_points () 
{
  for (int i=0; i<bone_list.size(); i++) 
    {
      for (int j=0; j<frame_count; j++) 
	{
	  if (j == current_frame) 
	    {
	      points[bone_list[i]->a[j]].hidden = 0;
	      points[bone_list[i]->b[j]].hidden = 0;
	    } 
	  else 
	    {
	      point_db[bone_list[i]->a[j]].hidden = 1;
	      point_db[bone_list[i]->b[j]].hidden = 1;
	    }
	}
    }
}
*/

/*:
  @subsection EW_bool EW_Bone::uses_point (int p)

  Returns 1 if point p is any point referenced by a[] or b[] in
  any frame.
*/

EW_bool 
EW_Bone::uses_point (int c)
{
  for (int i=0; i < frame_count; i++)
    {
      if ((c == a[i]) || (c == b[i]))
	return 1;
    }
  
  return 0;
}

/*:
  @subsection EW_bool EW_Bone::uses_point (int p)

  Returns 1 if point p is any point referenced by a[] or b[] in
  any frame.
*/

EW_bool 
EW_Bone::select_point_down (int c, sgVec3 d)
{
  if ((c == a[current_frame]) || (c == b[current_frame]))
    return 1;
  return 0;
}

/*:
  Drag either the base or the end effector of the bone.  If
  the current_frame > 0, then the end effector is a fixed
  distance from the bone bone origin.  If the current_frame
  == 0, then the distance is free (and will be used to set
  the other frames.)
*/

EW_bool
EW_Bone::select_point_drag(int i, sgVec3 v)
{
  EW_bool rvalue = 0;
  // Origin
  if (i == a[current_frame])
    {
      sgVec3 delta;
      sgSubVec3(delta, v, point_db[i].xyz);
      sgCopyVec3(point_db[i].xyz, v);
      if (current_frame > 0)
	{
	  sgAddVec3(point_db[b[current_frame]].xyz, delta);
	}
      rvalue = 1;
      t_bit = 1;
      scene->set_bone_dirty_bit(current_bone);
    }
  // Effector
  else if (i == b[current_frame])
    {
      if (current_frame > 0)
	{
	  double limiting_distance = sgDistanceVec3(point_db[b[0]].xyz, 
						    point_db[a[0]].xyz);
	  sgVec3 d;
	  sgSubVec3(d, v, point_db[a[current_frame]].xyz);
	  sgUnitVec3(d);
	  sgScaleVec3(d, limiting_distance);
	  sgAddVec3(point_db[i].xyz, point_db[a[current_frame]].xyz, d);
	}
      else
	{
	  sgCopyVec3(point_db[i].xyz, v);
	}
      rvalue = 1;
      t_bit = 1;
      scene->set_bone_dirty_bit(current_bone);
    }
  return rvalue;
}

EW_bool 
EW_Bone::select_point_release (int a, sgVec3 b)
{
  return select_point_drag(a, b);
}

EW_bool 
EW_Bone::add_point_down (sgVec3 b)
{
  return 1;
}

EW_bool 
EW_Bone::add_point_drag (sgVec3 b)
{
  return 1;
}

EW_bool 
EW_Bone::add_point_release (sgVec3 b)
{
  return 1;
}

void
EW_Bone::set_draw_dirty_bit()
{
  t_bit = 1;
  d_bit = 1;
}

void
EW_Bone::set_bone_dirty_bit(int i)
{
}

void 
EW_Bone::translate (sgVec3 a) 
{
}

void 
EW_Bone::lock (EW_bool a) 
{
  /*
    point_db[center].lock = a;
  */
}

void cb_bones_browser_duplicate(Fl_Button* a, void* b) 
{
  EW_Bone * c = new EW_Bone();
  bone_list.append();
  bone_list.last() = c;
}

void cb_bones_browser_remove (Fl_Button* a, void* b) 
{
  EW_Bone * c;
  int i, j;
  if (bone_list.size() < 0) 
    {
      fl_alert("Sorry, you must have at least zero bones.  duh.");
      return;
    }

  i = BonesBrowser->value() - 1;
  if (i<=0) 
    i=0;
   
  for (j=i+1; j<bone_list.size(); j++) 
    {
      c = bone_list[j-1];
      bone_list[j-1] = bone_list[j];
      bone_list[j] = c;
    }
  bone_list.shrink(1);
}

int 
bone_from_point (int apnt) 
{
  int i;

  if (apnt < 0) return -1;

  int size = (int)bone_list.size();

  for (i=0; i<size; i++) 
    {
      if (bone_list[i]->a[current_frame] == (int)apnt) 
	return i;
      if (bone_list[i]->b[current_frame] == (int)apnt) 
	return i;
    }
  return -1;
}

#if 0
EW_Bone *
EW_Bone::scm2obj(SCM a)
{
  return (EW_Bone *)EW_Object::scm2obj(a, EW_Bone::record.tag);
}

SCM
EW_Bone::obj2scm(EW_Object *a)
{
  return gh_cons(gh_int2scm(EW_Bone::record.tag), gh_long2scm((long)a));
}
#endif

void
EW_Bone::pdb_register(char * n, long tag)
{
  EW_Object::pdb_register(n, tag);
  pdb_register_tag(tag, record.tag);
}

