/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_camera.h>
#include <ewave/objects/ew_sphere.h>
#include <ewave/xpm_icons/camera.xpm>

EW_Class  EW_Camera::klass;

EW_Camera::EW_Camera () 
{
  style    = PTperspective;
  cylinder = PTfixedvc;
  angle    = 30.0;

  look_at  = new_point ();
  up       = new_coeff3 ();
  
  if (!mesh)
    mesh = EW_Sphere::new_sphere (3);
}

static EW_Object* 
EW_Camera_alloc()
{
  return ((EW_Object*)(new EW_Camera()));
}

void
EW_Camera::initialize()
{
  klass.name ("EW_Camera");
  klass.create (EW_Camera_alloc);
  klass.pixmap (new Fl_Pixmap(camera_xpm));
  klass.inspector (new Fl_Group(0,0,100,100,"Test"));
  kernel->register_object (&klass);
  extern void ew_camera_scheme_init ();
  ew_camera_scheme_init ();
}

int
EW_Camera::write (EIFF *eiff)
{
  EW_Object::write(eiff);
  eiff_write_long (eiff, 1); // EW_Camera version number
  look_at.write (eiff);
  up.write (eiff);

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int
EW_Camera::read (EIFF *eiff)
{
  EW_Object::read (eiff);
  long version = eiff_read_long (eiff);

  if (version >= 1)
    {
      look_at.read (eiff);
      up.read (eiff);
    }
  return 0;
}

Mesh* EW_Camera::mesh = 0;

void 
EW_Camera::draw (int gl_mode, EW_Drw ew_mode)
{
  Vec3 look_at_t, center_t;
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);

  switch (ew_mode) {

  case Drw_points:
    if (visible (ew_mode))
      {
	/*
	if (gl_mode == GL_SELECT)
	  glPushName (look_at[0]);
	glBegin(GL_POINTS);
	tmp = coeff.get (look_at);
	glVertex3fv( tmp.data );
	glEnd();
	if (gl_mode == GL_SELECT)
	  glPopName ();
	*/
      }
    break;
  case Drw_polygons:
    glBegin(GL_LINES);
      {
	look_at_t = get_coeff (look_at);
	glVertex3dv( look_at_t.value() );
	center_t = get_coeff (center);
	glVertex3dv( center_t.value() );
      }
    glEnd();
      {
	glPushMatrix();
	glTranslated (center_t[0], center_t[1], center_t[2]);
	glScaled (10, 10, 10);
	mesh->draw();
	glPopMatrix();
      }
    break;
  default:
    break;
  }
}

EW_Object *
EW_Camera::object_from_point(int a)
{
  if (uses_point(a)) 
    return this;

  return NULL;
}

short  
EW_Camera::select_point_down (Point p, Vec3 v)
{
  return 1;
}

short   
EW_Camera::select_point_drag (Point p, Vec3 v)
{
  set_point_absolute (p, v);
  return 1;
}

short   
EW_Camera::select_point_release (Point p, Vec3 v)
{
  return select_point_drag (p, v);
}

short   
EW_Camera::add_point_down (Vec3 v)
{
  kernel->click ();
  switch (kernel->get_clicks ())
    {
    case 1:
      set_coeff (center, v);
      set_coeff (look_at, v);
      break;
    case 2:
      set_coeff (look_at, v);
      break;
    default:
      return 0;
    }
  return 1;
}

short   
EW_Camera::add_point_drag (Vec3 v)
{
  switch (kernel->get_clicks ())
    {
    case 1:
      set_coeff (center, v);
      break;
    case 2:
      set_coeff (look_at, v);
    default:
      return 0;
    }  
  return 1;
}

short   
EW_Camera::add_point_release (Vec3 v)
{
  add_point_drag (v);

  if (kernel->get_clicks () == 2)
    kernel->current_object = NULL;

  if (kernel->get_clicks () >= 2) kernel->set_clicks (0); 

  return 1;
}

void
EW_Camera::refresh()
{
}

EW_Object *
EW_Camera::replicate()
{
  return 0;
}

/*
EW_Camera *
EW_Camera::scm2obj(SCM a)
{
  return (EW_Camera *)EW_Object::scm2obj(a, klass_.id());
}

SCM
EW_Camera::obj2scm(EW_Object *a)
{
  return gh_cons(gh_int2scm(klass_.id()), gh_long2scm((long)a));
}
*/

EW_Camera * 
EW_Camera::first_camera (EW_Object * a) 
{
  if (a->vklass().id() == klass.id())
    return (EW_Camera *)a;

  if (a->is_group()) 
    {
      EW_Group * c = (EW_Group *)a;
      for (EW_Object_List::cursor i = c->children.begin (); &i; i++)
	{
	  EW_Object * b;
	  b = first_camera (&i); 
	  if (b != 0) 
	    return (EW_Camera *)b;
	}
    }
  return 0;
}
