/* Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_capsul.h>
#include <ewave/objects/ew_sphere.h>
#include <ewave/objects/ew_cylinder.h>
#include <ewave/xpm_icons/sphere.xpm>
#include <FL/Fl_Pack.H>

EW_Class  EW_Capsul::klass;

EW_Capsul::EW_Capsul () 
{
  int a, b;
  for (a=0; a<=1; a++)
    {
      for (b=0; b<4; b++)
	qs[a*4+b] = EW_Sphere::new_quarter_sphere (3, b, a);
    }

  qc[0] = EW_Cylinder::new_quarter_cylinder (3, 1);
  qc[1] = EW_Cylinder::new_quarter_cylinder (3, 0);
  qc[2] = EW_Cylinder::new_quarter_cylinder (3, 2);
  qc[3] = EW_Cylinder::new_quarter_cylinder (3, 3);
}

int
EW_Capsul::write (EIFF *eiff)
{
  EW_Generic::write (eiff);
  eiff_write_long (eiff, 1); // EW_Sphere version number

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int
EW_Capsul::read (EIFF *eiff)
{
  EW_Generic::read (eiff);
  long version = eiff_read_long (eiff);

  if (version >= 1)
    {
      // .. Capsul specific stuff.
    }
  return 0;
}

void
EW_Capsul::refresh ()
{
  EW_Generic::refresh ();
}

EW_Object*
EW_Capsul::replicate ()
{
  /*
   EW_Sphere *rvalue = new EW_Sphere ();
   *rvalue = *this;
   rvalue->center = point_db.size ();
   point_db.append ();
   point_db.last () = point_db[center];
   return rvalue;
   */
   return 0;
}

static EW_Object* 
EW_Capsul_alloc()
{
  return ((EW_Object*)(new EW_Capsul()));
}

void
EW_Capsul::initialize()
{
  klass.name ("EW_Capsul");
  klass.create (EW_Capsul_alloc);
  klass.pixmap (new Fl_Pixmap(sphere_xpm));
  klass.inspector (EW_Generic::klass.inspector ());
  kernel->register_object (&klass);
  //extern void ew_capsul_scheme_init ();
  //ew_capsul_scheme_init ();
}

void 
EW_Capsul::draw (int gl_mode, EW_Drw ew_mode)
{
  Vec3 center_t, scale_t, rotate_t;
  double radius_t;
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);

  switch (ew_mode) {

  case Drw_points:
    if (visible (ew_mode))
      {
      }
    break;
  case Drw_polygons:
    center_t = get_coeff (center);
    scale_t = get_coeff (scale);
    radius_t = get_coeff (radius);
    rotate_t = get_coeff (rotate);
      {
	glPushMatrix();
	glTranslated (center_t[0], center_t[1], center_t[2]);
	glRotated (rotate_t[0], 1, 0, 0);
	glRotated (rotate_t[1], 0, 1, 0);
	glRotated (rotate_t[2], 0, 0, 1);
	
	for (int i=0; i<=1; i++)
	  {
	    for (int j=0; j<=1; j++)
	      {
		for (int k=0; k<=1; k++)
		  {
		    glPushMatrix ();
		    glTranslated ((k ? -1 : 1 ) * (j ? -1 : 1 ) * scale_t[0], 
				  (j ? -1 : 1 ) * scale_t[1], 
				  (i ? -1 : 1 ) * scale_t[2]);
		    
		    glScaled (radius_t, radius_t, radius_t);
		    qs[i*4+j*2+k]->draw ();
		    glPopMatrix ();
		  }
	      }
	  }
	
	for (int j=0; j<=1; j++)
	  {
	    for (int k=0; k<=1; k++)
	      {
		glPushMatrix ();
		glTranslated ((j ? 1 : -1 ) * scale_t[0], 0,
			      (k ? -1 : 1 ) * scale_t[2]);
		
		glScaled (1., scale_t[1] / radius_t, 1.);
		glScaled (radius_t, radius_t, radius_t);
		qc[k*2+j]-> draw ();
		glPopMatrix ();
	      }
	  }
	
	for (int j=0; j<=1; j++)
	  {
	    for (int k=0; k<=1; k++)
	      {
		glPushMatrix ();
		glRotated (90, 0, 0, 1);

		glTranslated ((j ? 1 : -1 ) * scale_t[1], 0,
			      (k ? -1 : 1 ) * scale_t[2]);
		
		glScaled (1., scale_t[0] / radius_t, 1.);
		glScaled (radius_t, radius_t, radius_t);
		qc[k*2+j]-> draw ();
		glPopMatrix ();
	      }
	  }
	
	for (int j=0; j<=1; j++)
	  {
	    for (int k=0; k<=1; k++)
	      {
		glPushMatrix ();
		glRotated (90, 1, 0, 0);

		glTranslated ((j ? 1 : -1 ) * scale_t[0], 0,
			      (k ? -1 : 1 ) * scale_t[1]);
		
		glScaled (1., scale_t[2] / radius_t, 1.);
		glScaled (radius_t, radius_t, radius_t);
		qc[k*2+j]-> draw ();
		glPopMatrix ();
	      }
	  }
	
	glPopMatrix();
      }
    break;
  default:
    break;
  }
}

