/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_cone.h>
#include <ewave/xpm_icons/cone.xpm>

EW_Class  EW_Cone::klass;

EW_Cone::EW_Cone () 
{
  mesh = new_cone (3);
}

int
EW_Cone::write (EIFF *eiff)
{
  EW_Generic::write (eiff);
  eiff_write_long (eiff, 1); // EW_Cone version number

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int
EW_Cone::read (EIFF *eiff)
{
  EW_Generic::read (eiff);
  long version = eiff_read_long (eiff);

  if (version >= 1)
    {
      // .. Cone specific stuff.
    }
  return 0;
}

void
EW_Cone::refresh ()
{
  EW_Generic::refresh ();
}

EW_Object*
EW_Cone::replicate ()
{
  /*
   EW_Cone *rvalue = new EW_Cone ();
   *rvalue = *this;
   rvalue->center = point_db.size ();
   point_db.append ();
   point_db.last () = point_db[center];
   return rvalue;
   */
   return 0;
}

void 
EW_Cone::draw (int gl_mode, EW_Drw ew_mode)
{
  Vec3 center_t, scale_t, rotate_t;
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);

  switch (ew_mode) {

  case Drw_points:
    if (visible (ew_mode))
      {
      }
    break;
  case Drw_polygons:
    center_t = get_coeff (center);
    scale_t = get_coeff (scale) * get_coeff (radius);
    rotate_t = get_coeff (rotate);
      {
	glPushMatrix();
	glTranslated (center_t[0], center_t[1], center_t[2]);
	glRotated (rotate_t[0], 1, 0, 0);
	glRotated (rotate_t[1], 0, 1, 0);
	glRotated (rotate_t[2], 0, 0, 1);
	glScaled (scale_t[0], scale_t[1], scale_t[2]);
	mesh->draw();
	glPopMatrix();
      }
    break;
  default:
    break;
  }
}

static EW_Object* 
EW_Cone_alloc ()
{
  return ((EW_Object*)(new EW_Cone()));
}

void
EW_Cone::initialize ()
{
  klass.name ("EW_Cone");
  klass.create (EW_Cone_alloc);
  klass.pixmap (new Fl_Pixmap(cone_xpm));
  klass.inspector (EW_Generic::klass.inspector ());
  kernel->register_object (&klass);
  extern void ew_cone_scheme_init ();
  ew_cone_scheme_init ();
}

Mesh *
EW_Cone::new_cone (int detail_)
{
  double detail = detail_ * 4.;
  Mesh *mesh = new Mesh ();
  mesh->allow_duplicate_vertices ();
  
  for (int n=0; n<detail; n++)
    {
      double x  = cos((((360. / detail) * n)     * DEGREES_TO_RADIANS));
      double xn = cos((((360. / detail) * (n+1)) * DEGREES_TO_RADIANS));
      double y  = sin((((360. / detail) * n)     * DEGREES_TO_RADIANS));
      double yn = sin((((360. / detail) * (n+1)) * DEGREES_TO_RADIANS));
	{
      Vec3 a, b, c, d, n;

      a[2] = y;
      b[2] = yn;
      //c[1] = yn;
      //d[1] = y;
      a[0] = x;
      b[0] = xn;
      //c[0] = xn;
      //d[0] = x;
      a[1] = b[1] = -1;
      //c[2] = d[2] = 1;
      Vec3 e;
      e[0] = 0;
      e[2] = 0;
      e[1] = 1;

      ///sgCopyVec3(cone_mesh.append(), a);
      ///sgCopyVec3(cone_mesh.append(), b);
      ///sgCopyVec3(cone_mesh.append(), e);
      ///sgCopyVec3(cone_mesh.append(), e);
      
      n = (a-b) % (e-b);
      mesh->add_face (e, e, b, a, n, n, n, n);

      e[1] = 0;
      n = (b-a) % (e-a);
      //if (end_caps)
	{
	  e[1] = -1;
	  // triangle between a b center
	  mesh->add_face (a, b, e, e, n, n, n, n);

	  //sgCopyVec3(cone_mesh.append(), e);
	  //sgCopyVec3(cone_mesh.append(), e);
	  //sgCopyVec3(cone_mesh.append(), b);
	  //sgCopyVec3(cone_mesh.append(), a);
	  // triangle between c d center
	}
	}
    }
  
  return mesh;
}
