/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_cube.h>
#include <ewave/xpm_icons/box.xpm>

EW_Class  EW_Cube::klass;

EW_Cube::EW_Cube () 
{
  mesh = new_cube (1);
}

int
EW_Cube::write (EIFF *eiff)
{
  EW_Generic::write (eiff);
  eiff_write_long (eiff, 1); // EW_Cube version number

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int
EW_Cube::read (EIFF *eiff)
{
  EW_Generic::read (eiff);
  long version = eiff_read_long (eiff);

  if (version >= 1)
    {
      // .. cube specific stuff.
    }
  return 0;
}

void
EW_Cube::refresh ()
{
  EW_Generic::refresh ();
}

EW_Object*
EW_Cube::replicate ()
{
  /*
  EW_Cube *rvalue = new EW_Cube();
  *rvalue = *this;
  rvalue->center = point_db.size();
  point_db.append();
  point_db.last() = point_db[center];
  return rvalue;
  */
  return 0;
}

void 
EW_Cube::draw (int gl_mode, EW_Drw ew_mode)
{
  Vec3 center_t, scale_t, rotate_t;
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);

  switch (ew_mode) {

  case Drw_points:
    break;
  case Drw_polygons:
    center_t = get_coeff (center);
    scale_t = get_coeff (scale) * get_coeff (radius);
    rotate_t = get_coeff (rotate);
      {
	glPushMatrix();
	glTranslated (center_t[0], center_t[1], center_t[2]);
	glRotated (rotate_t[0], 1, 0, 0);
	glRotated (rotate_t[1], 0, 1, 0);
	glRotated (rotate_t[2], 0, 0, 1);
	glScaled (scale_t[0], scale_t[1], scale_t[2]);
	mesh->draw();
	glPopMatrix();
      }
    break;
  default:
    break;
  }
}

static EW_Object* 
EW_Cube_alloc()
{
  return ((EW_Object*)(new EW_Cube()));
}

void
EW_Cube::initialize ()
{
  klass.name ("EW_Cube");
  klass.create (EW_Cube_alloc);
  klass.pixmap (new Fl_Pixmap(box_xpm));
  klass.inspector (EW_Generic::klass.inspector ());
  kernel->register_object (&klass);
  extern void ew_cube_scheme_init ();
  ew_cube_scheme_init ();
}

Mesh * 
EW_Cube::new_cube (int detail)
{
  Mesh *mesh = new Mesh ();

  /*
       v7----v6
      /     /
   v3-----v2 |
    |     |  |
    | v4--|-v5
    |/    |/
   v0-----v1
   
   */
  
  Vec3 v0 (-0.5, -0.5, -0.5);
  Vec3 v1 ( 0.5, -0.5, -0.5);
  Vec3 v2 ( 0.5,  0.5, -0.5);
  Vec3 v3 (-0.5,  0.5, -0.5);
  Vec3 v4 (-0.5, -0.5,  0.5);
  Vec3 v5 ( 0.5, -0.5,  0.5);
  Vec3 v6 ( 0.5,  0.5,  0.5);
  Vec3 v7 (-0.5,  0.5,  0.5);
  Vec3 n;
  
  mesh->allow_duplicate_vertices ();
  
  n = v0 % v3;
  mesh->add_face (v0, v3, v2, v1,     n, n, n, n);
  n = v1 % v2;
  mesh->add_face (v1, v2, v6, v5,     n, n, n, n);
  n = v5 % v6;
  mesh->add_face (v5, v6, v7, v4,     n, n, n, n);
  n = v4 % v7;
  mesh->add_face (v4, v7, v3, v0,     n, n, n, n);
  n = v3 % v7;
  mesh->add_face (v3, v7, v6, v2,     n, n, n, n);
  n = v4 % v0;
  mesh->add_face (v4, v0, v1, v5,     n, n, n, n);
  
  return mesh;
}
