/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_cylinder.h>
#include <ewave/xpm_icons/cylinder.xpm>

EW_Class  EW_Cylinder::klass;

EW_Cylinder::EW_Cylinder () 
{
  //mesh = new_cylinder (3);
  mesh = new_quarter_cylinder (3, 1);
}

int
EW_Cylinder::write (EIFF *eiff)
{
  EW_Generic::write (eiff);
  eiff_write_long (eiff, 1); // EW_Cylinder version number

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int
EW_Cylinder::read (EIFF *eiff)
{
  EW_Generic::read (eiff);
  long version = eiff_read_long (eiff);

  if (version >= 1)
    {
      // .. Cylinder specific stuff.
    }
  return 0;
}

void
EW_Cylinder::refresh ()
{
  EW_Generic::refresh ();
}

EW_Object*
EW_Cylinder::replicate ()
{
  /*
   EW_Cylinder *rvalue = new EW_Cylinder ();
   *rvalue = *this;
   rvalue->center = point_db.size ();
   point_db.append ();
   point_db.last () = point_db[center];
   return rvalue;
   */
	return 0;
}

void 
EW_Cylinder::draw (int gl_mode, EW_Drw ew_mode)
{
  Vec3 center_t, scale_t, rotate_t;
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);

  switch (ew_mode) {

  case Drw_points:
    if (visible (ew_mode))
      {
      }
    break;
  case Drw_polygons:
    center_t = get_coeff (center);
    scale_t = get_coeff (scale) * get_coeff (radius);
    rotate_t = get_coeff (rotate);
      {
	glPushMatrix();
	glTranslated (center_t[0], center_t[1], center_t[2]);
	glRotated (rotate_t[0], 1, 0, 0);
	glRotated (rotate_t[1], 0, 1, 0);
	glRotated (rotate_t[2], 0, 0, 1);
	glScaled (scale_t[0], scale_t[1], scale_t[2]);
	mesh->draw();
	glPopMatrix();
      }
    break;
  default:
    break;
  }
}

static EW_Object* 
EW_Cylinder_alloc ()
{
  return ((EW_Object*)(new EW_Cylinder()));
}

void
EW_Cylinder::initialize ()
{
  klass.name ("EW_Cylinder");
  klass.create (EW_Cylinder_alloc);
  klass.pixmap (new Fl_Pixmap(cylinder_xpm));
  klass.inspector (EW_Generic::klass.inspector ());
  kernel->register_object (&klass);
  extern void ew_cylinder_scheme_init ();
  ew_cylinder_scheme_init ();
}

Mesh *
EW_Cylinder::new_cylinder (int detail_)
{
  double detail = detail_ * 4.;
  Mesh *mesh = new Mesh ();
  mesh->allow_duplicate_vertices ();
  
  for (int i=0; i<detail; i++)
    {
      double x  = cos((((360. / detail) * i)     * DEGREES_TO_RADIANS));
      double xn = cos((((360. / detail) * (i+1)) * DEGREES_TO_RADIANS));
      double y  = sin((((360. / detail) * i)     * DEGREES_TO_RADIANS));
      double yn = sin((((360. / detail) * (i+1)) * DEGREES_TO_RADIANS));
      Vec3 a, b, c, d, n;

      a[2] = y;
      b[2] = yn;
      c[2] = yn;
      d[2] = y;
      a[0] = x;
      b[0] = xn;
      c[0] = xn;
      d[0] = x;
      a[1] = b[1] = -1;
      c[1] = d[1] = 1;

      n = (a-b) % (c-b);
      mesh->add_face (d, c, b, a, n, n, n, n);

      //if (end_caps)
	{	  
	  Vec3 e;
	  e[0] = 0;
	  e[2] = 0;
	  
	  e[1] = -1;
	  n = (a-b) % (e-b);
	  mesh->add_face (a, b, e, e, n, n, n, n);

	  e[1] = 1;
	  n = (c-d) % (e-d);
	  mesh->add_face (c, d, e, e, n, n, n, n);
	}
    }
  
  return mesh;
}

Mesh *
EW_Cylinder::new_quarter_cylinder (int detail_, int q0)
{
  double detail = detail_ * 4.;
  Mesh *mesh = new Mesh ();
  mesh->allow_duplicate_vertices ();
  
  for (int i=detail_*q0; i<(detail_*(q0+1)); i++)
    {
      double x  = cos((((360. / detail) * i)     * DEGREES_TO_RADIANS));
      double xn = cos((((360. / detail) * (i+1)) * DEGREES_TO_RADIANS));
      double y  = sin((((360. / detail) * i)     * DEGREES_TO_RADIANS));
      double yn = sin((((360. / detail) * (i+1)) * DEGREES_TO_RADIANS));
      Vec3 a, b, c, d, n;

      a[2] = y;
      b[2] = yn;
      c[2] = yn;
      d[2] = y;
      a[0] = x;
      b[0] = xn;
      c[0] = xn;
      d[0] = x;
      a[1] = b[1] = -1;
      c[1] = d[1] = 1;

      n = (a-b) % (c-b);
      mesh->add_face (d, c, b, a, n, n, n, n);

      //if (end_caps)
	{	  
	  Vec3 e;
	  e[0] = 0;
	  e[2] = 0;
	  
	  e[1] = -1;
	  n = (a-b) % (e-b);
	  mesh->add_face (a, b, e, e, n, n, n, n);

	  e[1] = 1;
	  n = (c-d) % (e-d);
	  mesh->add_face (c, d, e, e, n, n, n, n);
	}
    }
  
  return mesh;
}

