#include <unistd.h>
#include <ewave/objects/ew_group.h>
#include <ewave/xpm_icons/csg.xpm>

#define EW_CLASS_VERSION 1

EW_Class EW_Group::klass;

EW_Group::EW_Group ()
{
  children.init (EW_Object::evaluator);
}

EW_Group::~EW_Group() 
{
  for (EW_Object_List::cursor i = children.begin (); &i; i++)
    delete (i.value ());
}

static EW_Object *
EW_Group_alloc ()
{
  return new EW_Group();
}

void
EW_Group::initialize()
{
  klass.name ("EW_Group");
  klass.create (EW_Group_alloc);
  klass.pixmap (new Fl_Pixmap(csg_xpm));
  klass.inspector (new Fl_Group(0,0,100,100,"Test"));
  kernel->register_object (&klass);
  extern void ew_group_scheme_init ();
//  ew_group_scheme_init ();
}

void 
EW_Group::set_damage (EW_Dmg x) 
{
  for (EW_Object_List::cursor i = children.begin (); i.value (); i++)
    (&i)->set_damage (x);
}

void 
EW_Group::draw (int gl_mode, EW_Drw ew_mode) 
{  
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);
  Vec3 center_t = get_coeff (center);

  glPushMatrix();
  glTranslated (center_t[0], center_t[1], center_t[2]);
  for (EW_Object_List::cursor i = children.begin (); &i; i++)
    (&i)->draw (gl_mode, ew_mode);
  glPopMatrix();
}

void 
EW_Group::translate (Vec3 a) 
{
  EW_Object::translate (a);
}

void 
EW_Group::scale (Vec3 origin, Vec3 scale) 
{
  //  for (int i = 0; i < children.size (); i++) 
  //    children[i]->scale (origin, scale);
}

void 
EW_Group::rotate (Vec3 origin, Vec3 rotation) 
{
  //for (int i = 0; i < children.size (); i++) 
  //  children[i]->rotate (origin, rotation);
}

EW_Object *
EW_Group::object_from_point (int a)
{
  if (uses_point(a)) 
    return this;

  for (EW_Object_List::cursor i = children.begin (); i.value (); i++)
    {
      EW_Object * c = (&i)->object_from_point(a);
      if (c != NULL)
	return c;
    }
  return NULL;
}

int
EW_Group::write (char *filename)
{
  EIFF *eiff = eiff_open(filename, "w");
  return write (eiff);
}

int
EW_Group::write (EIFF *eiff)
{
  EW_Object::write (eiff);
  
  eiff_write_long (eiff, 17); // EW_Group version number
  eiff_write_long (eiff, open); // Group is open (1) or closed (0)
  eiff_write_long (eiff, children.size()); // Number of children

  for (EW_Object_List::cursor i = children.begin (); i.value (); i++)
    (&i)->write (eiff);

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int 
EW_Group::read (EIFF *eiff)
{
  EW_Object::read (eiff);
  
  long version = eiff_read_long (eiff);
  printf ("version = %ld\n", version);
  
  if (version >= 17)
    {
      open = eiff_read_long (eiff);
      printf ("open = %ld\n", open);
      long children_cnt = eiff_read_long (eiff);
      
      printf ("children_cnt = %ld\n", children_cnt);
      
      for (int i=0; i<children_cnt; i++)
	eiff_read_chunk (eiff, this);
    }
  return 0;
}

short 
EW_Group::remove_child(EW_Object *a)
{
  for (EW_Object_List::cursor i = children.begin (); &i; i++)
    if ((&i) == a)
      {
	children.erase (i);
	return 1;
      }
  return 0;
}

short 
EW_Group::add (EW_Object *a) 
{
//  children.append ();
//  children.last () = a;
  children.insert (a);
  a->set_parent (this);
   
  if (!global.loading)
    {
      //scene_graph->find(this);
      //scene_graph->add_sub (strdup (a->name ()), 
	//		    (int)a->is_group (), 
//			    a->klass.pixmap (), 
//			    (void *)children.last ());
    }
  return 1;
}

void
EW_Group::refresh ()
{
  //Group_name->value(name ());
}

EW_Object *
EW_Group::replicate()
{
  EW_Group * rvalue = new EW_Group();
  *rvalue = *this;
  rvalue->children.clear ();

/*  for (i=0; i<children.size(); i++)
    rvalue->add(children[i]->replicate());
*/
  return rvalue;
}

short 
EW_Group::clear() 
{
  children.clear ();
  return 1;
}

