/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_nurbs.h>
#include <ewave/xpm_icons/camera.xpm>

EW_Class  EW_NURBS::klass;

EW_NURBS::EW_NURBS () 
{

   vector <Point> a(4);
   cv.push_back (a);
   cv.push_back (a);
   cv.push_back (a);
   cv.push_back (a);
   
   degree = 3;     // default is 3 for cubic.
   
   for (int i=0; i<4; i++)
     {
	for (int j=0; j<4; j++)
	  {
	     Point tmp = new_coeff3 ();
	     set_coeff (tmp, Vec3(0,10.*i,10.*j));
	     (cv[i])[j] = tmp;
	  }
     }
}

static EW_Object* 
EW_NURBS_alloc()
{
  return ((EW_Object*)(new EW_NURBS()));
}

void
EW_NURBS::initialize()
{
   klass.name ("EW_NURBS");
   klass.create (EW_NURBS_alloc);
   klass.pixmap (new Fl_Pixmap(camera_xpm));
   klass.inspector (new Fl_Group(0,0,100,100,"Test"));
   kernel->register_object (&klass);
   //extern void ew_camera_scheme_init ();
   //ew_camera_scheme_init ();
}

int
EW_NURBS::write (EIFF *eiff)
{
   EW_Object::write(eiff);
   eiff_write_long (eiff, 1); // EW_NURBS version number
   
   eiff_write_long (eiff, cv.size());    // EW_NURBS rows
   eiff_write_long (eiff, cv[0].size()); // EW_NURBS columns
   
   for (unsigned int i=0; i < cv.size(); i++) //rows
     for (unsigned int j=0; j < cv[i].size(); j++) //columns
       (cv[i])[j].write (eiff);

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int
EW_NURBS::read (EIFF *eiff)
{
   EW_Object::read (eiff);
   long version = eiff_read_long (eiff);
   
   unsigned long col = eiff_read_long (eiff);
   unsigned long row = eiff_read_long (eiff);

   if (version >= 1)
     {

	for (unsigned int i=0; i<row; i++)
	  {
	     vector<Point> tmp (col);
	     for (unsigned int j=0; j<col; j++)
	       {
		  tmp[j].read (eiff);
	       }
	     cv.push_back (tmp);
	  }
    }
   
  return 0;
}

/*

typedef vector <vector <Vec3> > control_mesh;

Vec3
EW_NURBS::create_nurbs_value(degree, level, control_mesh CVs, t, s)
{
   create newCVs..
   return create_nurbs_value(degree, level-1, newCVs t, s);
}
*/

void 
EW_NURBS::draw (int gl_mode, EW_Drw ew_mode)
{
  Vec3 look_at_t, center_t;
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);

  switch (ew_mode) {

  case Drw_points:
    if (visible (ew_mode))
      {
      }
    break;
  case Drw_polygons:
    glBegin(GL_LINES);
      {
	 for (unsigned int i=0; i<cv.size(); i++) //rows
	   for (unsigned int j=0; j<cv[i].size(); j++) //columns
	   {
	      Vec3 a = get_coeff ((cv[i])[j]);
	      if ((j+1) < cv[i].size())
		{
		   Vec3 b = get_coeff ((cv[i])[j+1]);
		   glVertex3dv( a.value() );
		   glVertex3dv( b.value() );		   
		}	      
	   }
	 
	 for (unsigned int j=0; j<cv[0].size(); j++) //rows
	   for (unsigned int i=0; i<cv.size(); i++) //columns
	   {
	      Vec3 a = get_coeff ((cv[i])[j]);
	      if ((i+1) < cv[0].size())
		{
		   Vec3 b = get_coeff ((cv[i+1])[j]);
		   glVertex3dv( a.value() );
		   glVertex3dv( b.value() );		   
		}	      
	   }
      }
    glEnd();
       {
	  /*
	 for (unsigned int j=0; j<cv[0].size(); j++) //rows
	   for (unsigned int i=0; i<cv.size(); i++) //columns
	   {
	      Vec3 a = get_coeff ((cv[i])[j]);
	      // convert to Vec3 mesh...
	   }
	  
	  for (t=0..1, s=0..1, s+=delta, t+=delta)
	    {
	       pnt0 = create_nurbs_value(degree, degree, CVs, t, s);
	       pnt1 = create_nurbs_value(degree, degree, CVs, t+delta, s);
	       pnt2 = create_nurbs_value(degree, degree, CVs, t, s+delta);
	       pnt3 = create_nurbs_value(degree, degree, CVs t+delta, s+delta);
	       
	       glBegin ();
		 {
		    glvertex (pnt0);
		    glvertex (pnt1);
		    glvertex (pnt2);
	       
		    glvertex (pnt1);
		    glvertex (pnt2);
		    glvertex (pnt3);
		 }
	       glEnd ();
	    }
	   */
	 // NURBS DRAWING CODE HERE..  FILL MESH WITH VALUES...
	 // mesh->draw();
      }
    break;
  default:
    break;
  }
}

EW_Object *
EW_NURBS::replicate()
{
  return 0;
}

void
EW_NURBS::refresh ()
{
  return;
}
