/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/xpm_icons/default.xpm>
#include <ewave/objects/ew_object.h>

EW_Class  EW_Object::klass;
int       EW_Object::default_name_cnt;
int       EW_Class::uid = 0;
int       read_CHUNK_EW_OBJECT (EIFF* eiff, long unsigned parent);

long
EW_Object::evaluator (EW_Object *x)
{
  return x->get_id ();
}

static long 
Point_List_v (Point *t)
{
  return ((long)t->x());
}

static long 
Coefficient_List_v (Coefficient *t)
{
  return (t->get_id ());
}

EW_Object::EW_Object() 
{ 
  name = 0;
  parent = 0;

  set_default_name ("object");
  coeff.init (&Coefficient_List_v);
  attribute.init (&Coefficient_List_v);
  point.init (&Point_List_v);
  
  center = new_point ();

  /*
  printf("testing coefficients..\n");
  set_coeff (center, Vec3(5, 20, 50));
  set_coeff (2, center, Vec3(10, 30, 60));
  //set_coeff (2, center, Vec3(20.5, 21.5, 22.5));

  
  Vec3 rv = get_coeff (center);
  printf("rv=%g, %g, %g\n", rv[0], rv[1], rv[2]);
  
  for (double i=0; i<2.5; i=i+.1)
    {
      rv = get_coeff (i, center);
      printf("rv(%g)=%g, %g, %g\n", i, rv[0], rv[1], rv[2]);
    }
  printf("testing end..\n");
*/
  
  visible (Drw_object, 1);
  visible (Drw_points, 1);
  visible (Drw_polygons, 1);
}

EW_Object::~EW_Object () { }

void
EW_Object::initialize ()
{
  klass.name ("EW_Object");
  klass.pixmap (new Fl_Pixmap(default_xpm));
  eiff_init ();
  eiff_register_chunk (CHUNK_EW_OBJECT, read_CHUNK_EW_OBJECT);
  extern void ew_object_scheme_init ();
//  ew_object_scheme_init ();
}

short
EW_Object::is_group ()
{
  return 0;
}

void
EW_Object::translate(Vec3 delta) 
{
  set_coeff (center, (get_coeff (center) + delta));
}

void
EW_Object::scale (Vec3 origin, Vec3 scale) 
{}

void
EW_Object::rotate (Vec3 origin, Vec3 rotation) {}

short
EW_Object::uses_point (int j) 
{
  if (point.search (j)) return 1;
  return 0;
}

long
EW_Object::new_coeff ()
{
  Coefficient *y = new Coefficient ();
  coeff.insert (y);
  return (y->get_id());
}

Point
EW_Object::new_coeff3 ()
{
  Point x;

  x[0] = new_coeff ();
  x[1] = new_coeff ();
  x[2] = new_coeff ();

  // set_coeff (x, Vec3(0, 0, 0));
  
  return x;
}

Point
EW_Object::new_point ()
{
  Point x, *z;

  x = new_coeff3 ();

  z = new Point();
  *z = x;
  point.insert (z);
  return x;
}

/*
Vec3
EW_Object::get_point_relative (int j)
{
  int i, s = point.size();
  for (i = 0; i < s; i++)
    if (point[i]->x() == j)
      return (get_coeff (*point[i]));
  return Vec3 (0, 0, 0);
}
*/

Point
EW_Object::lookup_point (int j) 
{
  const Point *p = point.search (j);
  if (p)
    return *p;
  return Point(-1, -1, -1);
}

Vec3
EW_Object::get_point_relative (Point j)
{
  return (get_coeff (j));
}

Vec3
EW_Object::get_point_absolute (Point j)
{
  if (parent)
    return get_point_relative (j) + parent->get_center_absolute ();
  return get_point_relative (j);
}

void
EW_Object::set_point_relative (Point j, Vec3 k)
{
  set_coeff (j, k);
}

void
EW_Object::set_point_absolute (Point j, Vec3 k)
{
  if (parent)
    return set_point_relative (j, k - parent->get_center_absolute ());
  return set_point_relative (j, k);
}

short 
EW_Object::select_point_down (Point p, Vec3 v)
{
  return 1;
}

short 
EW_Object::select_point_drag (Point p, Vec3 v)
{
  set_point_absolute (p, v);
  return 1;
}

short 
EW_Object::select_point_release (Point p, Vec3 v)
{
  return select_point_drag (p, v);
}

short 
EW_Object::add_point_down (Vec3 v)
{
  set_coeff (center, v);
  return 1;
}

short 
EW_Object::add_point_drag (Vec3 v)
{
  set_coeff (center, v);
  return 1;
}

short 
EW_Object::add_point_release (Vec3 v)
{
  add_point_drag (v);
  return 1;
}

EW_Object * 
EW_Object::replicate()
{
  return NULL;
}

short 
EW_Object::is_subset_of (EW_Object *a) 
{
  if (a->is_superset_of(this))
    return 1;
  return 0;
}

short 
EW_Object::is_superset_of (EW_Object *a) 
{
  if (a == this) 
    return 1;
  if (! is_group())
    return 0;

  EW_Group *b = (EW_Group *)this;

  for (EW_Object_List::cursor i = b->children.begin (); &i; i++)
    {
      if ((&i)->is_superset_of (a))
	return 1;
    }
  return 0;
}

short 
EW_Object::is_parent_of (EW_Object *a) 
{
  if (a == this) 
    return 0;
  if (! is_group())
    return 0;

  EW_Group *b = (EW_Group *)this;

  for (EW_Object_List::cursor i = b->children.begin (); &i; i++)
    if ((&i) == a)
      return 1;

  return 0;
}

short 
EW_Object::is_child_of (EW_Object *a) 
{
  if (a->is_parent_of(this))
    return 1;
  return 0;
}

short
EW_Object::remove()
{
  if (parent != NULL)
    if (parent->remove_child(this))
      return 1;
  return 0;
}

EW_Object *
EW_Object::object_from_point(int a)
{
  if (uses_point(a)) 
    return this;

  return NULL;
}

void 
EW_Object::visible (int mode, int state)
{
  switch (mode) {
  case Drw_points:
    ew_points_visible = state;
    break;
  case Drw_polygons:
    ew_polygons_visible = state;
    break;
  case Drw_object:
    ew_object_visible = state;
    break;
  };
}

void 
EW_Object::show ()
{
  ew_object_visible = 1;
}

void 
EW_Object::hide ()
{
  ew_object_visible = 0;
}

int
EW_Object::visible (int mode)
{
  switch (mode) {
  case Drw_points:
    return ew_points_visible;
  case Drw_polygons:
    return ew_polygons_visible;
  case Drw_object:
    return ew_object_visible;
  };
  return 0;
}

void
EW_Object::draw_point (Point p, int gl_mode)
{
  if (p[0] == -1) 
    printf ("EW_Object::draw_point - invalid point\n");

  Vec3 tmp = get_coeff (p);

  if (gl_mode == GL_SELECT)
    glPushName (p[0]);

  if (kernel->current_point[0] == p[0])
    {
      // update_context (tmp);
      glColor3dv (kernel->selected_point_color.value ());
      glBegin (GL_POINTS);
      glVertex3dv (tmp.value ());
      glEnd ();
      glColor3dv (kernel->point_color.value ());
    }
  else
    {
      glBegin (GL_POINTS);
      glVertex3dv (tmp.value ());
      glEnd();
    }

  if (gl_mode == GL_SELECT)
    glPopName ();
}

void
EW_Object::draw (int gl_mode, EW_Drw ew_mode) 
{
  if (!visible (Drw_object)) return;

  switch (ew_mode) {

   case Drw_points:
    if (visible (ew_mode))
      {
	// draw the points
	/*
	int i, s = point.size();
	for (i = 0; i < s; i++)
	  if (1) // hide invisible points 
	    draw_point (point[i], gl_mode);
	 */
	Point_List::cursor Cursor;
	for (Cursor = point.begin(); Cursor.value(); Cursor++)
	  {
	    if (1) // hide invisible points 
	      draw_point (*Cursor.value(), gl_mode);
	  }	
      }
    break;

  case Drw_polygons:
    if (this != kernel->current_object) 
      glColor3f (0.8f, 0.8f, 1.0f);
    else 
      glColor3f (0.8f, 0.8f, 0.4f);
    break;
   default:
    break;
  }

}

void
EW_Object::set_default_name (const char *s)
{
  char t[80];
  if (name) free (name);
  sprintf (t, "%s_%d", s, default_name_cnt);
  default_name_cnt++;
  name = strdup (t);
}

/*
EW_Object *
EW_Object::scm2obj(SCM a, long b)
{
  return (EW_Object *)gh_scm2long(pdb_lookup_object(a, b));
}
*/

char * indenter(int j)
{
  char * r = (char *)malloc(j+1);
  for (int i=0; i<j; i++)
    r[i] = ' ';
  r[j] = '\0';
  return r;
}

void
EW_Object::set_name (char *c)
{
  // 2UNDO PUSH: name (name_);
  if (name) free (name);
  name = strdup (c);
}

Vec3
EW_Object::get_center_relative ()
{
  return get_coeff (center);
}

Vec3
EW_Object::get_center_absolute ()
{
  Vec3 relative_center = get_center_relative ();
  if (parent)
    {
      Vec3 absolute_center = parent->get_center_absolute ();
      return absolute_center + relative_center;
    }
  return relative_center;
}

void 
EW_Object::set_center_relative (Vec3 v)
{
  set_coeff (center, v);
}

void
EW_Object::set_center_absolute (Vec3 v)
{
  Vec3 absolute_center (0, 0, 0);
  
  if (parent)
    absolute_center = parent->get_center_absolute ();

  set_center_relative (v - absolute_center);
}

void
EW_Object::set_locked (short x)
{
  locked = x;
}

int
EW_Object::write (EIFF *eiff)
{
  eiff_write_chunk (eiff, CHUNK_EW_OBJECT);
  eiff_write_string (eiff, (char *)vklass().name());
  eiff_write_long (eiff, 1); // EW_Object version number
  eiff_write_string (eiff, name);

  UBase::write (eiff);
  eiff_write_long (eiff, locked);
  eiff_write_long (eiff, ew_polygons_visible); // visibility flags

  write_coeff (eiff);
  write_point (eiff);
  write_attribute (eiff);
  center.write (eiff);

  if (&klass == &vklass())
    {
      printf("This should *never* happen!!\n");
      eiff_finish_chunk (eiff);
    }
  return 0;
}

int
EW_Object::read (EIFF *eiff)
{
  long version = eiff_read_long (eiff);
  printf("EW_Object::read : version = %ld\n", version);
  
  if (version >= 1)
    {
      char *c = eiff_read_string (eiff); set_name (c); free (c); // name
      printf("EW_Object::read : name = %s\n", get_name());

      UBase::read (eiff);
      locked = eiff_read_long (eiff); // lock flag
      printf("EW_Object::read : locked = %d\n", locked);
      long vis = eiff_read_long (eiff); // visibility flags
      printf("EW_Object::read : vis = %ld\n", vis);
      
      read_coeff (eiff);
      read_point (eiff);
      read_attribute (eiff);
      center.read (eiff);
      printf("EW_Object::read : center = %ld, %ld, %ld\n", center[0], center[1], center[2]);
    }
  return 0;
}

int 
EW_Object::read (char *filename)
{
  EIFF *eiff = eiff_open(filename, "r");
  eiff_read_chunk (eiff, 0);
  return 1;
}

int
read_CHUNK_EW_OBJECT (EIFF* eiff, long unsigned parent)
{
  char* klass_name = eiff_read_string (eiff);
  printf("read_CHUNK_EW_OBJECT : klass_name = %s\n", klass_name);
  
  global.set_object (klass_name);
  EW_Object *obj = global.new_object ();
  free (klass_name);

  if (parent)
    ((EW_Group *)parent)->add (obj);
  else
    kernel->scene = (EW_Group *)obj;

  return obj->read (eiff);
}

void EW_Object::set_attribute_real (char *, double) {}
void EW_Object::set_attribute_string (char *, char *) {}
void EW_Object::set_attribute_binary (char *, void *, int size) {}
double EW_Object::get_attribute_real (char *, double) { return 0.0;}
const char * EW_Object::get_attribute_string (char *, char *) { return "bug: unimplemented";}
void * EW_Object::get_attribute_binary (char *, void *, int size) {return 0;}
EW_Object::EW_Att EW_Object::attribute_type (char *) { return Att_undefined; }

void
kb_tangents (Coefficient_Value a0, Coefficient_Value a1, 
	     Coefficient_Value a2, Coefficient_Value a3, 
	     double & ax, double & ay, double & bx, double & by) 
{
  // The "outgoing" tangent for the first point, a1:
  
  ax = ( (0.5)
	 * (1. - a1.tension) 
	 * (1. - a1.bias) 
	 * (1. - a1.continuity) 
	 * (a2.value[0] - a1.value[0]) )
    + ( (0.5)
	* (1. - a1.tension)
	* (1. + a1.bias)
	* (1. + a1.continuity)
	* (a1.value[0] - a0.value[0]) );
  
  ay = ( (0.5) 
	 * (1. - a1.tension) 
	 * (1. - a1.bias) 
	 * (1. - a1.continuity) 
	 * (a2.value[1] - a1.value[1]) )
    + ( (0.5)
	* (1. - a1.tension)
	* (1. + a1.bias)
	* (1. + a1.continuity)
	* (a1.value[1] - a0.value[1]) );
  
  // The "incoming" tangent for the second point, a2:
  
  bx = ( (0.5) 
	 * (1. - a2.tension) 
	 * (1. - a2.bias) 
	 * (1. + a2.continuity) 
	 * (a3.value[0] - a2.value[0]) )
    + ( (0.5)
	* (1. - a2.tension)
	* (1. + a2.bias)
	* (1. - a2.continuity)
	* (a2.value[0] - a1.value[0]) );
  
  by = ( (0.5) 
	 * (1. - a2.tension) 
	 * (1. - a2.bias) 
	 * (1. + a2.continuity) 
	 * (a3.value[1] - a2.value[1]) )
    + ( (0.5)
	* (1. - a2.tension)
	* (1. + a2.bias)
	* (1. - a2.continuity)
	* (a2.value[1] - a1.value[1]) );
}

#ifdef REM
static void
print_point (Coefficient_Value a)
{
  printf ("CE_V: %g, %g\n", a.value[0], a.value[1]);
}
#endif

static double
calculate_coeff (Coefficient *a, double t)
{
  double time_t;
  for (unsigned long i = 0; i < a->size(); i++)
    {
      if ((*a)[i].value[0] > t)
	{
	  double ax, ay, bx, by;
	  Coefficient_Value p0, p1, p2, p3;

	  if (i == 0)
	    return (*a)[i].value[1];

	  p2 = (*a)[i];
	  p1 = (*a)[i-1];

	  if (i == 1)
	    {
	      p0 = (*a)[i-1];
	      p0.value[0] = (2 * p1.value[0]) - p2.value[0];
	      p0.value[1] = (2 * p1.value[1]) - p2.value[1];
	    }
	  else
	    p0 = (*a)[i-2];

	  if ((i+1) >= a->size())
	    {
	      p3 = p2;
	      p3.value[0] = (2 * p2.value[0]) - p1.value[0];
	      p3.value[1] = (2 * p2.value[1]) - p1.value[1];
	    }
	  else 
	    p3 = (*a)[i+1];

	  kb_tangents (p0, p1, p2, p3, ax, ay, bx, by);
	    {
	      time_t = (t - p1.value[0]) / 
		(p2.value[0] - p1.value[0]);
	      
	      // printf ("time_t = %g\n", time_t);
	      // print_point (p0);
	      // print_point (p1);
	      // print_point (p2);
	      // print_point (p3);
	      
	      double t3 = time_t*time_t*time_t;
	      double t2 = time_t*time_t;
	      double q1 = (2*t3 - 3*t2 + 1);
	      double q2 = (-2*t3 + 3*t2);
	      double q3 = (t3 - 2*t2 + time_t);
	      double q4 = (t3 - t2);
	      // double dx = q1*x + q2*x3    + q3*X1 + q4*X2;
	      double dy = (q1*p1.value[1]) + (q2*p2.value[1]) + q3*ay + q4*by;
	      return dy;
	    }
	}
      if ((i+1) == a->size())
	return (*a)[i].value[1];
    }
	return 0.0;
}

double 
EW_Object::get_coeff (double t, long c)
{
  double r = 0;
  //printf("IN = %ld %ld %ld\n", p[0], p[1], p[2]);
  Coefficient *a = coeff.search (c);
  if (a)
    r = calculate_coeff (a, t);
  else
    printf("BAD ERROR\n");
  //printf("OUT = %g %g %g\n", r[0], r[1], r[2]);
  return r;
}

Vec3 
EW_Object::get_coeff (double t, Point p)
{
  Vec3 r;
  r[0] = get_coeff (t, p[0]);
  r[1] = get_coeff (t, p[1]);
  r[2] = get_coeff (t, p[2]);
  return r;
}

void 
EW_Object::set_coeff (double t, long c, double d)
{
  Coefficient *a = coeff.search (c);
  // printf ("searching for %d (t=%g).. found %d.\n", (int)c, t, (int)a);
  if (a)
    {
      if (a->size() == 0)
	{
	  Coefficient_Value k;
	  k.value[0] = t;
	  k.value[1] = d;
	  k.tension = 0;
	  k.continuity = 0;
	  k.bias = 0;
	  a->push_back (k);
	  return;
	}
      for (unsigned long i=0; i<a->size(); i++)
	{
	  if ((*a)[i].value[0] == t)
	    {
	      (*a)[i].value[1] = d;
	      break;
	    }
	  if ((*a)[i].value[0] > t)
	    {
	      Coefficient_Value k;
	      k.value[0] = t;
	      k.value[1] = d;
	      k.tension = 0;
	      k.continuity = 0;
	      k.bias = 0;
	      a->insert (i, k);
	      break;
	    }
	  if ((i+1) == a->size())
	    {
	      Coefficient_Value k;
	      k.value[0] = t;
	      k.value[1] = d;
	      k.tension = 0;
	      k.continuity = 0;
	      k.bias = 0;
	      a->push_back (k);
	    }
	}
    }  
}

void EW_Object::set_coeff (double t, Point p, Vec3 v)
{
  set_coeff (t, p[0], v[0]);
  set_coeff (t, p[1], v[1]);
  set_coeff (t, p[2], v[2]);
}

void EW_Object::read_coeff (EIFF*)
{
}

void EW_Object::write_coeff (EIFF*)
{
}

void EW_Object::read_point (EIFF*)
{
}

void EW_Object::write_point (EIFF*)
{
}

void EW_Object::read_attribute (EIFF*)
{
}

void EW_Object::write_attribute (EIFF*)
{
}

