#ifndef _INCLUDE_EW_OBJECT_H_
#define _INCLUDE_EW_OBJECT_H_

#include <stdio.h>

//#include <libguile.h>
//#include <guile/gh.h>
//#include <gFL/gfl.h>

#include <FL/Fl.H>
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Pack.H>
#include <FL/glut.H>

#include <ewave/kernel/base.h>
#include <ewave/kernel/iavltree.h>
#include <ewave/geometry/math.h>
#include <ewave/geometry/coefficients.h>
#include <ewave/geometry/mesh.h>
#include <ewave/materials/material.h>
#include <ewave/kernel/ew_kernel.h>
#include <ewave/FakeGuile.h>

typedef bool EW_bool;
typedef int EW_int;
typedef int EW_int_array[3];

struct EW_Point
{	float x;
	float y;
	float z;
	EW_Point(float x,float y,float z)
	:	x(x),y(y),z(z)
	{}
	EW_Point(EW_Point& pt)
	{	*this=pt;
	}
};

struct EW_Vec3 
{	EW_Point pt;
	double jtime;
	EW_Vec3(EW_Point& pt,double jtime) 
	:	pt(pt),jtime(jtime)
	{}
};

//extern Fl_ToggleTree * scene_graph;
extern double global_time;

class Fl_Pixmap;

enum { 
  TS_null,      // No current tool state.
  TS_pick,      // "Point picking" state
  TS_zoom,      // View Zoom
  TS_move,      // View move
  TS_translate, // group/object/point translation
  TS_pick_bone, // bone picking state
  TS_weld,
  TS_add_object
    //, DTentity, DTgroup, DTpoint, DTpolygon, DTbezier, 
  //DTlight, DTcamera, DTcube, 
  //DTsphere, DTblobby, DTblobby_group, DTbone 
};

enum { PTperspective, PTorthographic, PTfisheye, PTultra_wide_angle, 
       PTomnimax, PTpanoramic, PTcylinder };
enum { PTfixedvc, PTfixedhc, PTfreevc, PTfreehc };
enum { Lpoint, Lspot, Lcylindrical, Larea };

// render visibility flags

typedef double Nurbs;

typedef IAVLTree<Point> Point_List;

class EW_Object : public UBase
{
 public:
//rsr added missing:
void pdb_register(char * n, long tag)
{	printf("NOT IMPLEMENTED: EW_Object::pdb_register\n");
}
void scm2obj(SCM a)
{	printf("NOT IMPLEMENTED: EW_Object::scm2obj\n");
}
void write_class(FILE *file, int ind)
{	printf("NOT IMPLEMENTED: EW_Object::write_class\n");
}
  // Constructor, Destructor, and Replicator
  EW_Object ();
  virtual ~EW_Object ();
  virtual EW_Object*   replicate ();
  static void          initialize ();

  // Class information
  static  EW_Class     klass;
  virtual EW_Class&    vklass () { return klass; }

  // Common EW_Object methods. (don't override)
  const char*          get_name () { return name; }
  void                 set_name (char *x);
  void                 set_default_name (const char *);
  short                get_locked () { return locked; }
  void                 set_locked (short x);

  Vec3                 get_center_relative ();
  Vec3                 get_center_absolute ();
  void                 set_center_relative (Vec3 v);
  void                 set_center_absolute (Vec3 v);

  Point                get_center_coeff () { return center; }
  void                 set_center_coeff (Point a) { center = a; }
  
  Vec3                 get_point_relative (Point);
  Vec3                 get_point_absolute (Point);
  void                 set_point_relative (Point, Vec3);
  void                 set_point_absolute (Point, Vec3);

  Point                lookup_point (int);
  short                uses_point (int);
  void                 set_parent (EW_Group *o) { parent = o; }
  EW_Group*            get_parent () { return parent; }
  short                remove ();


  // Object Inspector Interface
  virtual void         refresh () {}

  
  // Viewport Interface
  virtual short        select_point_down (Point, Vec3);
  virtual short        select_point_drag (Point, Vec3);
  virtual short        select_point_release (Point, Vec3);
  virtual short        add_point_down (Vec3);
  virtual short        add_point_drag (Vec3);
  virtual short        add_point_release (Vec3);

  
  // Common manipulators.
  virtual void         translate (Vec3);
  virtual void         scale (Vec3, Vec3);
  virtual void         rotate (Vec3, Vec3);

  
  // Native Read and Write
  virtual int          write (EIFF *);
  virtual int          read (EIFF *);
  static int           read (char *);
  
  // Geometry Export
  virtual short        provides_mesh () { return 0; }
  virtual short        provides_nurbs () { return 0; }
  virtual short        provides_implicit () { return 0; }
  virtual Mesh         evaluate_mesh () { return Mesh(); }
  virtual Nurbs        evaluate_nurbs () { return 0; }
  virtual double       evaluate_implicit (Vec3 x) { return 0; }
  
  // Geometry Import
  virtual short        accepts_mesh () { return 0; }
  virtual short        accepts_nurbs () { return 0; }
  virtual short        accepts_implicit () { return 0; }  
  
  // Attributes
  enum EW_Att { Att_undefined, Att_real, Att_string, Att_binary };
  virtual void         set_attribute_real (char *, double);
  virtual void         set_attribute_string (char *, char *);
  virtual void         set_attribute_binary (char *, void *, int size);
  virtual double       get_attribute_real (char *, double);
  virtual const char * get_attribute_string (char *, char *);
  virtual void *       get_attribute_binary (char *, void *, int size);
  virtual EW_Att       attribute_type (char *);
  void                 read_attribute (EIFF*);
  void                 write_attribute (EIFF*);
  
  // OpenGL Rendering Engine
  enum EW_Dmg { Dmg_none, Dmg_translate, Dmg_scale, Dmg_all };
  enum EW_Drw { Drw_points, Drw_polygons, Drw_object }; 
  virtual void         set_damage (EW_Dmg i) { damage = i; }
  virtual void         draw (int, EW_Drw);
  void                 draw_point (Point p, int render_mode);
  int                  visible (int i);
  void                 visible (int i, int j);
  void                 show ();
  void                 hide ();
  
  // Relationships
  virtual short        is_group ();
  short                is_superset_of (EW_Object *);
  short                is_subset_of (EW_Object *);
  short                is_child_of (EW_Object *);
  short                is_parent_of (EW_Object *);  
  EW_Object *          object_from_point (int);
  
  // Coefficients
  long                 new_coeff ();
  Point                new_coeff3 ();
  void                 delete_coeff (long);
  void                 delete_coeff (Point);
  double               get_coeff (double t, long c);
  double               get_coeff (long c) { return get_coeff (kernel->time, c); }
  Vec3                 get_coeff (double t, Point);
  Vec3                 get_coeff (Point z) { return get_coeff (kernel->time, z); }
  void                 set_coeff (double t, long c, double d);
  void                 set_coeff (long c, double d) { set_coeff (kernel->time, c, d); }
  void                 set_coeff (double t, Point p, Vec3 z);
  void                 set_coeff (Point y, Vec3 z) { set_coeff (kernel->time, y, z); }
  void                 read_coeff (EIFF*);
  void                 write_coeff (EIFF*);

  // Evaluates a value for AVL trees.
  static long          evaluator (EW_Object *);

 protected:

  // Points
  Point                new_point ();
  void                 read_point (EIFF*);
  void                 write_point (EIFF*);
  
  Coefficient_List     coeff;
  Coefficient_List     attribute;
  Point_List           point;
  Point                center;
  short                locked;
  int                  damage;
  static int           default_name_cnt;  
  
 private:

  char                 *name;
  EW_Group             *parent;
  short                ew_polygons_visible;
  short                ew_points_visible;
  short                ew_object_visible;
};

typedef IAVLTree<EW_Object> EW_Object_List;

class EW_ptr_array
{	int a_size;
public:
	EW_ptr_array()
	:	a_size(0)
	{}
	int size()
	{	return a_size;
	}
	void shrink(int n)
	{	//BUG: 
		return;
	}
};

class EW_ObjectRecord
{	  
public:
	const char* name;
	long tag;
	int create; ;
	Fl_Pixmap* pixmap;
	int property_window;

};

class EW_Polygon3
{public:
	int control_points[3];
};

#include <ewave/kernel/global.h>
#include <ewave/objects/ew_group.h>

#endif
