#include <unistd.h>
#include <ewave/objects/ew_object.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM smob)
{
  delete (EW_Object *) gh_cdr (smob);
  return (sizeof sizeof (EW_Object));
}

static int
Sprint (SCM smob, SCM port, scm_print_state *pstate)
{
  EW_Object *obj = (EW_Object *) gh_cdr (smob);
  scm_puts ("#<obj EW_Object ", port);
  scm_puts ((char *)obj->klass.name (), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("ewave-object", 
		   "The base class for all ewave objects.",
		   ew_object, ew_base, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_1_1
  ( ew_object_get_name,
   "ewave-object::get-name",
   "Gets the object's name.",
   obj,
   ew_object,
   name,
   str )
{
  GFL_ASSERT_1 (ew_object_get_name, obj);
  EW_Object * s = (EW_Object *) gh_cdr (obj);
  return gh_str02scm((char *)s->get_name ());
}

GFL_PROCEDURE_2_0
  ( ew_object_set_name,
   "ewave-object::set-name",
   "Sets the object's name.",
   obj, name,
   ew_object, str )
{
  GFL_ASSERT_2 (ew_object_set_name, obj, name);  
  EW_Object * s = (EW_Object *) gh_cdr (obj);
  char * tmp = gh_scm2newstr(gh_cadr(name), NULL);
  s->set_name (tmp);
  free (tmp);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( ew_object_get_type,
   "ewave-object::get-type",
   "Gets the object's type.",
   obj,
   ew_object,
   name,
   str )
{
  printf ("(ewave-object::get-type)\n");
  printf ("  object_id = %ld\n", SCM_CAR (obj) );
  GFL_ASSERT_1 (ew_object_get_type, obj);
  EW_Object * s = (EW_Object *) gh_cdr (obj);
  return gh_str02scm ((char *)s->vklass().name ());
}

GFL_PROCEDURE_1_1
  ( ew_object_get_locked,
   "ewave-object::get-locked",
   "Get the lock state.",
   obj,
   ew_object,
   locked,
   str )
{
  GFL_ASSERT_1 (ew_object_get_locked, obj);
  EW_Object * s = (EW_Object *) gh_cdr (obj);
  if (gh_int2scm (s->get_locked()))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
  ( ew_object_set_locked,
   "ewave-object::set-locked",
   "Set the lock state.",
   obj, locked,
   ew_object, int )
{
  GFL_ASSERT_2 (ew_object_set_locked, obj, locked);
  EW_Object * s = (EW_Object *) gh_cdr (obj);
  if (locked == SCM_BOOL_T)
    s->set_locked (1);
  else s->set_locked (0);
  return SCM_UNSPECIFIED;
}

void ew_object_scheme_init ()
{
  GFL_NEW_TYPE (ew_object);
  GFL_NEW_PROCEDURE (ew_object_get_type);
  GFL_NEW_PROCEDURE (ew_object_set_name);
  GFL_NEW_PROCEDURE (ew_object_get_name);
  GFL_NEW_PROCEDURE (ew_object_get_locked);
  GFL_NEW_PROCEDURE (ew_object_set_locked);  
}

