/* 
   Extreme Wave -- A 3d modelling and animation environment
   Copyright (C) 1999 James Dean Palmer
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Please report all bugs and problems to "james@tiger-marmalade.com".
*/
#include <unistd.h>
#include <ewave/objects/ew_sphere.h>
#include <ewave/xpm_icons/sphere.xpm>

EW_Class  EW_Sphere::klass;

EW_Sphere::EW_Sphere () 
{
  //mesh = new_sphere (3);
  mesh = new_quarter_sphere (3, 0, 0);
}

int
EW_Sphere::write (EIFF *eiff)
{
  EW_Generic::write (eiff);
  eiff_write_long (eiff, 1); // EW_Sphere version number

  if (&klass == &vklass())
    eiff_finish_chunk (eiff);
  return 0;
}

int
EW_Sphere::read (EIFF *eiff)
{
  EW_Generic::read (eiff);
  long version = eiff_read_long (eiff);

  if (version >= 1)
    {
      // .. Sphere specific stuff.
    }
  return 0;
}

void
EW_Sphere::refresh ()
{
  EW_Generic::refresh ();
}

EW_Object*
EW_Sphere::replicate ()
{
  /*
   EW_Sphere *rvalue = new EW_Sphere ();
   *rvalue = *this;
   rvalue->center = point_db.size ();
   point_db.append ();
   point_db.last () = point_db[center];
   return rvalue;
   */
   return 0;
}

void 
EW_Sphere::draw (int gl_mode, EW_Drw ew_mode)
{
  Vec3 center_t, scale_t, rotate_t;
  if (!visible (Drw_object)) return;
  EW_Object::draw (gl_mode, ew_mode);

  switch (ew_mode) {

  case Drw_points:
    if (visible (ew_mode))
      {
      }
    break;
  case Drw_polygons:
    center_t = get_coeff (center);
    scale_t = get_coeff (scale) * get_coeff (radius);
    rotate_t = get_coeff (rotate);
      {
	glPushMatrix();
	glTranslated (center_t[0], center_t[1], center_t[2]);
	glRotated (rotate_t[0], 1, 0, 0);
	glRotated (rotate_t[1], 0, 1, 0);
	glRotated (rotate_t[2], 0, 0, 1);
	glScaled (scale_t[0], scale_t[1], scale_t[2]);
	mesh->draw();
	glPopMatrix();
      }
    break;
  default:
    break;
  }
}

static EW_Object* 
EW_Sphere_alloc ()
{
  return ((EW_Object*)(new EW_Sphere()));
}

void
EW_Sphere::initialize ()
{
  klass.name ("EW_Sphere");
  klass.create (EW_Sphere_alloc);
  klass.pixmap (new Fl_Pixmap(sphere_xpm));
  klass.inspector (EW_Generic::klass.inspector ());
  kernel->register_object (&klass);
  extern void ew_sphere_scheme_init ();
  ew_sphere_scheme_init ();
}

Mesh *
EW_Sphere::new_sphere (int detail)
{
  double inc = 1. / detail;
  Mesh *mesh = new Mesh ();

  for (int n=0; n < (2 * detail); n++)
    {
      double z = cos ((((180. / (2. * detail)) * n) * DEGREES_TO_RADIANS));
      double zn = cos ((((180. / (2. * detail))* (n + 1.)) * DEGREES_TO_RADIANS));

      double r = cos (asin (z));
      double rn = cos (asin (zn));

      for (double theta = .0; theta < 360.; theta += (90. * inc))
	{
	  double rtheta = DEGREES_TO_RADIANS * theta;
	  double rinc = DEGREES_TO_RADIANS * 90. * inc;
	  Vec3 
	    a, e, // lower right
	    b, f, // upper right
	    c, g, // upper left
	    d, h; // lower left

	  a[2] = b[2] = z;
	  c[2] = d[2] = zn;

	  a[0] = r * cos (rtheta);
	  a[1] = r * sin (rtheta);
	  b[0] = r * cos (rtheta+rinc);
	  b[1] = r * sin (rtheta+rinc);

	  c[0] = rn * cos (rtheta+rinc);
	  c[1] = rn * sin (rtheta+rinc);
	  d[0] = rn * cos (rtheta);
	  d[1] = rn * sin (rtheta);

	  // Normals (e-h):
	  e=a;
	  f=b;
	  g=c;
	  h=d;
	  
	  mesh->add_face (c, b, a, d, g, f, e, h);
	}
    }
  return mesh;
}

Mesh *
EW_Sphere::new_quarter_sphere (int detail, int q0, int q1)  // q0 = 0..3 q1 = 0..1
{
  double inc = 1. / detail;
  Mesh *mesh = new Mesh ();

  for (int n=q1*detail; n < ((q1+1) * detail); n++)
    {
      printf ("N=%d\n",n);
      double z = cos((((180 / (2 * detail)) * n) * DEGREES_TO_RADIANS));
      double zn = cos((((180 / (2 * detail)) * (n + 1)) * DEGREES_TO_RADIANS));
      
      double r = cos (asin (z));
      double rn = cos (asin (zn));
      
      for (double theta = (90.*q0); (theta + .001) < (90.*(q0+1)); theta += (90.*inc)) // was <
	{
	  printf ("theta=%g + inc = %g < %g\n", theta, inc, (90.*(q0+1)));
	  double rtheta = DEGREES_TO_RADIANS * theta;
	  double rinc = DEGREES_TO_RADIANS * 90 * inc;
	  Vec3 
	    a, e, // lower right
	    b, f, // upper right
	    c, g, // upper left
	    d, h; // lower left

	  a[2] = b[2] = z;
	  c[2] = d[2] = zn;

	  a[0] = r * cos (rtheta);
	  a[1] = r * sin (rtheta);
	  b[0] = r * cos (rtheta+rinc);
	  b[1] = r * sin (rtheta+rinc);

	  c[0] = rn * cos (rtheta+rinc);
	  c[1] = rn * sin (rtheta+rinc);
	  d[0] = rn * cos (rtheta);
	  d[1] = rn * sin (rtheta);
	  
	  // Normals (e-h):
	  e=a;
	  f=b;
	  g=c;
	  h=d;
	  
	  mesh->add_face (c, b, a, d, g, f, e, h);
	  //sgCopyVec3(qsphere.append(), d); // lower left
	  //sgCopyVec3(qsphere.append(), a); // upper left
	  //sgCopyVec3(qsphere.append(), b); // upper right
	  //sgCopyVec3(qsphere.append(), c); // lower right

	}
    }
  return mesh;
}
