; This script is executed after the Guile core is booted.
; It needs to be divided up better..

(use-modules (utils goosh))

; A couple of functions to make writing file more convenient.
; If the first argument is a port, then the following arguments
; are written to that port.

(define (println . x)
  "Print the passed objects to the specified output-port."
  (define args x)
  (define fh (current-output-port))
  (if (output-port? (car x))
      (begin 
	(set! args (cdr x))
	(set! fh (car x))))
  (for-each 
   (lambda (a) (display a fh)) 
   args)
  (newline fh))

(define print 
  (lambda x 
    (define args x)
    (define fh (current-output-port))
    (if (output-port? (car x))
	(begin 
	  (set! args (cdr x))
	  (set! fh (car x))))
    (for-each 
     (lambda (a) (display a fh)) 
     args)))

(define vector->string 
  (lambda (p)
    (string-append 
     "<"
     (number->string (car p)) ", "
     (number->string (cadr p)) ", "
     (number->string (caddr p)) ">")))

(define point->string 
  (lambda (point) 
    (let ((p (point-db point)))
      (vector->string p))))

(define hash-mset!
  (lambda x
    (define args (cdr x))
    (for-each 
     (lambda (a) (hash-set! (car x) (car a) (cadr a)))
     args)))

(define dummy-callback
   (lambda ()
      (fltk::message "Sorry, this is just a dummy callback.") ))

(define ew-quit-callback
   (lambda ()
      (if (fltk-fl::ask "Are you sure you want to quit?")
	 (quit)) ))

;(println "Extreme Wave Version " (get-major-version) "." (get-minor-version))

; Preferences
(set-tool-delta 1.0)
(set-aspect-ratio 1.0)

; Define some menu callbacks

(define render_image 
  (lambda (w h) 
    (povray-export "/tmp/ewave_temp.pov")
    (run-concurrently "povray" "+i/tmp/ewave_temp.pov" 
		      (string-append "+w" (number->string w))
		      (string-append "+h" (number->string h))
		      "-P" "+v" "-d" "+O/tmp/ewave_temp.tga")))

(define render_thumbnail_callback (lambda () (render_image 320 240)))
(define render_fullscreen_callback (lambda () (render_image 640 480)))

(define render_view_callback
  (lambda ()
    (run-concurrently "xv" "/tmp/ewave_temp.tga")))

; prime-tile is the main fltk-tile.
; prime-window is the main window.

(define ew-active-vp-insert-left
   (lambda ()
     (define vv (ew-viewport-new 
       (fltk-widget::x prime-tile)
       (fltk-widget::y prime-tile)
       (/ (fltk-widget::w prime-tile) 2)
       (/ (fltk-widget::h prime-tile) 2)
       ortho_left))
     (fltk-group::end vv)
     (fltk-tile::insert-left prime-tile (ewave::get-active-viewport-window) vv) 
     (ew-viewport-show vv) ))

(define ew-active-vp-insert-right
   (lambda ()
     (define vv (ew-viewport-new 
       (fltk-widget::x prime-tile)
       (fltk-widget::y prime-tile)
       (/ (fltk-widget::w prime-tile) 2)
       (/ (fltk-widget::h prime-tile) 2)
       ortho_right))
     (fltk-group::end vv)
     (fltk-tile::insert-right prime-tile (ewave::get-active-viewport-window) vv) 
     (ew-viewport-show vv) ))

(define ew-active-vp-insert-down
   (lambda ()
     (define vv (ew-viewport-new 
       (fltk-widget::x prime-tile)
       (fltk-widget::y prime-tile)
       (/ (fltk-widget::w prime-tile) 2)
       (/ (fltk-widget::h prime-tile) 2)
       ortho_bottom))
     (fltk-group::end vv)
     (fltk-tile::insert-down prime-tile (ewave::get-active-viewport-window) vv) 
     (ew-viewport-show vv) ))

(define ew-active-vp-insert-up
   (lambda ()
     (define vv (ew-viewport-new 
       (fltk-widget::x prime-tile)
       (fltk-widget::y prime-tile)
       (/ (fltk-widget::w prime-tile) 2)
       (/ (fltk-widget::h prime-tile) 2)
       ortho_top))
     (fltk-group::end vv)
     (fltk-tile::insert-up prime-tile (ewave::get-active-viewport-window) vv) 
     (ew-viewport-show vv) ))

(define ew-active-vp-fill-left 
   (lambda () 
      (define goner (fltk-tile::fill-left prime-tile (ewave::get-active-viewport-window)))
      (if (not (boolean? goner)) (ew-viewport-hide goner) )))

(define ew-active-vp-fill-right 
   (lambda () 
      (define goner (fltk-tile::fill-right prime-tile (ewave::get-active-viewport-window)))
      (if (not (boolean? goner)) (ew-viewport-hide goner) )))

(define ew-active-vp-fill-up
  (lambda () 
     (define goner (fltk-tile::fill-up prime-tile (ewave::get-active-viewport-window)))
     (if (not (boolean? goner)) (ew-viewport-hide goner) )))

(define ew-active-vp-fill-down
  (lambda () 
     (define goner (fltk-tile::fill-down prime-tile (ewave::get-active-viewport-window)))
     (if (not (boolean? goner)) (ew-viewport-hide goner) )))
(define pick-callback (lambda () (set-tool tool_pick)))

(define vp-context-menu
   (lambda (menu)
      (fltk-menu::add menu "Free Pick" "" pick-callback 0)

      ; These should show up only if something is selected:
      (fltk-menu::add menu "Translate Selection" "" pick-callback 0)
      (fltk-menu::add menu "Scale Selection" "" pick-callback 0)
      (fltk-menu::add menu "Rotate Selection" "" pick-callback FL_MENU_DIVIDER)

      (fltk-menu::add menu "Add" "" #f FL_SUBMENU)
      (fltk-menu::add menu "Add/Implicit"  "" dummy-callback FL_SUBMENU)
      (fltk-menu::add menu "Add/Implicit/Capsul"  "" 
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Capsul")) 0)
      (fltk-menu::add menu "Add/Implicit/Cone"  ""
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Cone")) 0)
      (fltk-menu::add menu "Add/Implicit/Cube"  ""
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Cube")) 0)
      (fltk-menu::add menu "Add/Implicit/Cylinder"  ""
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Cylinder")) 0)
      (fltk-menu::add menu "Add/Implicit/Plane"  "" 
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Plane")) 0)
      (fltk-menu::add menu "Add/Implicit/Sphere"  "" 
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Sphere")) 0)

      (fltk-menu::add menu "Add/Optics"  "" dummy-callback FL_SUBMENU)
      (fltk-menu::add menu "Add/Optics/Camera"  "" 
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Camera")) 0)
      (fltk-menu::add menu "Add/Optics/Light"  "" 
	 (lambda ()
	    (set-tool tool_add)
	    (kernel::set-object "EW_Light")) 0)

      ;(fltk-menu::add menu "Add/NURBS"  "" dummy-callback FL_SUBMENU)
      (fltk-menu::add menu "Add/TriMesh"  "" dummy-callback FL_SUBMENU)
      (fltk-menu::add menu "Add/TriMesh/Sphere" "" dummy-callback 0)
      (fltk-menu::add menu "Add/TriMesh/Plane" "" dummy-callback 0)
      
      (fltk-menu::add menu "Viewport" "" #f FL_SUBMENU)
      (fltk-menu::add menu "Viewport/Pan"  "" (lambda () (set-tool tool_move)) 0)
      (fltk-menu::add menu "Viewport/Zoom"  "" (lambda () (set-tool tool_zoom)) FL_SUBMENU)
      (fltk-menu::add menu "Viewport/Zoom/Arbitrary" "" (lambda () (set-tool tool_zoom)) 0)
      (fltk-menu::add menu "Viewport/Zoom/Zoom In (X2)" "" (lambda () (zoom 2.0)) 0)
      (fltk-menu::add menu "Viewport/Zoom/Zoom Out (X2)" "" (lambda () (zoom 0.5)) 0)
      (fltk-menu::add menu "Viewport/Rotate"  "" (lambda () (set-tool tool_null)) FL_MENU_DIVIDER)
      (fltk-menu::add menu "Viewport/Expand" "" #f FL_SUBMENU)
      (fltk-menu::add menu "Viewport/Expand/Left"  "" ew-active-vp-fill-left 0)
      (fltk-menu::add menu "Viewport/Expand/Right" "" ew-active-vp-fill-right 0)
      (fltk-menu::add menu "Viewport/Expand/Up"    "" ew-active-vp-fill-up 0)
      (fltk-menu::add menu "Viewport/Expand/Down"  "" ew-active-vp-fill-down 0) 
      (fltk-menu::add menu "Viewport/Divide" "" #f FL_SUBMENU)
      (fltk-menu::add menu "Viewport/Divide/Horizontal" "" ew-active-vp-insert-right 0)
      (fltk-menu::add menu "Viewport/Divide/Vertical" "" ew-active-vp-insert-down 0)
      (fltk-menu::add menu "Viewport/Divide/Quadrants" "" ew-active-vp-insert-left 0)
      (fltk-menu::add menu "Viewport/Layout" "" #f FL_SUBMENU)
      (fltk-menu::add menu "Viewport/Layout/1 - Perspective" "" dummy-callback 0)
      (fltk-menu::add menu "Viewport/Layout/4 - Classical" "" dummy-callback FL_MENU_DIVIDER)
      (fltk-menu::add menu "Viewport/Layout/Save As.." "" #f 0)
      (fltk-menu::add menu "Viewport/GL Render" "" #f FL_SUBMENU)
      (fltk-menu::add menu "Viewport/GL Render/Wireframe" "" (lambda () (shade-wire)) 0)
      (fltk-menu::add menu "Viewport/GL Render/Flat Shaded" "" (lambda () (shade-flat)) 0)
      (fltk-menu::add menu "Viewport/GL Render/Smooth Shaded" "" (lambda () (shade-smooth)) 0)
      
      (fltk-menu::add menu "Render" "" #f FL_SUBMENU)
      (fltk-menu::add menu "Render/Thumbnail" "" render_thumbnail_callback 0)
      (fltk-menu::add menu "Render/Full Screen" "" render_fullscreen_callback 0)
      (fltk-menu::add menu "Render/Animation" "" #f 0)
      (fltk-menu::add menu "Render/View" "" render_view_callback 0)

      (fltk-menu::add menu "Help" "" #f FL_SUBMENU)

      ; This is the context help item:
      (fltk-menu::add menu "Help/Manual: Rotation Tool" "" dummy-callback FL_MENU_DIVIDER)
      (fltk-menu::add menu "Help/About.." "" dummy-callback 0)
      (fltk-menu::add menu "Help/Tutorial" "" dummy-callback 0)
      (fltk-menu::add menu "Help/Manual: TOC" "" dummy-callback 0)
      (fltk-menu::add menu "Help/Manual: Index" "" dummy-callback 0)
      
      (fltk-menu::add menu "Quit" "" ew-quit-callback 0) ))


(define v0 (ew-viewport-new 
           (fltk-widget::x prime-tile)
	   (fltk-widget::y prime-tile)
	   (/ (fltk-widget::w prime-tile) 2)
	   (/ (fltk-widget::h prime-tile) 2)
	   ortho_right))
(fltk-group::end v0)
(fltk-group::add prime-tile v0)

(define v1 (ew-viewport-new 
	   (fltk-widget::x prime-tile)
	   (+ (/ (fltk-widget::h prime-tile) 2) 
	      (fltk-widget::y prime-tile))
	   (/ (fltk-widget::w prime-tile) 2)
	   (/ (fltk-widget::h prime-tile) 2)
	   ortho_top))
(fltk-group::end v1)
(fltk-group::add prime-tile v1)

(define v2 (ew-viewport-new 
           (/ (fltk-widget::w prime-tile) 2)
	   (+ (/ (fltk-widget::h prime-tile) 2) 
	      (fltk-widget::y prime-tile))
	   (/ (fltk-widget::w prime-tile) 2)
	   (/ (fltk-widget::h prime-tile) 2)
	   perspective_arbitrary))
(fltk-group::end v2)
(fltk-group::add prime-tile v2)

(define v3 (ew-viewport-new 
	   (/ (fltk-widget::w prime-tile) 2)
	   (fltk-widget::y prime-tile)
	   (/ (fltk-widget::w prime-tile) 2)
	   (/ (fltk-widget::h prime-tile) 2)
	   ortho_front))
(fltk-group::end v3)
(fltk-group::add prime-tile v3)

(ew-viewport-show v0)
(ew-viewport-show v1)
(ew-viewport-show v2)
(ew-viewport-show v3)

(fltk-group::begin prime-window)
(define context-menu (fltk-menu-button::new 
			(fltk-widget::x prime-tile) 
			(fltk-widget::y prime-tile)
			(fltk-widget::w prime-tile)
			(fltk-widget::h prime-tile) "Context Menu"))
(fltk-widget::set-label-size context-menu 12)
(fltk-menu::set-text-size context-menu 12)
(fltk-menu-button::set-type context-menu POPUP3)
(vp-context-menu context-menu)
(fltk-group::end prime-window)

(define flip-x 
  (lambda (object)
    (if (is-group object)
	(begin
	  (define children (car (get-children object)))
	  (for-each (lambda (a) (flip-x a))
		    children)))
    (for-each (lambda (a)
		(define p (point-db a))
		(define q (cons (caddr p) '()))
		(set! q (cons (cadr p) q))
		(set! q (cons (* -1 (car p)) q))
		(set-point-db a q))
	      (get-attribute object "all-points"))))

(ewave::bind-key "ctrl-x ctrl-c" ew-quit-callback)
(ewave::bind-key "1" (lambda () (zoom 0.5)))
(ewave::bind-key "2" (lambda () (zoom 2.0)))
(ewave::bind-key "shift-1" (lambda () (zoom 0.75)))
(ewave::bind-key "shift-2" (lambda () (zoom 1.5)))
(ewave::bind-key "q" (lambda () (set-tool tool_pick)))
(ewave::bind-key "w" (lambda () (set-tool tool_zoom)))
(ewave::bind-key "e" (lambda () (set-tool tool_move)))
(ewave::bind-key "b" (lambda () (set-tool tool_weld)))
(ewave::bind-key "s" (lambda () (shade-toggle)))
(ewave::bind-key "meta-x" (lambda () (activate-command-line)))

