; ewave::bind-key is a wrapper for ewave::bind-key-raw that converts
; a keyboard shortcut description into a list of keyboard events 
; to trap.

(use-modules (ice-9 string-fun))
(use-modules (ice-9 regex))

; We need a standard string library!!! 
; See :  http://srfi.schemers.org/srfi-13/srfi-13.txt
;        http://sunland.gsfc.nasa.gov/info/scheme/

(define string-prefix?
  (lambda (a b)
    (let ((c (lambda () (string-match a b))))
      (if (c)
	  (if (= 0 (car (vector-ref (c) 1)))
	      #t 
	      #f)
	  #f))))

(define (ewave::bind-key a b)
  "Bind a scheme procedure to a keyboard shortcut."

  (define evaluate-key
    (lambda (x)
      (if (string-prefix? "shift" x) 
	  FL_SHIFT
	  (if (string-prefix? "ctrl" x) 
	      FL_CTRL
	      (if (string-prefix? "alt" x) 
		  FL_ALT
		  (if (string-prefix? "meta" x) 
		      FL_META
		      (char->integer (string-ref x 0))))))))

  (define evaluate-keys
    (lambda (x)
      (if (not (null? (cdr x)))
	  (+ (evaluate-keys (cdr x)) (evaluate-key (car x)))
	  (evaluate-key (car x)))))

  (define evaluate-combinations
    (lambda (x)
      (define evaluate-combination
	(lambda (x)
	  (evaluate-keys 
	   (separate-fields-discarding-char #\- x list))))
      (if (not (null? (cdr x)))
	  (cons (evaluate-combination (car x))
		(evaluate-combinations (cdr x)))
	  (cons (evaluate-combination (car x)) '()))))

  (ewave::bind-key-raw
   (evaluate-combinations 
    (separate-fields-discarding-char #\  a list)) b))
