; This script exports the current scene to a povray file.  Use
; "povray-export" to export the current frame, or use 
; "povray-export-anim" to export the entire animation.

; Povray uses a left handed coordinate system where as Extreme Wave 
; uses a right handed coordinate system.  This means all points
; must be multiplied by x=-1, and all rotations should be multiplied
; by z, y = -1

(println "povray.scm")

(define frame-offset 0)

(define LH-vector->string 
  (lambda (p)
    (string-append 
     "<"
     (number->string (car p)) ", "
     (number->string (cadr p)) ", "
     (number->string (* -1 (caddr p))) ">")))

(define LH-point->string 
  (lambda (point) 
    (println "point==" point)
    (let ((p (point-db point)))
      (LH-vector->string p))))

(define LH-rotation-vector->string 
  (lambda (p)
    (string-append 
     "<"
     (number->string (* -1 (car p))) ", "
     (number->string (* -1 (cadr p))) ", "
     (number->string (caddr p)) ">")))

(println "povray-export")

(define povray-export 
  (lambda (filename)
    (define fh0 (open-output-file (string-append filename ".ini")))
    (define fh (open-output-file filename)) 
    (println fh0 "Buffer_Output=true")
    (println fh0 "Input_File_Name=" filename)
    (println fh0 "Output_File_Name=" filename ".ppm")
    (println fh0 "Output_File_Type=P")
    (println fh0 "Height=200")
    (println fh0 "Width=320")
    (close-port fh0)
    (set! frame-offset 0)
    (println fh "// Povray code generated by Extreme Wave\n")
    (println "GOT HERE 1")
    (println (scene))
    (println "Type : " (ewave-object::get-type (scene)))
    (println "GOT HERE 2")
    (povray-export-object fh (scene))
    (close-port fh)))

(println "povray-export-anim")

(define povray-export-anim
  (lambda (filename)
    (define fps 10)
    (define fh0 (open-output-file (string-append filename ".ini")))
    (println fh0 "Initial_Clock=0")
    (println fh0 "Final_Clock=" (* (- (frame-count) 2) fps))
    (println fh0 "Initial_Frame=0.0")
    (println fh0 "Final_Frame=" (* (- (frame-count) 2) fps))
    (println fh0 "Buffer_Output=true")
    (println fh0 "Input_File_Name=" filename)
    (println fh0 "Output_File_Name=" filename ".ppm")
    (println fh0 "Output_File_Type=P")
    (println fh0 "Height=200")
    (println fh0 "Width=320")
    (close-port fh0)

    (define fh (open-output-file filename))
    (println fh "// Povray code generated by Extreme Wave\n")
    (define outer 
      (lambda (x)
	(if (< x (- (frame-count) 2))
	    (begin
	      (set-current-frame (+ x 1))
	      (define inner
		(lambda (y)
		  (if (< y fps)
		      (begin 
			(if (= (+ y x) 0)
			    (println fh "\n#if (( clock >= 0 )"
				     " & (clock <= 1))")
			    (println fh "\n#if (( clock > " 
				     (+ y (* fps x)) " )"
				     " & (clock <= " (+ y (* fps x) 1) "))"))
			(set! frame-offset y)
			(povray-export-object fh (scene))
			(println fh "\n#end")
			(inner (+ y 1))))))
	      (inner 0)
	      (outer (+ x 1))))))
    (outer 0)
    (close-port fh)))

;; povray-export support functions:
(println "povray-export-object")

(define povray-export-object
  (lambda (fh object)
    (define object-type #f)
    (println "Exporting Object of type : " (ewave-object::get-type object))
    (set! object-type (ewave-object::get-type object))
    (println "Object type is: " object-type)
    (if (string=? object-type "EW_Light")
	(povray-export-light fh object))
    (if (string=? object-type "EW_Camera")
	(povray-export-camera fh object))
    (if (string=? object-type "EW_Group")
	(povray-export-group fh object))
    (if (string=? object-type "EW_DifferenceGroup")
	(povray-export-difference-group fh object))
    (if (string=? object-type "EW_BlobbyGroup")
	(povray-export-blobby-group fh object))
    (if (string=? object-type "EW_Sphere")
	(povray-export-sphere fh object))
    (if (string=? object-type "EW_Cube")
	(povray-export-cube fh object))
    (if (string=? object-type "EW_BlobbySphere")
	(povray-export-blobby-sphere fh object))))

(define povray-export-group
  (lambda (fh object)
    (define children (ewave-group::get-children object))
    (define group_name (ewave-object::get-name object))
    (println fh "// start of group, \"" group_name "\"")
    (for-each
     (lambda (a) (povray-export-object fh a))
     children)
    (println fh "// end of group, \"" group_name "\"")
     ))
    
(define povray-export-blobby-group
  (lambda (fh object)
    (define children (get-children object))
    (println fh "blob { threshold " (car (get-attribute object "BlobbyGroup::get-threshold")))
    (for-each
     (lambda (a) (povray-export-object fh a))
     children)
    (println fh "}")))

(define povray-export-difference-group
  (lambda (fh object)
    (define children (get-children object))
    (println fh "difference { ")
    (for-each
     (lambda (a) (povray-export-object fh a))
     children)
    (println fh "}")))

(println "povray-export-sphere")

(define povray-export-sphere
  (lambda (fh object)
    (println fh "sphere { ") 
    (println fh "<0, 0, 0>, " (get-attribute object "ewave-sphere::get-radius"))
    (println fh "  scale " 
             (vector->string (get-attribute object "ewave-sphere::get-scale"))) 
    (println fh "  rotate " 
                (LH-rotation-vector->string (get-attribute object "ewave-sphere::get-rotate"))) 
    (println fh "  translate "
      (LH-point->string (car (get-attribute object "ewave-sphere::get-center"))))
    
    ; povray-bone-trick needs to be re-written in scheme at some point.
    (println fh (povray-bone-trick object frame-offset))

    (povray-export-material fh (car (get-attribute object "ewave-sphere::get-material"))) 
    (println fh "}")))

(define povray-export-blobby-sphere
  (lambda (fh object)
    (println fh "sphere { ") 
    (println fh "<0, 0, 0>, " (get-attribute object "EW_BlobbySphere::get-radius") ", "
      (car (get-attribute object "EW_BlobbySphere::get-strength")))
    (println fh "  scale " 
             (vector->string (get-attribute object "EW_BlobbySphere::get-scale"))) 
    (println fh "  rotate " 
             (LH-rotation-vector->string (get-attribute object "EW_BlobbySphere::get-rotate"))) 
;    (println fh "  translate " 
;             (LH-vector->string (get-attribute object "translate"))) 
    (println fh "  translate "
      (LH-point->string (car (get-attribute object "EW_BlobbySphere::get-center"))))

    ; povray-bone-trick needs to be re-written in scheme at some point.
    (println fh (povray-bone-trick object frame-offset))

    (povray-export-material fh (car (get-attribute object "EW_BlobbySphere::get-material"))) 
    (println fh "}"))) 

(define povray-export-cube
  (lambda (fh object)
    (println fh "box { ") 
    (define radius (car (get-attribute object "EW_Cube::get-radius")))
    (println fh "<" radius ", " radius ", " radius ">, ")
    (println fh "<" (* radius -1) ", " (* radius -1) ", " (* radius -1) "> ")
    (println fh "  scale " 
             (vector->string (get-attribute object "EW_Cube::get-scale"))) 
    (println fh "  rotate " 
                (LH-rotation-vector->string (get-attribute object "EW_Cube::get-rotate"))) 
;    (println fh "  translate " 
;             (LH-vector->string (get-attribute object "translate"))) 
    (println fh "  translate "
      (LH-point->string (car (get-attribute object "EW_Cube::get-center"))))
    (povray-export-material fh (car (get-attribute object "EW_Cube::get-material"))) 
    (println fh "}")))

(define povray-export-height-field
  (lambda (fh object)
    (println fh "height_field { ") 
    (define img-src (get-attribute object "EW_HeightField::get-image-src"))
    (define ext "ppm")
    (if (string-match img-src ".png$")
	(set! ext "png"))
    (if (string-match img-src ".gif$")
	(set! ext "gif"))
    (if (string-match img-src ".jpg$")
	(set! ext "jpg"))
    (if (string-match img-src ".tga$")
	(set! ext "tga"))
    (println ext " \"" img-src "\"")

    (define radius (get-attribute object "EW_HeightField::get-radius"))
    (define scale (get-attribute object "EW_HeightField::get-scale"))

    (println fh "  scale " 
	     (string-append 
	      "<"
	      (number->string (* radius (car scale))) ", "
	      (number->string (* radius (cadr scale))) ", "
	      (number->string (* radius (caddr scale))) ">"))

    (println fh "  rotate " 
                (LH-rotation-vector->string (get-attribute object "EW_HeightField::get-rotate"))) 
    (println fh "  translate "
      (LH-point->string (car (get-attribute object "EW_HeightField::get-center"))))
    (povray-export-material fh (car (get-attribute object "EW_HeightField::get-material"))) 
    (println fh "}")))


(define povray-export-camera
  (lambda (fh object)
    (println fh "camera { ") 
    (println fh "location " (LH-point->string (car (get-attribute object "EW_Camera::get-center"))))
    (println fh "look_at " (LH-point->string (car (get-attribute object "EW_Camera::get-look-at"))))
    (println fh "}")))

(define povray-export-light
  (lambda (fh object)
    (println fh "light_source { ") 
    (println fh (LH-point->string (car (get-attribute object "EW_Light::get-center"))))
    (println fh "color " (vector->string (get-attribute object "EW_Light::get-color")))
; more if spotlight
;    (println fh "look_at " (LH-point->string (car (get-attribute object "look-at"))))
    (println fh "}")))

(define povray-export-material
  (lambda (fh material)
    (println fh "  pigment { color rgb " 
             (vector->string (get-material-attribute material "diffuse_color"))
             " }")
    (if (not (string=? "none" (get-material-attribute material "normal_type")))
	(println fh "normal { " (get-material-attribute material "normal_type")
		 " }")
	(println fh "finish { " 
		 " diffuse "
		 (number->string (car (get-material-attribute material "finish_diffuse")))
		 " brilliance "
		 (number->string (car (get-material-attribute material "finish_brilliance")))
		 " ambient "
		 (number->string (car (get-material-attribute material "finish_ambient")))
		 " reflection "
		 (number->string (car (get-material-attribute material "finish_reflection")))
		 " phong "
		 (number->string (car (get-material-attribute material "finish_phong")))
		 " phong_size "
		 (number->string (car (get-material-attribute material "finish_phong_size")))
		 " refraction "
		 (number->string (car (get-material-attribute material "finish_refraction")))
		 " ior "
		 (number->string (car (get-material-attribute material "finish_ior")))
		 " } "))))
  

