// BufferBase.h 2/19/05
// Copyright 2005 Robin.Rowe@MovieEditor.com
// License OSI MIT

#ifndef BUFFER_BASE_H
#define BUFFER_BASE_H

#include <string.h>
#include "../io/IoFile.h"

namespace gel {

#ifdef _DEBUG
extern const char* buffer_h_stop;
extern const char* buffer_h_drop;
#endif

class BufferBase
{public:
	void Clone(Buffer<T>& buffer)
	{	const unsigned desired_size=buffer.GetSize();
		if(!Create(desired_size))
		{	return;
		}
		Copy(buffer.GetSafeString(),desired_size);
	}
	void Truncate();
	
/*	void Eat(Buffer<T>& b)
	{	Close();
		data=b.data;
		size=b.size;
		b.Reset();
	}*/
	bool Open(const T* string)
	{	if(!string)
		{	return false;
		}
		const unsigned len=strlen(string)+1;// null-terminated
		if(!Create(len))
		{	return false;
		}
		Copy(string);//,len);
		return true;
	}
	bool Open(unsigned size)	
;/*	{	if(data && this->size==size)
		{	return true;
		}
		Close();
#ifdef _DEBUG
		const size_t guard_size=sizeof(unsigned);
#else
		const size_t guard_size=0;
#endif
		data=new T[size+guard_size];
		if(!data)
		{	this->size=0;
			return false;
		}
		this->size=size;
		SetGuard();
		Truncate();
		return true;
	}
*/	void Close()
	{	DropGuard();
		delete[] data;
		data=0;
		size=0;
	}
	const T* GetSafeData() const
	{	return data? data:"";
	}
	T* GetUnsafeData()
	{	return data;
	}
	T* GetUnsafeEndOfBuffer() 
	{	return data+size;
	}
	void UnsafeDisableDelete()
	{	Reset();// deliberate memory leak, to be able to live beyond dtor
	}
	unsigned GetSize() const
	{	return size;
	}
	unsigned GetSize(const char* ptr) const
	{	const char* end_of_buffer=data+size;
		if(ptr<data || ptr>=end_of_buffer)
		{	return 0;
		}
		return end_of_buffer-ptr;
	}
	void Copy(const T* string,unsigned size)
	{	if(!data || GetSize()<size)
		{	return;
		}
		memcpy(data,string,size);
	}
	void Copy(const T* string)
	{	if(!string)
		{	return;
		}
		const unsigned size=strlen(string)+1;
		Copy(string,size);
	}
	void Append(const T* string)
	{	if(!string || !data)
		{	return;
		}
		const unsigned append_length=strlen(string);
		const unsigned existing_length=strlen(data);
		const unsigned size=append_length+existing_length+1;
		if(GetSize()<size)
		{	return;
		}
		memcpy(data+existing_length,string,append_length+1);
	}
	bool IsBlank() const
	{	return data? !data[0]:true;
	}
	void Zero()
	{	if(!size)
		{	return;
		}
		memset(data,0,size);
	}
protected:
	void Breakpoint()
	{
#ifdef _DEBUG
		__asm int 03h;// Breakpoint triggered by bad buffer!
#endif
	}
	void SetGuard()
	{
#ifdef _DEBUG	
		if(!data)
		{	return;
		}
		const char* buffer_h_stop="STOP";
		unsigned* tail=(unsigned*)GetUnsafeEndOfBuffer();
		*tail=*(const unsigned*)buffer_h_stop;
#endif
	}
	void DropGuard()
	{
#ifdef _DEBUG	
		if(!data)
		{	return;
		}
		const char* buffer_h_stop="STOP";
		const char* buffer_h_drop="DROP";
		unsigned* tail=(unsigned*)GetUnsafeEndOfBuffer();
		if(*tail!=*(const unsigned*)buffer_h_stop)
		{	Breakpoint();
		}
		*tail=*(const unsigned*)buffer_h_drop;
#endif
	}
};

//typedef Buffer<char> CharBuffer;
typedef Buffer<unsigned char> ByteBuffer;

}

template <class T>
gel::IoFile& Write(gel::IoFile& f,const gel::Buffer<T>& b)
{	f.Write((void*)b.GetSafeData(),b.GetSize());
	return f;
} 

#endif
