// Substring_t.h
// rsr 7/8/04

#ifndef SUBSTRING_H
#define SUBSTRING_H

#include <string.h>

namespace gel {
char* SkipWhitespaceReverse(char* string,char* end);
char* SkipWhitespace(char* string,char* end);
char* SkipDarkspace(char* string,char* end);
const char* SkipWhitespace(const char* string,const char* end);
const char* SkipDarkspace(const char* string,const char* end);

inline
unsigned GetLength(char* string)
{	return (unsigned) strlen(string);
}

template <class T>
struct Substring_t
{	T* data;
	unsigned length;
	Substring_t<T>()
	{	Reset();
	}
	void Reset()
	{	data=0;
		length=0;
	}
	Substring_t<T>(T* data,unsigned length)
	:	data(data),
		length(length)
	{}
	Substring_t<T>(const T* data)
	:	data((T*)data)
	{	length=data? strlen(data):0;
	}

	Substring_t<T>(T* data)
	{	Set(data);
	}
	Substring_t<T>(const Substring_t<T>& ss)
	:	data(ss.data),
		length(ss.length)
	{}
	Substring_t<T>& operator=(const Substring_t<T>& rh)
	{	data=rh.data;
		length=rh.length;
		return *this;
	}
	void Set(T* string)
	{	if(!string)
		{	Reset();
			return;
		}
		data=string;
		length=GetLength(string);
	}
	bool IsGood() const
	{	return 0!=data && 0!=length;
	}
	void EatWhitespace()
	{	T* end=data+length;
		T* ptr=gel::SkipWhitespace(data,end);
		data=ptr;
		length=end-data;
	}
	void EatWhitespaceReverse()
	{	T* end=data+length;
		end=gel::SkipWhitespaceReverse(data,end);
		length=end-data;
	}
	void EatDarkspace()
	{	T* end=data+length;
		T* ptr=gel::SkipDarkspace(data,end);
		data=ptr;
		length=end-data;
	}
	Substring_t<T> EatWord()
	{	EatWhitespace();
		T* ptr=data;
		EatDarkspace();
		return Substring_t(ptr,data-ptr);
	}
	Substring_t<T> EatLine()
	{	T* newline=(T*)memchr(data,'\n',length);
		T* ptr=data;
		const unsigned len=newline-ptr+1;
		if(newline)
		{	data+=len;
			length-=len;
		}
		else
		{	data+=length;
			length=0;
		}
		return Substring_t<T>(ptr,len);
	}
	void Squeeze()
	{	EatWhitespace();
		EatWhitespaceReverse();
	}
/*	void Advance()
	{	data+=length;
		length=0;
	}*/
	T Peek(unsigned offset)
	{	return offset<length? data[offset]:0;
	}
	T* GetUnsafeEnd() const
	{	return data+length;
	}
//	T* GetAfter(T* end_of_data) const
//	{	return length? GetEnd()+1:0;
//	}
	// returns one by one strings separated by newlines, a destructive operation
/*	Substring_t<T> ExtractString()
	{	Substring_t<T> temp(0,0);
		if(!data || !length)
		{	return temp; // null substring
		}
		if(!newline)
		{	temp=*this;
			Reset();
			return temp; // last substring
		}
		temp.data=data;
		temp.length=newline-data-1;
		data=newline+1;
		length-=(temp.length+1);
		return temp;
	}*/
	void SetEnd(T* end)
	{	if(!end || end<data)
		{	return;
		}
		length=end-data;
	}
	bool IsEqual(const Substring_t<T>& ss) const
	{	if(length!=ss.length)
		{	return false;
		}
		enum {match=0};
		const int status=strncmp(data,ss.data,length);
		if(match==status)
		{	return true;
		}
		return false;
	}
	T* Find(T c)
	{	return data? (T*) memchr(data,c,length):0;
	}
//	const T* Find(T c) const
//	{	return data? (const T*) memchr(data,c,length):0;
//	}
	bool IsBlank() const
	{	return length==0;
	}
	bool operator!() const
	{	return IsBlank();
	}
};

} //namespace

typedef gel::Substring_t<char> Substring;
typedef gel::Substring_t<unsigned> Bytestring;

template <class T>
bool operator==(const gel::Substring_t<T>& lh,const gel::Substring_t<T>& rh)
{	return lh.IsEqual(rh);
}

template <class T>
bool operator==(const gel::Substring_t<T>& lh,const char* s)
{	const gel::Substring_t<T> rh(s);
	return lh.IsEqual(rh);
}

#	ifdef IO_FILE_H

template <class T>
gel::IoFile& Write(gel::IoFile& os,const gel::Substring_t<T>& ss)
{	if(ss.IsGood())
	{	const size_t bytes=fwrite(ss.data,sizeof(char),ss.length,os.file);
	}
	return os;
}

template <class T>
bool operator==(gel::Substring_t<T>& a,gel::Substring_t<T>& b)
{	return a.IsEqual(b);
}

#	endif

#endif
