// DllLoader.h 10/23/04 (total rewrite)
// Copyright 2004 Robin.Rowe@MovieEditor.com
// License OSI MIT

#ifndef DLL_LOADER_H
#define DLL_LOADER_H

#ifdef WIN32
#	include <windows.h>
#else
	typedef void* HMODULE
#endif

#include <dlfcn.h>
#include <cppstring.h>

class DllLoader
{	HMODULE handle;
	string lib_name;
	void Zero()
	{	handle = 0;
        lib_name.clear();
	}
public:
	~DllLoader(void)
	{	Close();
	}
	DllLoader() 
	{	Zero();
	}
	const string& GetLibName() const
	{	return lib_name;
	}
	void Close()
	{	if(handle)
		{	dlclose(handle);
		}
		Zero();
	}
	bool Open(const string& lib_name)
	{	if(IsOpen() && this->lib_name == lib_name)
		{	return true;
		}
		if(IsOpen())
		{	Close();
		}
		this->lib_name = lib_name;
//   const char* s="D:\\code.oss\\cinepaint-project\\img_img\\vcpp6\\Debug\\img_ppm.dll";
		const char* s=lib_name.c_str();
		handle = dlopen(s, RTLD_NOW);
		return IsOpen();
	}
	void* GetFunction(const char* func_name) const
	{	if(!handle)
		{	return 0;
		}
		return dlsym(handle,func_name);
	}
	const char* GetError(void) const
	{	return dlerror();
	}
	bool IsOpen() const
	{	return 0!=handle;
	}
};

/*  gcc -rdynamic -o foo foo.c -ldl   

       dlopen  loads  a  dynamic library from the file named by the null terminated string filename
       and returns an opaque "handle" for the dynamic library.  If filename is not an absolute path
       (i.e.,  it  does not begin with a "/"), then the file is searched for in the following loca
       tions:
 
              A colon-separated list of directories in the user's LD_LIBRARY_PATH environment vari
              able.
 
              The list of libraries cached in /etc/ld.so.cache.
 
              /usr/lib, followed by /lib.
 
       If filename is a NULL pointer, then the returned handle is for the main program.
*/

#endif
