/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is CinePaint, an image sequence manipulation program
 *                      ColorChangeListener - Color Change Listener interface
 *
 * The Initial Developer of the Original Code is
 * the University of Glasgow.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Donald MacVicar
 *   Colin Law
 *   Stuart Ford
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"),
 * in which case the provisions of the GPL are applicable instead of those
 * above. If you wish to allow use of your version of this file only under
 * the terms of the GPL, and not to allow others to use your version of this
 * file under the terms of the MPL, indicate your decision by deleting the
 * provisions above and replace them with the notice and other provisions
 * required by the GPL. If you do not delete the provisions above, a recipient
 * may use your version of this file under the terms of any one of the MPL,
 * or the GPL.
 *
 * ***** END LICENSE BLOCK *****
 *
 * $Id: ColorChangeListener.h,v 1.1 2006/01/04 20:18:33 robinrowe Exp $
 */

#ifndef _COLOR_CHANGE_LISTENER_H_
#define _COLOR_CHANGE_LISTENER_H_

#include <dll_api.h>

#include <plug-ins/Color.h>

/**
 * Purely Abstract class defining a ColorChange interface.
 * Classes interested in recieving color change notifications should implement this.
 *
 */
class CINEPAINT_COLOR_API ColorChangeListener
{
	public:
		virtual ~ColorChangeListener() {} ;

		/** standard color types */
		enum ColorType { NOTYPE_ENUM, FOREGROUND_ENUM, BACKGROUND_ENUM };

		/**
		 * Invoked to indicate that a Color change has taken place.
		 *
		 * @param color_type the color which has changed
		 * @param color the new color
		 */
		virtual void ColorChanged(ColorType color_type, const Color& color) = 0;

	protected:
		ColorChangeListener() {} ;

}; /* class ColorChangeListener */

#endif /* _COLOR_CHANGE_LISTENER_H_ */
