/* win32/unistd.h: replaces *nix header of same name
// Windows emulation of common *nix functions
// Copyright Nov 10, 2002, Robin.Rowe@MovieEditor.com
// License MIT (http://opensource.org/licenses/mit-license.php)
*/

// link with Ws2_32.lib! 

#ifndef WIN32_UNISTD_H
#define WIN32_UNISTD_H

#include <windows.h>
#include <winsock.h>
#include <fcntl.h>
#include <process.h>
#include <io.h>
#include <malloc.h>
#include <time.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <direct.h>
#include <sys/utime.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include <math.h>
#include "fix_vcast.h"

#ifdef Fl_H

#include <string.h>
inline 
int fl_strlen(const char* s)
{	return int (s ? strlen(s):0);
}

#define strlen fl_strlen

#endif

#define pow10(x) pow(x,10)
#define alloca _alloca

/* use with limits.h */
#define LONG_LONG_MAX LLONG_MAX     
#define LONG_LONG_MIN LLONG_MIN     
/* #define ULLONG_MAX */

#define snprintf _snprintf
/*#define snwprintf _snwprintf*/
#define vsnprintf _vsnprintf

#undef MAX_PRIORITY /* remove winspool.h warning */

#define strcasecmp _stricmp
#define strncasecmp _strnicmp

#define popen _popen
#define pclose _pclose

//#define send send2
typedef int ssize_t;

#define lstat stat
typedef unsigned short mode_t;
#define mkdir _mkdir
#define F_OK 0
#define X_OK 0
#define W_OK 2

#ifndef S_ISREG
#define S_ISREG(x) (_S_IFREG & x)
#endif

#define S_ISLNK(x) 0

#ifndef S_ISDIR
#define S_ISDIR(x) (_S_IFDIR & x)
#endif

#define S_IXUSR _S_IEXEC
#define S_IRUSR _S_IREAD
#define S_IWUSR _S_IWRITE
#define S_IXOTH 0
#define S_IXGRP 0

/*

From WIN32 sys/stat.h:

#define S_IFMT   _S_IFMT
#define S_IFDIR  _S_IFDIR
#define S_IFCHR  _S_IFCHR
#define S_IFREG  _S_IFREG
#define S_IREAD  _S_IREAD
#define S_IWRITE _S_IWRITE
#define S_IEXEC  _S_IEXEC

*/

/* strings.h */
int ffs(int i);

#define STDIN_FILENO fileno(stdin)

/*
SIGABRT Abnormal termination 
SIGFPE Floating-point error 
SIGILL Illegal instruction 
SIGINT CTRL+C signal 
SIGSEGV Illegal storage access 
SIGTERM Termination request 
*/

#define SIGALRM NSIG+1
#define SIGHUP NSIG+2
#define SIGQUIT NSIG+3
#define SIGBUS NSIG+4
#define SIGPIPE NSIG+5
#define SIGKILL SIGTERM
#define SIGCHLD NSIG+6
#define SIGUSR1 NSIG+7
#define SIGUSR2 NSIG+8

#define WNOHANG 0

/* SIGTERM */
/* SIGFPE */
#if 0
/* 7-1-06 rsr
None of these POSIX signal handlers are implemented by libunistd.
Included here only as potential future implementation effort */
typedef int sigset_t;
typedef int siginfo_t;
#define SIG_SETMASK 0
#define SIGTSTP 0
int sigemptyset(sigset_t *set);
int sigfillset(sigset_t *set);
int sigaddset(sigset_t *set, int signum);
int sigdelset(sigset_t *set, int signum);
int sigismember(const sigset_t *set, int signum);
int sigaction(int signum, const struct sigaction *act, struct sigaction *oldact);
int sigprocmask(int how, const sigset_t *set, sigset_t *oldset);
int sigpending(sigset_t *set);
int sigsuspend(const sigset_t *mask);
int sigaction(int signum, const struct sigaction *act, struct sigaction *oldact);
int sigprocmask(int how, const sigset_t *set, sigset_t *oldset);
int sigpending(sigset_t *set);
int sigsuspend(const sigset_t *mask);

struct sigaction 
{	void (*sa_handler)(int);
	void (*sa_sigaction)(int, siginfo_t *, void *);
	sigset_t sa_mask;
	int sa_flags;
	void (*sa_restorer)(void);
};
#endif

#define rint(x) floor ((x) + 0.5)

#define getpid _getpid
typedef int pid_t;
typedef int gid_t;
typedef int uid_t;

inline 
int setgid(gid_t gid)
{	return -1;
}

inline 
int setuid(uid_t uid)
{	return -1;
}

#define kill(pid,sig) ExitProcess(sig)

/* Above only kills my process. To do it right you'd need this:
inline 
BOOL TerminateProcess(
  HANDLE hProcess,
  UINT uExitCode
);
*/

#define lround floor
#define roundl floor

// Call this function before using Winsock in a console app!

inline
bool StartWinsock()
{	WORD version_requested=MAKEWORD(2,0);
	WSADATA data;
	return 0==WSAStartup(version_requested,&data);
}

#pragma warning( error : 4013)
#pragma warning( error : 4047) 

#endif
