/**
 * mlt.i - swig bindings for MLT
 * Copyright (C) 2004-2005 Charles Yates
 * Author: Charles Yates <charles.yates@pandora.be>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

%module(directors="1") mlt
%include "carrays.i"
%array_class(unsigned char, unsignedCharArray);

%{
#include <time.h>
#include <mlt/framework/mlt.h>

int factory_init( )
{
	return mlt_factory_init( NULL );
}

mlt_consumer factory_consumer( char *id, char *arg )
{
	return mlt_factory_consumer( id, arg );
}

int consumer_valid( mlt_consumer consumer )
{
	return consumer != NULL;
}

mlt_filter factory_filter( char *id, char *arg )
{
	return mlt_factory_filter( id, arg );
}

mlt_transition factory_transition( char *id, char *arg )
{
	return mlt_factory_transition( id, arg );
}

mlt_producer factory_producer( char *id, char *arg )
{
	return mlt_factory_producer( id, arg );
}

int producer_valid( mlt_producer producer )
{
	return producer != NULL;
}

void factory_close( )
{
	mlt_factory_close( );
}

int consumer_connect( mlt_consumer consumer, mlt_producer producer )
{
	mlt_service service = mlt_producer_service( producer );
	return mlt_consumer_connect( consumer, service );
}

%}

%include <framework/mlt_types.h>
%include <framework/mlt_properties.h>
%include <framework/mlt_service.h>
%include <framework/mlt_consumer.h>
%include <framework/mlt_producer.h>
%include <framework/mlt_filter.h>
%include <framework/mlt_transition.h>
%include <framework/mlt_factory.h>

extern int factory_init( );
extern mlt_consumer factory_consumer( char *id, char *arg );
extern int consumer_valid( mlt_consumer consumer );
extern mlt_filter factory_filter( char *id, char *arg );
extern mlt_transition factory_transition( char *id, char *arg );
extern mlt_producer factory_producer( char *id, char *arg );
extern int producer_valid( mlt_producer producer );
extern void factory_close( );
extern int consumer_connect( mlt_consumer consumer, mlt_producer producer );

%inline %{

typedef struct
{
#ifdef SWIG
	%immutable;
#endif
	int destruct;
	mlt_properties properties;
}
MltProperties;

typedef struct 
{
#ifdef SWIG
	%immutable;
#endif
	int destruct;
	mlt_producer producer;
	mlt_service service;
	MltProperties props;
}
MltProducer;

typedef struct 
{
#ifdef SWIG
	%immutable;
#endif
	mlt_filter filter;
	MltProperties props;
}
MltFilter;

typedef struct
{
#ifdef SWIG
	%immutable;
#endif
	int destruct;
	mlt_transition transition;
	MltProperties props;
}
MltTransition;

typedef struct 
{
#ifdef SWIG
	%immutable;
#endif
	mlt_consumer consumer;
	mlt_service service;
	mlt_producer dummy;
	MltProperties props;
}
MltConsumer;

typedef struct
{
#ifdef SWIG
	%immutable;
#endif
	int destruct;
	mlt_playlist playlist;
	MltProducer prod;
	MltProperties props;
}
MltPlaylist;

typedef struct
{
#ifdef SWIG
	%immutable;
#endif
	mlt_field field;
	MltProducer prod;
	MltProperties props;
}
MltJoin;

typedef struct
{
#ifdef SWIG
	%immutable;
#endif
	mlt_frame frame;
	MltProperties props;
}
MltFrame;

MltFrame *create_mlt_frame( MltProducer *producer, int index )
{
    mlt_frame frame;
	MltFrame *ptr = calloc( 1, sizeof( MltFrame ) );
	mlt_service_get_frame( producer->service, &frame, index);
	ptr->frame = frame;
	ptr->props.properties = mlt_frame_properties( ptr->frame );
	return ptr;
}

%}

%extend MltProperties
{
	MltProperties( char *file = NULL )
	{
		MltProperties *ptr = calloc( 1, sizeof( MltProperties ) );
		if ( ptr != NULL )
		{
			ptr->destruct = 1;
			if ( file != NULL )
				ptr->properties = mlt_properties_load( file );
			else
				ptr->properties = mlt_properties_new( );
		}
		return ptr;
	}

	~MltProperties( )
	{
		if ( self->destruct == 1 )
		{
			mlt_properties_close( self->properties );
			free( self );
		}
	}

	int set( char *name, char *value = NULL )
	{
		return mlt_properties_set( self->properties, name, value );
	}

	char *get( char *name )
	{
		return mlt_properties_get( self->properties, name );
	}

	int get_int( char *name )
	{
		return mlt_properties_get_int( self->properties, name );
	}

	int set_int( char *name, int value )
	{
		return mlt_properties_set_int( self->properties, name, value );
	}

	unsigned char *get_data( char *name )
	{
		return mlt_properties_get_data( self->properties, name, NULL );
	}

	int dump_data( char *name, int length )
	{
		uint8_t *data = mlt_properties_get_data( self->properties, name, NULL );
		return fwrite( data, length, 1, stdout );
	}

	int parse( char *namevalue )
	{
		return mlt_properties_parse( self->properties, namevalue );
	}

	int count( )
	{
		return mlt_properties_count( self->properties );
	}

	char *get_name( int index )
	{
		return mlt_properties_get_name( self->properties, index );
	}

	char *get_value( int index )
	{
		return mlt_properties_get_value( self->properties, index );
	}

	void dump( FILE *output = stderr )
	{
		mlt_properties_dump( self->properties, output );
	}

	int save( char *file )
	{
		int error = 0;
		FILE *f = fopen( file, "w" );
		if ( f != NULL )
		{
			mlt_properties_dump( self->properties, f );
			fclose( f );
		}
		else
		{
			error = 1;
		}
		return error;
	}

	void inherit( MltProperties those )
	{
		mlt_properties_inherit( self->properties, those.properties );
	}
};

%extend MltConsumer 
{
	MltConsumer( char *id, char *arg = NULL ) 
	{
		MltConsumer *ptr = calloc( 1, sizeof( MltConsumer ) );
		if ( arg != NULL )
		{
			ptr->consumer = mlt_factory_consumer( id, arg );
		}
		else
		{
			if ( strchr( id, ':' ) )
			{
				char *temp = strdup( id );
				char *arg = strchr( temp, ':' ) + 1;
				*( arg - 1 ) = '\0';
				ptr->consumer = mlt_factory_consumer( temp, arg );
				free( temp );
			}
			else
			{
				ptr->consumer = mlt_factory_consumer( id, NULL );
			}
		}

		if ( ptr->consumer != NULL )
		{
			ptr->service = mlt_consumer_service( ptr->consumer );
			ptr->props.properties = mlt_consumer_properties( ptr->consumer );
		}
		ptr->dummy = mlt_factory_producer( "colour", "black" );
		return ptr;
	}

	~MltConsumer( ) 
	{
		if ( self->consumer != NULL )
			mlt_consumer_close( self->consumer );
		mlt_producer_close( self->dummy );
		free( self );
	}

	void destroy( )
	{
		if ( self->consumer != NULL )
		{
			mlt_consumer_close( self->consumer );
			self->consumer = NULL;
		}
	}

	int is_valid( )
	{
		return self->consumer != NULL;
	}

	MltProperties properties( )
	{
		return self->props;
	}

	mlt_consumer get_consumer( )
	{
		return self->consumer;
	}

	int connect( MltProducer producer )
	{
		return mlt_service_connect_producer( self->service, producer.service, 0 );
	}

	int set( char *name, char *value = NULL )
	{
		return mlt_properties_set( mlt_consumer_properties( self->consumer ), name, value );
	}

	char *get( char *name )
	{
		return mlt_properties_get( mlt_consumer_properties( self->consumer ), name );
	}

	int start( )
	{
		return mlt_consumer_start( self->consumer );
	}

	int stop( )
	{
		return mlt_consumer_stop( self->consumer );
	}

	int is_stopped( )
	{
		return mlt_consumer_is_stopped( self->consumer );
	}

	int attach_filter( MltFilter filter )
	{
		mlt_service producer = mlt_service_producer( self->service );
		int error = 0;
		if ( producer != NULL )
			error = mlt_service_connect_producer( mlt_filter_service( filter.filter ), producer, 0 );
		if ( error == 0 )
		{
			mlt_service_connect_producer( self->service, mlt_filter_service( filter.filter ),  0 );
			self->service = mlt_filter_service( filter.filter );
		}
		return error;
	}

	int detach_filter( MltFilter filter )
	{
		mlt_service it = self->service;
		while ( it != NULL && it != mlt_filter_service( filter.filter ) )
			it = mlt_service_consumer( it );
		if ( it == mlt_filter_service( filter.filter ) )
		{
			mlt_service producer = mlt_service_producer( it );
			mlt_service consumer = mlt_service_consumer( it );
			mlt_service_connect_producer( consumer, producer,  0 );
			mlt_service_connect_producer( mlt_filter_service( filter.filter ), mlt_producer_service( self->dummy ), 0 );
			if ( self->service == mlt_filter_service( filter.filter ) )
				self->service = consumer;
		}
		return 0;
	}
};

%extend MltFilter 
{
	MltFilter( char *id, char *arg = NULL ) 
	{
		MltFilter *ptr = calloc( 1, sizeof( MltFilter ) );
		if ( arg != NULL )
		{
			ptr->filter = mlt_factory_filter( id, arg );
		}
		else
		{
			if ( strchr( id, ':' ) )
			{
				char *temp = strdup( id );
				char *arg = strchr( temp, ':' ) + 1;
				*( arg - 1 ) = '\0';
				ptr->filter = mlt_factory_filter( temp, arg );
				free( temp );
			}
			else
			{
				ptr->filter = mlt_factory_filter( id, NULL );
			}
		}
		if ( ptr != NULL && ptr->filter != NULL )
			ptr->props.properties = mlt_filter_properties( ptr->filter );
		return ptr;
	}

	~MltFilter( ) 
	{
		if ( self->filter != NULL )
			mlt_filter_close( self->filter );
		free( self );
	}

	void destroy( )
	{
		if ( self->filter != NULL )
		{
			mlt_filter_close( self->filter );
			self->filter = NULL;
		}
	}

	int is_valid( )
	{
		return self->filter != NULL;
	}

	MltProperties properties( )
	{
		return self->props;
	}

	int set( char *name, char *value = NULL )
	{
		return mlt_properties_set( mlt_filter_properties( self->filter ), name, value );
	}

	char *get( char *name )
	{
		return mlt_properties_get( mlt_filter_properties( self->filter ), name );
	}
};

%extend MltTransition 
{
	MltTransition( char *id, char *arg = NULL ) 
	{
		MltTransition *ptr = calloc( 1, sizeof( MltTransition ) );
		if ( arg != NULL )
		{
			ptr->transition = mlt_factory_transition( id, arg );
		}
		else
		{
			if ( strchr( id, ':' ) )
			{
				char *temp = strdup( id );
				char *arg = strchr( temp, ':' ) + 1;
				*( arg - 1 ) = '\0';
				ptr->transition = mlt_factory_transition( temp, arg );
				free( temp );
			}
			else
			{
				ptr->transition = mlt_factory_transition( id, NULL );
			}
		}
		if ( ptr != NULL && ptr->transition != NULL )
			ptr->props.properties = mlt_transition_properties( ptr->transition );
		return ptr;
	}

	~MltTransition( ) 
	{
		if ( self->transition != NULL )
			mlt_transition_close( self->transition );
		free( self );
	}

	void destroy( )
	{
		if ( self->transition != NULL )
		{
			mlt_transition_close( self->transition );
			self->transition = NULL;
		}
	}

	MltProperties properties( )
	{
		return self->props;
	}

	int is_valid( )
	{
		return self->transition != NULL;
	}

	int set( char *name, char *value = NULL )
	{
		return mlt_properties_set( mlt_transition_properties( self->transition ), name, value );
	}

	char *get( char *name )
	{
		return mlt_properties_get( mlt_transition_properties( self->transition ), name );
	}
};

%extend MltFrame
{
	MltFrame( MltProducer producer, int index ) 
	{
		return create_mlt_frame( &producer, index );
	}

	~MltFrame( ) 
	{
	    if ( self->frame != NULL )
		{
			mlt_frame_close( self->frame );
			free( self );
		}
	}

	void destroy( )
	{
		if ( self->frame != NULL )
		{
			mlt_frame_close( self->frame );
			self->frame = NULL;
		}
	}

	MltProperties properties( )
	{
		return self->props;
	}

	int is_valid( )
	{
		return self->frame != NULL;
	}
	
	int prepare_image( int width = 192, int height = 144 )
	{
	    uint8_t *buffer;
	    mlt_image_format format = mlt_image_yuv422;
	    return mlt_frame_get_image( self->frame, &buffer, &format, &width, &height, 0);
	}

	int fetch_image( int width = 192, int height = 144, mlt_image_format format = mlt_image_rgb24 )
	{
	    uint8_t *buffer;
		if ( mlt_properties_get_int( self->props.properties, "consumer_aspect_ratio" ) == 0 )
			mlt_properties_set_int( self->props.properties, "consumer_aspect_ratio", 1 );
	    return mlt_frame_get_image( self->frame, &buffer, &format, &width, &height, 0);
	}
};

%extend MltProducer 
{
	MltProducer( char *id, char *arg = NULL ) 
	{
		MltProducer *ptr = calloc( 1, sizeof( MltProducer ) );
		if ( arg != NULL )
			ptr->producer = mlt_factory_producer( id, arg );
		else
			ptr->producer = mlt_factory_producer( "fezzik", id );
		if ( ptr->producer != NULL )
		{
			ptr->destruct = 1;
			ptr->service = mlt_producer_service( ptr->producer );
			ptr->props.properties = mlt_producer_properties( ptr->producer );
		}
		return ptr;
	}

	~MltProducer( ) 
	{
		if ( self->destruct )
		{
			if ( self->producer != NULL )
			{
				mlt_producer_close( self->producer );
			}
			free( self );
		}
	}

	void destroy( )
	{
		if ( self->destruct && self->producer != NULL )
		{
			mlt_producer_close( self->producer );
			self->producer = NULL;
		}
	}

	MltProperties properties( )
	{
		return self->props;
	}

	int is_valid( )
	{
		return self->producer != NULL;
	}

	int seek( int position )
	{
		return mlt_producer_seek( self->producer, position );
	}

	int get_position( )
	{
		return mlt_producer_position( self->producer );
	}

	int get_in( )
	{
		return mlt_producer_get_in( self->producer );
	}

	int get_out( )
	{
		return mlt_producer_get_out( self->producer );
	}
	
	int get_length( )
	{
		return mlt_producer_get_length( self->producer );
	}
	
	int set_in_and_out( int in, int out)
	{
	    return mlt_producer_set_in_and_out( self->producer, in, out );
	}

	int set( char *name, char *value = NULL )
	{
		return mlt_properties_set( mlt_producer_properties( self->producer ), name, value );
	}

	char *get( char *name )
	{
		return mlt_properties_get( mlt_producer_properties( self->producer ), name );
	}

	int set_speed( int speed )
	{
		return mlt_producer_set_speed( self->producer, speed );
	}

	int get_speed( )
	{
		return mlt_producer_get_speed( self->producer );
	}

	int attach_filter( MltFilter filter )
	{
		mlt_service consumer = mlt_service_consumer( self->service );
		int error = mlt_filter_connect( filter.filter, self->service, 0 );
		if ( error == 0 )
		{
			self->service = mlt_filter_service( filter.filter );
			if ( consumer != NULL )
				mlt_service_connect_producer( consumer, self->service, 0 );
		}
		return error;
	}

	int detach_filter( MltFilter filter )
	{
		mlt_service it = self->service;
		while ( it != NULL && it != mlt_filter_service( filter.filter ) )
			it = mlt_service_consumer( it );
		if ( it == mlt_filter_service( filter.filter ) )
		{
			mlt_service producer = mlt_service_producer( it );
			mlt_service consumer = mlt_service_consumer( it );
			if ( consumer != NULL && producer != NULL )
				mlt_service_connect_producer( consumer, producer,  0 );
			if ( self->service == mlt_filter_service( filter.filter ) )
				self->service = producer;
		}
		return 0;
	}
	
	MltFrame get_frame( int index = 0 )
	{
		return *create_mlt_frame( self, index );
	}
};

%extend MltPlaylist
{
	MltPlaylist( ) 
	{
		MltPlaylist *ptr = calloc( 1, sizeof( MltPlaylist ) );
		if ( ptr != NULL )
			ptr->playlist = mlt_playlist_init( );
		if ( ptr->playlist != NULL )
		{
			ptr->destruct = 1;
			ptr->prod.producer = mlt_playlist_producer( ptr->playlist );
			ptr->prod.service = mlt_producer_service( ptr->prod.producer );
			ptr->props.properties = mlt_playlist_properties( ptr->playlist );
		}
		return ptr;
	}

	~MltPlaylist( ) 
	{
		if ( self->destruct )
		{
			if ( self->playlist != NULL )
				mlt_playlist_close( self->playlist );
			free( self );
		}
	}

	void destroy( )
	{
		if ( self->destruct && self->playlist != NULL )
		{
			mlt_playlist_close( self->playlist );
			self->playlist = NULL;
		}
	}

	int is_valid( )
	{
		return self->playlist != NULL;
	}

	MltProperties properties( )
	{
		return self->props;
	}

	MltProducer producer( )
	{
		return self->prod;
	}

	int count( )
	{
		return mlt_playlist_count( self->playlist );
	}

	int clear( )
	{
		return mlt_playlist_clear( self->playlist );
	}

	int append( MltProducer producer, int in = -1, int out = -1 )
	{
		return mlt_playlist_append_io( self->playlist, producer.producer, in, out );
	}

	int insert( MltProducer producer, int where, int in = -1, int out = -1 )
	{
		return mlt_playlist_insert( self->playlist, producer.producer, where, in, out );
	}

	int remove( int where )
	{
		return mlt_playlist_remove( self->playlist, where );
	}

	int move( int what, int where )
	{
		return mlt_playlist_move( self->playlist, what, where );
	}

	int resize( int clip, int in, int out )
	{
		return mlt_playlist_resize_clip( self->playlist, clip, in, out );
	}

	int blank( int length )
	{
		return mlt_playlist_blank( self->playlist, length );
	}
};

%extend MltJoin
{
	MltJoin( ) 
	{
		MltJoin *ptr = calloc( 1, sizeof( MltJoin ) );
		if ( ptr != NULL )
			ptr->field = mlt_field_init( );
		if ( ptr->field != NULL )
		{
			mlt_multitrack multitrack = mlt_field_multitrack( ptr->field );
    		mlt_tractor tractor = mlt_field_tractor( ptr->field );
    		mlt_properties props = mlt_tractor_properties( tractor );
    		mlt_properties_set_data( props, "multitrack", multitrack, 0, ( mlt_destructor )mlt_multitrack_close, NULL );
			ptr->prod.producer = mlt_tractor_producer( mlt_field_tractor( ptr->field ) );
			ptr->prod.props.properties = mlt_producer_properties( ptr->prod.producer );
			ptr->prod.service = mlt_producer_service( ptr->prod.producer );
			ptr->props.properties = mlt_field_properties( ptr->field );
		}
		return ptr;
	}

	~MltJoin( ) 
	{
		if ( self->field != NULL )
			mlt_field_close( self->field );
		free( self );
	}

	void destroy( )
	{
		if ( self->field != NULL )
		{
			mlt_field_close( self->field );
			self->field = NULL;
		}
	}

	MltProperties properties( )
	{
		return self->props;
	}

	int is_valid( )
	{
		return self->field != NULL;
	}

	int set_top( MltProducer producer )
	{
		return mlt_multitrack_connect( mlt_field_multitrack( self->field ), producer.producer, 0 );
	}

	int set_bottom( MltProducer producer )
	{
		return mlt_multitrack_connect( mlt_field_multitrack( self->field ), producer.producer, 1 );
	}

	int attach_transition( MltTransition transition, int a_track = 0, int b_track = 1 )
	{
		return mlt_field_plant_transition( self->field, transition.transition, a_track, b_track );
	}
};


