#!/bin/env perl

# Import required modules
use mlt;

# Start the mlt system
mlt::factory_init( );

# Check the arguments
if ( $#ARGV < 1 ) {
	print "Usage: thumbs.pl file name\n";
	exit;
}

# Get the arguments
$file = $ARGV[0];
$name = $ARGV[1];
$size = "192x144";
if ( $#ARGV == 2 ) {
	$size = $ARGV[2];
}

# Create the producer
$p = new mlt::MltProducer( $file );

# Check that the producer is valid
if ( $p->is_valid != 0 ) {

	# Construct the playlist
	$l = new mlt::MltPlaylist( );

	# Get the out point
	$o = $p->properties->get_int( "out" );

	# Create the playlist
	for ( $i = 0; $i < 5; $i ++ ) {
		$l->append( $p, $o * 0.25 * $i, $o * 0.25 * $i );
	}

	# Create the consumer which will generate the thumb nails
	$c = new mlt::MltConsumer( "avformat", $name . "%d.jpg" );
	$c->set( "real_time", "0" );
	$c->set( "progressive", "1" );
	$c->set( "size", $size );

	# Connect the playlist and start it
	$c->connect( $l->producer );
	$c->start;

	# Wait until the user stops the consumer
	while ( $c->is_stopped == 0 ) {
		sleep( 1 );
	}

	# Clean up
	$c->destroy;
	$l->destroy;
	$p->destroy;
}
else {
	print "Unable to open $ARGV[0]\n";
}

# Close the factory
#mlt::factory_close( );
