#!/bin/env ruby

# Required modules
require 'mlt'

# Create the mlt system
Mlt::factory_init

# Get and check the argument
file = ARGV.shift
name = ARGV.shift
size = ARGV.shift
size = "192x144" if size.nil?
raise "Usage: thumbs.rb file name [ size ]" if file.nil? || name.nil?

# Create the producer
producer = Mlt::MltProducer.new( file )
raise "Unable to load #{file}" if producer.is_valid == 0

# Construct the playlist
playlist = Mlt::MltPlaylist.new( )

# Get the out point
out = producer.properties.get_int( "out" );

# Calculate position of frames
[ 0, 0.25, 0.5, 0.75, 1 ].each { |x| playlist.append( producer, x*out, x*out ) }

# Create the thumb nail generator
generator = Mlt::MltConsumer.new( "avformat", "#{name}%d.jpg" )
generator.set( "real_time", "0" )
generator.set( "progressive", "1" )
generator.set( "size", size )

# Connect the consumer
generator.connect( playlist.producer );
generator.start

# Wait until we're done
while generator.is_stopped == 0 
	sleep 1
end

# Clean up
generator.destroy
playlist.destroy
producer.destroy

