#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>

#include "htp.h"

int print_help()
{	printf(PROGRAM_NAME " version " VERSION "\n");
	printf("Usage: " PROGRAM_NAME "[-anh] [-P proxy] [-U user] [-p pass]\n");
	printf("               <host> [<host> [<host> [...]]]\n");
	printf("   -a        Allow htpdate to slew the time instead of setting it.\n");
	printf("   -n        Do not actually set the clock, just print the new time.\n");
	printf("   -h        This help information.\n");
	printf("   -P proxy  Proxy to connect through.\n");
	printf("   -U user   Username for proxy authentication.\n");
	printf("   -p pass   Password for proxy authentication.\n");
	printf("   host      Host to query for time, in the format of hostname[:port]\n");
	exit(EXIT_SUCCESS);
}

unsigned int timeind = 0;
unsigned int totaltimeservers = 0;
struct timeserver_st timeservers[32];
char *host = NULL;
char* portstr = NULL;
int port = -1;
int proxy_port = -1;
int allowadj = 0;
int queryonly = 0;
char *proxy_host = NULL;
char *proxy_portstr = NULL;
char *proxy_username = NULL;
char *proxy_password = NULL;

void ProcessCmdLine(int argc,char** argv)
{	for(int i=1;i<argc;i++)
	{	if (i >= (sizeof(timeservers)/sizeof(timeservers[0]))) {
			break;
		}
		if('-'==argv[i][0])
		{	switch(argv[i][1])
			{case 'a':			/* adjust time */
				allowadj = 1;
				break;
			case 'n':			
				queryonly = 1;
				break;
			case 'h':			/* show help */
				print_help();
				exit(0);
			case 'P':
				proxy_host = argv[++i];
				proxy_portstr = strchr(proxy_host, ':');
				if ( proxy_portstr != NULL ) {
					proxy_portstr[0] = '\0';
					proxy_portstr++;
					if ( (proxy_port = atoi(proxy_portstr)) <= 0 ) {
						puts("Invalid port number" );
						exit(1);
					}
				}
				break;
			case 'p':
				proxy_password=argv[++i];
				break;
			case 'U':
				proxy_username=argv[++i];
				break;
			default:
				break;
			}
			continue;	
		}
		host = argv[i];
		portstr = strchr(host, ':');
		if (portstr != NULL) {
			/* Here, we modify argv.. good idea? */
			portstr[0] = '\0';
			portstr++;
			port = atoi(portstr);
			if (port <= 0) {
				fprintf(stderr, "Invalid port: %s.\n", portstr);
				continue;
			}
		} else {
			port = 80;
		}
		timeservers[timeind].host = host;
		timeservers[timeind].port = port;
		timeind++;
	}
}

int main(int argc, char *argv[]) {
	struct timeval *newtime;
	time_t temp;
	double delta = 0.0;
	char strtime[128];
	htp_init();
#ifdef HAVE_OPENLOG
	openlog("htpdate", LOG_PID, LOG_USER);
#endif
	/* Process each argument as a host:port pair. */
	ProcessCmdLine(argc,argv);
	totaltimeservers = timeind;
	if (totaltimeservers == 0) {
		fprintf(stderr, "No servers specified. Using google.com.\n");
		
		totaltimeservers=1;
		timeservers[0].host = "google.com";
		timeservers[0].port = 80;

	
//		return(EXIT_FAILURE);
	}

	if (proxy_host == NULL) {
		proxy_host = getenv("http_proxy");
		if (proxy_host == NULL) {
			proxy_host = getenv("HTTP_PROXY");
		}
	}

	if (proxy_host != NULL) {
		proxy_portstr = strrchr(proxy_host, ':');
		if (proxy_portstr != NULL) {
			*proxy_portstr = '\0';
			proxy_portstr++;
			proxy_port = atoi(proxy_portstr);
			if (proxy_port <= 0) {
				printf("Illegal proxy port: %s.\n", proxy_portstr);
				return(EXIT_FAILURE);
			}
		} else {
			proxy_port = 8080;
		}
	}

	newtime = htp_calctime(timeservers, totaltimeservers, proxy_host, proxy_port, proxy_username, proxy_password);

	if (newtime == NULL) {
		fprintf(stderr, "No servers could be reached.\n");
		return(EXIT_FAILURE);
	}

	if (!queryonly) {
		delta = set_clock(newtime, allowadj, 1);
	}
	temp=newtime->tv_sec;
	strftime(strtime, sizeof(strtime), "%a %b %d %H:%M:%S %Y", localtime(&temp));

	printf("%s [+ %f] (delta: %f)\n", strtime, ((double) newtime->tv_usec) / 1000000.0, delta);

	return(EXIT_SUCCESS);
}
