#ifndef Fl_Multi_Value_Slider_H
#define Fl_Multi_Value_Slider_H

#include <FL/Fl_Slider.H>

typedef struct {    // segment values, normalized to 0-1
   double left;      // if any of this values is <0 then handle inexistant
   double right;
   double middle;
} _Fl_Value_Segment;
#define FL_VALUESEG_INVALID -1

#define FL_MVS_MVALS_ON   0x01
#define FL_MVS_MVALS_OFF  0xFE
#define FL_MVS_LVALS_ON   0x02
#define FL_MVS_LVALS_OFF  0xFD
#define FL_MVS_RVALS_ON   0x04
#define FL_MVS_RVALS_OFF  0xFB
#define FL_MVS_LIMITS_ON (FL_MVS_LVALS_ON | FL_MVS_RVALS_ON)
#define FL_MVS_LIMITS_OFF  (FL_MVS_LVALS_ON | FL_MVS_RVALS_ON)
#define FL_MVS_VALS_ALL   (FL_MVS_LVALS_ON | FL_MVS_RVALS_ON | FL_MVS_MVALS_ON)

#define FL_MVS_SIZE_LOCK     0x01
#define FL_MVS_SIZE_UNLOCK   0xFE
#define FL_MVS_SIZE_SAME_ON  0x02
#define FL_MVS_SIZE_SAME_OFF 0xFD


class  Fl_Multi_Value_Slider : public Fl_Slider {
    _Fl_Value_Segment *segments;
    int nsegs;
    int currseg;
    char dragelem; //0 nop, 1 left, 2 rigth, 4 middle, 8 segment
    char mvals; // bit 0 mvals on/off, bit 1 lvals on/off, bit2 rvals on/off
    char flags; // bit 0 lock size on/off
    uchar textfont_, textsize_;
    unsigned textcolor_;
public:
    double normval(double v);
    int findseg(double v,char t=0);
    int splitseg(int iseg=FL_VALUESEG_INVALID);
    int removeseg(int iseg=FL_VALUESEG_INVALID);
    int setsegments(int n, double *slist);
    double * getsegments(int &n);
    int current_segment(int cseg=-1);
    int numsegs() { return nsegs; }

    void draw_peg(int X,int Y,int W,int H,double val, int t=0);
    void draw_bg(int X, int Y, int W, int H);
    void draw();

    int handle(int event, int X, int Y, int W, int H);
    int handle(int);
    int usevals(int m=-1);
    int locksizes(int l=-1); // lock current segments number, cant add/remove
                            // only allows to change left/middle/right vals
    int samesize(int l=-1); // if they arent of same size and not locked
                            // recalculate sizes, otherwise just return
                            //status (only if unlocked sizes!!!)
    void forcesamesize();
    Fl_Multi_Value_Slider(int x,int y,int w,int h, const char *l = 0);

    Fl_Font textfont() const {return (Fl_Font)textfont_;}
    void textfont(uchar s) {textfont_ = s;}
    uchar textsize() const {return textsize_;}
    void textsize(uchar s) {textsize_ = s;}
    Fl_Color textcolor() const {return (Fl_Color)textcolor_;}
    void textcolor(unsigned s) {textcolor_ = s;}
};

#endif
