#ifndef _Gradient_H_
#define _Gradient_H_
/* File format is either:
*   GIMP Gradient (1.2):
*      number_of_segments
*      left middle right r0 g0 b0 a0 r1 g1 b1 a1 type coloring
*      left middle right r0 g0 b0 a0 r1 g1 b1 a1 type coloring
*      ...
The format seems to be
start middle end [range 0...1]
R G B A left endpoint
R G B A right endpoint
type where type is one of linear (0), curved, sine,
sphere_increasing and shere_decreasing
coloring, one of rgb, hsv_ccw, hsv_cw

The type & colouring are accessible to the gradient editor (the
user, I mean) in the "blending function for segment" and "Colouring 
type for segment" menus of the gradient editor menu.

This information is available in app/gradient.c and
app/gradient_header.h.


*   Vu Gradient:
*Scale11: MaPalette
*Scale11_1:     DarkGoldenrod
*Scale11_2:     SlateBlue
*Scale11_3:     yellow
*Scale11_4:     tan
*Scale11_4:     #ccffcc
*Scale11_5:     SandyBrown
*      
*/

#define GRAD_NOP    0
#define GRAD_GIMP   1
#define GRAD_VU     2
#define GRAD_AUTO   0

#define GRAD_LEFT   1
#define GRAD_RIGHT  2
#define GRAD_MIDDLE 4

typedef struct _grad_segment_t {
        double       left, middle, right; // Left pos, midpoint, right pos
        double       r0, g0, b0, a0;    // Left color 
        double       r1, g1, b1, a1;    // Right color 
        int          type;              // Segment's blending function 
        int          color;             // Segment's coloring type 
        char         flags;             // same color as left,rigth
                                        // bit 0 link left, bit 1 link right
} grad_segment_t;


class Gradient {
    grad_segment_t *segs;
    int nsegs;
   protected:
    int load_gimp(const char *grname);
    int load_vu(const char *grname);
    int save_gimp(const char *grname);
    int save_vu(const char *grname);

   public:
    float Tmin,Tmax;
    float Tdist;
    char *gradfilename;
    char *gradname;
    int grad_type;

    Gradient(int g_type=GRAD_GIMP);

    int load(const char *grname,int g_type=GRAD_GIMP);
    int save(const char *grname=NULL,int g_type=GRAD_NOP);

    void setTminmax(float T0,float T1);

    int remove_segment(int iseg, int action=-1);
    int split_segment(int iseg, float splitpoint=-1);

    int numsegs() { return nsegs; }
    grad_segment_t * getseg(int i);
    double * getsegments(int &n);
    int updatesegments(int n, double *slist);

    int setColor(int iseg, int side, float r,float g, float b,float a,
                char fl=-1); 
    int getColor(float T,float &r,float &g,float &b, float &a);

    void make_default();
    void clean_gradient();
    void setdefaults();

    ~Gradient();
};

#endif
