#include <FL/Fl.H>
#include <FL/Fl_Single_Window.H>
#include <FL/fl_draw.H>
#include "Fl_Color_Menu.H"

#define BOXSIZE 14
#define BORDER 4
#define XDIVS  16
#define YDIVS  16

Fl_Color_Menu::Fl_Color_Menu(unsigned oldcol) :
  Fl_Group(0,0,BOXSIZE*8+1+2*BORDER, BOXSIZE*32+1+2*BORDER) {
  initial = which = (Fl_Color)oldcol;
  boxsizex=BOXSIZE;
  boxsizey=BOXSIZE;
  border=BORDER;
}

Fl_Color_Menu::Fl_Color_Menu(int x, int y, int w, int h, char const *t) 
: Fl_Group(x, y, w, h, t) {
  int dd=(w>h)?(Fl::box_dw(box())+Fl::box_dx(box())):
               (Fl::box_dh(box())+Fl::box_dy(box()));
  Fl_Group::color(FL_GRAY);
  border=(dd>BORDER)?dd:BORDER;
  boxsizex=(w-Fl::box_dw(box()))/XDIVS;
  boxsizey=(h-Fl::box_dh(box()))/YDIVS;
}

void Fl_Color_Menu::drawbox(unsigned c) {
  if (c < 0 || c > 255) return;
  int X = (c%XDIVS)*boxsizex+border+x();
  int Y = (c/XDIVS)*boxsizey+border+y();
#if BORDER_WIDTH < 3
  if ((Fl_Color)c == which) fl_draw_box(FL_DOWN_BOX, X+1, Y+1, boxsizex-1, boxsizey-1,
                 (Fl_Color)c);
  else fl_draw_box(FL_BORDER_BOX, X, Y, boxsizex+1, boxsizey+1, (Fl_Color)c);
#else
  fl_draw_box((Fl_Color)c == which ? FL_DOWN_BOX : FL_BORDER_BOX,
	      X, Y, boxsizex+1, boxsizey+1, (Fl_Color)c);
#endif
}

void Fl_Color_Menu::draw() {
  boxsizex=(w()-Fl::box_dw(box()))/XDIVS;
  boxsizey=(h()-Fl::box_dh(box()))/YDIVS;
  if (damage() != FL_DAMAGE_CHILD) {
    fl_draw_box(box(),x(),y(),w(),h(),color());
    for (int c = 0; c < 256; c++) drawbox((Fl_Color)c);
  } else {
    drawbox(previous);
    drawbox(which);
  }
  previous = which;
}

int Fl_Color_Menu::handle(int e) {
  int c = which;
  switch (e) {
  case FL_PUSH:
  case FL_DRAG: {
    int X = (Fl::event_x() - x() - border);
    if (X >= 0) X = X/boxsizex;
    int Y = (Fl::event_y() - y() - border);
    if (Y >= 0) Y = Y/boxsizey;
    if (X >= 0 && X < XDIVS && Y >= 0 && Y < YDIVS)
      c = XDIVS*Y + X;
    else {
      c = which;
     }
    } 
    if (when() & FL_WHEN_CHANGED) do_callback();
    break;
  case FL_RELEASE:
    done = 1;
    if (when() & FL_WHEN_RELEASE) do_callback();
    return 1;
  case FL_KEYBOARD:
    switch (Fl::event_key()) {
    case FL_Up: if (c > 15) c -= XDIVS; break;
    case FL_Down: if (c < 256-16) c += XDIVS; break;
    case FL_Left: if (c > 0) c--; break;
    case FL_Right: if (c < 255) c++; break;
    case FL_Escape: which = initial; done = 1; return 1;
    case FL_Enter: done = 1;     
                   if (when() & FL_WHEN_ENTER_KEY) do_callback();
                   return 1;
    default: return 0;
    }
    break;
  default:
    return 0;
  }
  if (c != which) {
    which = (Fl_Color)c; damage(FL_DAMAGE_CHILD);
    do_callback();
  }
  return 1;
}

Fl_Color Fl_Color_Menu::color() {
  return which;
}

static unsigned fl_cmap[256] = {
#include "fl_cmap.h" // this is a file produced by "cmap.cxx":
};

Fl_Color Fl_Color_Menu::color(unsigned char r, unsigned char g, unsigned char b) {
  Fl_Color col=fl_rgb_color(r,g,b);
  int numcolors=256;
 
  int mindist = 0x7FFFFFFF;
  unsigned int bestmatch = 0;
  for (unsigned int n = numcolors; n--;) {
    unsigned a = fl_cmap[n];
    int d, t;
    t = int(r)-int( (a>>24) & 0x0FF); d = t*t;
    t = int(g)-int( (a>>16) & 0x0FF); d += t*t;
    t = int(b)-int( (a>>8)  & 0x0FF); d += t*t;
    if (d <= mindist) {bestmatch = n; mindist = d;}
  }
  col =(Fl_Color)bestmatch;
  color(col);
  return which;
}

void Fl_Color_Menu::color(unsigned col) {
  Fl_Group::color((Fl_Color)col);
  previous=initial = which = (Fl_Color)col;
  damage(FL_DAMAGE_CHILD);
  redraw();
}
