#include <stdio.h>
#include <stdlib.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include "gradient.h"
#include "Fl_Gradient_Box.H" 

Fl_Gradient_Box::Fl_Gradient_Box(int X, int Y, int W, int H, const char*l): Fl_Widget(X,Y,W,H,l) 
{
 px=0;
 cgrad=NULL;
}

Gradient* Fl_Gradient_Box::grad(Gradient *g)
{
 if(g!=NULL)
   cgrad=g;
 return cgrad;
}

double * Fl_Gradient_Box::getsegments(int &n)
{
 if(cgrad)
   return cgrad->getsegments(n);
 return NULL;
}

void Fl_Gradient_Box::generate_gradimage(void* wbox, int X, int Y, int W, uchar* buf) {
  Fl_Gradient_Box* v = (Fl_Gradient_Box*)wbox;
  Gradient *gr=v->grad();
  float r,g,b,a,aa,rr,gg,bb;
  if(gr) {
   for (int x = X; x < X+W; x++) {
     gr->getColor((float)x,r,g,b,a); 
     if ((x&0x07)==4 || (Y&0x07)==4) {
       rr=0.8; gg=0.4; bb=0.4;
     } else {
       rr=0.8; gg=0.8; bb=0.8;
     }
     aa=1.0-a;
     *buf++ = uchar((r*a+aa*rr)*255); 
     *buf++ = uchar((g*a+aa*gg)*255); 
     *buf++ = uchar((b*a+aa*bb)*255); 
   }
 }
}

void Fl_Gradient_Box::draw()
{
 int sxx = x(), syy = y(), sww = w(), shh = h();
 int X=sxx+Fl::box_dx(box());
 int Y=syy+Fl::box_dy(box());
 int W=sww-Fl::box_dw(box());
 int H=shh-Fl::box_dh(box());
 
 if (damage()& FL_DAMAGE_ALL ) draw_box();
 fl_clip(X,Y,W,H);
 if(grad()) {
   cgrad->setTminmax(X,X+W);
   fl_draw_image(generate_gradimage, this, X,Y,W,H);
 } else {
  if(active_r()){
    fl_color(selection_color());  
  } else {
    fl_color(selection_color() | 8);
  }  
  W/=4; H/=4; X+=W*2; Y+=H; 
  fl_loop(X,Y, X+W*2, Y+H , X,Y+H*2, X-W*2, Y+H );
 }
 fl_pop_clip();
}

int Fl_Gradient_Box::handle(int event, int X, int Y, int W, int H)
{
 switch (event) {
    case FL_PUSH:
    case FL_DRAG: 
      px=Fl::event_x()-X-Fl::box_dx(box());
      if(when() & FL_WHEN_CHANGED) do_callback();
      return 1;
    case FL_RELEASE: 
      px=Fl::event_x()-X-Fl::box_dx(box());
      if(when() & FL_WHEN_RELEASE) do_callback();
      return 1;
  default:
      break;
  }
  return 0;
}

int Fl_Gradient_Box::handle(int event) 
{
  return handle(event,
		x()+Fl::box_dx(box()),
		y()+Fl::box_dy(box()),
		w()-Fl::box_dw(box()),
		h()-Fl::box_dh(box()));
}
