#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "fl_named_rgb.H"
#ifndef FALSE 
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE  1
#endif

//--------------------------------------------------------------------------

#define MAX_RGB 3000
/* the RGB data file does not have a standard location on unix. */

#ifdef __VMS
  static const char *rgbfile = "SYS$MANAGER:DECW$RGB.DAT";
#else
#ifdef __EMX__   /* OS2 */
#include <X11/XlibInt.h>
  static const char *rgbfile = "/XFree86/lib/X11/rgb.txt";
#else
#ifdef __FreeBSD__
  static const char *rgbfile = "/usr/X11R6/lib/X11/rgb.txt";
#else
   static const char *rgbfile = "/usr/lib/X11/rgb.txt";
#endif
#endif
#endif



static RGBdb rgbdb[MAX_RGB];
static int rgbdbsize=0;

static int read_entry(FILE * fp, int *r, int *g, int *b, char *name)
{
    int  n;
    char buf[512], *p;

    if (!fgets(buf, sizeof(buf) - 1, fp))
      return 0;

    if(buf[0] == '!')
      fgets(buf,sizeof(buf)-1,fp);

    if(sscanf(buf, " %d %d %d %n", r, g, b, &n) < 3)
	return 0;

    p = buf + n;

    /* squeeze out all spaces */
    while (*p)
    {
	if (*p != ' ' && *p != '\n')
	    *name++ = *p;
	p++;
    }
    *name = 0;

    return (feof(fp) || ferror(fp)) ? 0 : 1;
}

static int load_rgbdatabase(char *fname)
{
    FILE *fp;
    RGBdb *db = rgbdb, *dbs = db + MAX_RGB;
    int r, g, b,  lr  = -1 , lg = -1, lb = -1;
    char name[256];
#ifdef __EMX__
    if (!(fp = fopen(__XOS2RedirRoot(fname), "r")))
#else
    if (!(fp = fopen(fname, "r")))
#endif
    {
	printf("Load %s Can't Open", fname);
	return 0;
    }

    /* read the items */

    for (; db < dbs && read_entry(fp, &r, &g, &b, name);)
    {
	db->r = r;
	db->g = g;
	db->b = b;
        db->name=strdup(name);
	/* unique the entries on the fly NOT!!*/
//	if (lr != r || lg != g || lb != b)
	{
	    db++;
	    lr = r;
	    lg = g;
	    lb = b;
            rgbdbsize++;
	}
    }
    fclose(fp);

    if (db < dbs)
	db->r = 1000;		/* sentinel */
    else
    {
	db--;
	db->r = 1000;
    }

    return 1;
}

static int
search_entry(int r, int g, int b)
{
    register RGBdb *db = rgbdb;
    int i, j, diffr, diffg, diffb;
    unsigned int diff, mindiff;

    mindiff = ~0;
    for (i = j = 0; db->r < 256; db++, i++)
    {
       diffr = r - db->r;
       diffg = g - db->g;
       diffb = b - db->b;

#ifdef FL_LINEAR
	diff = unsigned(3.0 * (FL_abs(r - db->r)) +
			(5.9 * FL_abs(g - db->g)) +
			(1.1 * (FL_abs(b - db->b)));
#else
        diff = unsigned(3.0 * (diffr *diffr) +
			5.9 * (diffg *diffg) +
			1.1 * (diffb *diffb));
#endif

	if (mindiff > diff)
	{
	    mindiff = diff;
	    j = i;
	}
    }

    return j;
}

// get rgb color from unix rgb or hex web style rgb partly based on fltk sample 
// colbrowser..
static int loadedRGBDatabase=0;

int getRgbfromString(const char *rgbname, double &r,double &g, double &b)
{
 int i;
 char tmpclr[3];
 if(!loadedRGBDatabase) {
   if(!load_rgbdatabase((char*)rgbfile))
     return FALSE;
   loadedRGBDatabase=1;     
 }
 if(rgbname[0]=='\0')
   return FALSE;
 if(rgbname[0]=='#') {
   tmpclr[2]='\0';
   i=1;
   tmpclr[0]=rgbname[i++];
   tmpclr[1]=rgbname[i++];
   r=(int)strtol(tmpclr,NULL,16)/255.0;
   tmpclr[0]=rgbname[i++];
   tmpclr[1]=rgbname[i++];
   g=(int)strtol(tmpclr,NULL,16)/255.0;
   tmpclr[0]=rgbname[i++];
   tmpclr[1]=rgbname[i++];
   b=(int)strtol(tmpclr,NULL,16)/255.0;
 } else {
   for(i=0;i<rgbdbsize && strcmp(rgbname,rgbdb[i].name)!=0; i++);
   if(i==rgbdbsize)
     return FALSE;
   r=rgbdb[i].r/255.0;
   g=rgbdb[i].g/255.0;
   b=rgbdb[i].b/255.0; 
 }
 
 return TRUE;
}

int getRgbEntry(int r,int g, int b)
{
 if(!loadedRGBDatabase) {
   if(!load_rgbdatabase((char *)rgbfile))
     return -1;
   loadedRGBDatabase=1;     
 }
 return search_entry(r,g,b);
}

char *getStringfromRgb(const char *rgbname, double r,double g, double b)
{
 int i;
 if(!loadedRGBDatabase) {
   if(!load_rgbdatabase((char *)rgbfile))
     return NULL;
   loadedRGBDatabase=1;     
 }
 i=search_entry((int)(r*255),(int)(g*255),(int)(b*255));
 return rgbdb[i].name;
}

int RgbDatabaseSize()
{
 if(!loadedRGBDatabase) {
   if(!load_rgbdatabase((char*)rgbfile))
     return -1;
   loadedRGBDatabase=1;     
 }
 return rgbdbsize;
}
void getRgbData(int i,char *&name, int &r, int &g, int &b)
{
 if(!loadedRGBDatabase) {
   if(!load_rgbdatabase((char*)rgbfile))
     return;
   loadedRGBDatabase=1;     
 }
 if(i>=rgbdbsize || i<0 )
   return;
 name=rgbdb[i].name;
 r=rgbdb[i].r;
 g=rgbdb[i].g;
 b=rgbdb[i].b;
}

RGBdb *getRgbData(int i)
{
 if(!loadedRGBDatabase) {
   if(!load_rgbdatabase((char*)rgbfile))
     return NULL;
   loadedRGBDatabase=1;     
 }
 if(i>=rgbdbsize || i<0 )
   return NULL;
 return &(rgbdb[i]);
}

//------------------------------------------------------------------------
