#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "fl_named_rgb.H"
#include "gradient.h" 
#ifndef FALSE 
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE  1
#endif

#define D_FCLAMP(x,a,b)  (((x)<(a))?(a):(((x)>(b))?(b):(x)))

Gradient::Gradient(int g_type) 
{ 
 grad_type=g_type; 
 Tmin=0; 
 Tmax=0; 
 Tdist=0; 
 segs=NULL;
 nsegs=0;
 gradfilename=strdup("null"); 
 gradname=strdup("null"); 
 make_default(); 
}

Gradient::~Gradient()
{
  if(nsegs!=0 && segs!=NULL)
    free(segs);
}

void Gradient::setdefaults() 
{
 gradfilename=strdup("null"); 
 gradname=strdup("null"); 
 make_default(); 
}

void Gradient::clean_gradient()
{
  if(nsegs!=0 && segs!=NULL) {
    free(segs);
    nsegs=0;
    segs=NULL;
  }
} 

void Gradient::setTminmax(float T0,float T1) 
{ 
 Tmin=T0; 
 Tmax=T1; 
 Tdist=1.0/(Tmax-Tmin);
}

void Gradient::make_default()
{
 clean_gradient();
 segs=(grad_segment_t*)malloc(sizeof(grad_segment_t));
 nsegs=1;
 segs[0].left=0;
 segs[0].middle=0.5;
 segs[0].right=1.0;
 segs[0].r0=1.0;
 segs[0].g0=0.0;
 segs[0].b0=0.0;
 segs[0].a0=0.5;
 segs[0].r1=1.0;
 segs[0].g1=1.0;
 segs[0].b1=0.0;
 segs[0].a1=1.0;
 segs[0].type=0;
 segs[0].flags=0;
}

int Gradient::load(const char *grname,int g_type)
{
 FILE *fp;
 char tmpbuf[1024];
 if(grname==NULL) {
     return FALSE;
 }

 if(g_type==GRAD_AUTO) {
   if((fp=fopen(grname, "rb"))==NULL) 
     return FALSE;    
   if(fgets(tmpbuf, 1024, fp)==NULL)
     return FALSE;
   if(strncmp(tmpbuf, "GIMP Gradient\n", strlen("GIMP Gradient\n"))== 0) {
      g_type=GRAD_GIMP;
   } else if(strncmp(tmpbuf,"*Scale",strlen("*Scale")) ==0) {
      g_type=GRAD_VU;
   }
   fclose(fp);
 }
 switch (g_type) {
    case GRAD_GIMP: 
             return load_gimp(grname);
             break;
    case GRAD_VU:
             return load_vu(grname);
             break;
    default:
         break;
 }
 return FALSE;
}

int Gradient::save(const char *grname,int g_type)
{
 char *grfilename;
 if(grname!=NULL) {
   grfilename=(char *)grname;
 } else {
   grfilename=gradfilename;
 }
 if(g_type==GRAD_NOP) 
   g_type=grad_type;

 switch (g_type) {
    case GRAD_GIMP: 
             return save_gimp(grfilename);
             break;
    case GRAD_VU:
             return save_vu(grfilename);
             break;
    default:
         break;
 }
 return FALSE;
}

int Gradient::load_gimp(const char *grname)
{
 FILE *fp;
 int numseg,i;
 grad_segment_t *seg;
 char gimpline[1024];
 char gname[1024];
 if(grname==NULL) {
   return FALSE;
 }
 if(grname[0]=='\0') {
   if(segs!=NULL) free(segs);
    make_default();
   return TRUE;
 }
 if((fp=fopen(grname, "rb"))==NULL) {
      printf("\nError: gradient file %s doesn't exist using the old one.\n",grname);
   return FALSE;    
 }

 for(i=strlen(grname); i>=0 && (grname[i]!='/' && grname[i]!='\\'); i--);
 i+=(i<int(strlen(grname)-1))?0:1;
 strcpy(gname,&(grname[i+1]));
 
 if(fgets(gimpline, 1024, fp)==NULL || strcmp(gimpline, "GIMP Gradient\n") != 0) {
    printf("Error: %s is not a GIMP gradient file using the old one.\n",grname);
    fclose(fp);
    return FALSE;    
 }
 numseg=0;
 if(fgets(gimpline, 1024, fp)==NULL || strcmp(gimpline, "Name:\n") !=0) {
   if((numseg=atoi(gimpline))<=0) {
      printf("Error: %s doesn't have a gradient segment using the old one.\n",grname);
      fclose(fp);
      return FALSE;
   }
 } 
 if(numseg<=0){
   strcpy(gname,gimpline+strlen("Name:")); 
   fscanf(fp,"%i\n",&numseg);
   if(numseg<=0) {
      printf("Error: %s doesn't have a gradient segment using the old one.\n",grname);
      fclose(fp);
      return FALSE;
   }
 }
 clean_gradient();
 nsegs=numseg;
 segs=(grad_segment_t*)malloc(sizeof(grad_segment_t)*nsegs);
 int resp;
 for(numseg=0, seg=segs ; numseg<nsegs; numseg++, seg++) {
    resp=fscanf(fp, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d %d\n",
			&(seg->left), &(seg->middle), &(seg->right),
			&(seg->r0), &(seg->g0), &(seg->b0), &(seg->a0),
			&(seg->r1), &(seg->g1), &(seg->b1), &(seg->a1),
			&(seg->type), &(seg->color));
    seg->flags=0;
 }
 fclose(fp);
 free(gradfilename);
 free(gradname);
 grad_type=GRAD_GIMP;
 gradfilename=strdup(grname);
 gradname=strdup(gname);
 return TRUE;
}

int Gradient::load_vu(const char *grname)
{
 FILE *fp;
 int numseg;
 grad_segment_t *seg;
 char vuline[1024];
 int baseoffset;
 char *gname,tbuf[64],*cname;
 double r,g,b;

 baseoffset=strlen("*Scale");

 if(grname==NULL) {
   return FALSE;
 }
 if(grname[0]=='\0') {
   if(segs!=NULL) free(segs);
    make_default();
   return TRUE;
 }
 if((fp=fopen(grname, "rb"))==NULL) {
      printf("Error: gradient file %s doesn't exist using the old one.",grname);
   return FALSE;    
 }
 
  fgets(vuline, 1024, fp);
  if(strncmp(vuline,"*Scale",baseoffset) !=0 ) {
    printf("Error: %s is not a VU gradient file using the old one.",grname);
    fclose(fp);
    return FALSE;    
 }
 if((gname=strstr(vuline,":"))==NULL)  {
    printf("Error: %s is not a VU gradient file using the old one.",grname);
    fclose(fp);
    return FALSE;    
 }
 do {
   gname++;
 } while(gname[0]!='\0' && (gname[0]==' ' || gname[0]=='\t'));

 if(gname[0]!='\0')
  strcpy(tbuf,gname);
 else 
   return FALSE;
 if(tbuf[0]=='\0') {
    fclose(fp);
   return FALSE;
 }
 free(gradname);
 if(strlen(tbuf)>=1)
   tbuf[strlen(tbuf)-1]='\0';
 gradname=strdup(tbuf);
 
 clean_gradient();

 numseg=0;
 // calculate number of segments from file
 while(fgets(vuline,1024,fp) && strncmp(vuline,"*Scale",baseoffset)==0 ) {
    numseg++;
 }
 fclose(fp);
 if(numseg<=0) {
   printf("Error: Vu gradient file contains no segments\n");
   return FALSE;
 }
 fp=fopen(grname, "rb");

 fgets(vuline,1024,fp); // eat gradient name

 nsegs=numseg;
 segs=(grad_segment_t*)malloc(sizeof(grad_segment_t)*nsegs);
 for(numseg=0, seg=segs ; numseg<nsegs; numseg++, seg++) {
      // in a vu file segment distribution is uniform..
      seg->left=double(numseg)/double(nsegs-1);
      seg->middle=(double(numseg)+0.5)/double(nsegs-1); 
      seg->right=(double(numseg)+1.0)/double(nsegs-1);
      // if we get to this point it's guaranteed that the line exist on the file
      fgets(vuline,1024,fp);  
      if((cname=strstr(vuline,":"))==NULL) {
          fclose(fp);
          return FALSE;
      }
      do {
        cname++;
      } while(cname[0]!='\0' && (cname[0]==' ' || cname[0]=='\t'));
      if(cname[0]!='\0')
         strcpy(tbuf,cname);
      else 
        return FALSE;
      if(strlen(tbuf)>=1)
        tbuf[strlen(tbuf)-1]='\0';
      getRgbfromString(tbuf,r,g,b);
      seg->r0=r; seg->g0=r; seg->b0=b;
      seg->flags=GRAD_LEFT | GRAD_RIGHT;
      seg->a0=1.0;
      seg->type=0;
      seg->color=0; 
 } 
 for(numseg=0, seg=segs ; numseg<nsegs-1; numseg++, seg++) {
     seg->r1=seg[1].r0;
     seg->g1=seg[1].g0;
     seg->b1=seg[1].b0;
     seg->a1=seg[1].a0;
 }
 nsegs--;
 segs=(grad_segment_t*)realloc(segs,sizeof(grad_segment_t)*nsegs);

 fclose(fp);
 free(gradfilename);
 grad_type=GRAD_VU;
 gradfilename=strdup(grname);
 return TRUE;
}

int Gradient::save_gimp(const char *grname)
{
 FILE *fp;
 int numseg;
 grad_segment_t *seg;
 
 if(grname==NULL) {
   return FALSE;
 }
 if(grname[0]=='\0') {
   return FALSE;
 }
 if((fp=fopen(grname, "wb"))==NULL) {
      printf("Error: can't create gradient file %s.",grname);
   return FALSE;    
 }
 fprintf(fp,"GIMP Gradient\n");
 fprintf(fp,"%d\n",nsegs);
 for(numseg=0,seg=segs; numseg<nsegs; numseg++,seg++) {
   fprintf(fp,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d %d\n",
			(seg->left), (seg->middle), (seg->right),
			(seg->r0), (seg->g0), (seg->b0), (seg->a0),
			(seg->r1), (seg->g1), (seg->b1), (seg->a1),
			(seg->type), (seg->color));
 }

 fclose(fp);
 return TRUE;
}

int Gradient::save_vu(const char *grname) 
{ 
 FILE *fp; 
 int numseg;
 grad_segment_t *seg;
 char tmpbuf[64];
 
 if(grname==NULL) {
   return FALSE;
 }
 if(grname[0]=='\0') {
   return FALSE;
 }
 if((fp=fopen(grname, "wb"))==NULL) {
      printf("Error: can't create gradient file %s.",grname);
   return FALSE;     
 }
 fprintf(fp,"*Scale11:  %s\n",gradname);
 for(numseg=0,seg=segs; numseg<nsegs; numseg++,seg++) {
   fprintf(fp,"*Scale11_%d:  %s\n",numseg+1,
			getStringfromRgb(tmpbuf,seg->r0, seg->g0, seg->b0));
 }
 if(nsegs>0 && numseg>0) {
    seg--;
    fprintf(fp,"*Scale11_%d:  %s\n",numseg+1,
			getStringfromRgb(tmpbuf,seg->r1, seg->g1, seg->b1));
 }

 fclose(fp);
 return TRUE;
}
  
int Gradient::remove_segment(int iseg, int action)
{
 int i;
 double mid;

 if(iseg>=nsegs || iseg<0 )
   return -1;

 if(nsegs<=1)
   return 1;

 if(iseg >0 && iseg<nsegs-1)
   mid=(segs[iseg].left+segs[iseg].right)*0.5;
 else if(iseg<=0)
   mid=0.0;
 else 
   mid=1.0;

 if(iseg>0) 
  segs[iseg-1].right=mid;

 if(iseg<nsegs-1)  
  segs[iseg+1].left=mid;

 nsegs--;
 for(i=iseg; i<nsegs; i++) {
   segs[i].left=segs[i+1].left;
   segs[i].right=segs[i+1].right;
   segs[i].middle=segs[i+1].middle;
   segs[i].color=segs[i+1].color;
   segs[i].flags=segs[i+1].flags;
   segs[i].type=segs[i+1].type;
   segs[i].r0=segs[i+1].r0;
   segs[i].g0=segs[i+1].g0;
   segs[i].b0=segs[i+1].b0;
   segs[i].a0=segs[i+1].a0;
   segs[i].r1=segs[i+1].r1;
   segs[i].g1=segs[i+1].g1;
   segs[i].b1=segs[i+1].b1;
   segs[i].a1=segs[i+1].a1;
 }
 segs=(grad_segment_t*)realloc(segs,sizeof(grad_segment_t)*nsegs);
 
 return iseg;
}

grad_segment_t *Gradient::getseg(int i)
{
 if(i<0 || i>nsegs || nsegs==0 || segs==NULL)
   return NULL;
 return &(segs[i]);
}

double * Gradient::getsegments(int &n)
{
 int i;
 double *slist;

 n=nsegs;

 if(nsegs<=0) 
   return NULL;

 slist=(double *)malloc(sizeof(double)*nsegs*3);
 for(i=0; i<n; i++) {
    slist[i*3]=segs[i].left;
    slist[i*3+1]=segs[i].middle;
    slist[i*3+2]=segs[i].right;
 }
 return slist;
}

int Gradient::updatesegments(int n, double *slist)
{
 int i;

 if(n<=0 || slist==NULL || n!=nsegs)
   return -1;
 for(i=0; i<n; i++) {
    segs[i].left=slist[i*3];
    segs[i].middle=slist[i*3+1];
    segs[i].right=slist[i*3+2];
 }
 return nsegs;
}

int Gradient::split_segment(int iseg, float splitpoint)
{
 int i;
 float mid, middist;
 if(iseg>=nsegs || iseg<0 )
   return -1;
 nsegs++;
 segs=(grad_segment_t*)realloc(segs,sizeof(grad_segment_t)*nsegs);
 for(i=nsegs-1;i>iseg; i--) {
   segs[i].left=segs[i-1].left;
   segs[i].right=segs[i-1].right;
   segs[i].middle=segs[i-1].middle;
   segs[i].color=segs[i-1].color;
   segs[i].flags=segs[i-1].flags;
   segs[i].type=segs[i-1].type;
   segs[i].r0=segs[i-1].r0;
   segs[i].g0=segs[i-1].g0;
   segs[i].b0=segs[i-1].b0;
   segs[i].a0=segs[i-1].a0;
   segs[i].r1=segs[i-1].r1;
   segs[i].g1=segs[i-1].g1;
   segs[i].b1=segs[i-1].b1;
   segs[i].a1=segs[i-1].a1;
  }

 if(segs[iseg].middle<0) {
   mid=(segs[iseg].left+segs[iseg].right)*0.5;
 } else {
   mid=segs[iseg].middle;
 }
 if(splitpoint<=0 || splitpoint>=1.0 ) {
   middist=(segs[iseg].right>segs[iseg].left)?
         (mid-segs[iseg].left)/(segs[iseg].right-segs[iseg].left)
         :0.5;
 } else {
   middist=splitpoint;
 }

 segs[iseg+1].left=mid;
 segs[iseg].right=mid;

 if(segs[iseg].middle>=0) {
  segs[iseg].middle=(segs[iseg].left+segs[iseg].right)*0.5;
  segs[iseg].r1=segs[iseg].r0 + (segs[iseg].r1 - segs[iseg].r0) * middist;
  segs[iseg].g1=segs[iseg].g0 + (segs[iseg].g1 - segs[iseg].g0) * middist;
  segs[iseg].b1=segs[iseg].b0 + (segs[iseg].b1 - segs[iseg].b0) * middist;
  segs[iseg].a1=segs[iseg].a0 + (segs[iseg].a1 - segs[iseg].a0) * middist;
  segs[iseg+1].middle=(segs[iseg+1].left+segs[iseg+1].right)*0.5;
  segs[iseg+1].r0=segs[iseg].r1;
  segs[iseg+1].g0=segs[iseg].g1;
  segs[iseg+1].b0=segs[iseg].b1;
  segs[iseg+1].a0=segs[iseg].a1;
  segs[iseg].flags|=GRAD_RIGHT;
  segs[iseg+1].flags|=GRAD_LEFT;
 } 
 return iseg;
}

int Gradient::setColor(int iseg, int side, float r,float g, float b, float a,
  char fl)
{
 if(iseg<0 || iseg>=nsegs || 0==nsegs || segs==NULL) 
   return FALSE;
 
 if(fl!=-1) {
   segs[iseg].flags=fl;
   if(iseg>0 && (fl & GRAD_LEFT ))  {
      segs[iseg-1].flags|=GRAD_RIGHT;
   }
   if(iseg<nsegs-1 && (segs[iseg].flags & GRAD_RIGHT)) {
      segs[iseg+1].flags|=GRAD_LEFT;
   }
 } 
 switch(side) {
    case GRAD_LEFT: 
      segs[iseg].r0=r;
      segs[iseg].g0=g;
      segs[iseg].b0=b;
      segs[iseg].a0=a;
      if(iseg>0 && (segs[iseg].flags & GRAD_LEFT)) {
        segs[iseg-1].r1=r;
        segs[iseg-1].g1=g;
        segs[iseg-1].b1=b;
        segs[iseg-1].a1=a;
      }
      break;
    case GRAD_RIGHT:
      segs[iseg].r1=r;
      segs[iseg].g1=g;
      segs[iseg].b1=b;
      segs[iseg].a1=a;
      if(iseg<nsegs-1 && (segs[iseg].flags & GRAD_RIGHT)) {
        segs[iseg+1].r0=r;
        segs[iseg+1].g0=g;
        segs[iseg+1].b0=b;
        segs[iseg+1].a0=a;
      }
      break;
    default:
      return FALSE;
      break;
 }
 return TRUE;
}

int Gradient::getColor(float T,float &r,float &g,float &b,float &a)
{
 int ns;
 grad_segment_t *seg;
 float pos;
 float factor;
 float middle,seg_len;
 //pos=(T-Tmin)/(Tmax-Tmin);
 pos=T*Tdist;
 pos=D_FCLAMP(pos,0.0,1.0);

 for(ns=0,seg=segs; ns<nsegs && !((seg->left)<=pos && pos<=(seg->right)); ns++, seg++) {
//   fprintf(stderr,"ns %d seg %f %f %f ",ns,seg->left,seg->middle, seg->right);
 }
  seg_len = seg->right - seg->left;

	if (seg_len < 1e-5) {
		middle = 0.5;
		pos    = 0.5;
	} else {
		middle = (seg->middle - seg->left) / seg_len;
		pos    = (pos - seg->left) / seg_len;
	} /* else */


  if (pos <= middle) {
		  if (middle < 1e-5)
		 	  factor=0.0;
		  else
			  factor= 0.5 * pos / middle;
    } else {
		   pos -= middle;
		   middle = 1.0 - middle;
		   if (middle < 1e-5)
			   factor=1.0;
		   else
			   factor= 0.5 + 0.5 * pos / middle;
    }
    a = seg->a0 + (seg->a1 - seg->a0) * factor;
    r = seg->r0 + (seg->r1 - seg->r0) * factor;
    g = seg->g0 + (seg->g1 - seg->g0) * factor;
    b = seg->b0 + (seg->b1 - seg->b0) * factor;

  return TRUE;
}
