#include <stdio.h>
#include <stdlib.h>
#include <FL/x.H>
#include  "gradientui.h"

int g_input_type=GRAD_AUTO,
    g_out_type=GRAD_AUTO,
    show_gui=1;
    
char *inputfilename=NULL,
     *outputfilename=NULL;
Gradient cgradient;
  
//**************************************************************************
//    Command Line processing Section

void showCmdLineHelp(void)
{
 fprintf(stderr,"YAGE - Yet anoter Gradient Editor -- by Daniel Barrero\n");
 fprintf(stderr,"Usage: yage [-h] [filename | [-if filename]] [-ti gimp|vu|auto] [-to gimp|vu|auto ] [ -of filename ]\n");
 fprintf(stderr, " the -of option converts between fileformats without loading the GUI\n");
}

//------
// cleanup the comand line arguments from removing the ones already processed
// to be able to pass a set of valid command line arguments to VU 
// initialization functions.
void removeArgs(int *argcp, char **argv, int numToRemove)
{
  int i, j;
  for (i = 0, j = numToRemove; argv[j]; i++, j++) {
    argv[i] = argv[j];
  }
  argv[i] = NULL;
  *argcp -= numToRemove;
}

//-------
//-------
// Parse command line arguments to initialize local client options
int parseCmdLine(int *argcp,char **argv)
{
  int nargs,i,j;
  char **oargv;

// Make internal copy of comand line arguments
  nargs=*argcp;
  if(nargs==1) 
    return 1;
  
  oargv= new char *[nargs];
  if(!oargv) {
    fprintf(stderr,"Error out of memory\n");
  }

  for(i=0;i<nargs; i++) {
    oargv[i]=strdup(argv[i]);
    if(!oargv[i]) {
      fprintf(stderr,"Error out of memory\n");
    }
  }

  if(nargs==2) {
    if(argv[1][0]!='-') {
      inputfilename=strdup(oargv[1]);
      removeArgs(argcp, &argv[1],1);
    } else if(!strcmp(argv[1],"-h")) {
       showCmdLineHelp();
       fprintf(stderr,"to control the appearance of the UI the options are:\n");
       fprintf(stderr," [-bg2 color] [-bg color] [-di[splay] host:n.n]");
       fprintf(stderr," [-fg color] [-i[conic]]");
       fprintf(stderr," [ -g[eometry] WxH+X+Y]");
       fprintf(stderr," [-not[ooltips]]");
       fprintf(stderr," [-s[cheme] scheme] [-to[oltips]]\n");
       return 0;
    } 
    return 1;
  }

// Process arguments for options
  for( i=j=1; i<nargs; i++) {
     if(!strcmp(oargv[i],"-ti")) {
         if ( ((i+1) >= nargs) || 
              ((i+1)<nargs &&  (oargv[i+1][0]=='-' || oargv[i+1][0]=='+') )
            ) {
            fprintf(stderr,"Error: explicit gradient input type option missing parameter\n");
            showCmdLineHelp();
            return 0; 
         } else {
           i++;  
           if(strcmp(oargv[i], "gimp"))
              g_input_type=GRAD_GIMP;
           else if(strcmp(oargv[i], "vu"))
              g_input_type=GRAD_VU;
           else if(strcmp(oargv[i], "auto"))
              g_input_type=GRAD_AUTO;
           else {
            fprintf(stderr,"Error: unknown gradient filetype\n");
            showCmdLineHelp();
            return 0; 
           }
           removeArgs(argcp, &argv[j],2);
         }  
     } if(!strcmp(oargv[i],"-h")) {
           showCmdLineHelp();
            return 0;  
     } if(!strcmp(oargv[i],"-to")) {
         if ( ((i+1) >= nargs) || 
              ((i+1)<nargs &&  (oargv[i+1][0]=='-' || oargv[i+1][0]=='+') )
            ) {
            fprintf(stderr,"Error: explicit gradient output type option missing parameter\n");
            showCmdLineHelp();
            return 0; 
         } else {
           i++;  
           if(strcmp(oargv[i], "gimp"))
              g_input_type=GRAD_GIMP;
           else if(strcmp(oargv[i], "vu"))
              g_input_type=GRAD_GIMP;
           else {
            fprintf(stderr,"Error: unknown ouput gradient filetype\n");
            showCmdLineHelp();
            return 0; 
           }
           removeArgs(argcp, &argv[j],2);
         }  
     } if(!strcmp(oargv[i],"-if")) {
         if ( ((i+1) >= nargs) || 
              ((i+1)<nargs &&  (oargv[i+1][0]=='-' || oargv[i+1][0]=='+') )
            ) {
            fprintf(stderr,"Error: missing input filename\n");
            showCmdLineHelp();
            return 0; 
         } else {
           i++;  
           inputfilename=strdup(oargv[i]);
           removeArgs(argcp, &argv[j],2);
         }  
     } if(!strcmp(oargv[i],"-of")) {
         if ( ((i+1) >= nargs) || 
              ((i+1)<nargs &&  (oargv[i+1][0]=='-' || oargv[i+1][0]=='+') )
            ) {
            fprintf(stderr,"Error: missing output filename\n");
            showCmdLineHelp();
            return 0; 
         } else {
           i++;  
           outputfilename=strdup(oargv[i]);
           removeArgs(argcp, &argv[j],2);
           show_gui=0;
         }  
     } else {
       j++;
     }
  }
  if(((outputfilename!=NULL && g_out_type==GRAD_AUTO) 
     || (outputfilename==NULL && g_out_type!=GRAD_AUTO))) {
    fprintf(stderr,"Error: invalid combination of parameters\n");
    showCmdLineHelp();
    return 0;
  }
  return 1;
}
// End Command Line Processing Section
//--------------------------------------------------------------------------

#include "icon.xbm"

int main(int argc,char **argv)
{
 if(!parseCmdLine(&argc,argv) )
    return 0;

 if(show_gui) {
   if( inputfilename !=NULL ) {
     if(!cgradient.load(inputfilename,g_input_type)) {
         fprintf(stderr,"Error Can't open input file: %s\n",inputfilename);
         return 0;
     }
   }
   make_window(&cgradient);
#ifdef _WIN32

#else
   Pixmap p = XCreateBitmapFromData(fl_display, DefaultRootWindow(fl_display),
                                 icon_bits, icon_width, icon_height);
   yagemainwin->icon((char *)p);
#endif
   yagemainwin->show(argc,argv);
   return Fl::run();
 } else if(inputfilename!=NULL && outputfilename!=NULL && g_input_type!=g_out_type && g_out_type!=GRAD_AUTO ){
   if( inputfilename !=NULL ) {
     if(!cgradient.load(inputfilename,g_input_type)) {
         fprintf(stderr,"Error Can't open input: file: %s\n",inputfilename);
         return 0;
     }
   }
   char tmpbuf[1024];
   strcpy(tmpbuf, inputfilename);
   strcat(tmpbuf, (g_out_type==GRAD_GIMP)?"_yage.gimpg":"_yage.vug");
   if(!cgradient.save(tmpbuf, g_out_type)) {
      fprintf(stderr,"Error Can't create output: file %s\n",tmpbuf);
      return 0;
   } else {
      fprintf(stderr,"Conversion succesful, Created output file: %s\n",tmpbuf);
   }
 } else {
   showCmdLineHelp();
 }
 return 0;
}
